@@ -54,43 +54,6 @@ ConverterTest::ConverterTest()
 {
 }
 
-namespace convertertest
-{
-
-const MyMoneyMoney moCheckingOpen(1418.0);
-const MyMoneyMoney moCreditOpen(-418.0);
-const MyMoneyMoney moZero(0.0);
-const MyMoneyMoney moSolo(234.12);
-const MyMoneyMoney moParent1(88.01);
-const MyMoneyMoney moParent2(133.22);
-const MyMoneyMoney moParent(moParent1+moParent2);
-const MyMoneyMoney moChild(14.00);
-const MyMoneyMoney moThomas(5.11);
-const MyMoneyMoney moNoPayee(8944.70);
-
-QCString acAsset;
-QCString acLiability;
-QCString acExpense;
-QCString acIncome;
-QCString acChecking;
-QCString acCredit;
-QCString acSolo;
-QCString acParent;
-QCString acChild;
-QCString acForeign;
-QCString acCanChecking;
-QCString acJpyChecking;
-QCString acCanCash;
-QCString acJpyCash;
-QCString inBank;
-QCString eqStock1;
-QCString eqStock2;
-QCString acInvestment;
-QCString acStock1;
-QCString acStock2;
-QCString acDividends;
-
-}
 using namespace convertertest;
 
 void ConverterTest::setUp () {
@@ -99,6 +62,8 @@ void ConverterTest::setUp () {
   file = MyMoneyFile::instance();
   file->attachStorage(storage);
 
+  MyMoneyFileTransaction ft;
+
   file->addCurrency(MyMoneySecurity("CAD", "Canadian Dollar",        "C$"));
   file->addCurrency(MyMoneySecurity("USD", "US Dollar",              "$"));
   file->addCurrency(MyMoneySecurity("JPY", "Japanese Yen",           QChar(0x00A5), 100, 1));
@@ -114,31 +79,28 @@ void ConverterTest::setUp () {
   acLiability = (MyMoneyFile::instance()->liability().id());
   acExpense = (MyMoneyFile::instance()->expense().id());
   acIncome = (MyMoneyFile::instance()->income().id());
-  acChecking = makeAccount(QString("Checking Account"),MyMoneyAccount::Checkings,moCheckingOpen,QDate(2004,5,15),acAsset);
-  acCredit = makeAccount(QString("Credit Card"),MyMoneyAccount::CreditCard,moCreditOpen,QDate(2004,7,15),acLiability);
-  acSolo = makeAccount(QString("Solo"),MyMoneyAccount::Expense,0,QDate(2004,1,11),acExpense);
-  acParent = makeAccount(QString("Parent"),MyMoneyAccount::Expense,0,QDate(2004,1,11),acExpense);
-  acChild = makeAccount(QString("Child"),MyMoneyAccount::Expense,0,QDate(2004,2,11),acParent);
-  acForeign = makeAccount(QString("Foreign"),MyMoneyAccount::Expense,0,QDate(2004,1,11),acExpense);
+  acChecking = makeAccount("Checking Account",MyMoneyAccount::Checkings,moConverterCheckingOpen,QDate(2004,5,15),acAsset);
+  acCredit = makeAccount("Credit Card",MyMoneyAccount::CreditCard,moConverterCreditOpen,QDate(2004,7,15),acLiability);
+  acSolo = makeAccount("Solo",MyMoneyAccount::Expense,0,QDate(2004,1,11),acExpense);
+  acParent = makeAccount("Parent",MyMoneyAccount::Expense,0,QDate(2004,1,11),acExpense);
+  acChild = makeAccount("Child",MyMoneyAccount::Expense,0,QDate(2004,2,11),acParent);
+  acForeign = makeAccount("Foreign",MyMoneyAccount::Expense,0,QDate(2004,1,11),acExpense);
   
   MyMoneyInstitution i("Bank of the World","","","","","","");
-  MyMoneyKeyValueContainer ofxsettings;
-  ofxsettings.setValue("iban","123456789");
-  i.setOfxConnectionSettings(ofxsettings);
   file->addInstitution(i);
   inBank = i.id();
+  ft.commit();
 }
 
 void ConverterTest::tearDown ()
 {
   file->detachStorage(storage);
-  delete file;
   delete storage;
 }
 
 void ConverterTest::testOfxImport(void)
 {
-// These tests don't work now that the OFX importing logic is in a plugin
+// These tests don't work now that the OFX importing logic is in a plugin.
 #if 0
   // Of course, the TRUE test will be to import these back through 
   // MyMoneyStatementReader into transactions, HOWEVER, that class currently
