@@ -25,7 +25,6 @@ email                : mte@users.sourceforge.net
 #include <qfile.h>
 #include <qmap.h>
 #include <qobject.h>
-#include <qmessagebox.h>
 #include <qfiledialog.h>
 #include <qinputdialog.h>
 #include <qdatetime.h>
@@ -52,7 +51,7 @@ email                : mte@users.sourceforge.net
   #include "../mymoney/mymoneyprice.h"
   #include "../dialogs/kgncimportoptionsdlg.h"
   #include "../dialogs/kgncpricesourcedlg.h"
-  #include "../dialogs/ieditscheduledialog.h"
+  #include "../dialogs/keditscheduledlg.h"
   #include "../widgets/kmymoneyedit.h"
   #define TRY try {
   #define CATCH } catch (MyMoneyException *e) {
@@ -81,7 +80,7 @@ void MyMoneyGncReader::setOptions () {
     // set users input options
     m_dropSuspectSchedules = dlg.scheduleOption();
     m_investmentOption = dlg.investmentOption();
-//    m_useFinanceQuote = dlg.quoteOption(); not implemented in 0.8
+    m_useFinanceQuote = dlg.quoteOption();
     m_decoder = dlg.decodeOption();
     gncdebug = dlg.generalDebugOption();
     xmldebug = dlg.xmlDebugOption();
@@ -113,8 +112,8 @@ void GncObject::checkVersion (const QString& elName, const QXmlAttributes& elAtt
   TRY
   if (map.contains(elName)) { // if it's not in the map, there's nothing to check
     if (!map[elName].contains(elAttrs.value("version"))) {
-      QString em = i18n("checkVersion: Sorry. This importer cannot handle version %1 of element %2")
-                          .arg(elAttrs.value("version")).arg(elName);
+      QString em = i18n("%1: Sorry. This importer cannot handle version %2 of element %3")
+                          .arg(__func__).arg(elAttrs.value("version")).arg(elName);
       throw new MYMONEYEXCEPTION (em);
     }
   }
@@ -220,7 +219,7 @@ QString GncObject::hide (QString data, unsigned int anonClass) {
     mresult  = MyMoneyMoney(m_moneyHideFactor) * in;
     mresult.convert(10000);
     mresult.setThousandSeparator (' ');
-    result = mresult.formatMoney();
+    result = mresult.formatMoney("", 2);
     break;
   }
   return (result);
@@ -886,8 +885,14 @@ bool XmlReader::startElement (const QString&, const QString&, const QString& elN
     if (elName == "gnc:book" || elName == "gnc:count-data" || elName == "book:id") lastType = -1;
     pMain->oStream << endl;
     switch (lastType) {
-    case 0: indentCount += 2;
-    case 2: spaces.fill (' ', indentCount); pMain->oStream << spaces.latin1(); break;
+      case 0:
+        indentCount += 2;
+        // tricky fall through here
+
+      case 2:
+        spaces.fill (' ', indentCount);
+        pMain->oStream << spaces.latin1();
+        break;
     }
     pMain->oStream << '<' << elName;
     for (i = 0; i < elAttrs.count(); i++) {
@@ -908,22 +913,27 @@ bool XmlReader::startElement (const QString&, const QString&, const QString& elN
       m_co = m_os.top();
       m_co->setVersion(elAttrs.value("version"));
       m_co->setPm (pMain); // pass the 'main' pointer to the sub object
-      return (true);
+      // return true;   // removed, as we hit a return true anyway
     }
+#if 0
     // check for a data element
-    if (m_co->isDataElement (elName, elAttrs)) return (true);
-    return (true);
+    if (m_co->isDataElement (elName, elAttrs))
+      return (true);
+#endif
+    else {
+      // reduced the above to
+      m_co->isDataElement(elName, elAttrs);
+    }
   } catch (MyMoneyException *e) {
 #ifndef _GNCFILEANON
     // we can't pass on exceptions here coz the XML reader won't catch them and we just abort
-    QMessageBox::critical (0, PACKAGE, i18n("Import failed\n\n") + e->what(),
-                           QMessageBox::Abort, QMessageBox::NoButton , QMessageBox::NoButton);
+    KMessageBox::error(0, i18n("Import failed:\n\n%1").arg(e->what()), PACKAGE);
     qFatal ("%s", e->what().latin1());
 #else
     qFatal ("%s", e->latin1());
 #endif // _GNCFILEANON
   }
-  return (true); // to keep compiler happy
+  return true; // to keep compiler happy
 }
 
 bool XmlReader::endElement( const QString&, const QString&, const QString&elName ) {
@@ -952,8 +962,7 @@ bool XmlReader::endElement( const QString&, const QString&, const QString&elName
   } catch (MyMoneyException *e) {
 #ifndef _GNCFILEANON
     // we can't pass on exceptions here coz the XML reader won't catch them and we just abort
-    QMessageBox::critical (0, PACKAGE, i18n("Import failed\n\n") + e->what(),
-                           QMessageBox::Abort, QMessageBox::NoButton , QMessageBox::NoButton);
+    KMessageBox::error(0, i18n("Import failed:\n\n%1").arg(e->what()), PACKAGE);
     qFatal ("%s", e->what().latin1());
 #else
     qFatal ("%s", e->latin1());
@@ -1036,15 +1045,14 @@ void MyMoneyGncReader::readFile(QIODevice* pDevice, IMyMoneySerialize* storage)
   if (bAnonymize) setFileHideFactor ();
   //m_defaultPayee = createPayee (i18n("Unknown payee"));
 
-//  MyMoneyFileTransaction ft;
+  MyMoneyFileTransaction ft;
   m_xr = new XmlReader (this);
   try {
     m_xr->processFile (pDevice);
     terminate (); // do all the wind-up things
-//    ft.commit();
+    ft.commit();
   } catch (MyMoneyException *e) {
-    QMessageBox::critical (0, PACKAGE, i18n("Import failed\n\n") + e->what(),
-                           QMessageBox::Abort, QMessageBox::NoButton , QMessageBox::NoButton);
+    KMessageBox::error(0, i18n("Import failed:\n\n%1").arg(e->what()), PACKAGE);
     qFatal ("%s", e->what().latin1());
   } // end catch
   signalProgress (0, 1, i18n("Import complete")); // switch off progress bar
@@ -1329,6 +1337,13 @@ void MyMoneyGncReader::convertTransaction (const GncTransaction *gtx) {
   for (i = 0; i < gtx->splitCount(); i++) {
     convertSplit (static_cast<const GncSplit *>(gtx->getSplit (i)));
   }
+  // handle the odd case of just one split, which gnc allows,
+  // by just duplicating the split
+  // of course, we should change the sign but this case has only ever been seen
+  // when the balance is zero, and can cause kmm to crash, so...
+  if (gtx->splitCount() == 1) {
+    convertSplit (static_cast<const GncSplit *>(gtx->getSplit (0)));
+  }
   m_splitList += m_liabilitySplitList += m_otherSplitList;
   // the splits are in order in splitList. Link them to the tx. also, determine the
   // action type, and fill in some fields which gnc holds at transaction level
@@ -1707,7 +1722,7 @@ void MyMoneyGncReader::convertSchedule (const GncSchedule *gsc) {
   } else {
     tx = convertTemplateTransaction (sc.name(), *itt);
   }
-  tx.setId(QString());
+  tx.clearId();
 
 // define the conversion table for intervals
   struct convIntvl {
@@ -1726,10 +1741,13 @@ void MyMoneyGncReader::convertSchedule (const GncSchedule *gsc) {
                            {"once", 'o', 1, MyMoneySchedule::OCCUR_ONCE, MyMoneySchedule::MoveNothing },
                            {"daily" , 'd', 1, MyMoneySchedule::OCCUR_DAILY, MyMoneySchedule::MoveNothing },
                            //{"daily_mf", 'd', 1, MyMoneySchedule::OCCUR_DAILY, MyMoneySchedule::MoveMonday }, doesn't work, need new freq in kmm
+                           {"30-days" , 'd', 30, MyMoneySchedule::OCCUR_EVERYTHIRTYDAYS, MyMoneySchedule::MoveNothing },
                            {"weekly", 'w', 1, MyMoneySchedule::OCCUR_WEEKLY, MyMoneySchedule::MoveNothing },
                            {"bi_weekly", 'w', 2, MyMoneySchedule::OCCUR_EVERYOTHERWEEK, MyMoneySchedule::MoveNothing },
+                           {"three-weekly", 'w', 3, MyMoneySchedule::OCCUR_EVERYTHREEWEEKS, MyMoneySchedule::MoveNothing },
                            {"four-weekly", 'w', 4, MyMoneySchedule::OCCUR_EVERYFOURWEEKS,
                            MyMoneySchedule::MoveNothing },
+                           {"eight-weekly", 'w', 8, MyMoneySchedule::OCCUR_EVERYEIGHTWEEKS, MyMoneySchedule::MoveNothing },
                            {"monthly", 'm', 1, MyMoneySchedule::OCCUR_MONTHLY, MyMoneySchedule::MoveNothing },
                            {"two-monthly", 'm', 2, MyMoneySchedule::OCCUR_EVERYOTHERMONTH,
                             MyMoneySchedule::MoveNothing },
@@ -1878,7 +1896,7 @@ void MyMoneyGncReader::terminate () {
   signalProgress (0, m_accountCount, i18n ("Reorganizing accounts..."));
   QValueList<MyMoneyAccount> list;
   QValueList<MyMoneyAccount>::Iterator acc;
-  list = m_storage->accountList();
+  m_storage->accountList(list);
   for (acc = list.begin(); acc != list.end(); ++acc) {
     if ((*acc).parentAccountId() == m_storage->asset().id()) {
       MyMoneyAccount assets = m_storage->asset();
@@ -1959,39 +1977,41 @@ void MyMoneyGncReader::terminate () {
   bool exit = false;
   for (i = 0; (i < sectionsToReport.count()) && !exit; i++) {
     QString button0Text = i18n("More");
-    if (i + 1 == sectionsToReport.count()) button0Text = i18n("Done"); // last section
-    switch (QMessageBox::information (0, PACKAGE,
-                                      buildReportSection (*sectionsToReport.at(i)),
-                                      button0Text, i18n("Save Report"), i18n("Cancel"),
-                                      0, 2))    // Enter == button 0, Escape == button 2
-    {
-    case 0:
-      break; // more
-    case 1:
-      exit = writeReportToFile (sectionsToReport);
-      break;
-    case 2:   // Cancel clicked or Escape pressed
-      exit = true;
-      break;
+    if (i + 1 == sectionsToReport.count())
+      button0Text = i18n("Done"); // last section
+    KGuiItem yesItem(button0Text, QIconSet(), "", "");
+    KGuiItem noItem(i18n("Save Report"), QIconSet(), "", "");
+
+    switch(KMessageBox::questionYesNoCancel(0,
+           buildReportSection (*sectionsToReport.at(i)),
+                                PACKAGE,
+                                yesItem, noItem)) {
+      case KMessageBox::Yes:
+        break;
+      case KMessageBox::No:
+        exit = writeReportToFile (sectionsToReport);
+        break;
+      default:
+        exit = true;
+        break;
     }
   }
+
   for (i = 0; i < m_suspectList.count(); i++) {
     MyMoneySchedule sc = m_storage->schedule(m_suspectList[i]);
-    QMessageBox mb (PACKAGE,
-            i18n(QString("Problems were encountered in converting schedule '%1'.\n"
-                "Do you want to review or edit it now?").arg(sc.name())),
-                    QMessageBox::Question,
-                    QMessageBox::Yes | QMessageBox::Default,
-                    QMessageBox::No | QMessageBox::Escape,
-                    QMessageBox::NoButton);
-    switch (mb.exec()) {
-    case QMessageBox::Yes:
-         KEditScheduleDialog *s = new  KEditScheduleDialog (sc.transaction().splits()[0].action(), sc, 0, "");
+      KEditScheduleDlg *s;
+      switch(KMessageBox::warningYesNo(0, i18n("Problems were encountered in converting schedule '%1'.\nDo you want to review or edit it now?").arg(sc.name()), PACKAGE)) {
+      case KMessageBox::Yes:
+        s = new KEditScheduleDlg (sc);
         // FIXME: connect newCategory to something useful, so that we
         // can create categories from within the dialog
         if (s->exec())
           m_storage->modifySchedule (s->schedule());
         delete s;
+        break;
+
+      default:
+        break;
     }
   }
   PASS
@@ -2041,8 +2061,8 @@ const QString MyMoneyGncReader::buildReportSection (const QString source) {
     for (i = 0; i < m_messageList.count(); i++) {
       GncMessageArgs *m = m_messageList.at(i);
       if (m->source == source) {
-        if (gncdebug) qDebug(QString("build text source %1, code %2, argcount %3")
-              .arg(m->source).arg(m->code).arg(m->args.count()));
+        if (gncdebug) qDebug("%s", QString("build text source %1, code %2, argcount %3")
+              .arg(m->source).arg(m->code).arg(m->args.count()).data());
         QString ss = GncMessages::text (m->source, m->code);
         // add variable args. the .arg function seems always to replace the
         // lowest numbered placeholder it finds, so translating messages
@@ -2062,10 +2082,16 @@ bool MyMoneyGncReader::writeReportToFile (const QValueList<QString> sectionsToRe
   unsigned int i;
   QFileDialog* fd = new QFileDialog (0, "Save report as", TRUE);
   fd->setMode (QFileDialog::AnyFile);
-  if (fd->exec() != QDialog::Accepted) return (false);
+  if (fd->exec() != QDialog::Accepted) {
+    delete fd;
+    return (false);
+  }
   QFile reportFile(fd->selectedFile());
   QFileInfo fi (reportFile);
-  if (!reportFile.open (IO_WriteOnly)) return (false);
+  if (!reportFile.open (IO_WriteOnly))  {
+    delete fd;
+    return (false);
+  }
   QTextStream stream (&reportFile);
   for (i = 0; i < sectionsToReport.count(); i++) {
     stream << buildReportSection (*sectionsToReport.at(i)).latin1() << endl;
@@ -2216,7 +2242,7 @@ void MyMoneyGncReader::checkInvestmentOption (QString stockId) {
     QStringList accList;
     QValueList<MyMoneyAccount> list;
     QValueList<MyMoneyAccount>::Iterator acc;
-    list = m_storage->accountList();
+    m_storage->accountList(list);
     // build a list of candidates for the input box
     for (acc = list.begin(); acc != list.end(); ++acc) {
       //      if (((*acc).accountGroup() == MyMoneyAccount::Asset) && ((*acc).accountType() != MyMoneyAccount::Stock)) accList.append ((*acc).name());
@@ -2247,6 +2273,7 @@ void MyMoneyGncReader::checkInvestmentOption (QString stockId) {
           ok = true;
         } else {
           // this code is probably not going to be implemented coz we can't change account types (??)
+#if 0
           QMessageBox mb (PACKAGE,
                           i18n ("%1 is not an Investment Account. Do you wish to make it one?").arg(invAcc.name()),
                           QMessageBox::Question,
@@ -2262,6 +2289,17 @@ void MyMoneyGncReader::checkInvestmentOption (QString stockId) {
             ok = true;
             break;
           }
+#endif
+          switch(KMessageBox::questionYesNo(0, i18n ("%1 is not an Investment Account. Do you wish to make it one?").arg(invAcc.name(), PACKAGE))) {
+            case KMessageBox::Yes:
+              // convert it - but what if it has splits???
+              qFatal ("Not yet implemented");
+              ok = true;
+              break;
+            default:
+              ok = false;
+              break;
+          }
         }
       } // end if ok - user pressed Cancel
     } // end while !ok
@@ -2341,9 +2379,8 @@ void MyMoneyGncReader::postMessage (const QString source, const unsigned int cod
   // get the number of args this message requires
   const unsigned int argCount = GncMessages::argCount (source, code);
   if ((gncdebug) && (argCount != argList.count()))
-    qDebug
-    (QString("MyMoneyGncReader::postMessage debug: Message %1, code %2, requires %3 arguments, got %4")
-        .arg(source).arg(code).arg(argCount).arg(argList.count()));
+    qDebug("%s", QString("MyMoneyGncReader::postMessage debug: Message %1, code %2, requires %3 arguments, got %4")
+        .arg(source).arg(code).arg(argCount).arg(argList.count()).data());
   // store the arguments
   for (i = 0; i < argCount; i++) {
     if (i > argList.count()) m->args.append(QString());
