@@ -1,5 +1,5 @@
 /***************************************************************************
-                          mymoneyqifreader.cpp 
+                          mymoneyqifreader.cpp
                              -------------------
     begin                : Mon Jan 27 2003
     copyright            : (C) 2000-2003 by Michael Edwardes
@@ -39,6 +39,7 @@
 #include <kconfig.h>
 #include <kdebug.h>
 #include <kprogress.h>
+#include <kio/netaccess.h>
 
 // ----------------------------------------------------------------------------
 // Project Headers
@@ -89,9 +90,9 @@ void MyMoneyQifReader::setAutoCreatePayee(const bool create)
   m_autoCreatePayee = create;
 }
 
-void MyMoneyQifReader::setFilename(const QString& name)
+void MyMoneyQifReader::setURL(const KURL& url)
 {
-  m_filename = name;
+  m_url = url;
 }
 
 void MyMoneyQifReader::setProfile(const QString& profile)
@@ -140,17 +141,17 @@ void MyMoneyQifReader::slotReceivedDataFromFilter(KProcess* /* proc */, char *bu
 void MyMoneyQifReader::slotProcessBuffers(void)
 {
   QValueList<QByteArray>::Iterator it = m_data.begin();
-  while(!m_userAbort && it != m_data.end()) 
+  while(!m_userAbort && it != m_data.end())
   {
     QBuffer databuffer(*it);
     databuffer.open( IO_ReadOnly );
-    
-    while (!m_userAbort && ! databuffer.atEnd() )
+
+    while ( ! m_userAbort && ! databuffer.atEnd() )
     {
       QCString line(80);
       databuffer.readLine( line.data(), line.size() );
       m_qifLine += QString::fromUtf8(line);  // or fromLocal8Bit?
-      
+
       if ( m_qifLine.right(1) == "\n" )
       {
         m_qifLine.remove(QRegExp("[\r\n]"));
@@ -158,11 +159,11 @@ void MyMoneyQifReader::slotProcessBuffers(void)
         m_qifLine = QCString();
       }
     }
-    
+
     databuffer.close();
 
     // remove this chunk from the list.  In case ANOTHER chunk came in while
-    // we were doing this operation, we can't just ++it and clear() at the end.    
+    // we were doing this operation, we can't just ++it and clear() at the end.
     it = m_data.remove(it);
   }
 
@@ -210,9 +211,17 @@ const bool MyMoneyQifReader::startImport(void)
   m_userAbort = false;
   m_pos = 0;
   m_linenumber = 0;
-
+  m_filename = QString::null;
   m_data.clear();
 
+  if(!KIO::NetAccess::download(m_url, m_filename, NULL)) {
+    KMessageBox::detailedError(0,
+                               i18n("Error while loading file '%1'!").arg(m_url.prettyURL()),
+                               KIO::NetAccess::lastErrorString(),
+                               i18n("File access error"));
+    return false;
+  }
+
   m_file = new QFile(m_filename);
   if(m_file->open(IO_ReadOnly)) {
 
@@ -303,6 +312,11 @@ const bool MyMoneyQifReader::finishImport(void)
   }
 #endif
 
+  // if a temporary file was constructed by NetAccess::download,
+  // then it will be removed with the next call. Otherwise, it
+  // stays untouched on the local filesystem
+  KIO::NetAccess::removeTempFile(m_filename);
+
   // Add the transaction entries
   KProgressDialog dlg(0,"transactionaddprogress",i18n("Adding transactions"),i18n("Now adding the transactions to your ledger..."));
   dlg.progressBar()->setTotalSteps(m_transactionCache.count());
@@ -311,21 +325,30 @@ const bool MyMoneyQifReader::finishImport(void)
   dlg.show();
   kapp->processEvents();
   MyMoneyFile* file = MyMoneyFile::instance();
-  file->suspendNotify(true);
   QValueList<MyMoneyTransaction>::iterator it = m_transactionCache.begin();
-  while( it != m_transactionCache.end() )
+  MyMoneyFileTransaction ft;
+  try
   {
-    if ( dlg.wasCancelled() )
+    while( it != m_transactionCache.end() )
     {
-      m_userAbort = true;
-      rc = false;
-      break;
+      if ( dlg.wasCancelled() )
+      {
+        m_userAbort = true;
+        rc = false;
+        break;
+      }
+      file->addTransaction(*it);
+      dlg.progressBar()->advance(1);
+      ++it;
     }
-    file->addTransaction(*it);
-    dlg.progressBar()->advance(1);
-    ++it;
+    if(rc)
+      ft.commit();
+  } catch(MyMoneyException *e) {
+    KMessageBox::detailedSorry(0, i18n("Unable to add transactions"),
+    (e->what() + " " + i18n("thrown in") + " " + e->file()+ ":%1").arg(e->line()));
+    delete e;
+    rc = false;
   }
-  file->suspendNotify(false);
 
   return rc;
 }
@@ -406,14 +429,15 @@ void MyMoneyQifReader::processQifEntry(void)
     } else {
       // Process entry of same type
       switch(m_entryType) {
-        case EntryUnknown:
-          kdDebug(2) << "Line " << m_linenumber << ":Found an entry without a type being specified. Entry skipped." << endl;
-          break;
-
         case EntryCategory:
           processCategoryEntry();
           break;
 
+        case EntryUnknown:
+          kdDebug(2) << "Line " << m_linenumber << ": Warning: Found an entry without a type being specified. Checking assumed." << endl;
+          processTransactionEntry();
+          break;
+
         case EntryTransaction:
           processTransactionEntry();
           break;
@@ -426,7 +450,7 @@ void MyMoneyQifReader::processQifEntry(void)
           m_account = MyMoneyAccount();
           processAccountEntry();
           break;
-          
+
         case EntrySecurity:
           processSecurityEntry();
           break;
@@ -438,7 +462,7 @@ void MyMoneyQifReader::processQifEntry(void)
         case EntryMemorizedTransaction:
           kdDebug(2) << "Line " << m_linenumber << ": Memorized transactions are not yet implemented!" << endl;
           break;
-        
+
         default:
           kdDebug(2) << "Line " << m_linenumber<< ": EntryType " << m_entryType <<" not yet implemented!" << endl;
           break;
@@ -482,7 +506,6 @@ void MyMoneyQifReader::processMSAccountEntry(const MyMoneyAccount::accountTypeE
     m_account.setAccountType(accountType);
     QString txt = extractLine('T');
     MyMoneyMoney balance = m_qifProfile.value('T', txt);
-    m_account.setOpeningBalance(balance);
 
     QDate date = m_qifProfile.date(extractLine('D'));
     m_account.setOpeningDate(date);
@@ -492,34 +515,36 @@ void MyMoneyQifReader::processMSAccountEntry(const MyMoneyAccount::accountTypeE
       name = name.mid(1, name.length()-2);
     }
     m_account.setName(name);
-    selectOrCreateAccount(Select, m_account);
-       
+    selectOrCreateAccount(Select, m_account, balance);
+
     if ( ! balance.isZero() )
-    {          
+    {
       MyMoneyFile* file = MyMoneyFile::instance();
       QCString openingtxid = file->openingBalanceTransaction(m_account);
+      MyMoneyFileTransaction ft;
       if ( ! openingtxid.isEmpty() )
       {
         MyMoneyTransaction openingtx = file->transaction(openingtxid);
         MyMoneySplit split = openingtx.splitByAccount(m_account.id());
-        
+
         if ( split.shares() != balance )
         {
+          const MyMoneySecurity& sec = file->security(m_account.currencyId());
           if ( KMessageBox::questionYesNo(
             qApp->mainWidget(),
-            i18n("The %1 account currently has an opening balance of %2. This QIF file reports an opening balance of %3. Would you like to overwrite the current balance with the one from the QIF file?").arg(m_account.name(),split.shares().formatMoney(),balance.formatMoney()),
+            i18n("The %1 account currently has an opening balance of %2. This QIF file reports an opening balance of %3. Would you like to overwrite the current balance with the one from the QIF file?").arg(m_account.name(), split.shares().formatMoney(m_account, sec),balance.formatMoney(m_account, sec)),
             i18n("Overwrite opening balance"),
             KStdGuiItem::yes(),
             KStdGuiItem::no(),
             "OverwriteOpeningBalance" )
             == KMessageBox::Yes )
-          { 
+          {
             file->removeTransaction( openingtx );
             m_account.setOpeningDate( date );
             file->createOpeningBalanceTransaction( m_account, balance );
           }
         }
-        
+
       }
       else
       {
@@ -527,8 +552,9 @@ void MyMoneyQifReader::processMSAccountEntry(const MyMoneyAccount::accountTypeE
         m_account.setOpeningDate( date );
         file->createOpeningBalanceTransaction( m_account, balance );
       }
+      ft.commit();
     }
-    
+
   } else {
     // for some unknown reason, Quicken 2001 generates the following (somewhat
     // misleading) sequence of lines:
@@ -658,11 +684,11 @@ void MyMoneyQifReader::processTransactionEntry(void)
   // Assign the "#" field to the transaction's bank id
   // This is the custom KMM extension to QIF for a unique ID
   tmp = extractLine('#');
-  if(!tmp.isEmpty()) 
+  if(!tmp.isEmpty())
   {
-    t.setBankID("ID " + tmp);
+    s1.setBankID("ID " + tmp);
   }
-  
+
   // Collect data for the account's split
   s1.setAccountId(m_account.id());
   tmp = extractLine('S');
@@ -671,14 +697,14 @@ void MyMoneyQifReader::processTransactionEntry(void)
     tmp = tmp.left(pos);
   }
   s1.setMemo(tmp);
-  
-  // TODO Deal with currencies more gracefully.  QIF cannot deal with multiple
+
+  // TODO (Ace) Deal with currencies more gracefully.  QIF cannot deal with multiple
   // currencies, so we should assume that transactions imported into a given
   // account are in THAT ACCOUNT's currency.  If one of those involves a transfer
   // to an account with a different currency, value and shares should be
   // different.  (Shares is in the target account's currency, value is in the
   // transaction's)
-  
+
   s1.setValue(m_qifProfile.value('T', extractLine('T')));
   s1.setShares(m_qifProfile.value('T', extractLine('T')));
   s1.setNumber(extractLine('N'));
@@ -715,10 +741,11 @@ void MyMoneyQifReader::processTransactionEntry(void)
         // of the payee.
         payee.setName(tmp);
 
+        MyMoneyFileTransaction ft;
         try {
           file->addPayee(payee);
           s1.setPayeeId(payee.id());
-
+          ft.commit();
         } catch(MyMoneyException *e) {
           KMessageBox::detailedSorry(0, i18n("Unable to add payee/receiver"),
             (e->what() + " " + i18n("thrown in") + " " + e->file()+ ":%1").arg(e->line()));
@@ -758,7 +785,7 @@ void MyMoneyQifReader::processTransactionEntry(void)
     MyMoneySplit s2 = s1;
     s2.setValue(-s1.value());
     s2.setShares(-s1.value());
-    s2.setId(QCString());
+    s2.clearId();
 
     // standard transaction
     tmp = extractLine('L');
@@ -790,26 +817,25 @@ void MyMoneyQifReader::processTransactionEntry(void)
       try {
         MyMoneyAccount account = file->account(accountId);
         // FIXME: check that the type matches and ask if not
-        
+
         if ( account.accountType() == MyMoneyAccount::Investment )
         {
           kdDebug(2) << "Line " << m_linenumber << ": Cannot transfer to an investment account. Transaction ignored." << endl;
           return;
         }
-	
         if ( account.id() == m_account.id() )
         {
           kdDebug(2) << "Line " << m_linenumber << ": Cannot transfer to the same account. Transfer ignored." << endl;
           accountId = QCString();
         }
-        
+
       } catch (MyMoneyException *e) {
         kdDebug(2) << "Line " << m_linenumber << ": Account with id " << accountId.data() << " not found" << endl;
         accountId = QCString();
         delete e;
       }
     }
-    
+
     if(!accountId.isEmpty()) {
       s2.setAccountId(accountId);
       try {
@@ -820,12 +846,12 @@ void MyMoneyQifReader::processTransactionEntry(void)
       }
     }
   } else {
-    // splitted transaction
+    // split transaction
     int   count;
 
     for(count = 1; !extractLine('$', count).isEmpty(); ++count) {
       MyMoneySplit s2 = s1;
-      s2.setId(QCString());
+      s2.clearId();
       s2.setValue(-m_qifProfile.value('$', extractLine('$', count)));
       s2.setShares(-m_qifProfile.value('$', extractLine('$', count)));
       s2.setMemo(extractLine('E', count));
@@ -858,7 +884,7 @@ void MyMoneyQifReader::processTransactionEntry(void)
         try {
           MyMoneyAccount account = file->account(accountId);
           // FIXME: check that the type matches and ask if not
-        
+
           if ( account.accountType() == MyMoneyAccount::Investment )
           {
             kdDebug(2) << "Line " << m_linenumber << ": Cannot transfer to an investment account. Transaction ignored." << endl;
@@ -869,7 +895,7 @@ void MyMoneyQifReader::processTransactionEntry(void)
             kdDebug(2) << "Line " << m_linenumber << ": Cannot transfer to the same account. Transfer ignored." << endl;
             accountId = QCString();
           }
-        
+
         } catch (MyMoneyException *e) {
           kdDebug(2) << "Line " << m_linenumber << ": Account with id " << accountId.data() << " not found" << endl;
           accountId = QCString();
@@ -906,9 +932,9 @@ void MyMoneyQifReader::processTransactionEntry(void)
       }
     }
 
-    // Second, if no bank-id diplicates, try using the more error-prone
+    // Second, if no bank-id duplicates, try using the more error-prone
     // MMTransaction::isDuplicate() method, as long as the user asked for
-    // this behaviour        
+    // this behaviour
     if ( oktoadd & m_qifProfile.attemptMatchDuplicates() )
     {
       for(it = list.begin(); it != list.end(); ++it) {
@@ -922,12 +948,12 @@ void MyMoneyQifReader::processTransactionEntry(void)
 
     if ( oktoadd )
     {
-      m_transactionCache.push_back(t);      
+      m_transactionCache.push_back(t);
     }
-  } 
-  catch (MyMoneyException *e) 
+  }
+  catch (MyMoneyException *e)
   {
-    kdDebug(2) << "Line " << m_linenumber << ": Problem adding imported transaction: " << e->what() << endl;  
+    kdDebug(2) << "Line " << m_linenumber << ": Problem adding imported transaction: " << e->what() << endl;
     delete e;
   }
 
@@ -938,27 +964,27 @@ void MyMoneyQifReader::processInvestmentTransactionEntry()
 //   kdDebug(2) << "Investment Transaction:" << m_qifEntry.count() << " lines" << endl;
   /*
   Items for Investment Accounts
-  Field 	Indicator Explanation
-  D 	Date
-  N 	Action
-  Y 	Security (NAME, not symbol)
-  I 	Price
-  Q 	Quantity (number of shares or split ratio)
-  T 	Transaction amount
-  C 	Cleared status
-  P 	Text in the first line for transfers and reminders (Payee)
-  M 	Memo
-  O 	Commission
-  L 	Account for the transfer
-  $ 	Amount transferred
-  ^ 	End of the entry
-  
-  It will be presumed all transactions are to the associated cash account, if 
+  Field   Indicator Explanation
+  D   Date
+  N   Action
+  Y   Security (NAME, not symbol)
+  I   Price
+  Q   Quantity (number of shares or split ratio)
+  T   Transaction amount
+  C   Cleared status
+  P   Text in the first line for transfers and reminders (Payee)
+  M   Memo
+  O   Commission
+  L   Account for the transfer
+  $   Amount transferred
+  ^   End of the entry
+
+  It will be presumed all transactions are to the associated cash account, if
   one exists, unless otherwise noted by the 'L' field.
 
   Expense/Income categories will be automatically generated, "_Dividend",
   "_InterestIncome", etc.
-  
+
   */
 
   MyMoneyFile* file = MyMoneyFile::instance();
@@ -967,7 +993,7 @@ void MyMoneyQifReader::processInvestmentTransactionEntry()
   if ( m_account.accountType() != MyMoneyAccount::Investment )
   {
     kdDebug(2) << "Line " << m_linenumber << ": Can't import investment transactions into this account." << endl;
-    
+
     int rc = KMessageBox::warningContinueCancel(0,
          i18n("This QIF file contains investment transactions.  You are trying to import this file into "
               "an account which is not an investment account.  These transactions will be ignored."),
@@ -983,7 +1009,7 @@ void MyMoneyQifReader::processInvestmentTransactionEntry()
         break;
     }
   }
-   
+
   MyMoneyTransaction t;
 
   // mark it imported for the view
@@ -1016,21 +1042,21 @@ void MyMoneyQifReader::processInvestmentTransactionEntry()
     }
   }
 
+  MyMoneySplit s1;
+
   // 'M' field: Memo
   QString memo = extractLine('M');
   t.setMemo(memo);
+  s1.setMemo(extractLine('M'));
 
   // '#' field: BankID
   QString bankid = extractLine('#');
   if ( ! bankid.isEmpty() )
-    t.setBankID(bankid);
+    s1.setBankID(bankid);
 
-  MyMoneySplit s1;
-  s1.setMemo(extractLine('M'));
-  
   // 'O' field: Fees
   MyMoneyMoney fees = m_qifProfile.value('T', extractLine('O'));
-  
+
   // 'T' field: Amount
   MyMoneyMoney amount = m_qifProfile.value('T', extractLine('T'));
   s1.setValue(amount-fees);
@@ -1054,7 +1080,7 @@ void MyMoneyQifReader::processInvestmentTransactionEntry()
   //
   // Therefore, generally it is not recommended to import a QIF file containing
   // investment transactions but NOT containing security records.
-  
+
   QString securitysymbol = m_investmentMap[securityname];
 
   // the correct account is the stock account which matches two criteria:
@@ -1073,14 +1099,14 @@ void MyMoneyQifReader::processInvestmentTransactionEntry()
     MyMoneySecurity security = file->security( currencyid );
     QString symbol = security.tradingSymbol().lower();
     QString name = security.name().lower();
-    
+
     if ( securityname == name || securitysymbol == symbol )
     {
       s1.setAccountId(*it_account);
       thisaccount = file->account(*it_account);
       found = true;
 
-#if 0      
+#if 0
       // update the price, while we're here.  in the future, this should be
       // an option
       QCString basecurrencyid = file->baseCurrency().id();
@@ -1090,7 +1116,7 @@ void MyMoneyQifReader::processInvestmentTransactionEntry()
         MyMoneyPrice newprice( currencyid, basecurrencyid, t_in.m_datePosted, t_in.m_moneyAmount / t_in.m_dShares, i18n("Statement Importer") );
         file->addPrice(newprice);
       }
-#endif    
+#endif
     }
 
     ++it_account;
@@ -1099,40 +1125,40 @@ void MyMoneyQifReader::processInvestmentTransactionEntry()
   if (!found)
   {
     kdDebug(2) << "Line " << m_linenumber << ": Security " << securityname << " not found in this account.  Transaction ignored." << endl;
-    
+
     // If the security is not known, notify the user
-    // TODO: A "SelectOrCreateAccount" interface for investments
+    // TODO (Ace) A "SelectOrCreateAccount" interface for investments
     KMessageBox::information(0, i18n("This investment account does not contain the \"%1\" security.  "
                                       "Transactions involving this security will be ignored.").arg(securityname),
                                 i18n("Security not found"),
                                 QString("MissingSecurity%1").arg(securityname.stripWhiteSpace()));
     return;
   }
-  
+
   // 'Q' field: Quantity
   MyMoneyMoney quantity = m_qifProfile.value('T', extractLine('Q'));
 
   // 'N' field: Action
   QString action = extractLine('N').lower();
-  
+
   // remove trailing X, which seems to have no purpose (?!)
   if ( action.endsWith("x") )
     action = action.left( action.length() - 1 );
 
   // Whether to create a cash split for the other side of the value
   MyMoneyMoney cashsplit;
-  
+
   if ( action == "reinvdiv" || action == "reinvlg" || action == "reinvsh" )
   {
     s1.setShares(quantity);
     s1.setAction(MyMoneySplit::ActionReinvestDividend);
-    
+
     MyMoneySplit s2;
     s2.setMemo(memo);
-    
+
     QString incomeaccount = QString("_") + extractLine('N');
     s2.setAccountId(findOrCreateIncomeAccount(incomeaccount));
-        
+
     s2.setValue(-amount);
     s2.setShares(-amount);
     t.addSplit(s2);
@@ -1145,7 +1171,7 @@ void MyMoneyQifReader::processInvestmentTransactionEntry()
     // amount into the transaction.
 
     QString incomeaccount = QString("_") + extractLine('N');
-    
+
     s1.setAccountId(findOrCreateIncomeAccount(incomeaccount));
     s1.setValue(-amount-fees);
     s1.setShares(-amount-fees);
@@ -1159,7 +1185,7 @@ void MyMoneyQifReader::processInvestmentTransactionEntry()
     s2.setAction(MyMoneySplit::ActionDividend);
     s2.setAccountId(thisaccount.id());
     t.addSplit(s2);
-    
+
     cashsplit = amount;
   }
   else if (action == "buy")
@@ -1167,14 +1193,14 @@ void MyMoneyQifReader::processInvestmentTransactionEntry()
     s1.setShares(quantity);
     s1.setAction(MyMoneySplit::ActionBuyShares);
 
-    cashsplit = -amount;      
+    cashsplit = -amount;
   }
   else if (action == "sell")
   {
     s1.setShares(-quantity);
     s1.setValue(-amount-fees);
     s1.setAction(MyMoneySplit::ActionBuyShares);
-    
+
     cashsplit = amount;
   }
   else if ( action == "shrsin" )
@@ -1192,10 +1218,15 @@ void MyMoneyQifReader::processInvestmentTransactionEntry()
   else if ( action == "stksplit" )
   {
     MyMoneyMoney splitfactor = (quantity / MyMoneyMoney(10.0)).reduce();
-    
+
     // Stock splits not supported
-    kdDebug(2) << "Line " << m_linenumber << ": Stock split not supported (date=" << date << " security=" << securityname << " factor=" << splitfactor.toString() << ")" << endl;
-    return;
+//     kdDebug(2) << "Line " << m_linenumber << ": Stock split not supported (date=" << date << " security=" << securityname << " factor=" << splitfactor.toString() << ")" << endl;
+
+    s1.setShares(splitfactor);
+    s1.setValue(0);
+    s1.setAction(MyMoneySplit::ActionSplitShares);
+
+//     return;
   }
   else
   {
@@ -1206,7 +1237,7 @@ void MyMoneyQifReader::processInvestmentTransactionEntry()
 
   t.addSplit(s1);
 
-  // The fees has to be added AFTER the interest, because 
+  // The fees has to be added AFTER the interest, because
   // KLedgerViewInvestments::preloadInvestmentSplits expects the splits to be
   // ordered this way.
   if ( ! fees.isZero() )
@@ -1218,7 +1249,7 @@ void MyMoneyQifReader::processInvestmentTransactionEntry()
     s.setAccountId(findOrCreateExpenseAccount("_Fees"));
     t.addSplit(s);
   }
-  
+
   // If there is a brokerage account, add a final split to stash the -value of the
   // transaction there, if needed based on the type of transaction
 
@@ -1230,7 +1261,7 @@ void MyMoneyQifReader::processInvestmentTransactionEntry()
   }
   else
   {
-    if(accountname.left(1) == m_qifProfile.accountDelimiter().left(1)) 
+    if(accountname.left(1) == m_qifProfile.accountDelimiter().left(1))
     {
       accountname = accountname.mid(1, accountname.length()-2);
       accountid = file->nameToAccount(accountname);
@@ -1240,14 +1271,14 @@ void MyMoneyQifReader::processInvestmentTransactionEntry()
         selectOrCreateAccount(Select, account);
         accountid = account.id();
       }
-    } 
-    else 
+    }
+    else
     {
       // L field with income/expense categories not supported in investments
       kdDebug(2) << "Line " << m_linenumber << ": L field with income/expense categories not supported in investments" << endl;
     }
-  }  
-    
+  }
+
   if ( ! accountid.isEmpty() && ! cashsplit.isZero() )
   {
     // FIXME This may not deal with foreign currencies properly
@@ -1258,7 +1289,7 @@ void MyMoneyQifReader::processInvestmentTransactionEntry()
     s.setAccountId(accountid);
     t.addSplit(s);
   }
-  
+
   // Add the transaction
   try {
     // check for duplicates ONLY by Bank ID in this account.
@@ -1271,23 +1302,25 @@ void MyMoneyQifReader::processInvestmentTransactionEntry()
     QValueList<MyMoneyTransaction> list = file->transactionList(filter);
     QValueList<MyMoneyTransaction>::Iterator it;
     for(it = list.begin(); it != list.end(); ++it) {
-      if(t.bankID() == (*it).bankID() && !t.bankID().isNull() && !(*it).bankID().isNull())
+      //if(t.bankID() == (*it).bankID() && !t.bankID().isNull() && !(*it).bankID().isNull())
+      MyMoneySplit thissplit = (*it).splitByAccount(thisaccount.id());
+      if(t.bankID() == thissplit.bankID() && !t.bankID().isNull() && !thissplit.bankID().isNull())
         break;
     }
     if(it == list.end())
     {
-      m_transactionCache.push_back(t);      
+      m_transactionCache.push_back(t);
     }
   } catch (MyMoneyException *e) {
-  
+
     kdDebug(2) << "Line " << m_linenumber << ": Problem adding imported transaction: " << e->what() << endl;
-    
+
     QString message(i18n("Problem adding imported transaction: "));
     message += e->what();
     KMessageBox::information(0, message);
     delete e;
   }
-  
+
   /*************************************************************************
    *
    * These transactions are natively supported by KMyMoney
@@ -1405,7 +1438,7 @@ void MyMoneyQifReader::processInvestmentTransactionEntry()
    * does not support, but may support in the future.
    *
    *************************************************************************/
-  /*   Note the Q field is the split ratio per 10 shares, so Q12.5 is a 
+  /*   Note the Q field is the split ratio per 10 shares, so Q12.5 is a
         12.5:10 split, otherwise known as 5:4.
   D1/14' 5
   NStkSplit
@@ -1511,7 +1544,7 @@ void MyMoneyQifReader::processInvestmentTransactionEntry()
 const QCString MyMoneyQifReader::findOrCreateIncomeAccount(const QString& searchname)
 {
   QCString result;
-  
+
   MyMoneyFile *file = MyMoneyFile::instance();
 
   // First, try to find this account as an income account
@@ -1525,7 +1558,7 @@ const QCString MyMoneyQifReader::findOrCreateIncomeAccount(const QString& search
     {
       result = *it_accid;
       break;
-    }  
+    }
     ++it_accid;
   }
 
@@ -1536,19 +1569,21 @@ const QCString MyMoneyQifReader::findOrCreateIncomeAccount(const QString& search
     acc.setName( searchname );
     acc.setAccountType( MyMoneyAccount::Income );
     MyMoneyAccount income = file->income();
+    MyMoneyFileTransaction ft;
     file->addAccount( acc, income );
+    ft.commit();
     result = acc.id();
   }
 
   return result;
 }
 
-// TODO: Combine this and the previous function
+// TODO (Ace) Combine this and the previous function
 
 const QCString MyMoneyQifReader::findOrCreateExpenseAccount(const QString& searchname)
 {
   QCString result;
-  
+
   MyMoneyFile *file = MyMoneyFile::instance();
 
   // First, try to find this account as an income account
@@ -1562,7 +1597,7 @@ const QCString MyMoneyQifReader::findOrCreateExpenseAccount(const QString& searc
     {
       result = *it_accid;
       break;
-    }  
+    }
     ++it_accid;
   }
 
@@ -1572,8 +1607,10 @@ const QCString MyMoneyQifReader::findOrCreateExpenseAccount(const QString& searc
     MyMoneyAccount acc;
     acc.setName( searchname );
     acc.setAccountType( MyMoneyAccount::Expense );
+    MyMoneyFileTransaction ft;
     MyMoneyAccount expense = file->expense();
     file->addAccount( acc, expense );
+    ft.commit();
     result = acc.id();
   }
 
@@ -1663,7 +1700,7 @@ void MyMoneyQifReader::processAccountEntry(void)
     // I can't fathom why I originally wrote this warning, so I am taking it out for now.
     // Perhaps a !Type:Invst is used to refer to BOTH the brokerage account (which is a checkings account in KMM)
     // and the investment account.
-  
+
   } else {
     m_account.setAccountType(MyMoneyAccount::Checkings);
     kdDebug(2) << "Line " << m_linenumber << ": Unknown account type '" << type << "', checkings assumed" << endl;
@@ -1671,7 +1708,7 @@ void MyMoneyQifReader::processAccountEntry(void)
   selectOrCreateAccount(Select, m_account);
 }
 
-void MyMoneyQifReader::selectOrCreateAccount(const SelectCreateMode mode, MyMoneyAccount& account)
+void MyMoneyQifReader::selectOrCreateAccount(const SelectCreateMode mode, MyMoneyAccount& account, const MyMoneyMoney& balance)
 {
   MyMoneyFile* file = MyMoneyFile::instance();
 
@@ -1706,7 +1743,7 @@ void MyMoneyQifReader::selectOrCreateAccount(const SelectCreateMode mode, MyMone
   if(!msg.isEmpty())
     accountSelect.setCaption(msg);
 
-  it = m_accountTranslation.find((leadIn + ":" + account.name()).lower());
+  it = m_accountTranslation.find((leadIn + MyMoneyFile::AccountSeperator + account.name()).lower());
   if(it != m_accountTranslation.end()) {
     try {
       account = file->account(*it);
@@ -1785,32 +1822,34 @@ void MyMoneyQifReader::selectOrCreateAccount(const SelectCreateMode mode, MyMone
       if(!accountSelect.selectedAccount().isEmpty()) {
         accountId = accountSelect.selectedAccount();
 
-        m_accountTranslation[(leadIn + ":" + account.name()).lower()] = accountId;
+        m_accountTranslation[(leadIn + MyMoneyFile::AccountSeperator + account.name()).lower()] = accountId;
 
         // MMAccount::openingBalance() is where the accountSelect dialog has
         // stashed the opening balance that the user chose.
         MyMoneyAccount importedAccountData(account);
-        MyMoneyMoney balance = importedAccountData.openingBalance();
+        // MyMoneyMoney balance = importedAccountData.openingBalance();
         account = file->account(accountId);
         if ( ! balance.isZero() )
-        {          
+        {
           QCString openingtxid = file->openingBalanceTransaction(account);
+          MyMoneyFileTransaction ft;
           if ( ! openingtxid.isEmpty() )
           {
             MyMoneyTransaction openingtx = file->transaction(openingtxid);
             MyMoneySplit split = openingtx.splitByAccount(account.id());
-            
+
             if ( split.shares() != balance )
             {
+              const MyMoneySecurity&  sec = file->security(account.currencyId());
               if ( KMessageBox::questionYesNo(
                 qApp->mainWidget(),
-                i18n("The %1 account currently has an opening balance of %2. This QIF file reports an opening balance of %3. Would you like to overwrite the current balance with the one from the QIF file?").arg(account.name(),split.shares().formatMoney(),balance.formatMoney()),
+                i18n("The %1 account currently has an opening balance of %2. This QIF file reports an opening balance of %3. Would you like to overwrite the current balance with the one from the QIF file?").arg(account.name(), split.shares().formatMoney(account, sec), balance.formatMoney(account, sec)),
                 i18n("Overwrite opening balance"),
                 KStdGuiItem::yes(),
                 KStdGuiItem::no(),
                 "OverwriteOpeningBalance" )
                 == KMessageBox::Yes )
-              { 
+              {
                 file->removeTransaction( openingtx );
                 file->createOpeningBalanceTransaction( account, balance );
               }
@@ -1821,7 +1860,8 @@ void MyMoneyQifReader::selectOrCreateAccount(const SelectCreateMode mode, MyMone
             // Add an opening balance
             file->createOpeningBalanceTransaction( account, balance );
           }
-        }        
+          ft.commit();
+        }
         break;
       }
 
@@ -1864,7 +1904,7 @@ void MyMoneyQifReader::processPriceEntry(void)
   prices with bogus dates.  Hopefully that will catch all of these.
 
   Also note that prices can be in fractional units, e.g. 141 9/16.
-  
+
 */
 
   MyMoneyFile* file = MyMoneyFile::instance();
@@ -1875,7 +1915,7 @@ void MyMoneyQifReader::processPriceEntry(void)
     ++it_line;
 
   QValueList<MyMoneySecurity> securities = file->securityList();
-    
+
   // Make a price for each line
   while ( it_line != m_qifEntry.end() )
   {
@@ -1884,7 +1924,7 @@ void MyMoneyQifReader::processPriceEntry(void)
     QString pricestr = column[1];
     QString datestr = column[2].remove("\"");
     kdDebug(2) << "Price:" << column[0] << " / " << column[1] << " / " << column[2] << endl;
-  
+
     // Only add the price if the date is valid.  If invalid, fail silently.  See note above.
     // Also require the price value to not have any slashes.  Old prices will be something like
     // "25 9/16", which we do not support.  So we'll skip the price for now.
@@ -1895,6 +1935,7 @@ void MyMoneyQifReader::processPriceEntry(void)
       // universal in the file, so all securities in the file must be
       // searched.
 
+      MyMoneyFileTransaction ft;
       QValueList<MyMoneySecurity>::const_iterator it_security = securities.begin();
       while ( it_security != securities.end() )
       {
@@ -1907,9 +1948,10 @@ void MyMoneyQifReader::processPriceEntry(void)
           file->addPrice(price);
           break;
         }
-        ++ it_security;      
+        ++ it_security;
       }
-      
+      ft.commit();
+
       // if the security mentioned in this price was not found in the file
       if ( it_security == securities.end() )
       {
@@ -1917,9 +1959,9 @@ void MyMoneyQifReader::processPriceEntry(void)
         kdDebug(2) << "Line " << m_linenumber << ": Security " << symbol << " not found." << endl;
       }
     }
-    
+
     ++it_line;
-  }  
+  }
 }
 
 void MyMoneyQifReader::processSecurityEntry(void)
@@ -1933,7 +1975,7 @@ void MyMoneyQifReader::processSecurityEntry(void)
       throw new MYMONEYEXCEPTION("USERABORT");
   }
 #endif
-  
+
   /*
   !Type:Security
   NVANGUARD 500 INDEX
@@ -1945,16 +1987,16 @@ void MyMoneyQifReader::processSecurityEntry(void)
   QString type = extractLine('T').lower();
   QString name = extractLine('N').lower();
   QString symbol = extractLine('S').lower();
-  
+
   kdDebug(2) << "Line " << m_linenumber << ": Security (" << type << "): " << name << " (" << symbol << ")" << endl;
 
-  // TODO: If this investment isn't already in the account, add it
+  // TODO (Ace) If this investment isn't already in the account, add it
 
   // Add it to our name-to-symbol mapping.  This will allow the investment transaction
   // importer to translate from the QIF file's security name to the symbol, which we
   // can then use to look up the security in KMM.
 
-  m_investmentMap[name] = symbol;  
+  m_investmentMap[name] = symbol;
 }
 
 #include "mymoneyqifreader.moc"
