@@ -35,6 +35,7 @@
 #include <klocale.h>
 #include <kmessagebox.h>
 #include <kconfig.h>
+#include <kdebug.h>
 
 // ----------------------------------------------------------------------------
 // Project Headers
@@ -45,11 +46,12 @@
 #include "../dialogs/kaccountselectdlg.h"
 #include "../kmymoney2.h"
 
-MyMoneyStatementReader::MyMoneyStatementReader()
+MyMoneyStatementReader::MyMoneyStatementReader() :
+  m_userAbort(false),
+  m_autoCreatePayee(false),
+  m_ft(0),
+  m_progressCallback(0)
 {
-  m_progressCallback = 0;
-  m_userAbort = false;
-  m_autoCreatePayee = false;
 }
 
 MyMoneyStatementReader::~MyMoneyStatementReader()
@@ -69,6 +71,12 @@ const bool MyMoneyStatementReader::startImport(const MyMoneyStatement& s)
   // same place.
   bool result = true;
 
+  //
+  // For testing, save the statement to an XML file
+  // (uncomment this line)
+  //
+  //MyMoneyStatement::writeXMLFile(s,"Imported.Xml");
+
   //
   // Select the account
   //
@@ -77,10 +85,6 @@ const bool MyMoneyStatementReader::startImport(const MyMoneyStatement& s)
 
   m_account.setName(s.m_strAccountName);
   m_account.setNumber(s.m_strAccountNumber);
-  m_account.setValue("lastStatementBalance", s.m_closingBalance.toString());
-
-  if ( s.m_dateEnd.isValid() )
-    m_account.setValue("lastStatementDate", s.m_dateEnd.toString("yyyy-MM-dd"));
 
   switch ( s.m_eType )
   {
@@ -104,11 +108,38 @@ const bool MyMoneyStatementReader::startImport(const MyMoneyStatement& s)
       break;
   }
 
+  m_ft = new MyMoneyFileTransaction();
+
   if ( !m_userAbort )
     m_userAbort = ! selectOrCreateAccount(Select, m_account);
 
+  m_account.setValue("lastStatementBalance", s.m_closingBalance.toString());
+  if ( s.m_dateEnd.isValid() ) {
+    m_account.setValue("lastImportedTransactionDate", s.m_dateEnd.toString(Qt::ISODate));
+  }
+
+  try {
+    MyMoneyFile::instance()->modifyAccount(m_account);
+  } catch(MyMoneyException* e) {
+    qDebug("Updating account in MyMoneyStatementReader::startImport failed");
+    delete e;
+  }
+
   signalProgress(0, s.m_listTransactions.count(), "Importing Statement ...");
 
+  //
+  // Process the securities
+  //
+  if ( s.m_eType == MyMoneyStatement::etInvestment )
+  {
+    QValueList<MyMoneyStatement::Security>::const_iterator it_s = s.m_listSecurities.begin();
+    while ( it_s != s.m_listSecurities.end() )
+    {
+      processSecurityEntry(*it_s);
+      ++it_s;
+    }
+  }
+
   //
   // Process the transactions
   //
@@ -147,9 +178,62 @@ const bool MyMoneyStatementReader::finishImport(void)
   signalProgress(-1, -1);
   rc = !m_userAbort;
 
+  // finish the transaction
+  if(rc)
+    m_ft->commit();
+  delete m_ft;
+  m_ft = 0;
+
   return rc;
 }
 
+void MyMoneyStatementReader::processSecurityEntry(const MyMoneyStatement::Security& sec_in)
+{
+  // For a security entry, we will just make sure the security exists in the
+  // file. It will not get added to the investment account until it's called
+  // for in a transaction.
+  MyMoneyFile* file = MyMoneyFile::instance();
+
+  // check if we already have the security
+  // In a statement, we do not know what type of security this is, so we will
+  // not use type as a matching factor.
+  MyMoneySecurity security;
+  QValueList<MyMoneySecurity> list = file->securityList();
+  QValueList<MyMoneySecurity>::ConstIterator it = list.begin();
+  while ( it != list.end() && security.id().isEmpty() )
+  {
+    if(sec_in.m_strSymbol.isEmpty()) {
+      if((*it).name() == sec_in.m_strName)
+        security = *it;
+    } else if((*it).tradingSymbol() == sec_in.m_strSymbol)
+      security = *it;
+    ++it;
+  }
+
+  // if the security was not found, we have to create it while not forgetting
+  // to setup the type
+  if(security.id().isEmpty())
+  {
+    security.setName(sec_in.m_strName);
+    security.setTradingSymbol(sec_in.m_strSymbol);
+    security.setSmallestAccountFraction(1000);
+    security.setTradingCurrency(file->baseCurrency().id());
+    security.setValue("kmm-security-id", sec_in.m_strId);
+    security.setValue("kmm-online-source", "Yahoo");
+    security.setSecurityType(MyMoneySecurity::SECURITY_STOCK);
+    MyMoneyFileTransaction ft;
+    try {
+      file->addSecurity(security);
+      ft.commit();
+      kdDebug(0) << "Created " << security.name() << " with id " << security.id() << endl;
+    } catch(MyMoneyException *e) {
+      KMessageBox::error(0, i18n("Error creating security record: %1").arg(e->what()), i18n("Error"));
+    }
+  } else {
+    kdDebug(0) << "Found " << security.name() << " with id " << security.id() << endl;
+  }
+}
+
 void MyMoneyStatementReader::processTransactionEntry(const MyMoneyStatement::Transaction& t_in)
 {
   MyMoneyFile* file = MyMoneyFile::instance();
@@ -159,15 +243,26 @@ void MyMoneyStatementReader::processTransactionEntry(const MyMoneyStatement::Tra
   // mark it imported for the view
   t.setValue("Imported", "true");
 
-  // TODO: We can get the commodity from the statement!!
+  // TODO (Ace) We can get the commodity from the statement!!
   // Although then we would need UI to verify
   t.setCommodity(m_account.currencyId());
 
   t.setPostDate(t_in.m_datePosted);
   t.setMemo(t_in.m_strMemo);
 
+#if 0
+  // (acejones) removing this code.  keeping it around for reference.
+  //
+  // this is the OLD way of handling bank ID's, which unfortunately was wrong.
+  // bank ID's actually need to go on the split which corresponds with the
+  // account we're importing into.
+  //
+  // thus anywhere "this account" is put into a split is also where we need
+  // to put the bank ID in.
+  //
   if ( ! t_in.m_strBankID.isEmpty() )
     t.setBankID(t_in.m_strBankID);
+#endif
 
   MyMoneySplit s1;
 
@@ -179,54 +274,110 @@ void MyMoneyStatementReader::processTransactionEntry(const MyMoneyStatement::Tra
   // set these values if a transfer split is needed at the very end.
   MyMoneyMoney transfervalue;
 
-  // If the user has chosent to import into an investment account, determine the correct account to use
+  // If the user has chosen to import into an investment account, determine the correct account to use
   MyMoneyAccount thisaccount = m_account;
   if ( thisaccount.accountType() == MyMoneyAccount::Investment )
   {
-    // the correct account is the stock account which matches two criteria:
-    // (1) it is a sub-account of the selected investment account, and
-    // (2) the symbol of the underlying security matches the security of the
-    // transaction
-
-    // search through each subordinate account
-    bool found = false;
-    QCStringList accounts = thisaccount.accountList();
-    QCStringList::const_iterator it_account = accounts.begin();
-    while( !found && it_account != accounts.end() )
+    // find the security transacted, UNLESS this transaction didn't
+    // involve any security.
+    if ( t_in.m_eAction != MyMoneyStatement::Transaction::eaNone )
     {
-      QCString currencyid = file->account(*it_account).currencyId();
-      MyMoneySecurity security = file->security( currencyid );
-      QString symbol = security.tradingSymbol();
+      // the correct account is the stock account which matches two criteria:
+      // (1) it is a sub-account of the selected investment account, and
+      // (2a) the symbol of the underlying security matches the security of the
+      // transaction, or
+      // (2b) the name of the security matches the name of the security of the transaction.
 
-      // startsWith(QString, bool) is not available in Qt 3.0
-      if ( t_in.m_strSecurity.lower().startsWith(QString(symbol+" ").lower()) )
+      // search through each subordinate account
+      bool found = false;
+      QCStringList accounts = thisaccount.accountList();
+      QCStringList::const_iterator it_account = accounts.begin();
+      while( !found && it_account != accounts.end() )
       {
-        s1.setAccountId(*it_account);
-        thisaccount = file->account(*it_account);
-        found = true;
+        QCString currencyid = file->account(*it_account).currencyId();
+        MyMoneySecurity security = file->security( currencyid );
+        QString symboltoken = security.tradingSymbol() + " ";
+        QString nametoken = " " + security.name();
 
-        // update the price, while we're here.  in the future, this should be
-        // an option
-        QCString basecurrencyid = file->baseCurrency().id();
-        MyMoneyPrice price = file->price( currencyid, basecurrencyid, t_in.m_datePosted, true );
-        if ( !price.isValid() )
+        if(t_in.m_strSecurity.startsWith(symboltoken,false)
+        || (t_in.m_strSecurity == nametoken))
         {
-          MyMoneyPrice newprice( currencyid, basecurrencyid, t_in.m_datePosted, t_in.m_amount / t_in.m_shares, i18n("Statement Importer") );
-          file->addPrice(newprice);
+          thisaccount = file->account(*it_account);
+          found = true;
+
+          if(t_in.m_eAction != MyMoneyStatement::Transaction::eaCashDividend) // Bug #1581788: Should not update a price for cash dividends
+          {
+            // update the price, while we're here.  in the future, this should be
+            // an option
+            QCString basecurrencyid = file->baseCurrency().id();
+            MyMoneyPrice price = file->price( currencyid, basecurrencyid, t_in.m_datePosted, true );
+            if ( !price.isValid() )
+            {
+              MyMoneyPrice newprice( currencyid, basecurrencyid, t_in.m_datePosted, t_in.m_amount / t_in.m_shares, i18n("Statement Importer") );
+              file->addPrice(newprice);
+            }
+          }
         }
+
+        ++it_account;
       }
 
-      ++it_account;
+      // If there was no stock account under the m_acccount investment account,
+      // add one using the security.
+      if (!found)
+      {
+        // The security should always be available, because the statement file
+        // should separately list all the securities referred to in the file,
+        // and when we found a security, we added it to the file.
+
+        MyMoneySecurity security;
+        QValueList<MyMoneySecurity> list = MyMoneyFile::instance()->securityList();
+        QValueList<MyMoneySecurity>::ConstIterator it = list.begin();
+        while ( it != list.end() && security.id().isEmpty() )
+        {
+          QString symboltoken = (*it).tradingSymbol() + " ";
+          QString nametoken = " " + (*it).name();
+          if (
+            t_in.m_strSecurity.startsWith(symboltoken,false)
+            ||
+            (t_in.m_strSecurity == nametoken)
+          )
+            security = *it;
+          ++it;
+        }
+
+        if(!security.id().isEmpty())
+        {
+          thisaccount = MyMoneyAccount();
+          thisaccount.setName(security.name());
+          thisaccount.setAccountType(MyMoneyAccount::Stock);
+          thisaccount.setCurrencyId(security.id());
+
+          file->addAccount(thisaccount, m_account);
+          kdDebug(0) << __func__ << ": created account " << thisaccount.id() << " for security " << t_in.m_strSecurity << " under account " << m_account.id() << endl;
+        }
+        // this security does not exist in the file.
+        else
+        {
+          if ( t_in.m_strSecurity.isEmpty() )
+          {
+            KMessageBox::information(0, i18n("This imported statement contains investment transactions with no security.  These transactions will be ignored.").arg(t_in.m_strSecurity),i18n("Security not found"),QString("BlankSecurity"));
+          }
+          else
+          {
+            // This should be rare.  A statement should have a security entry for any
+            // of the securities referred to in the transactions.  The only way to get
+            // here is if that's NOT the case.
+            KMessageBox::information(0, i18n("This investment account does not contain the \"%1\" security.  Transactions involving this security will be ignored.").arg(t_in.m_strSecurity),i18n("Security not found"),QString("MissingSecurity%1").arg(t_in.m_strSecurity.stripWhiteSpace()));
+          }
+          return;
+        }
+      }
     }
 
-    if (!found)
-    {
-      KMessageBox::information(0, i18n("This investment account does not contain the \"%1\" security.  "
-                                        "Transactions involving this security will be ignored.").arg(t_in.m_strSecurity),
-                                  i18n("Security not found"),
-                                  QString("MissingSecurity%1").arg(t_in.m_strSecurity.stripWhiteSpace()));
-      return;
-    }
+    s1.setAccountId(thisaccount.id());
+    if ( ! t_in.m_strBankID.isEmpty() )
+      s1.setBankID(t_in.m_strBankID);
 
     if (t_in.m_eAction==MyMoneyStatement::Transaction::eaReinvestDividend)
     {
@@ -240,20 +391,32 @@ void MyMoneyStatementReader::processTransactionEntry(const MyMoneyStatement::Tra
       s2.setValue(-t_in.m_amount);
       t.addSplit(s2);
     }
-    if (t_in.m_eAction==MyMoneyStatement::Transaction::eaCashDividend)
+    else if (t_in.m_eAction==MyMoneyStatement::Transaction::eaCashDividend)
     {
-      // NOTE: With CashDividend, the amount of the dividend should
-      // be in data.amount.  Since I've never seen an OFX file with
-      // cash dividends, this is an assumption on my part. (acejones)
-
       // Cash dividends require setting 2 splits to get all of the information
       // in.  Split #1 will be the income split, and we'll set it to the first
       // income account.  This is a hack, but it's needed in order to get the
       // amount into the transaction.
 
+      // There are some sign issues.  The OFX plugin universally reverses the sign
+      // for investment transactions.
+      //
+      // The way we interpret the sign on 'amount' is the s1 split, which is always
+      // the thing that's NOT the cash account.  For dividends, it's the income
+      // category, for buy/sell it's the stock account.
+      //
+      // For cash account transactions, the s1 split IS the cash account split,
+      // which explains why they have to be reversed for investment transactions
+      //
+      // Ergo, the 'amount' is negative at this point and needs to stay negative.
+      // The 'fees' is positive.
+      //
+      // This should probably change.  It would be more consistent to ALWAYS
+      // interpret the 'amount' as the cash account part.
+
       s1.setAccountId(findOrCreateIncomeAccount("_Dividend"));
-      s1.setShares(-t_in.m_amount);
-      s1.setValue(-t_in.m_amount);
+      s1.setShares(t_in.m_amount);
+      s1.setValue(t_in.m_amount);
 
       // Split 2 will be the zero-amount investment split that serves to
       // mark this transaction as a cash dividend and note which stock account
@@ -265,7 +428,7 @@ void MyMoneyStatementReader::processTransactionEntry(const MyMoneyStatement::Tra
       s2.setAccountId(thisaccount.id());
       t.addSplit(s2);
 
-      transfervalue = -t_in.m_amount - t_in.m_fees;
+      transfervalue = -t_in.m_amount-t_in.m_fees;
     }
     else if (t_in.m_eAction==MyMoneyStatement::Transaction::eaBuy || t_in.m_eAction==MyMoneyStatement::Transaction::eaSell)
     {
@@ -274,15 +437,43 @@ void MyMoneyStatementReader::processTransactionEntry(const MyMoneyStatement::Tra
 
       transfervalue = -t_in.m_amount;
     }
+    else if (t_in.m_eAction==MyMoneyStatement::Transaction::eaNone)
+    {
+      // User is attempting to import a non-investment transaction into this
+      // investment account.  This is not supportable the way KMyMoney is
+      // written.  However, if a user has an associated brokerage account,
+      // we can stuff the transaction there.
 
+      QCString brokerageactid = m_account.value("kmm-brokerage-account").utf8();
+      if ( ! brokerageactid.isEmpty() )
+      {
+        s1.setAccountId(brokerageactid);
+        if ( ! t_in.m_strBankID.isEmpty() )
+          s1.setBankID(t_in.m_strBankID);
+
+        if(!s1.value().isNegative())
+          s1.setAction(MyMoneySplit::ActionDeposit);
+        else
+          s1.setAction(MyMoneySplit::ActionWithdrawal);
+
+        // Needed to satisfy the bankid check below.
+        thisaccount = file->account(brokerageactid);
+      }
+      else
+      {
+        // Warning!! Your transaction is being thrown away.
+      }
+    }
   }
   else
   {
     // For non-investment accounts, just use the selected account
     // Note that it is perfectly reasonable to import an investment statement into a non-investment account
     // if you really want.  The investment-specific information, such as number of shares and action will
-    // be disacarded in that case.
+    // be discarded in that case.
     s1.setAccountId(m_account.id());
+    if ( ! t_in.m_strBankID.isEmpty() )
+      s1.setBankID(t_in.m_strBankID);
 
     if(!s1.value().isNegative())
       s1.setAction(MyMoneySplit::ActionDeposit);
@@ -364,7 +555,7 @@ void MyMoneyStatementReader::processTransactionEntry(const MyMoneyStatement::Tra
     // handle it.
     //
 
-    MyMoneyTransactionFilter filter(m_account.id());
+    MyMoneyTransactionFilter filter(thisaccount.id());
     filter.addPayee(payeeid);
     QValueList<MyMoneyTransaction> list = file->transactionList(filter);
     if(!list.empty())
@@ -380,7 +571,7 @@ void MyMoneyStatementReader::processTransactionEntry(const MyMoneyStatement::Tra
         QValueList<MyMoneyTransaction>::ConstIterator it_trans = list.fromLast();
         while ( it_trans != list.end() )
         {
-          MyMoneySplit s = (*it_trans).splitByAccount(m_account.id());
+          MyMoneySplit s = (*it_trans).splitByAccount(thisaccount.id());
           if ( s.value() == s1.value() )
           {
             t_old = *it_trans;
@@ -395,11 +586,12 @@ void MyMoneyStatementReader::processTransactionEntry(const MyMoneyStatement::Tra
       {
         // We don't need the split that covers this account,
         // we just need the other ones.
-        if ( (*it_split).accountId() != m_account.id() )
+        if ( (*it_split).accountId() != thisaccount.id() )
         {
           MyMoneySplit s(*it_split);
           s.setReconcileFlag(MyMoneySplit::NotReconciled);
-          s.setId(QCString());
+          s.clearId();
+          s.setBankID(QString());
 
           if ( t_old.splits().count() == 2 )
           {
@@ -432,6 +624,10 @@ void MyMoneyStatementReader::processTransactionEntry(const MyMoneyStatement::Tra
   if ( ! transfervalue.isZero() )
   {
     QCString brokerageactid = m_account.value("kmm-brokerage-account").utf8();
+    if (brokerageactid.isEmpty() )
+    {
+      brokerageactid = file->accountByName(m_account.brokerageName()).id();
+    }
 
     if ( !brokerageactid.isEmpty() )
     {
@@ -445,7 +641,6 @@ void MyMoneyStatementReader::processTransactionEntry(const MyMoneyStatement::Tra
     }
   }
 
-
   // Add the transaction
   try {
     // check for duplicates ONLY by Bank ID in this account.
@@ -458,7 +653,8 @@ void MyMoneyStatementReader::processTransactionEntry(const MyMoneyStatement::Tra
     QValueList<MyMoneyTransaction> list = file->transactionList(filter);
     QValueList<MyMoneyTransaction>::Iterator it;
     for(it = list.begin(); it != list.end(); ++it) {
-      if(t.bankID() == (*it).bankID() && !t.bankID().isNull() && !(*it).bankID().isNull())
+      MyMoneySplit thissplit = (*it).splitByAccount(thisaccount.id());
+      if(t_in.m_strBankID == thissplit.bankID() && !t_in.m_strBankID.isNull() && !thissplit.bankID().isNull())
         break;
     }
     if(it == list.end())
@@ -466,7 +662,7 @@ void MyMoneyStatementReader::processTransactionEntry(const MyMoneyStatement::Tra
       file->addTransaction(t);
     }
   } catch (MyMoneyException *e) {
-    QString message(i18n("Problem adding imported transaction: "));
+    QString message(i18n("Problem adding imported transaction #%1: ").arg(t_in.m_strBankID));
     message += e->what();
 
     int result = KMessageBox::warningContinueCancel(0, message);
@@ -494,7 +690,8 @@ bool MyMoneyStatementReader::selectOrCreateAccount(const SelectCreateMode /*mode
   if ( ! accountNumber.isEmpty() )
   {
     // Get a list of all accounts
-    QValueList<MyMoneyAccount> accounts = file->accountList(QCStringList());
+    QValueList<MyMoneyAccount> accounts;
+    file->accountList(accounts);
 
     // Iterate through them
     QValueList<MyMoneyAccount>::const_iterator it_account = accounts.begin();
@@ -505,7 +702,8 @@ bool MyMoneyStatementReader::selectOrCreateAccount(const SelectCreateMode /*mode
           ( (*it_account).number() == accountNumber )
         )
         {
-          account.setAccountId( (*it_account).id() );
+          MyMoneyAccount newAccount((*it_account).id(), account);
+          account = newAccount;
           accountId = (*it_account).id();
           break;
         }
@@ -558,8 +756,15 @@ bool MyMoneyStatementReader::selectOrCreateAccount(const SelectCreateMode /*mode
       account = file->account(accountId);
       if ( ! accountNumber.isEmpty() && account.value("StatementKey") != accountNumber )
       {
-        account.setValue("StatementKey",accountNumber);
-        MyMoneyFile::instance()->modifyAccount(account);
+        account.setValue("StatementKey", accountNumber);
+        MyMoneyFileTransaction ft;
+        try {
+          MyMoneyFile::instance()->modifyAccount(account);
+          ft.commit();
+        } catch(MyMoneyException* e) {
+          qDebug("Updating account in MyMoneyStatementReader::selectOrCreateAccount failed");
+          delete e;
+        }
       }
     }
     else
@@ -656,3 +861,4 @@ const QCString MyMoneyStatementReader::findOrCreateExpenseAccount(const QString&
 }
 
 #include "mymoneystatementreader.moc"
+// vim:cin:si:ai:et:ts=2:sw=2:
