@@ -35,7 +35,7 @@
 // Project Includes
 
 #include "../dialogs/kcurrencycalculatordecl.h"
-#include "../mymoney/mymoneyfile.h"
+#include <kmymoney/mymoneyfile.h>
 
 /**
   * @author Thomas Baumgart
@@ -63,6 +63,10 @@ public:
 
   const MyMoneyMoney price(void) const;
 
+  void setupPriceEditor(void);
+
+  static bool setupSplitPrice(MyMoneyMoney& shares, const MyMoneyTransaction& t, const MyMoneySplit& s, const QMap<QCString, MyMoneyMoney>& priceInfo, QWidget* parentWidget);
+
 protected:
   void updateExample(const MyMoneyMoney& price);
 
@@ -71,16 +75,14 @@ protected slots:
   void slotSetExchangeRate(void);
   void slotUpdateResult(const QString& txt);
   void slotUpdateRate(const QString& txt);
-  virtual void accept();
+  virtual void accept(void);
 
 private:
   MyMoneySecurity     m_fromCurrency;
   MyMoneySecurity     m_toCurrency;
   MyMoneyMoney        m_result;
   MyMoneyMoney        m_value;
-  QDate               m_date;
   signed64            m_resultFraction;
-  int                 m_ratePrec;
 };
 
 #endif
