@@ -38,13 +38,13 @@
 // ----------------------------------------------------------------------------
 // Project Includes
 
+#include <../dialogs/knewloanwizard.h>
 #include "keditloanwizard.h"
 #include "../widgets/kmymoneylineedit.h"
 #include "../widgets/kmymoneyedit.h"
 #include "../widgets/kmymoneycombo.h"
 #include "../widgets/kmymoneyaccountselector.h"
 #include "../widgets/kmymoneydateinput.h"
-#include "../widgets/kmymoneypayee.h"
 #include "../mymoney/mymoneyfile.h"
 #include "../kmymoneyutils.h"
 
@@ -111,8 +111,8 @@ void KEditLoanWizard::loadWidgets(const MyMoneyAccount& /* account */)
   QCString paymentAccountId, interestAccountId;
 
   m_nameEdit->loadText(m_account.name());
-  m_loanAmountEdit->loadText(m_account.loanAmount().formatMoney());
-  m_finalPaymentEdit->loadText(m_account.finalPayment().formatMoney());
+  m_loanAmountEdit->loadText(m_account.loanAmount().formatMoney(m_account.fraction(MyMoneyFile::instance()->security(m_account.currencyId()))));
+  m_finalPaymentEdit->loadText(m_account.finalPayment().formatMoney(m_account.fraction(MyMoneyFile::instance()->security(m_account.currencyId()))));
   m_firstDueDateEdit->setDate(m_account.openingDate());
 
   if(m_account.fixedInterestRate()) {
@@ -146,9 +146,10 @@ void KEditLoanWizard::loadWidgets(const MyMoneyAccount& /* account */)
   for(it_s = m_schedule.transaction().splits().begin();
       it_s != m_schedule.transaction().splits().end();
       ++it_s) {
+    MyMoneyAccount acc = file->account((*it_s).accountId());
     // if it's the split that references the source/dest
     // of the money, we check if we borrow or loan money
-    if((*it_s).action() == MyMoneySplit::ActionAmortization
+    if(acc.isAssetLiability() && !acc.isLoan()
     && (*it_s).value() != MyMoneyMoney::autoCalc) {
       if((*it_s).value().isNegative()) {
         m_lendButton->setChecked(false);
@@ -165,7 +166,7 @@ void KEditLoanWizard::loadWidgets(const MyMoneyAccount& /* account */)
       if(!(*it_s).payeeId().isEmpty()) {
         try {
           payee = file->payee((*it_s).payeeId());
-          m_payeeEdit->setText(payee.name());
+          m_payeeEdit->setSelectedItem(payee.id());
         } catch(MyMoneyException *e) {
           delete e;
           qWarning("Payee for schedule has been deleted");
@@ -175,11 +176,8 @@ void KEditLoanWizard::loadWidgets(const MyMoneyAccount& /* account */)
       // remove this split with one that will be replaced
       // later and has a phony id
       m_transaction.removeSplit(*it_s);
-
-      MyMoneySplit split;
-      split.setAccountId(QCString("Phony-ID"));
-      split.setValue(0);
-      m_transaction.addSplit(split);
+      m_split.clearId();
+      m_transaction.addSplit(m_split);
     }
 
     if((*it_s).action() == MyMoneySplit::ActionInterest) {
@@ -206,11 +204,12 @@ void KEditLoanWizard::loadWidgets(const MyMoneyAccount& /* account */)
   // we borrow or lend money
   loadAccountList();
 
-  m_paymentEdit->loadText(basePayment.formatMoney());
-  m_newPaymentEdit->loadText(basePayment.formatMoney());
-  m_paymentLabel->setText(QString(" ") + basePayment.formatMoney());
+  int fraction = m_account.fraction(MyMoneyFile::instance()->security(m_account.currencyId()));
+  m_paymentEdit->loadText(basePayment.formatMoney(fraction));
+  m_newPaymentEdit->loadText(basePayment.formatMoney(fraction));
+  m_paymentLabel->setText(QString(" ") + basePayment.formatMoney(fraction));
 
-  m_additionalCost->setText(addPayment.formatMoney());
+  m_additionalCost->setText(addPayment.formatMoney(fraction));
   m_interestAccountEdit->setSelected(interestAccountId);
   m_paymentAccountEdit->setSelected(paymentAccountId);
   m_nextDueDateEdit->setDate(m_schedule.nextPayment());
@@ -219,7 +218,7 @@ void KEditLoanWizard::loadWidgets(const MyMoneyAccount& /* account */)
   int amt = m_account.interestChangeFrequency(&changeFrequencyUnit);
   if(amt != -1) {
     m_interestFrequencyAmountEdit->setValue(amt);
-    m_interestFrequencyUnitEdit->loadCurrentItem(changeFrequencyUnit);
+    m_interestFrequencyUnitEdit->setCurrentItem(changeFrequencyUnit);
   }
 
   // keep track, if the loan should be fully repayed
@@ -347,7 +346,7 @@ void KEditLoanWizard::next()
         m_durationValueEdit->setValue(0);
       }
       if(m_fullyRepayLoan)
-        m_finalPaymentEdit->loadText(MyMoneyMoney(0).formatMoney());
+        m_finalPaymentEdit->loadText(MyMoneyMoney(0).formatMoney(m_account.fraction(MyMoneyFile::instance()->security(m_account.currencyId()))));
     }
 
 /*
@@ -366,7 +365,7 @@ void KEditLoanWizard::next()
     MyMoneySplit split;
     MyMoneyTransactionFilter filter(m_account.id());
 
-    filter.setDateFilter(QDate(), m_effectiveChangeDateEdit->getQDate().addDays(-1));
+    filter.setDateFilter(QDate(), m_effectiveChangeDateEdit->date().addDays(-1));
     list = MyMoneyFile::instance()->transactionList(filter);
 
     for(it = list.begin(); it != list.end(); ++it) {
@@ -386,7 +385,7 @@ void KEditLoanWizard::next()
     dontLeavePage = !calculateLoan();
 
     // reset the original loan amount to the widget
-    m_loanAmountEdit->setText(m_account.loanAmount().formatMoney());
+    m_loanAmountEdit->setValue(m_account.loanAmount());
 
     if(!dontLeavePage) {
       updateLoanInfo();
@@ -414,8 +413,8 @@ void KEditLoanWizard::slotCheckPageFinished(void)
   // appropriate, we just have to disable it.
 
   if(currentPage() == m_effectiveDatePage) {
-    if(m_effectiveChangeDateEdit->getQDate() < m_account.openingDate()
-    || m_effectiveChangeDateEdit->getQDate() > QDate::currentDate())
+    if(m_effectiveChangeDateEdit->date() < m_account.openingDate()
+    || m_effectiveChangeDateEdit->date() > QDate::currentDate())
       nextButton()->setEnabled(false);
 
   } else if(currentPage() == m_interestEditPage) {
@@ -432,11 +431,11 @@ void KEditLoanWizard::updateEditSummary(void)
   m_additionalFees7->setText(m_summaryAdditionalFees->text());
   m_totalPayment7->setText(m_summaryTotalPeriodicPayment->text());
   m_interestRate7->setText(m_summaryInterestRate->text());
-  m_startDateChanges->setText(KGlobal::locale()->formatDate(m_effectiveChangeDateEdit->getQDate(), true));
+  m_startDateChanges->setText(KGlobal::locale()->formatDate(m_effectiveChangeDateEdit->date(), true));
 
   // calculate the number of affected transactions
   MyMoneyTransactionFilter filter(m_account.id());
-  filter.setDateFilter(m_effectiveChangeDateEdit->getQDate(), QDate());
+  filter.setDateFilter(m_effectiveChangeDateEdit->date(), QDate());
 
   int count = 0;
   QValueList<MyMoneyTransaction> list;
@@ -466,8 +465,8 @@ const MyMoneySchedule KEditLoanWizard::schedule(void) const
   MyMoneySchedule sched = m_schedule;
   sched.setTransaction(transaction());
   sched.setOccurence(KMyMoneyUtils::stringToOccurence(m_paymentFrequencyUnitEdit->currentText()));
-  if(m_nextDueDateEdit->getQDate() < m_schedule.startDate())
-    sched.setStartDate(m_nextDueDateEdit->getQDate());
+  if(m_nextDueDateEdit->date() < m_schedule.startDate())
+    sched.setStartDate(m_nextDueDateEdit->date());
 
   return sched;
 }
@@ -485,21 +484,12 @@ const MyMoneyAccount KEditLoanWizard::account(void) const
   acc.setFinalPayment(MyMoneyMoney(m_finalPaymentEdit->text()));
   acc.setTerm(term());
   acc.setPeriodicPayment(m_paymentEdit->value());
-  acc.setInterestRate(m_effectiveChangeDateEdit->getQDate(), m_interestRateEdit->value());
+  acc.setInterestRate(m_effectiveChangeDateEdit->date(), m_interestRateEdit->value());
 
-  if(!m_payeeEdit->text().isEmpty()) {
-    try {
-      acc.setPayee(MyMoneyFile::instance()->payeeByName(m_payeeEdit->text()).id());
-    } catch(MyMoneyException *e) {
-      delete e;
-      qWarning("%s(%d): Someone has removed the selected payee", __FILE__, __LINE__);
-      acc.setPayee(QCString());
-    }
-  } else
-    acc.setPayee(QCString());
+  acc.setPayee(m_payeeEdit->selectedItem());
 
   if(m_variableInterestButton->isChecked()) {
-    acc.setNextInterestChange(m_interestChangeDateEdit->getQDate());
+    acc.setNextInterestChange(m_interestChangeDateEdit->date());
     acc.setInterestChangeFrequency(m_interestFrequencyAmountEdit->value(),
                                    m_interestFrequencyUnitEdit->currentItem());
   }
@@ -510,7 +500,7 @@ const MyMoneyAccount KEditLoanWizard::account(void) const
 const MyMoneyTransaction KEditLoanWizard::transaction() const
 {
   MyMoneyTransaction t = KNewLoanWizard::transaction();
-  MyMoneySplit s = t.splitByAccount(QCString());
+  MyMoneySplit s = t.splitByAccount(QCString("Phony-ID"));
 
   s.setAccountId(m_account.id());
   t.modifySplit(s);
