@@ -52,6 +52,7 @@
 #include "../mymoney/mymoneyfile.h"
 #include "../mymoney/mymoneysecurity.h"
 #include "../mymoney/mymoneyprice.h"
+#include "../kmymoneyglobalsettings.h"
 
 #define SYMBOL_COL      0
 #define NAME_COL        1
@@ -85,11 +86,6 @@ KEquityPriceUpdateDlg::KEquityPriceUpdateDlg(QWidget *parent, const QCString& se
 
   MyMoneyFile* file = MyMoneyFile::instance();
 
-  // Get the price precision from the configuration
-  KConfig *kconfig = KGlobal::config();
-  kconfig->setGroup("General Options");
-  m_pricePrecision = kconfig->readNumEntry("PricePrecision", 4);
-
   //
   // Add each price pair that we know about
   //
@@ -104,12 +100,14 @@ KEquityPriceUpdateDlg::KEquityPriceUpdateDlg(QWidget *parent, const QCString& se
   MyMoneyPriceList prices = file->priceList();
   for(MyMoneyPriceList::ConstIterator it_price = prices.begin(); it_price != prices.end(); ++it_price)
   {
-    MyMoneySecurityPair pair = it_price.key();
-
+    const MyMoneySecurityPair& pair = it_price.key();
     if ( file->security( pair.first ).isCurrency() && ( securityId.isEmpty() || ( pair == currencyIds ) ) )
     {
-      addPricePair(pair);
-      btnUpdateAll->setEnabled(true);
+      const MyMoneyPriceEntries& entries = (*it_price);
+      if(entries.count() > 0 && entries.begin().key() <= QDate::currentDate()) {
+        addPricePair(pair);
+        btnUpdateAll->setEnabled(true);
+      }
     }
   }
 
@@ -172,19 +170,42 @@ void KEquityPriceUpdateDlg::addPricePair(const MyMoneySecurityPair& pair)
   MyMoneyFile* file = MyMoneyFile::instance();
 
   QString symbol = QString("%1 > %2").arg(pair.first,pair.second);
-  if ( ! lvEquityList->findItem(symbol,SYMBOL_COL,Qt::ExactMatch) )
+  QString id = QString("%1 %2").arg(pair.first,pair.second);
+  if ( ! lvEquityList->findItem(id,ID_COL,Qt::ExactMatch) )
   {
     MyMoneyPrice pr = file->price(pair.first,pair.second);
     if(pr.source() != "KMyMoney") {
-      KListViewItem* item = new KListViewItem(lvEquityList,
-        symbol,
-        i18n("%1 units in %2").arg(pair.first,pair.second));
-      if(pr.isValid()) {
-        item->setText(PRICE_COL, pr.rate().formatMoney(file->currency(pair.second).tradingSymbol(), m_pricePrecision));
-        item->setText(DATE_COL, pr.date().toString(Qt::ISODate));
+      const QCString& foreignCurrency = file->foreignCurrency(pair.first, pair.second);
+      // check that the foreign currency is still in use
+      QValueList<MyMoneyAccount>::const_iterator it_a;
+      QValueList<MyMoneyAccount> list;
+      file->accountList(list);
+      for(it_a = list.begin(); it_a != list.end(); ++it_a) {
+        // if it's an account denominated in the foreign currency
+        // keep it
+        if(((*it_a).currencyId() == foreignCurrency)
+        && !(*it_a).isClosed())
+          break;
+        // if it's an investment traded in the foreign currency
+        // keep it
+        if((*it_a).isInvest() && !(*it_a).isClosed()) {
+          MyMoneySecurity sec = file->security((*it_a).currencyId());
+          if(sec.tradingCurrency() == foreignCurrency)
+            break;
+        }
+      }
+      // if it is in use, it_a is not equal to list.end()
+      if(it_a != list.end()) {
+        KListViewItem* item = new KListViewItem(lvEquityList,
+          symbol,
+          i18n("%1 units in %2").arg(pair.first,pair.second));
+        if(pr.isValid()) {
+          item->setText(PRICE_COL, pr.rate(pair.second).formatMoney(file->currency(pair.second).tradingSymbol(), KMyMoneyGlobalSettings::pricePrecision()));
+          item->setText(DATE_COL, pr.date().toString(Qt::ISODate));
+        }
+        item->setText(ID_COL,id);
+        item->setText(SOURCE_COL, "Yahoo Currency");  // This string value should not be localized
       }
-      item->setText(ID_COL,QString("%1 %2").arg(pair.first,pair.second));
-      item->setText(SOURCE_COL, "Yahoo Currency");  // This string value should not be localized
     }
   }
 }
@@ -194,25 +215,38 @@ void KEquityPriceUpdateDlg::addInvestment(const MyMoneySecurity& inv)
   MyMoneyFile* file = MyMoneyFile::instance();
 
   QString symbol = inv.tradingSymbol();
-  if ( ! lvEquityList->findItem(symbol,SYMBOL_COL,Qt::ExactMatch) )
+  QString id = inv.id();
+  if ( ! lvEquityList->findItem(id,ID_COL,Qt::ExactMatch) )
   {
-    KListViewItem* item = new KListViewItem(lvEquityList, symbol, inv.name());
-    MyMoneySecurity currency = file->currency(inv.tradingCurrency());
-    MyMoneyPrice pr = file->price(inv.id(), inv.tradingCurrency());
-    if(pr.isValid()) {
-      item->setText(PRICE_COL, pr.rate().formatMoney(currency.tradingSymbol(), m_pricePrecision));
-      item->setText(DATE_COL, pr.date().toString(Qt::ISODate));
+    // check that the security is still in use
+    QValueList<MyMoneyAccount>::const_iterator it_a;
+    QValueList<MyMoneyAccount> list;
+    file->accountList(list);
+    for(it_a = list.begin(); it_a != list.end(); ++it_a) {
+      if((*it_a).isInvest()
+      && ((*it_a).currencyId() == inv.id())
+      && !(*it_a).isClosed())
+        break;
     }
-    item->setText(ID_COL,inv.id());
-    item->setText(SOURCE_COL, inv.value("kmm-online-source"));
+    // if it is in use, it_a is not equal to list.end()
+    if(it_a != list.end()) {
+      KListViewItem* item = new KListViewItem(lvEquityList, symbol, inv.name());
+      MyMoneySecurity currency = file->currency(inv.tradingCurrency());
+      MyMoneyPrice pr = file->price(id.utf8(), inv.tradingCurrency());
+      if(pr.isValid()) {
+        item->setText(PRICE_COL, pr.rate(currency.id()).formatMoney(currency.tradingSymbol(), KMyMoneyGlobalSettings::pricePrecision()));
+        item->setText(DATE_COL, pr.date().toString(Qt::ISODate));
+      }
+      item->setText(ID_COL,id);
+      if (inv.value("kmm-online-quote-system") == "Finance::Quote")
+        item->setText(SOURCE_COL, QString("Finance::Quote %1").arg( inv.value("kmm-online-source")));
+      else
+        item->setText(SOURCE_COL, inv.value("kmm-online-source"));
 
-    // If this investment is denominated in a foreign currency, ensure that
-    // the appropriate price pair is also on the list
+      // If this investment is denominated in a foreign currency, ensure that
+      // the appropriate price pair is also on the list
 
-    if ( currency.id() != file->baseCurrency().id() )
-    {
-      QString symbol = QString("%1 > %2").arg(currency.id(),file->baseCurrency().id());
-      if ( ! lvEquityList->findItem(symbol,SYMBOL_COL,Qt::ExactMatch) )
+      if ( currency.id() != file->baseCurrency().id() )
       {
         addPricePair(MyMoneySecurityPair(currency.id(),file->baseCurrency().id()));
       }
@@ -236,45 +270,54 @@ void KEquityPriceUpdateDlg::slotOKClicked()
 
   MyMoneyFile* file = MyMoneyFile::instance();
   QValueList<MyMoneySecurity> equities = file->securityList();
-  MyMoneyFile::instance()->suspendNotify(true);
-  QListViewItem* item = lvEquityList->firstChild();
-  while ( item )
-  {
-    MyMoneyMoney rate(item->text(PRICE_COL));
-    if ( !rate.isZero() )
-    {
-      QCString id = item->text(ID_COL).utf8();
 
-      // if the ID has a space, then this is TWO ID's, so it's a currency quote
-      if ( QString(id).contains(" ") )
+  QListViewItem* item = lvEquityList->firstChild();
+  MyMoneyFileTransaction ft;
+  QString name;
+  try {
+    while ( item )
+    {
+      // turn on signals before we modify the last entry in the list
+      MyMoneyFile::instance()->blockSignals(item->nextSibling() != 0);
+
+      MyMoneyMoney rate(item->text(PRICE_COL));
+      if ( !rate.isZero() )
       {
-        QStringList ids = QStringList::split(" ",QString(id));
-        QCString fromid = ids[0].utf8();
-        QCString toid = ids[1].utf8();
-        MyMoneyPrice price(fromid,toid,QDate().fromString(item->text(DATE_COL), Qt::ISODate),rate,item->text(SOURCE_COL));
-        file->addPrice(price);
-      }
-      else
-      // otherwise, it's a security quote
-      {
-        MyMoneySecurity security = MyMoneyFile::instance()->security(id);
-        try {
+        QCString id = item->text(ID_COL).utf8();
+
+        // if the ID has a space, then this is TWO ID's, so it's a currency quote
+        if ( QString(id).contains(" ") )
+        {
+          QStringList ids = QStringList::split(" ",QString(id));
+          QCString fromid = ids[0].utf8();
+          QCString toid = ids[1].utf8();
+          name = QString("%1 --> %2").arg(fromid).arg(toid);
+          MyMoneyPrice price(fromid,toid,QDate().fromString(item->text(DATE_COL), Qt::ISODate),rate,item->text(SOURCE_COL));
+          file->addPrice(price);
+        }
+        else
+        // otherwise, it's a security quote
+        {
+          MyMoneySecurity security = MyMoneyFile::instance()->security(id);
+          name = security.name();
           MyMoneyPrice price(id, security.tradingCurrency(), QDate().fromString(item->text(DATE_COL), Qt::ISODate), rate, item->text(SOURCE_COL));
 
-          // TODO: Better handling of the case where there is already a price
+          // TODO (Ace) Better handling of the case where there is already a price
           // for this date.  Currently, it just overrides the old value.  Really it
           // should check to see if the price is the same and prompt the user.
           MyMoneyFile::instance()->addPrice(price);
-
-        } catch(MyMoneyException *e) {
-          qDebug("Unable to add price information for %s", security.name().data());
-          delete e;
         }
+
       }
+      item = item->nextSibling();
     }
-    item = item->nextSibling();
+    ft.commit();
+
+  } catch(MyMoneyException *e) {
+    qDebug("Unable to add price information for %s", name.data());
+    delete e;
   }
-  MyMoneyFile::instance()->suspendNotify(false);
+
   accept();
 }
 
@@ -338,26 +381,33 @@ void KEquityPriceUpdateDlg::slotQuoteFailed(const QString& _id, const QString& _
   QListViewItem* item = lvEquityList->findItem(_id,ID_COL,Qt::ExactMatch);
 
   // Give the user some options
-
   int result;
   if(_id.contains(" ")) {
-    result = KMessageBox::warningContinueCancel(this,i18n("Failed to retrieve an exchange rate for %1 from %2. It will be skipped this time.").arg(_symbol,item->text(SOURCE_COL)),i18n("Price Update  Failed"));
+    result = KMessageBox::warningContinueCancel(this, i18n("Failed to retrieve an exchange rate for %1 from %2. It will be skipped this time.").arg(_symbol, item->text(SOURCE_COL)), i18n("Price Update  Failed"));
   } else {
-    result = KMessageBox::questionYesNoCancel(this,i18n("Failed to retrieve a quote for %1 from %2.  Would you like to disable online price updates for this security?").arg(_symbol,item->text(SOURCE_COL)),i18n("Price Update Failed"),KStdGuiItem::yes(),KStdGuiItem::no(),"KEquityPriceUpdateDlg::slotQuoteFailed::Price Update Failed");
+    result = KMessageBox::questionYesNoCancel(this, i18n("Failed to retrieve a quote for %1 from %2.  Would you like to disable online price updates for this security?").arg(_symbol, item->text(SOURCE_COL)),i18n("Price Update Failed"), KStdGuiItem::yes(), KStdGuiItem::no(), "KEquityPriceUpdateDlg::slotQuoteFailed::Price Update Failed");
   }
 
   if ( result == KMessageBox::Yes )
   {
     // Disable price updates for this security
 
-    // Get this security (by ID)
-    MyMoneySecurity security = MyMoneyFile::instance()->security(_id.utf8());
+    MyMoneyFileTransaction ft;
+    try {
+      // Get this security (by ID)
+      MyMoneySecurity security = MyMoneyFile::instance()->security(_id.utf8());
 
-    // Set the quote source to blank
-    security.setValue("kmm-online-source",QString());
+      // Set the quote source to blank
+      security.setValue("kmm-online-source",QString());
+      security.setValue("kmm-online-quote-system",QString());
 
-    // Re-commit the security
-    MyMoneyFile::instance()->modifySecurity(security);
+      // Re-commit the security
+      MyMoneyFile::instance()->modifySecurity(security);
+      ft.commit();
+    } catch(MyMoneyException* e) {
+      KMessageBox::error(this, QString("<qt>")+i18n("Cannot update security <b>%1</b>: %2").arg(_symbol, e->what())+QString("</qt>"), i18n("Price Update Failed"));
+      delete e;
+    }
   }
 
   // As long as the user doesn't want to cancel, move on!
@@ -394,7 +444,7 @@ void KEquityPriceUpdateDlg::slotQuoteFailed(const QString& _id, const QString& _
 
 void KEquityPriceUpdateDlg::slotReceivedQuote(const QString& _id, const QString& _symbol,const QDate& _date, const double& _price)
 {
-  QListViewItem* item = lvEquityList->findItem(_symbol,SYMBOL_COL,Qt::ExactMatch);
+  QListViewItem* item = lvEquityList->findItem(_id,ID_COL,Qt::ExactMatch);
   QListViewItem* next = NULL;
 
   if ( item )
@@ -408,14 +458,12 @@ void KEquityPriceUpdateDlg::slotReceivedQuote(const QString& _id, const QString&
       double price = _price;
       QCString id = _id.utf8();
       MyMoneySecurity sec;
-
-      if(_id.contains(" ") == 0) {      // it's a normal symbol no currency conversion
+      if ( _id.contains(" ") == 0) {
         MyMoneySecurity security = MyMoneyFile::instance()->security(id);
         QString factor = security.value("kmm-online-factor");
         if(!factor.isEmpty()) {
           price *= MyMoneyMoney(factor).toDouble();
         }
-
         try {
           sec = MyMoneyFile::instance()->security(id);
           sec = MyMoneyFile::instance()->security(sec.tradingCurrency());
@@ -423,19 +471,21 @@ void KEquityPriceUpdateDlg::slotReceivedQuote(const QString& _id, const QString&
           sec = MyMoneySecurity();
           delete e;
         }
+
       } else {
         QRegExp splitrx("([0-9a-z\\.]+)[^a-z0-9]+([0-9a-z\\.]+)",false /*case sensitive*/);
         if ( splitrx.search(_id) != -1 ) {
           try {
             sec = MyMoneyFile::instance()->security(splitrx.cap(2).utf8());
           } catch(MyMoneyException *e) {
+            sec = MyMoneySecurity();
             delete e;
           }
         }
       }
-      item->setText(PRICE_COL, KGlobal::locale()->formatMoney(price,sec.tradingSymbol(),m_pricePrecision));
+      item->setText(PRICE_COL, KGlobal::locale()->formatMoney(price, sec.tradingSymbol(), KMyMoneyGlobalSettings::pricePrecision()));
       item->setText(DATE_COL, date.toString(Qt::ISODate));
-      logStatusMessage(i18n("Price for %1 updated").arg(_symbol));
+      logStatusMessage(i18n("Price for %1 updated (id %2)").arg(_symbol,_id));
     }
     else
     {
@@ -459,7 +509,7 @@ void KEquityPriceUpdateDlg::slotReceivedQuote(const QString& _id, const QString&
   }
   else
   {
-    logErrorMessage(i18n("Received a price for %1, but this symbol is not on the list!  Aborting entire update.").arg(_symbol));
+    logErrorMessage(i18n("Received a price for %1 (id %2), but this symbol is not on the list!  Aborting entire update.").arg(_symbol,_id));
   }
 
   if (next)
