@@ -1,7 +1,7 @@
 /***************************************************************************
                           kfindtransactiondlg.cpp
                              -------------------
-    copyright            : (C) 2003 by Thomas Baumgart
+    copyright            : (C) 2003, 2007 by Thomas Baumgart
     email                : ipwizard@users.sourceforge.net
  ***************************************************************************/
 
@@ -22,6 +22,7 @@
 #include <qcheckbox.h>
 #include <qtimer.h>
 #include <qtabwidget.h>
+#include <qlayout.h>
 
 // ----------------------------------------------------------------------------
 // KDE Includes
@@ -34,37 +35,36 @@
 #include <klineedit.h>
 #include <klistview.h>
 #include <kcombobox.h>
-#include <kguiitem.h>
+#include <kstdguiitem.h>
 #include <kiconloader.h>
 
 // ----------------------------------------------------------------------------
 // Project Includes
 
 #include "kfindtransactiondlg.h"
-#include "../widgets/kmymoneyregistersearch.h"
-#include "../widgets/kmymoneydateinput.h"
-#include "../widgets/kmymoneyedit.h"
-#include "../widgets/kmymoneylineedit.h"
-#include "../widgets/kmymoneyaccountselector.h"
-#include "../mymoney/mymoneyfile.h"
-#include "../mymoney/storage/imymoneystorage.h"
 
-KFindTransactionDlg::KFindTransactionDlg(QWidget *parent, const char *name, bool forceSearchEnabled)
- : KFindTransactionDlgDecl(parent, name, false),
-  m_transactionPtr(0),
-  m_forceSearchEnabled(forceSearchEnabled)
+#include <kmymoney/kmymoneydateinput.h>
+#include <kmymoney/kmymoneyedit.h>
+#include <kmymoney/kmymoneylineedit.h>
+#include <kmymoney/kmymoneyaccountselector.h>
+#include <kmymoney/mymoneyfile.h>
+#include <kmymoney/kmymoneychecklistitem.h>
+#include <kmymoney/kmymoneyglobalsettings.h>
+#include <kmymoney/register.h>
+#include <kmymoney/transaction.h>
+#include <kmymoney/kmymoneycombo.h>
+
+#include "ksortoptiondlg.h"
+
+KFindTransactionDlg::KFindTransactionDlg(QWidget *parent, const char *name) :
+  KFindTransactionDlgDecl(parent, name, false),
+  m_needReload(false)
 {
-  // hide the transaction register and make sure the dialog is
-  // displayed as small as can be. We make sure that the larger
-  // version (with the transaction register) will also fit on the screen
-  // by moving the dialog by (-45,-45).
-  m_register->setParent(this);
   m_register->installEventFilter(this);
-  m_registerFrame->hide();
-  KFindTransactionDlgDecl::update();
-  resize(minimumSizeHint());
+  m_tabWidget->setTabEnabled(m_resultPage, false);
 
-  move(x()-45, y()-45);
+  // 'cause we don't have a separate setupTextPage
+  connect(m_textEdit, SIGNAL(textChanged(const QString&)), this, SLOT(slotUpdateSelections()));
 
   setupAccountsPage();
   setupCategoriesPage();
@@ -82,66 +82,61 @@ KFindTransactionDlg::KFindTransactionDlg(QWidget *parent, const char *name, bool
   m_helpAnchor[m_payeeTab] = QString("details.search.payee");
   m_helpAnchor[m_detailsTab] = QString("details.search.details");
 
+  m_closeButton->setGuiItem(KStdGuiItem::close());
+  m_helpButton->setGuiItem(KStdGuiItem::help());
+
+#if KDE_IS_VERSION(3,4,0)
+  m_searchButton->setGuiItem(KStdGuiItem::find());
+  m_resetButton->setGuiItem(KStdGuiItem::reset());
+#else
   KIconLoader* il = KGlobal::iconLoader();
-  KGuiItem resetButtonItem( i18n( "&Reset" ),
-                    QIconSet(il->loadIcon("undo", KIcon::Small, KIcon::SizeSmall)),
-                    i18n("Reset all settings"),
-                    i18n("Use this to reset all settings to the state they were when the dialog was opened."));
-  m_resetButton->setGuiItem(resetButtonItem);
-
-  KGuiItem closeButtonItem( i18n( "&Close" ),
-                    QIconSet(il->loadIcon("fileclose", KIcon::Small, KIcon::SizeSmall)),
-                    i18n("Close the dialog"),
-                    i18n("Leave the dialog and return to where you came from."));
-  m_closeButton->setGuiItem(closeButtonItem);
-
   KGuiItem searchButtonItem( i18n( "&Search" ),
                     QIconSet(il->loadIcon("find", KIcon::Small, KIcon::SizeSmall)),
                     i18n("Start the search"),
                     i18n("Takes the current criteria and searches for matching transactions."));
   m_searchButton->setGuiItem(searchButtonItem);
 
-  KGuiItem helpButtonItem (i18n("&Help"),
-                    QIconSet(il->loadIcon("help", KIcon::Small, KIcon::SizeSmall)),
-                    i18n("Open online help"),
-                    i18n("Opens the online help and shows detailed information about transaction searching."));
-  m_helpButton->setGuiItem(helpButtonItem);
+  KGuiItem resetButtonItem( i18n( "&Reset" ),
+                            QIconSet(il->loadIcon("undo", KIcon::Small, KIcon::SizeSmall)),
+                            i18n("Reset all settings"),
+                            i18n("Use this to reset all settings to the state they were when the dialog was opened."));
+  m_resetButton->setGuiItem(resetButtonItem);
+#endif
+
+
+  // setup the register
+  QValueList<KMyMoneyRegister::Column> cols;
+  cols << KMyMoneyRegister::DateColumn;
+  cols << KMyMoneyRegister::AccountColumn;
+  cols << KMyMoneyRegister::DetailColumn;
+  cols << KMyMoneyRegister::ReconcileFlagColumn;
+  cols << KMyMoneyRegister::PaymentColumn;
+  cols << KMyMoneyRegister::DepositColumn;
+  m_register->setupRegister(MyMoneyAccount(), cols);
+  m_register->setSelectionMode(QTable::Single);
+
+  connect(m_register, SIGNAL(editTransaction()), this, SLOT(slotSelectTransaction()));
+  connect(m_register, SIGNAL(headerClicked()), this, SLOT(slotSortOptions()));
 
-  // readConfig();
   slotUpdateSelections();
 
-  QTimer::singleShot(0, this, SLOT(slotRightSize()));
-
   // setup the connections
   connect(m_searchButton, SIGNAL(clicked()), this, SLOT(slotSearch()));
-
   connect(m_resetButton, SIGNAL(clicked()), this, SLOT(slotReset()));
   connect(m_resetButton, SIGNAL(clicked()), m_accountsView, SLOT(slotSelectAllAccounts()));
   connect(m_resetButton, SIGNAL(clicked()), m_categoriesView, SLOT(slotSelectAllAccounts()));
-
   connect(m_closeButton, SIGNAL(clicked()), this, SLOT(deleteLater()));
-
   connect(m_helpButton, SIGNAL(clicked()), this, SLOT(slotShowHelp()));
 
-  connect(m_textEdit, SIGNAL(textChanged(const QString&)), this, SLOT(slotUpdateSelections()));
+  // only allow searches when a selection has been made
+  connect(this, SIGNAL(selectionEmpty(bool)), m_searchButton, SLOT(setDisabled(bool)));
 
-  connect(m_register, SIGNAL(clicked(int, int, int, const QPoint&)), this, SLOT(slotRegisterClicked(int, int, int, const QPoint&)));
-  connect(m_register, SIGNAL(doubleClicked(int, int, int, const QPoint&)), this, SLOT(slotRegisterDoubleClicked(int, int, int, const QPoint&)));
+  // get signal about engine changes
+  connect(MyMoneyFile::instance(), SIGNAL(dataChanged()), this, SLOT(slotRefreshView()));
 
-  connect(m_register, SIGNAL(signalNextTransaction()), this, SLOT(slotNextTransaction()));
-  connect(m_register, SIGNAL(signalPreviousTransaction()), this, SLOT(slotPreviousTransaction()));
-  connect(m_register, SIGNAL(signalSelectTransaction(const QCString&)), this, SLOT(slotSelectTransaction(const QCString&)));
+  slotUpdateSelections();
 
   m_textEdit->setFocus();
-
-  // make sure, we get a note when the engine changes state
-  MyMoneyFile::instance()->attach(MyMoneyFile::NotifyClassAnyChange, this);
-}
-
-KFindTransactionDlg::~KFindTransactionDlg()
-{
-  // detach ourself from the engine
-  MyMoneyFile::instance()->detach(MyMoneyFile::NotifyClassAnyChange, this);
 }
 
 void KFindTransactionDlg::slotReset(void)
@@ -175,19 +170,13 @@ void KFindTransactionDlg::slotReset(void)
   m_nrButton->setChecked(true);
   m_nrRangeButton->setChecked(false);
 
-  m_registerFrame->hide();
+  m_tabWidget->setTabEnabled(m_resultPage, false);
+  m_tabWidget->setCurrentPage(m_tabWidget->indexOf(m_criteriaTab));
 
   // the following call implies a call to slotUpdateSelections,
   // that's why we call it last
-  m_dateRange->setCurrentItem(allDates);
-  slotDateRangeChanged(allDates);
-
-  QTimer::singleShot(0, this, SLOT(slotRightSize()));
-}
-
-void KFindTransactionDlg::slotRightSize(void)
-{
-  resize(minimumSize());
+  m_dateRange->setCurrentItem(MyMoneyTransactionFilter::allDates);
+  slotDateRangeChanged(MyMoneyTransactionFilter::allDates);
 }
 
 void KFindTransactionDlg::slotUpdateSelections(void)
@@ -206,7 +195,7 @@ void KFindTransactionDlg::slotUpdateSelections(void)
   m_caseSensitive->setEnabled(!m_textEdit->text().isEmpty());
 
   // Account tab
-  if(!m_accountsView->allAccountsSelected()) {
+  if(!m_accountsView->allItemsSelected()) {
     if(!txt.isEmpty())
       txt += ", ";
     txt += i18n("Account");
@@ -229,7 +218,7 @@ void KFindTransactionDlg::slotUpdateSelections(void)
   }
 
   // Categories tab
-  if(!m_categoriesView->allAccountsSelected()) {
+  if(!m_categoriesView->allItemsSelected()) {
     if(!txt.isEmpty())
       txt += ", ";
     txt += i18n("Category");
@@ -256,9 +245,15 @@ void KFindTransactionDlg::slotUpdateSelections(void)
     txt += i18n("Details");
   }
 
+  //Show a warning about transfers if Categories are filtered - bug #1523508
+  if(!m_categoriesView->allItemsSelected()) {
+    m_transferWarning->setText( i18n("Warning: Filtering by Category will exclude all transfers from the results.") );
+  } else {
+    m_transferWarning->setText("");
+  }
+
   // disable the search button if no selection is made
-  if(!m_forceSearchEnabled)
-    m_searchButton->setDisabled(txt.isEmpty());
+  emit selectionEmpty(txt.isEmpty());
 
   if(txt.isEmpty()) {
     txt = i18n("(None)");
@@ -302,8 +297,10 @@ const bool KFindTransactionDlg::allItemsSelected(const QListView* view) const
 void KFindTransactionDlg::setupAccountsPage(void)
 {
   m_accountsView->setSelectionMode(QListView::Multi);
-  m_accountsView->loadList(static_cast<KMyMoneyUtils::categoryTypeE>
-                           (KMyMoneyUtils::asset | KMyMoneyUtils::liability));
+  AccountSet accountSet;
+  accountSet.addAccountGroup(MyMoneyAccount::Asset);
+  accountSet.addAccountGroup(MyMoneyAccount::Liability);
+  accountSet.load(m_accountsView);
   connect(m_accountsView, SIGNAL(stateChanged()), this, SLOT(slotUpdateSelections()));
 }
 
@@ -327,7 +324,7 @@ void KFindTransactionDlg::selectItems(QListView* view, const QCStringList& list,
   QListViewItem* it_v;
 
   for(it_v = view->firstChild(); it_v != 0; it_v = it_v->nextSibling()) {
-    kMyMoneyCheckListItem* it_c = static_cast<kMyMoneyCheckListItem*>(it_v);
+    KMyMoneyCheckListItem* it_c = static_cast<KMyMoneyCheckListItem*>(it_v);
     if(it_c->type() == QCheckListItem::CheckBox && list.contains(it_c->id())) {
       it_c->setOn(state);
     }
@@ -340,8 +337,10 @@ void KFindTransactionDlg::selectItems(QListView* view, const QCStringList& list,
 void KFindTransactionDlg::setupCategoriesPage(void)
 {
   m_categoriesView->setSelectionMode(QListView::Multi);
-  m_categoriesView->loadList(static_cast<KMyMoneyUtils::categoryTypeE>
-                           (KMyMoneyUtils::income | KMyMoneyUtils::expense));
+  AccountSet categorySet;
+  categorySet.addAccountGroup(MyMoneyAccount::Income);
+  categorySet.addAccountGroup(MyMoneyAccount::Expense);
+  categorySet.load(m_categoriesView);
   connect(m_categoriesView, SIGNAL(stateChanged()), this, SLOT(slotUpdateSelections()));
 }
 
@@ -360,7 +359,7 @@ void KFindTransactionDlg::selectSubItems(QListViewItem* item, const QCStringList
   QListViewItem* it_v;
 
   for(it_v = item->firstChild(); it_v != 0; it_v = it_v->nextSibling()) {
-    kMyMoneyCheckListItem* it_c = static_cast<kMyMoneyCheckListItem*>(it_v);
+    KMyMoneyCheckListItem* it_c = static_cast<KMyMoneyCheckListItem*>(it_v);
     if(list.contains(it_c->id()))
       it_c->setOn(state);
     selectSubItems(it_v, list, state);
@@ -369,36 +368,23 @@ void KFindTransactionDlg::selectSubItems(QListViewItem* item, const QCStringList
 
 void KFindTransactionDlg::setupDatePage(void)
 {
-  MyMoneyTransactionFilter::translateDateRange(allDates,m_startDates[allDates],m_endDates[allDates]);
-  MyMoneyTransactionFilter::translateDateRange(untilToday,m_startDates[untilToday],m_endDates[untilToday]);
-  MyMoneyTransactionFilter::translateDateRange(currentMonth,m_startDates[currentMonth],m_endDates[currentMonth]);
-  MyMoneyTransactionFilter::translateDateRange(currentYear,m_startDates[currentYear],m_endDates[currentYear]);
-  MyMoneyTransactionFilter::translateDateRange(monthToDate,m_startDates[monthToDate],m_endDates[monthToDate]);
-  MyMoneyTransactionFilter::translateDateRange(yearToDate,m_startDates[yearToDate],m_endDates[yearToDate]);
-  MyMoneyTransactionFilter::translateDateRange(lastMonth,m_startDates[lastMonth],m_endDates[lastMonth]);
-  MyMoneyTransactionFilter::translateDateRange(lastYear,m_startDates[lastYear],m_endDates[lastYear]);
-  MyMoneyTransactionFilter::translateDateRange(last30Days,m_startDates[last30Days],m_endDates[last30Days]);
-  MyMoneyTransactionFilter::translateDateRange(last3Months,m_startDates[last3Months],m_endDates[last3Months]);
-  MyMoneyTransactionFilter::translateDateRange(last6Months,m_startDates[last6Months],m_endDates[last6Months]);
-  MyMoneyTransactionFilter::translateDateRange(last12Months,m_startDates[last12Months],m_endDates[last12Months]);
-  MyMoneyTransactionFilter::translateDateRange(next30Days,m_startDates[next30Days],m_endDates[next30Days]);
-  MyMoneyTransactionFilter::translateDateRange(next3Months,m_startDates[next3Months],m_endDates[next3Months]);
-  MyMoneyTransactionFilter::translateDateRange(next6Months,m_startDates[next6Months],m_endDates[next6Months]);
-  MyMoneyTransactionFilter::translateDateRange(next12Months,m_startDates[next12Months],m_endDates[next12Months]);
-  MyMoneyTransactionFilter::translateDateRange(userDefined,m_startDates[userDefined],m_endDates[userDefined]);
+  int i;
+  for(i = MyMoneyTransactionFilter::allDates; i < MyMoneyTransactionFilter::dateOptionCount; ++i) {
+    MyMoneyTransactionFilter::translateDateRange(static_cast<MyMoneyTransactionFilter::dateOptionE>(i), m_startDates[i], m_endDates[i]);
+  }
 
-  connect(m_dateRange, SIGNAL(activated(int)), this, SLOT(slotDateRangeChanged(int)));
+  connect(m_dateRange, SIGNAL(itemSelected(int)), this, SLOT(slotDateRangeChanged(int)));
   connect(m_fromDate, SIGNAL(dateChanged(const QDate&)), this, SLOT(slotDateChanged()));
   connect(m_toDate, SIGNAL(dateChanged(const QDate&)), this, SLOT(slotDateChanged()));
 
-  slotDateRangeChanged(allDates);
+  slotDateRangeChanged(MyMoneyTransactionFilter::allDates);
 }
 
 void KFindTransactionDlg::slotDateRangeChanged(int idx)
 {
   switch(idx) {
-    case allDates:
-    case userDefined:
+    case MyMoneyTransactionFilter::allDates:
+    case MyMoneyTransactionFilter::userDefined:
       m_fromDate->loadDate(QDate());
       m_toDate->loadDate(QDate());
       break;
@@ -413,13 +399,15 @@ void KFindTransactionDlg::slotDateRangeChanged(int idx)
 void KFindTransactionDlg::slotDateChanged(void)
 {
   int idx;
-  for(idx = untilToday; idx < userDefined; ++idx) {
-    if(m_fromDate->getQDate() == m_startDates[idx]
-    && m_toDate->getQDate() == m_endDates[idx]) {
+  for(idx = MyMoneyTransactionFilter::untilToday; idx < MyMoneyTransactionFilter::userDefined; ++idx) {
+    if(m_fromDate->date() == m_startDates[idx]
+    && m_toDate->date() == m_endDates[idx]) {
       break;
     }
   }
-  m_dateRange->setCurrentItem(idx);
+  m_dateRange->blockSignals(true);
+  m_dateRange->setCurrentItem(static_cast<MyMoneyTransactionFilter::dateOptionE>(idx));
+  m_dateRange->blockSignals(false);
   slotUpdateSelections();
 }
 
@@ -474,7 +462,7 @@ void KFindTransactionDlg::loadPayees(void)
   list = file->payeeList();
   // load view
   for(it_l = list.begin(); it_l != list.end(); ++it_l) {
-    kMyMoneyCheckListItem* item = new kMyMoneyCheckListItem(m_payeesView, (*it_l).name(), (*it_l).id());
+    KMyMoneyCheckListItem* item = new KMyMoneyCheckListItem(m_payeesView, (*it_l).name(), QString(), (*it_l).id());
     connect(item, SIGNAL(stateChanged(bool)), this, SLOT(slotUpdateSelections()));
     item->setOn(true);
   }
@@ -542,7 +530,7 @@ void KFindTransactionDlg::scanCheckListItems(const QListViewItem* item, const op
 
   for(it_v = item->firstChild(); it_v != 0; it_v = it_v->nextSibling()) {
     if(it_v->rtti() == 1) {
-      kMyMoneyCheckListItem* it_c = static_cast<kMyMoneyCheckListItem*>(it_v);
+      KMyMoneyCheckListItem* it_c = static_cast<KMyMoneyCheckListItem*>(it_v);
       if(it_c->type() == QCheckListItem::CheckBox) {
         if(it_c->isOn())
           addItemToFilter(op, (*it_c).id());
@@ -558,7 +546,7 @@ void KFindTransactionDlg::scanCheckListItems(const QListView* view, const opType
 
   for(it_v = view->firstChild(); it_v != 0; it_v = it_v->nextSibling()) {
     if(it_v->rtti() == 1) {
-      kMyMoneyCheckListItem* it_c = static_cast<kMyMoneyCheckListItem*>(it_v);
+      KMyMoneyCheckListItem* it_c = static_cast<KMyMoneyCheckListItem*>(it_v);
       if(it_c->type() == QCheckListItem::CheckBox) {
         if(it_c->isOn())
           addItemToFilter(op, (*it_c).id());
@@ -579,14 +567,15 @@ void KFindTransactionDlg::setupFilter(void)
   }
 
   // Account tab
-  if(!m_accountsView->allAccountsSelected()) {
+  if(!m_accountsView->allItemsSelected()) {
     // retrieve a list of selected accounts
-    QCStringList list = m_accountsView->selectedAccounts();
+    QCStringList list;
+    m_accountsView->selectedItems(list);
 
     // if we're not in expert mode, we need to make sure
     // that all stock accounts for the selected investment
     // account are also selected
-    if(!KMyMoneyUtils::isExpertMode()) {
+    if(!KMyMoneyGlobalSettings::expertMode()) {
       QCStringList missing;
       QCStringList::const_iterator it_a, it_b;
       for(it_a = list.begin(); it_a != list.end(); ++it_a) {
@@ -607,7 +596,7 @@ void KFindTransactionDlg::setupFilter(void)
 
   // Date tab
   if(m_dateRange->currentItem() != 0) {
-    m_filter.setDateFilter(m_fromDate->getQDate(), m_toDate->getQDate());
+    m_filter.setDateFilter(m_fromDate->date(), m_toDate->date());
   }
 
   // Amount tab
@@ -627,9 +616,8 @@ void KFindTransactionDlg::setupFilter(void)
   }
 
   // Categories tab
-
-  if(!m_categoriesView->allAccountsSelected()) {
-    m_filter.addCategory(m_categoriesView->selectedAccounts());
+  if(!m_categoriesView->allItemsSelected()) {
+    m_filter.addCategory(m_categoriesView->selectedItems());
   }
 
   // Payees tab
@@ -672,117 +660,97 @@ void KFindTransactionDlg::slotSearch(void)
 
 void KFindTransactionDlg::slotRefreshView(void)
 {
-  try {
-    QValueList<MyMoneyTransaction> list = MyMoneyFile::instance()->transactionList(m_filter);
-    QValueList<MyMoneyTransaction>::ConstIterator it;
-
-    m_transactionList.clear();
-    QCString lastId;
-    int ofs = 0;
-
-    for(it = list.begin(); it != list.end(); ++it) {
-      KMyMoneyTransaction k(*it);
-      m_filter.match(*it, MyMoneyFile::instance()->storage());
-      if(lastId != (*it).id()) {
-        ofs = 0;
-        lastId = (*it).id();
-      } else
-        ofs++;
-
-      k.setSplitId(m_filter.matchingSplits()[ofs].id());
-      MyMoneyAccount acc = MyMoneyFile::instance()->account(m_filter.matchingSplits()[ofs].accountId());
-      if(acc.accountGroup() == MyMoneyAccount::Asset
-      || acc.accountGroup() == MyMoneyAccount::Liability)
-        m_transactionList.append(k);
-
-    }
-  } catch(MyMoneyException *e) {
-    delete e;
-    return;
+  m_needReload = true;
+  if(isVisible()) {
+    loadView();
+    m_needReload = false;
   }
-
-  QValueList<KMyMoneyTransaction>::ConstIterator it_t;
-
-  // setup the pointer vector
-  m_transactionPtrVector.clear();
-  m_transactionPtrVector.resize(m_transactionList.size());
-
-  int i;
-  for(i = 0, it_t = m_transactionList.begin(); it_t != m_transactionList.end(); ++it_t) {
-    m_transactionPtrVector.insert(i, &(*it_t));
-    ++i;
-  }
-
-  // sort the transactions
-  m_transactionPtrVector.sort();
-
-  bool dateMarkPlaced = false;
-  m_register->setCurrentDateIndex();    // turn off date mark
-
-  try {
-    // the trick is to go backwards ;-)
-
-    while(--i >= 0) {
-      if(m_transactionPtrVector.sortType() == KTransactionPtrVector::SortPostDate) {
-        if(m_transactionPtrVector[i]->postDate() > QDate::currentDate()) {
-          m_register->setCurrentDateIndex(i+1);
-
-        } else if(dateMarkPlaced == false) {
-          m_register->setCurrentDateIndex(i+1);
-          dateMarkPlaced = true;
-        }
-      }
-    }
-  } catch(MyMoneyException *e) {
-    qWarning("Unexpected exception in KFindTransactionDlg::refreshView");
-    delete e;
-    return;
-  }
-  m_foundText->setText(i18n(QString("Found %1 matching transactions")
-                      .arg(m_transactionPtrVector.count())));
-  m_register->setTransactionCount(m_transactionPtrVector.count());
-  m_register->setCurrentTransactionIndex(0);
-  m_registerFrame->show();
 }
 
-KMyMoneyTransaction* KFindTransactionDlg::transaction(const int idx) const
+void KFindTransactionDlg::show(void)
 {
-  if(idx >= 0 && static_cast<unsigned> (idx) < m_transactionPtrVector.count())
-    return m_transactionPtrVector[idx];
-  return 0;
+  if(m_needReload) {
+    loadView();
+    m_needReload = false;
+  }
+  KFindTransactionDlgDecl::show();
 }
 
-const QCString KFindTransactionDlg::accountId(const MyMoneyTransaction * const transaction, int match) const
+void KFindTransactionDlg::loadView(void)
 {
-  QValueList<MyMoneySplit>::ConstIterator it;
-  MyMoneyFile* file = MyMoneyFile::instance();
-  MyMoneyTransactionFilter filter = m_filter;
+  // setup sort order
+  m_register->setSortOrder(KMyMoneyGlobalSettings::sortSearchView());
 
-  if(filter.match(*transaction, file->storage())) {
-    for(it = filter.matchingSplits().begin(); it != filter.matchingSplits().end(); ++it) {
-      MyMoneyAccount acc = file->account((*it).accountId());
-      switch(acc.accountGroup()) {
-        case MyMoneyAccount::Income:
-        case MyMoneyAccount::Expense:
-          break;
-        default:
-          if(!match--)
-            return (*it).accountId();
-          break;
+  // clear out old data
+  m_register->clear();
+
+  // retrieve the list from the engine
+  MyMoneyFile::instance()->transactionList(m_transactionList, m_filter);
+
+    // create the elements for the register
+  QValueList<QPair<MyMoneyTransaction, MyMoneySplit> >::const_iterator it;
+  QMap<QCString, int>uniqueMap;
+  MyMoneyMoney deposit, payment;
+
+  int splitCount = 0;
+  for(it = m_transactionList.begin(); it != m_transactionList.end(); ++it) {
+    const MyMoneySplit& split = (*it).second;
+    MyMoneyAccount acc = MyMoneyFile::instance()->account(split.accountId());
+    ++splitCount;
+    uniqueMap[(*it).first.id()]++;
+
+    KMyMoneyRegister::Register::transactionFactory(m_register, (*it).first, (*it).second, uniqueMap[(*it).first.id()]);
+    { // debug stuff
+      if(split.shares().isNegative()) {
+        payment += split.shares().abs();
+      } else {
+        deposit += split.shares().abs();
       }
     }
   }
-  qFatal("KFindTransactionDlg::accountId(): No asset/liability account for transaction. This usually crashes");
-  return QCString();
+
+    // add the group markers
+  m_register->addGroupMarkers();
+
+    // sort the transactions according to the sort setting
+  m_register->sortItems();
+
+    // remove trailing and adjacent markers
+  m_register->removeUnwantedGroupMarkers();
+
+  // turn on the ledger lens for the register
+  m_register->setLedgerLensForced();
+
+  m_register->updateRegister(true);
+
+#if KMM_DEBUG
+  m_foundText->setText(i18n("Found %1 matching transactions (D %2 / P %3 = %4)")
+                      .arg(splitCount).arg(deposit.formatMoney("", 2)).arg(payment.formatMoney("", 2)).arg((deposit-payment).formatMoney("", 2)));
+#else
+  m_foundText->setText(i18n("Found %1 matching transactions") .arg(splitCount));
+#endif
+
+  m_tabWidget->setTabEnabled(m_resultPage, true);
+  m_tabWidget->setCurrentPage(m_tabWidget->indexOf(m_resultPage));
+
+  QTimer::singleShot(10, this, SLOT(slotRightSize()));
 }
 
-bool KFindTransactionDlg::focusNextPrevChild(bool next)
+void KFindTransactionDlg::slotRightSize(void)
 {
-  return KFindTransactionDlgDecl::focusNextPrevChild(next);
+  m_register->updateContents();
 }
 
 void KFindTransactionDlg::resizeEvent(QResizeEvent* ev)
 {
+  // Columns
+  // 1 = Date
+  // 2 = Account
+  // 4 = Detail
+  // 5 = C
+  // 6 = Payment
+  // 7 = Deposit
+
   // don't forget the resizer
   KFindTransactionDlgDecl::resizeEvent(ev);
 
@@ -795,109 +763,36 @@ void KFindTransactionDlg::resizeEvent(QResizeEvent* ev)
   int m_debitWidth = 80;
   int m_creditWidth = 80;
 
-  m_register->adjustColumn(0);
   m_register->adjustColumn(1);
   m_register->adjustColumn(2);
+  m_register->adjustColumn(5);
 
-  m_register->setColumnWidth(4, m_debitWidth);
-  m_register->setColumnWidth(5, m_creditWidth);
+  m_register->setColumnWidth(6, m_debitWidth);
+  m_register->setColumnWidth(7, m_creditWidth);
 
   for(int i = 0; i < m_register->numCols(); ++i) {
     switch(i) {
+      case 4:     // skip the one, we want to set
+        break;
       default:
         w -= m_register->columnWidth(i);
         break;
-      case 3:     // skip the one, we want to set
-        break;
     }
   }
-  m_register->setColumnWidth(3, w);
+
+  m_register->setColumnWidth(4, w);
 }
 
 
-void KFindTransactionDlg::slotRegisterClicked(int row, int /* col */, int /* button */, const QPoint& /* mousePos */)
-{
-  if(m_register->setCurrentTransactionRow(row) == true) {
-    m_register->ensureTransactionVisible();
-    m_register->repaintContents(false);
-  }
-}
-
-void KFindTransactionDlg::slotRegisterDoubleClicked(int row,
-                                                    int /* col */,
-                                                    int /* button */,
-                                                    const QPoint & /* mousePos */)
-{
-  if(m_register->setCurrentTransactionRow(row) == true) {
-    m_register->ensureTransactionVisible();
-    m_register->repaintContents(false);
-  }
-  slotSelectTransaction();
-}
-
 void KFindTransactionDlg::slotSelectTransaction(void)
 {
-  KMyMoneyTransaction *k = transaction(m_register->currentTransactionIndex());
-  if(k != 0) {
-    emit transactionSelected(k->splitById(k->splitId()).accountId(), k->id());
-    hide();
-  }
-}
-
-void KFindTransactionDlg::slotNextTransaction(void)
-{
-  if(static_cast<unsigned> (m_register->currentTransactionIndex() + 1) <= m_transactionPtrVector.count()) {
-    m_register->setCurrentTransactionIndex(m_register->currentTransactionIndex()+1);
-    m_register->ensureTransactionVisible();
-    m_register->repaintContents(false);
-  }
-}
-
-void KFindTransactionDlg::slotPreviousTransaction(void)
-{
-  if(m_register->currentTransactionIndex() > 0) {
-    m_register->setCurrentTransactionIndex(m_register->currentTransactionIndex()-1);
-    m_register->ensureTransactionVisible();
-    m_register->repaintContents(false);
-  }
-}
-
-void KFindTransactionDlg::slotSelectTransaction(const QCString& transactionId)
-{
-  int   idx = -1;
-
-  if(!transactionId.isEmpty()) {
-    for(unsigned i = 0; i < m_transactionPtrVector.count(); ++i) {
-      if(m_transactionPtrVector[i]->id() == transactionId) {
-        idx = i;
-        break;
-      }
+  QValueList<KMyMoneyRegister::RegisterItem*> list = m_register->selectedItems();
+  if(!list.isEmpty()) {
+    KMyMoneyRegister::Transaction* t = dynamic_cast<KMyMoneyRegister::Transaction*>(list[0]);
+    if(t) {
+      emit transactionSelected(t->split().accountId(), t->transaction().id());
+      hide();
     }
-  } else {
-    if(m_transactionPtrVector.count() > 0) {
-      idx = m_transactionPtrVector.count()-1;
-    }
-  }
-
-  if(idx != -1) {
-    // qDebug("KLedgerView::selectTransaction index is %d", idx);
-    m_transactionPtr = m_transactionPtrVector[idx];
-    m_register->setCurrentTransactionIndex(idx);
-    m_register->ensureTransactionVisible();
-    m_register->repaintContents(false);
-  }
-}
-
-void KFindTransactionDlg::update(const QCString& /* id */)
-{
-  // only calculate the new list if it is currently visible
-  if(m_registerFrame->isVisible()) {
-    KMyMoneyTransaction *k = transaction(m_register->currentTransactionIndex());
-    QCString id = k->id();
-
-    slotRefreshView();
-
-    slotSelectTransaction(id);
   }
 }
 
@@ -926,11 +821,6 @@ bool KFindTransactionDlg::eventFilter(QObject* o, QEvent* e)
   return rc;
 }
 
-const MyMoneyMoney KFindTransactionDlg::balance(const int /* idx */) const
-{
-  return 0;
-}
-
 void KFindTransactionDlg::slotShowHelp(void)
 {
   QString anchor = m_helpAnchor[m_criteriaTab->currentPage()];
@@ -940,6 +830,24 @@ void KFindTransactionDlg::slotShowHelp(void)
   kapp->invokeHelp(anchor);
 }
 
+void KFindTransactionDlg::slotSortOptions(void)
+{
+  KSortOptionDlg* dlg = new KSortOptionDlg(this);
+
+  dlg->setSortOption(KMyMoneyGlobalSettings::sortSearchView(), QString());
+  dlg->hideDefaultButton();
+
+  if(dlg->exec() == QDialog::Accepted) {
+    QString sortOrder = dlg->sortOption();
+    if(sortOrder != KMyMoneyGlobalSettings::sortSearchView()) {
+      KMyMoneyGlobalSettings::setSortSearchView(sortOrder);
+      slotRefreshView();
+    }
+  }
+  delete dlg;
+}
+
+
 // vim:cin:si:ai:et:ts=2:sw=2:
 
 
