@@ -30,10 +30,9 @@
 // ----------------------------------------------------------------------------
 // Project Includes
 
-#include "../views/kledgerview.h"
-#include "../mymoney/mymoneyutils.h"
-#include "../mymoney/mymoneytransactionfilter.h"
-#include "../mymoney/mymoneyobserver.h"
+// #include "../views/kledgerview.h"
+#include <kmymoney/mymoneyutils.h>
+#include <kmymoney/mymoneytransactionfilter.h>
 
 #include "../dialogs/kfindtransactiondlgdecl.h"
 
@@ -43,11 +42,12 @@ class QListViewItem;
 /**
   * @author Thomas Baumgart
   */
-
-class KFindTransactionDlg : public KFindTransactionDlgDecl, public IMyMoneyRegisterParent, public MyMoneyObserver {
+class KFindTransactionDlg : public KFindTransactionDlgDecl
+{
   Q_OBJECT
 public:
 
+  /*
   // Make sure to keep the following enum valus in sync with the values
   // used by the GUI in kfindtransactiondlgdecl.ui
   enum dateOptionE {
@@ -57,98 +57,50 @@ public:
     currentYear,
     monthToDate,
     yearToDate,
+    yearToMonth,
     lastMonth,
     lastYear,
+    last7Days,
     last30Days,
     last3Months,
     last6Months,
     last12Months,
+    next7Days,
     next30Days,
     next3Months,
     next6Months,
     next12Months,
     userDefined,
+    last3ToNext3Months,
+    last11Months,
     // insert new constants above of this line
     dateOptionCount
   };
-
-  /**
-    * Constructor for the find transaction dialog
-    *
-    * @param parent see QWidget()
-    * @param name see QWidget()
-    * @param forceSearchEnabled The Search/Ok button of the dialog is only
-    *                           enabled once a filter has been selected. Setting
-    *                           this to @a true leaves the button enabled
-    *                           at all times.
-    */
-  KFindTransactionDlg(QWidget *parent=0, const char *name=0, bool forceSearchEnabled=false);
-  ~KFindTransactionDlg();
-
-  /**
-    * This method returns a pointer to the transaction data
-    * in the ledger of this account. The transaction is identified
-    * by the parameter @p idx.
-    *
-    * @param idx index into ledger starting at 0
-    * @return pointer to MyMoneyTransaction object representing the
-    *         selected transaction. If idx is out of bounds,
-    *         0 will be returned.
-    */
-  KMyMoneyTransaction* transaction(const int idx) const;
-
-  /**
-    * This method returns the id of the account that should be
-    * shown by the view. The implementation depends on the type of view
-    * and might return a constant value or a value depending on the
-    * transaction passed as @p transaction.
-    *
-    * @param transaction pointer to a transaction
-    * @param match identifies the number of the matching split within the transaction
-    *              should be used to extract the account information. 0 will use
-    *              the first matching split.
-    *
-    * @return const QCString containing the account's id.
-    */
-  const QCString accountId(const MyMoneyTransaction * const transaction, const int match) const;
-
-  /**
-    * This method returns the balance of any visible transaction
-    * in the ledger of this account. The balance depends on filters
-    * and is automatically calculated when any view option is changed
-    * (e.g. filters, sort order, etc.)
-    *
-    * @param idx index into the ledger starting at 0
-    * @return Value of the balance for the account after the selected
-    *         transaction has been taken into account. If idx is out
-    *         of bounds, 0 will be returned as value. For liability type
-    *         accounts, the sign will be inverted for display purposes.
-    *
-    * @note Currently the fixed value 0 will be returned as the balance
-    *       is not shown in the find transaction dialog but the method
-    *       must be provided as part of the IMyMoneyRegisterParent interface.
-    */
-  const MyMoneyMoney balance(const int idx) const;
-
-  /**
-    * This method is called by the engine whenever a required notification
-    * is sent out for this object. Attachment of the observer is performed
-    * within the constructor, detachment within the destructor of this
-    * object.
-    */
-  void update(const QCString& id);
+*/
+  KFindTransactionDlg(QWidget *parent=0, const char *name=0);
+  ~KFindTransactionDlg() {}
 
   virtual bool eventFilter( QObject *o, QEvent *e );
 
 public slots:
-  void slotUpdateSelections(void);
-  void slotReset(void);
+  void show(void);
 
 protected:
   void resizeEvent(QResizeEvent*);
-  bool focusNextPrevChild(bool next);
 
 protected slots:
+  virtual void slotReset(void);
+  virtual void slotSearch(void);
+
+  /**
+    * This slot opens the detailed help page in khelpcenter. The
+    * anchor for the information is taken from m_helpAnchor.
+    */
+  virtual void slotShowHelp(void);
+
+
+  void slotUpdateSelections(void);
+
   void slotDateRangeChanged(int);
   void slotDateChanged(void);
 
@@ -161,31 +113,27 @@ protected slots:
   void slotNrSelected(void);
   void slotNrRangeSelected(void);
 
-  void slotSearch(void);
-
   void slotRefreshView(void);
 
   /**
-    *
+    * This slot selects the current selected transaction/split and emits
+    * the signal @a transactionSelected(const QCString& accountId, const QCString& transactionId)
     */
-  void slotRegisterClicked(int row, int col, int button, const QPoint &mousePos);
-  void slotRegisterDoubleClicked(int row, int col, int button, const QPoint &mousePos);
-  void slotNextTransaction(void);
-  void slotPreviousTransaction(void);
   void slotSelectTransaction(void);
-  void slotSelectTransaction(const QCString& id);
 
   void slotRightSize(void);
 
-  /**
-    * This slot opens the detailed help page in khelpcenter. The
-    * anchor for the information is taken from m_helpAnchor.
-    */
-  void slotShowHelp(void);
+  void slotSortOptions(void);
 
 signals:
   void transactionSelected(const QCString& accountId, const QCString& transactionId);
 
+  /**
+    * This signal is sent out when no selection has been made. It is
+    * used to control the state of the Search button.
+    */
+  void selectionEmpty(bool);
+
 protected:
   enum opTypeE {
     addAccountToFilter = 0,
@@ -207,15 +155,17 @@ protected:
   void selectItems(QListView* view, const QCStringList& list, const bool state);
   void selectSubItems(QListViewItem* item, const QCStringList& list, const bool state);
 
-  void readConfig(void);
-  void writeConfig(void);
-
   /**
     * This method loads the m_payeesView with the payees name
     * found in the engine.
     */
   void loadPayees(void);
 
+  /**
+    * This method loads the register with the matching transactions
+    */
+  void loadView(void);
+
   /**
     * This method returns information about the selection state
     * of the items in the m_accountsView.
@@ -235,32 +185,19 @@ protected:
   void addItemToFilter(const opTypeE op, const QCString& id);
 
 protected:
-  QDate                m_startDates[dateOptionCount];
-  QDate                m_endDates[dateOptionCount];
+  QDate                m_startDates[MyMoneyTransactionFilter::dateOptionCount];
+  QDate                m_endDates[MyMoneyTransactionFilter::dateOptionCount];
 
   /**
     * This member holds a list of all transactions matching the filter criteria
     */
-  QValueList<KMyMoneyTransaction> m_transactionList;
-
-  /**
-    * This member keeps a vector of pointers to all visible (filtered)
-    * transaction in m_transactionList in sorted order. Sorting is done
-    * in KTransactionPtrVector::compareItems
-    */
-  KTransactionPtrVector           m_transactionPtrVector;
-
-  /**
-    * This member keeps a pointer to the currently selected transaction
-    * It is NULL, if an empty (new) transaction is selected.
-    */
-  KMyMoneyTransaction*            m_transactionPtr;
+  QValueList<QPair<MyMoneyTransaction, MyMoneySplit> > m_transactionList;
 
   MyMoneyTransactionFilter        m_filter;
 
   QMap<QWidget*, QString>         m_helpAnchor;
 
-  bool                            m_forceSearchEnabled;
+  bool                            m_needReload;
 };
 
 #endif
