@@ -16,6 +16,10 @@
  *                                                                         *
  ***************************************************************************/
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 // ----------------------------------------------------------------------------
 // QT Includes
 
@@ -29,6 +33,8 @@
 #include <qtabwidget.h>
 #include <qbuttongroup.h>
 #include <qradiobutton.h>
+#include <qtextedit.h>
+#include <qlayout.h>
 
 // ----------------------------------------------------------------------------
 // KDE Headers
@@ -38,31 +44,53 @@
 #include <kmessagebox.h>
 #include <kcombobox.h>
 #include <klistview.h>
-#include <kconfig.h>
 #include <kstandarddirs.h>
 #include <kiconloader.h>
 #include <kguiitem.h>
 #include <kpushbutton.h>
+#include <kled.h>
+#include <kdebug.h>
+#include <kglobalsettings.h>
 
 // ----------------------------------------------------------------------------
 // Project Includes
 
 #include "knewaccountdlg.h"
-#include "../widgets/kmymoneyedit.h"
-#include "../widgets/kmymoneydateinput.h"
+
+#include <kmymoney/kmymoneyedit.h>
+#include <kmymoney/kmymoneydateinput.h>
+#include <kmymoney/mymoneyexception.h>
+#include <kmymoney/mymoneyfile.h>
+#include <kmymoney/kmymoneyaccounttree.h>
+#include <kmymoney/kmymoneyglobalsettings.h>
+#include <kmymoney/mymoneyreport.h>
+#include <kmymoney/kguiutils.h>
+#include <kmymoney/kmymoneycombo.h>
+
 #include "../widgets/kmymoneycurrencyselector.h"
-#include "../widgets/kmymoneyequity.h"
 #include "../widgets/kmymoneyaccountselector.h"
 
 #include "../mymoney/mymoneyexception.h"
-#include "../mymoney/mymoneyfile.h"
+#include "../mymoney/mymoneykeyvaluecontainer.h"
 #include "../dialogs/knewbankdlg.h"
-#include "../views/kbanklistitem.h"
 #include "../views/kmymoneyfile.h"
 #include "../kmymoneyutils.h"
 
-#define TAB_GENERAL      0
-#define TAB_TAX          1
+#include "../reports/kreportchartview.h"
+#include "../reports/pivottable.h"
+
+#ifdef USE_OFX_DIRECTCONNECT
+#include "../dialogs/konlinebankingsetupwizard.h"
+#endif
+
+// in KOffice version < 1.5 KDCHART_PROPSET_NORMAL_DATA was a static const
+// but in 1.5 this has been changed into a #define'd value. So we have to
+// make sure, we use the right one.
+#ifndef KDCHART_PROPSET_NORMAL_DATA
+#define KMM_KDCHART_PROPSET_NORMAL_DATA KDChartParams::KDCHART_PROPSET_NORMAL_DATA
+#else
+#define KMM_KDCHART_PROPSET_NORMAL_DATA KDCHART_PROPSET_NORMAL_DATA
+#endif
 
 KNewAccountDlg::KNewAccountDlg(const MyMoneyAccount& account, bool isEditing, bool categoryEditor, QWidget *parent, const char *name, const QString& title)
   : KNewAccountDlgDecl(parent,name,true),
@@ -85,7 +113,7 @@ KNewAccountDlg::KNewAccountDlg(const MyMoneyAccount& account, bool isEditing, bo
 
   m_subAccountLabel->setText(i18n("Is a sub account"));
 
-  m_qlistviewParentAccounts->header()->setFont(KMyMoneyUtils::headerFont());
+  m_qlistviewParentAccounts->header()->setFont(KMyMoneyGlobalSettings::listHeaderFont());
 
   accountNameEdit->setText(account.name());
   descriptionEdit->setText(account.description());
@@ -93,14 +121,37 @@ KNewAccountDlg::KNewAccountDlg(const MyMoneyAccount& account, bool isEditing, bo
   typeCombo->setEnabled(true);
   MyMoneyFile *file = MyMoneyFile::instance();
 
+  // load the price mode combo
+  m_priceMode->insertItem(i18n("default price mode", "<default>"), 0);
+  m_priceMode->insertItem(i18n("Price per share"), 1);
+  m_priceMode->insertItem(i18n("Total for all shares"), 2);
+
+  int priceMode = 0;
+  if(m_account.accountType() == MyMoneyAccount::Investment) {
+    m_priceMode->setEnabled(true);
+    if(!m_account.value("priceMode").isEmpty())
+      priceMode = m_account.value("priceMode").toInt();
+  }
+  m_priceMode->setCurrentItem(priceMode);
+
+  bool haveMinBalance = false;
+  bool haveMaxCredit = false;
   if (categoryEditor)
   {
+    // get rid of the tabs that are not used for categories
+    QWidget* tab = m_tab->page(m_tab->indexOf(m_institutionTab));
+    if(tab)
+      m_tab->removePage(tab);
+    tab = m_tab->page(m_tab->indexOf(m_limitsTab));
+    if(tab)
+      m_tab->removePage(tab);
 
-    m_qlistviewParentAccounts->setEnabled(true);
+    //m_qlistviewParentAccounts->setEnabled(true);
     startDateEdit->setEnabled(false);
     accountNoEdit->setEnabled(false);
 
     m_institutionBox->hide();
+    m_onlineBankingBox->hide();
     m_qcheckboxNoVat->hide();
 
     typeCombo->insertItem(KMyMoneyUtils::accountTypeToString(MyMoneyAccount::Income));
@@ -118,23 +169,61 @@ KNewAccountDlg::KNewAccountDlg(const MyMoneyAccount& account, bool isEditing, bo
         typeCombo->setCurrentItem(1);
         break;
     }
+    m_currency->setEnabled(true);
     if (m_isEditing)
     {
       typeCombo->setEnabled(false);
+      m_currency->setDisabled(MyMoneyFile::instance()->isReferenced(m_account));
     }
     m_qcheckboxPreferred->hide();
-    m_currency->setEnabled(false);
-    m_equityText->hide();
-    m_equity->hide();
 
     m_qcheckboxTax->setChecked(account.value("Tax") == "Yes");
     loadVatAccounts();
   }
   else
   {
-    QWidget* tab = m_tab->page(TAB_TAX);
-    if(tab) {
+    // get rid of the tabs that are not used for accounts
+    QWidget* tab = m_tab->page(m_tab->indexOf(m_taxTab));
+    if(tab)
       m_tab->removePage(tab);
+
+    switch(m_account.accountType()) {
+      case MyMoneyAccount::Savings:
+      case MyMoneyAccount::Cash:
+        haveMinBalance = true;
+        break;
+
+      case MyMoneyAccount::Checkings:
+        haveMinBalance = true;
+        haveMaxCredit = true;
+        break;
+
+      case MyMoneyAccount::CreditCard:
+        haveMaxCredit = true;
+        break;
+
+      default:
+        // no limit available, so we might get rid of the tab
+        tab = m_tab->page(m_tab->indexOf(m_limitsTab));
+        if(tab)
+          m_tab->removePage(tab);
+        // don't try to hide the widgets we just wiped
+        // in the next step
+        haveMaxCredit = haveMinBalance = true;
+        break;
+    }
+
+    if(!haveMaxCredit) {
+      m_maxCreditLabel->setEnabled(false);
+      m_maxCreditLabel->hide();
+      m_maxCreditEarlyEdit->hide();
+      m_maxCreditAbsoluteEdit->hide();
+    }
+    if(!haveMinBalance) {
+      m_minBalanceLabel->setEnabled(false);
+      m_minBalanceLabel->hide();
+      m_minBalanceEarlyEdit->hide();
+      m_minBalanceAbsoluteEdit->hide();
     }
 
     typeCombo->insertItem(KMyMoneyUtils::accountTypeToString(MyMoneyAccount::Checkings));
@@ -182,6 +271,7 @@ KNewAccountDlg::KNewAccountDlg(const MyMoneyAccount& account, bool isEditing, bo
         break;
       case MyMoneyAccount::Stock:
         m_institutionBox->hide();
+        m_onlineBankingBox->hide();
         typeCombo->setCurrentItem(8);
         break;
 /*
@@ -204,33 +294,45 @@ KNewAccountDlg::KNewAccountDlg(const MyMoneyAccount& account, bool isEditing, bo
     accountNoEdit->setText(account.number());
     m_qcheckboxPreferred->setChecked(account.value("PreferredAccount") == "Yes");
     m_qcheckboxNoVat->setChecked(account.value("NoVat") == "Yes");
+    loadKVP("iban", ibanEdit);
+    loadKVP("minBalanceAbsolute", m_minBalanceAbsoluteEdit);
+    loadKVP("minBalanceEarly", m_minBalanceEarlyEdit);
+    loadKVP("maxCreditAbsolute", m_maxCreditAbsoluteEdit);
+    loadKVP("maxCreditEarly", m_maxCreditEarlyEdit);
+    // reverse the sign for display purposes
+    if(!m_maxCreditAbsoluteEdit->lineedit()->text().isEmpty())
+      m_maxCreditAbsoluteEdit->setValue(m_maxCreditAbsoluteEdit->value()*MyMoneyMoney(-1,1));
+    if(!m_maxCreditEarlyEdit->lineedit()->text().isEmpty())
+      m_maxCreditEarlyEdit->setValue(m_maxCreditEarlyEdit->value()*MyMoneyMoney(-1,1));
+    loadKVP("lastNumberUsed", m_lastCheckNumberUsed);
+
 
     // we do not allow to change the account type once an account
-    // was created. Same applies to currency.
+    // was created. Same applies to currency if it is referenced.
     if (m_isEditing)
     {
       typeCombo->setEnabled(false);
-      m_currency->setEnabled(false);
+      m_currency->setDisabled(MyMoneyFile::instance()->isReferenced(m_account));
     }
-    if(m_account.accountType() == MyMoneyAccount::Stock) {
+    if(m_account.isInvest()) {
       typeCombo->setEnabled(false);
       m_qcheckboxPreferred->hide();
       m_currencyText->hide();
       m_currency->hide();
     } else {
-      m_equityText->hide();
-      m_equity->hide();
+      // use the old field and override a possible new value
+      if(!MyMoneyMoney(account.value("minimumBalance")).isZero()) {
+        m_minBalanceAbsoluteEdit->setValue(MyMoneyMoney(account.value("minimumBalance")));
+      }
     }
 
     m_qcheckboxTax->hide();
+
+    displayOnlineBankingStatus();
+
   }
 
-  // setup the currency / equity widgets
-  if(account.accountType() == MyMoneyAccount::Stock) {
-    m_equity->loadEquity(account.currencyId());
-  } else {
-    m_currency->setSecurity(file->currency(account.currencyId()));
-  }
+  m_currency->setSecurity(file->currency(account.currencyId()));
 
   // Load the institutions
   // then the accounts
@@ -250,12 +352,11 @@ KNewAccountDlg::KNewAccountDlg(const MyMoneyAccount& account, bool isEditing, bo
   }
 
   initParentWidget(account.parentAccountId(), account.id());
-
-  if(m_account.accountType() == MyMoneyAccount::Stock)
+  if(m_account.isInvest())
     m_qlistviewParentAccounts->setEnabled(false);
 
   if (!categoryEditor)
-    loadInstitutions(institutionName);
+    slotLoadInstitutions(institutionName);
 
   accountNameEdit->setFocus();
 
@@ -281,12 +382,11 @@ KNewAccountDlg::KNewAccountDlg(const MyMoneyAccount& account, bool isEditing, bo
   connect(m_qlistviewParentAccounts, SIGNAL(selectionChanged(QListViewItem*)),
     this, SLOT(slotSelectionChanged(QListViewItem*)));
   connect(m_qbuttonNew, SIGNAL(clicked()), this, SLOT(slotNewClicked()));
+  connect(m_buttonOnlineSetup, SIGNAL(clicked()), this, SLOT(slotOnlineSetupClicked()));
   connect(typeCombo, SIGNAL(activated(const QString&)),
     this, SLOT(slotAccountTypeChanged(const QString&)));
 
   connect(accountNameEdit, SIGNAL(textChanged(const QString&)), this, SLOT(slotCheckFinished()));
-  // connect(m_equity, SIGNAL(equityChanged(const QCString&)), this, SLOT(slotCheckFinished()));
-  connect(m_equity, SIGNAL(textChanged(const QString&)), this, SLOT(slotCheckFinished()));
 
   connect(m_vatCategory, SIGNAL(toggled(bool)), this, SLOT(slotVatChanged(bool)));
   connect(m_vatAssignment, SIGNAL(toggled(bool)), this, SLOT(slotVatAssignmentChanged(bool)));
@@ -295,13 +395,20 @@ KNewAccountDlg::KNewAccountDlg(const MyMoneyAccount& account, bool isEditing, bo
   connect(m_vatRate, SIGNAL(textChanged(const QString&)), this, SLOT(slotCheckFinished()));
   connect(m_vatAccount, SIGNAL(stateChanged()), this, SLOT(slotCheckFinished()));
 
+  connect(m_minBalanceEarlyEdit, SIGNAL(valueChanged(const QString&)), this, SLOT(slotAdjustMinBalanceAbsoluteEdit(const QString&)));
+  connect(m_minBalanceAbsoluteEdit, SIGNAL(valueChanged(const QString&)), this, SLOT(slotAdjustMinBalanceEarlyEdit(const QString&)));
+  connect(m_maxCreditEarlyEdit, SIGNAL(valueChanged(const QString&)), this, SLOT(slotAdjustMaxCreditAbsoluteEdit(const QString&)));
+  connect(m_maxCreditAbsoluteEdit, SIGNAL(valueChanged(const QString&)), this, SLOT(slotAdjustMaxCreditEarlyEdit(const QString&)));
+
+  connect(m_qcomboboxInstitutions, SIGNAL(activated(const QString&)), this, SLOT(slotLoadInstitutions(const QString&)));
+
   m_vatCategory->setChecked(false);
   m_vatAssignment->setChecked(false);
 
   // make sure our account does not have an id and no parent assigned
   // and certainly no children in case we create a new account
   if(!m_isEditing) {
-    m_account.setAccountId(QCString());
+    m_account.clearId();
     m_account.setParentAccountId(QCString());
     QCStringList::ConstIterator it;
     while((it = m_account.accountList().begin()) != m_account.accountList().end())
@@ -342,6 +449,10 @@ KNewAccountDlg::KNewAccountDlg(const MyMoneyAccount& account, bool isEditing, bo
   slotVatAssignmentChanged(m_vatAssignment->isChecked());
   slotCheckFinished();
 
+  kMandatoryFieldGroup* requiredFields = new kMandatoryFieldGroup (this);
+  requiredFields->setOkButton(createButton); // button to be enabled when all fields present
+  requiredFields->add(accountNameEdit);
+
   // using a timeout is the only way, I got the 'ensureItemVisible'
   // working when creating the dialog. I assume, this
   // has something to do with the delayed update of the display somehow.
@@ -357,8 +468,14 @@ void KNewAccountDlg::timerDone(void)
   m_qlistviewParentAccounts->repaintContents(false);
 }
 
-KNewAccountDlg::~KNewAccountDlg()
+void KNewAccountDlg::setOpeningBalance(const MyMoneyMoney& balance)
 {
+  m_openingBalanceEdit->setValue(balance);
+}
+
+void KNewAccountDlg::setOpeningBalanceShown(bool shown)
+{
+  m_openingBalanceEdit->setShown(shown);
 }
 
 void KNewAccountDlg::okClicked()
@@ -411,6 +528,25 @@ void KNewAccountDlg::okClicked()
 
   m_account.setName(accountNameText);
   m_account.setNumber(accountNoEdit->text());
+  storeKVP("iban", ibanEdit);
+  storeKVP("minBalanceAbsolute", m_minBalanceAbsoluteEdit);
+  storeKVP("minBalanceEarly", m_minBalanceEarlyEdit);
+
+  // the figures for credit line with reversed sign
+  if(!m_maxCreditAbsoluteEdit->lineedit()->text().isEmpty())
+    m_maxCreditAbsoluteEdit->setValue(m_maxCreditAbsoluteEdit->value()*MyMoneyMoney(-1,1));
+  if(!m_maxCreditEarlyEdit->lineedit()->text().isEmpty())
+    m_maxCreditEarlyEdit->setValue(m_maxCreditEarlyEdit->value()*MyMoneyMoney(-1,1));
+  storeKVP("maxCreditAbsolute", m_maxCreditAbsoluteEdit);
+  storeKVP("maxCreditEarly", m_maxCreditEarlyEdit);
+  if(!m_maxCreditAbsoluteEdit->lineedit()->text().isEmpty())
+    m_maxCreditAbsoluteEdit->setValue(m_maxCreditAbsoluteEdit->value()*MyMoneyMoney(-1,1));
+  if(!m_maxCreditEarlyEdit->lineedit()->text().isEmpty())
+    m_maxCreditEarlyEdit->setValue(m_maxCreditEarlyEdit->value()*MyMoneyMoney(-1,1));
+
+  storeKVP("lastNumberUsed", m_lastCheckNumberUsed);
+  // delete a previous version of the minimumbalance information
+  storeKVP("minimumBalance", QString(), QString());
 
   MyMoneyAccount::accountTypeE acctype;
   if (!m_categoryEditor)
@@ -419,21 +555,30 @@ void KNewAccountDlg::okClicked()
     // If it's a loan, check if the parent is asset or liability. In
     // case of asset, we change the account type to be AssetLoan
     if(acctype == MyMoneyAccount::Loan
-    && file->accountGroup(parent.accountType()) == MyMoneyAccount::Asset)
+    && parent.accountGroup() == MyMoneyAccount::Asset)
       acctype = MyMoneyAccount::AssetLoan;
 
+#if 0
+    // we do allow the same name for different accounts, so
+    // we don't need this check anymore.
     if(!file->nameToAccount(accountNameText).isEmpty()
     && (file->nameToAccount(accountNameText) != m_account.id())) {
-      KMessageBox::error(this, i18n("Account with that name already exists."));
+      KMessageBox::error(this, QString("<qt>")+i18n("An account named <b>%1</b> already exists. You cannot create a second account with the same name.").arg(accountNameText)+QString("</qt>"));
       return;
     }
+#endif
   }
   else
   {
-    acctype = MyMoneyFile::instance()->accountGroup(parent.accountType());
-    if(!file->categoryToAccount(accountNameText).isEmpty()
-    && (file->categoryToAccount(accountNameText) != m_account.id())) {
-      KMessageBox::error(this, i18n("Category with that name already exists."));
+    acctype = parent.accountGroup();
+    QString newName;
+    if(!MyMoneyFile::instance()->isStandardAccount(parent.id())) {
+      newName = MyMoneyFile::instance()->accountToCategory(parent.id()) + MyMoneyFile::AccountSeperator;
+    }
+    newName += accountNameText;
+    if(!file->categoryToAccount(newName, acctype).isEmpty()
+    && (file->categoryToAccount(newName, acctype) != m_account.id())) {
+      KMessageBox::error(this, QString("<qt>")+i18n("A category named <b>%1</b> already exists. You cannot create a second category with the same name.").arg(newName)+QString("</qt>"));
       return;
     }
   }
@@ -443,12 +588,8 @@ void KNewAccountDlg::okClicked()
 
   if (!m_categoryEditor)
   {
-    m_account.setOpeningDate(startDateEdit->getQDate());
-    if(m_account.accountType() == MyMoneyAccount::Stock) {
-      m_account.setCurrencyId(m_equity->id());
-    } else {
-      m_account.setCurrencyId(m_currency->security().id());
-    }
+    m_account.setOpeningDate(startDateEdit->date());
+    m_account.setCurrencyId(m_currency->security().id());
 
     if(m_qcheckboxPreferred->isChecked())
       m_account.setValue("PreferredAccount", "Yes");
@@ -458,12 +599,14 @@ void KNewAccountDlg::okClicked()
       m_account.setValue("NoVat", "Yes");
     else
       m_account.deletePair("NoVat");
+
+    if(m_minBalanceAbsoluteEdit->isVisible()) {
+      m_account.setValue("minimumBalance", m_minBalanceAbsoluteEdit->value().toString());
+    }
   }
   else
   {
-    KConfig *kconfig = KGlobal::config();
-    kconfig->setGroup("List Options");
-    if(kconfig->readBoolEntry("HideUnusedCategory", false) == true && !m_isEditing) {
+    if(KMyMoneyGlobalSettings::hideUnusedCategory() && !m_isEditing) {
       KMessageBox::information(this, i18n("You have selected to suppress the display of unused categories in the KMyMoney configuration dialog. The category you just created will therefore only be shown if it is used. Otherwise, it will be hidden in the accounts/categories view."), i18n("Hidden categories"), "NewHiddenCategory");
     }
   }
@@ -483,7 +626,7 @@ void KNewAccountDlg::okClicked()
       m_account.setValue("VatRate", (m_vatRate->value().abs() / MyMoneyMoney(100,1)).toString());
     } else {
       if(m_vatAssignment->isChecked()) {
-        m_account.setValue("VatAccount", m_vatAccount->selectedAccounts().first());
+        m_account.setValue("VatAccount", m_vatAccount->selectedItems().first());
         if(m_netAmount->isChecked())
           m_account.setValue("VatAmount", "Net");
       }
@@ -493,10 +636,60 @@ void KNewAccountDlg::okClicked()
   accept();
 }
 
+void KNewAccountDlg::loadKVP(const QCString& key, kMyMoneyEdit* widget)
+{
+  if(!widget)
+    return;
+
+  if(m_account.value(key).isEmpty()) {
+    widget->clearText();
+  } else {
+    widget->setValue(MyMoneyMoney(m_account.value(key)));
+  }
+}
+
+void KNewAccountDlg::loadKVP(const QCString& key, KLineEdit* widget)
+{
+  if(!widget)
+    return;
+
+  widget->setText(m_account.value(key));
+}
+
+void KNewAccountDlg::storeKVP(const QCString& key, const QString& text, const QString& value)
+{
+  if(text.isEmpty())
+    m_account.deletePair(key);
+  else
+    m_account.setValue(key, value);
+}
+
+void KNewAccountDlg::storeKVP(const QCString& key, kMyMoneyEdit* widget)
+{
+  storeKVP(key, widget->lineedit()->text(), widget->text());
+}
+
+void KNewAccountDlg::storeKVP(const QCString& key, KLineEdit* widget)
+{
+  storeKVP(key, widget->text(), widget->text());
+}
+
 const MyMoneyAccount& KNewAccountDlg::account(void)
 {
   // assign the right currency to the account
   m_account.setCurrencyId(m_currency->security().id());
+
+  // and the price mode
+  switch(m_priceMode->currentItem()) {
+    case 0:
+      m_account.deletePair("priceMode");
+      break;
+    case 1:
+    case 2:
+      m_account.setValue("priceMode", QString("%1").arg(m_priceMode->currentItem()));
+      break;
+  }
+
   return m_account;
 }
 
@@ -568,7 +761,7 @@ void KNewAccountDlg::initParentWidget(QCString parentId, const QCString& account
   MyMoneyAccount::accountTypeE type;
   MyMoneyAccount::accountTypeE groupType;
   type = KMyMoneyUtils::stringToAccountType(typeCombo->currentText());
-  groupType = file->accountGroup(type);
+  groupType = MyMoneyAccount::accountGroup(type);
 
   m_qlistviewParentAccounts->clear();
 
@@ -579,8 +772,7 @@ void KNewAccountDlg::initParentWidget(QCString parentId, const QCString& account
     {
       if(groupType == MyMoneyAccount::Asset || type == MyMoneyAccount::Loan) {
         // Asset
-        KAccountListItem *assetTopLevelAccount = new KAccountListItem(m_qlistviewParentAccounts,
-                          assetAccount);
+        KMyMoneyAccountTreeItem *assetTopLevelAccount = new KMyMoneyAccountTreeItem(m_qlistviewParentAccounts, assetAccount);
 
         if(m_parentAccount.id().isEmpty()) {
           m_parentAccount = assetAccount;
@@ -596,30 +788,31 @@ void KNewAccountDlg::initParentWidget(QCString parentId, const QCString& account
               it != assetAccount.accountList().end();
               ++it )
         {
-          KAccountListItem *accountItem = new KAccountListItem(assetTopLevelAccount,
-              file->account(*it));
+          MyMoneyAccount acc = file->account(*it);
+          if(acc.isClosed())
+            continue;
 
-          QCString id = file->account(*it).id();
-          if(parentId == id) {
+          KMyMoneyAccountTreeItem *accountItem = new KMyMoneyAccountTreeItem(assetTopLevelAccount, acc);
+
+          if(parentId == acc.id()) {
             m_parentItem = accountItem;
-          } else if(accountId == id) {
+          } else if(accountId == acc.id()) {
             if(m_isEditing)
               accountItem->setSelectable(false);
             m_accountItem = accountItem;
           }
 
-          QCStringList subAccounts = file->account(*it).accountList();
+          QCStringList subAccounts = acc.accountList();
           if (subAccounts.count() >= 1)
           {
-            showSubAccounts(subAccounts, accountItem, parentId, accountId);
+            showSubAccounts(subAccounts, accountItem, parentId, acc.id());
           }
         }
       }
 
       if(groupType == MyMoneyAccount::Liability) {
         // Liability
-        KAccountListItem *liabilityTopLevelAccount = new KAccountListItem(m_qlistviewParentAccounts,
-                          liabilityAccount);
+        KMyMoneyAccountTreeItem *liabilityTopLevelAccount = new KMyMoneyAccountTreeItem(m_qlistviewParentAccounts, liabilityAccount);
 
         if(m_parentAccount.id().isEmpty()) {
           m_parentAccount = liabilityAccount;
@@ -635,22 +828,24 @@ void KNewAccountDlg::initParentWidget(QCString parentId, const QCString& account
               it != liabilityAccount.accountList().end();
               ++it )
         {
-          KAccountListItem *accountItem = new KAccountListItem(liabilityTopLevelAccount,
-              file->account(*it));
+          MyMoneyAccount acc = file->account(*it);
+          if(acc.isClosed())
+            continue;
 
-          QCString id = file->account(*it).id();
-          if(parentId == id) {
+          KMyMoneyAccountTreeItem *accountItem = new KMyMoneyAccountTreeItem(liabilityTopLevelAccount, acc);
+
+          if(parentId == acc.id()) {
             m_parentItem = accountItem;
-          } else if(accountId == id) {
+          } else if(accountId == acc.id()) {
             if(m_isEditing)
               accountItem->setSelectable(false);
             m_accountItem = accountItem;
           }
 
-          QCStringList subAccounts = file->account(*it).accountList();
+          QCStringList subAccounts = acc.accountList();
           if (subAccounts.count() >= 1)
           {
-            showSubAccounts(subAccounts, accountItem, parentId, accountId);
+            showSubAccounts(subAccounts, accountItem, parentId, acc.id());
           }
         }
       }
@@ -659,7 +854,7 @@ void KNewAccountDlg::initParentWidget(QCString parentId, const QCString& account
     {
       if(groupType == MyMoneyAccount::Income) {
         // Income
-        KAccountListItem *incomeTopLevelAccount = new KAccountListItem(m_qlistviewParentAccounts,
+        KMyMoneyAccountTreeItem *incomeTopLevelAccount = new KMyMoneyAccountTreeItem(m_qlistviewParentAccounts,
                           incomeAccount);
 
         if(m_parentAccount.id().isEmpty()) {
@@ -676,7 +871,7 @@ void KNewAccountDlg::initParentWidget(QCString parentId, const QCString& account
               it != incomeAccount.accountList().end();
               ++it )
         {
-          KAccountListItem *accountItem = new KAccountListItem(incomeTopLevelAccount,
+          KMyMoneyAccountTreeItem *accountItem = new KMyMoneyAccountTreeItem(incomeTopLevelAccount,
               file->account(*it));
 
           QCString id = file->account(*it).id();
@@ -698,7 +893,7 @@ void KNewAccountDlg::initParentWidget(QCString parentId, const QCString& account
 
       if(groupType == MyMoneyAccount::Expense) {
         // Expense
-        KAccountListItem *expenseTopLevelAccount = new KAccountListItem(m_qlistviewParentAccounts,
+        KMyMoneyAccountTreeItem *expenseTopLevelAccount = new KMyMoneyAccountTreeItem(m_qlistviewParentAccounts,
                           expenseAccount);
 
         if(m_parentAccount.id().isEmpty()) {
@@ -715,7 +910,7 @@ void KNewAccountDlg::initParentWidget(QCString parentId, const QCString& account
               it != expenseAccount.accountList().end();
               ++it )
         {
-          KAccountListItem *accountItem = new KAccountListItem(expenseTopLevelAccount,
+          KMyMoneyAccountTreeItem *accountItem = new KMyMoneyAccountTreeItem(expenseTopLevelAccount,
               file->account(*it));
 
           QCString id = file->account(*it).id();
@@ -754,14 +949,14 @@ void KNewAccountDlg::initParentWidget(QCString parentId, const QCString& account
   m_qlistviewParentAccounts->setEnabled(true);
 }
 
-void KNewAccountDlg::showSubAccounts(QCStringList accounts, KAccountListItem *parentItem,
+void KNewAccountDlg::showSubAccounts(QCStringList accounts, KMyMoneyAccountTreeItem *parentItem,
                                      const QCString& parentId, const QCString& accountId)
 {
   MyMoneyFile *file = MyMoneyFile::instance();
 
   for ( QCStringList::ConstIterator it = accounts.begin(); it != accounts.end(); ++it )
   {
-    KAccountListItem *accountItem  = new KAccountListItem(parentItem,
+    KMyMoneyAccountTreeItem *accountItem  = new KMyMoneyAccountTreeItem(parentItem,
           file->account(*it));
 
     QCString id = file->account(*it).id();
@@ -791,13 +986,13 @@ void KNewAccountDlg::resizeEvent(QResizeEvent* e)
 
 void KNewAccountDlg::slotSelectionChanged(QListViewItem *item)
 {
-  KAccountListItem *accountItem = (KAccountListItem*)item;
+  KMyMoneyAccountTreeItem *accountItem = dynamic_cast<KMyMoneyAccountTreeItem*>(item);
   try
   {
     MyMoneyFile *file = MyMoneyFile::instance();
 
     //qDebug("Selected account id: %s", accountItem->accountID().data());
-    m_parentAccount = file->account(accountItem->accountID());
+    m_parentAccount = file->account(accountItem->id());
     m_subAccountLabel->setText(i18n("Is a sub account of %1").arg(m_parentAccount.name()));
     if(m_qlistviewParentAccounts->isEnabled()) {
       m_bSelectedParentAccount = true;
@@ -812,7 +1007,8 @@ void KNewAccountDlg::slotSelectionChanged(QListViewItem *item)
 
 void KNewAccountDlg::loadVatAccounts(void)
 {
-  QValueList<MyMoneyAccount> list = MyMoneyFile::instance()->accountList();
+  QValueList<MyMoneyAccount> list;
+  MyMoneyFile::instance()->accountList(list);
   QValueList<MyMoneyAccount>::Iterator it;
   QCStringList loadListExpense;
   QCStringList loadListIncome;
@@ -824,16 +1020,21 @@ void KNewAccountDlg::loadVatAccounts(void)
         loadListIncome += (*it).id();
     }
   }
-  m_vatAccount->loadList(i18n("Income"), loadListIncome, true);
-  m_vatAccount->loadList(i18n("Expense"), loadListExpense, false);
+  AccountSet vatSet;
+  vatSet.load(m_vatAccount, i18n("Income"), loadListIncome, true);
+  vatSet.load(m_vatAccount, i18n("Expense"), loadListExpense, false);
 }
 
-void KNewAccountDlg::loadInstitutions(const QString& name)
+void KNewAccountDlg::slotLoadInstitutions(const QString& name)
 {
   int id=-1, counter=0;
   m_qcomboboxInstitutions->clear();
+  QString bic;
   // Are we forcing the user to use institutions?
   m_qcomboboxInstitutions->insertItem(i18n("<No Institution>"));
+  m_bicValue->setText(" ");
+  ibanEdit->setEnabled(false);
+  accountNoEdit->setEnabled(false);
   try
   {
     MyMoneyFile *file = MyMoneyFile::instance();
@@ -842,8 +1043,12 @@ void KNewAccountDlg::loadInstitutions(const QString& name)
     QValueList<MyMoneyInstitution>::ConstIterator institutionIterator;
     for (institutionIterator = list.begin(), counter=1; institutionIterator != list.end(); ++institutionIterator, counter++)
     {
-      if ((*institutionIterator).name() == name)
+      if ((*institutionIterator).name() == name) {
         id = counter;
+        ibanEdit->setEnabled(true);
+        accountNoEdit->setEnabled(true);
+        m_bicValue->setText((*institutionIterator).value("bic"));
+      }
       m_qcomboboxInstitutions->insertItem((*institutionIterator).name());
     }
 
@@ -863,26 +1068,53 @@ void KNewAccountDlg::slotNewClicked()
 {
   MyMoneyInstitution institution;
 
-  KNewBankDlg dlg(institution, false, this, "newbankdlg");
+  KNewBankDlg dlg(institution, this);
   if (dlg.exec())
   {
+    MyMoneyFileTransaction ft;
     try
     {
       MyMoneyFile *file = MyMoneyFile::instance();
 
       institution = dlg.institution();
       file->addInstitution(institution);
-      loadInstitutions(institution.name());
+      ft.commit();
+      slotLoadInstitutions(institution.name());
     }
     catch (MyMoneyException *e)
     {
       delete e;
       KMessageBox::information(this, i18n("Cannot add institution"));
-      return;
     }
   }
 }
 
+void KNewAccountDlg::slotOnlineSetupClicked(void)
+{
+#ifdef USE_OFX_DIRECTCONNECT
+  // TODO (Ace) This is one place to add in logic for other online banking
+  // protocols.  We could have a dialog that asks the user which
+  // protocol to set up for this account, and then call the correct
+  // wizard (using a plugin)
+  //
+  // Or, the wizard itself could ask the user.
+
+  KOnlineBankingSetupWizard wiz(this,"onlinebankingsetup");
+  if(wiz.isInit()) {
+    if(wiz.exec() == QDialog::Accepted) {
+      MyMoneyKeyValueContainer settings;
+      if ( wiz.chosenSettings( settings ) )
+      {
+        m_account.setOnlineBankingSettings( settings );
+        displayOnlineBankingStatus();
+      }
+    }
+  }
+#else
+  KMessageBox::sorry(this,i18n("This version of KMyMoney has not been compiled with support for online banking"));
+#endif
+}
+
 void KNewAccountDlg::slotAccountTypeChanged(const QString& typeStr)
 {
   MyMoneyAccount::accountTypeE type;
@@ -894,7 +1126,7 @@ void KNewAccountDlg::slotAccountTypeChanged(const QString& typeStr)
     oldType = m_account.accountType();
     if(oldType != type) {
       QCString parentId;
-      switch(file->accountGroup(type)) {
+      switch(MyMoneyAccount::accountGroup(type)) {
         case MyMoneyAccount::Asset:
           parentId = file->asset().id();
           break;
@@ -927,16 +1159,10 @@ void KNewAccountDlg::slotCheckFinished(void)
     showButton = false;
   }
 
-  if(m_account.accountType() == MyMoneyAccount::Stock) {
-    if(m_equity->text().length() == 0) {
-      showButton = false;
-    }
-  }
-
   if(m_vatCategory->isChecked() && m_vatRate->value() <= MyMoneyMoney(0)) {
     showButton = false;
   } else {
-    if(m_vatAssignment->isChecked() && m_vatAccount->selectedAccounts().isEmpty())
+    if(m_vatAssignment->isChecked() && m_vatAccount->selectedItems().isEmpty())
       showButton = false;
   }
   createButton->setEnabled(showButton);
@@ -959,4 +1185,73 @@ void KNewAccountDlg::slotVatAssignmentChanged(bool state)
   m_amountGroup->setEnabled(state);
 }
 
+void KNewAccountDlg::displayOnlineBankingStatus(void)
+{
+// this should be a run-time conditional, checking whether there are any online
+// banking plugins loaded
+#ifdef USE_OFX_DIRECTCONNECT
+    // Set up online banking settings if applicable
+    MyMoneyKeyValueContainer settings = m_account.onlineBankingSettings();
+    QString protocol = settings.value("protocol");
+    if ( ! protocol.isEmpty() )
+    {
+      m_textOnlineStatus->setText(i18n("STATUS: Enabled & configured (%1)").arg(protocol));
+      m_ledOnlineStatus->on();
+
+      QString account = i18n("ACCOUNT: %1").arg(settings.value("accountid"));
+      QString bank = i18n("BANK/BROKER: %1").arg(settings.value("bankname"));
+      QString bankid = settings.value("bankid") + " " + settings.value("branchid");
+      if ( bankid.length() > 1 )
+        bank += " (" + bankid + ")";
+      m_textBank->setText(bank);
+      m_textOnlineAccount->setText(account);
+    }
+    else
+      m_textOnlineStatus->setText(i18n("STATUS: Account not configured"));
+
+#else
+    m_textOnlineStatus->setText(i18n("STATUS: Disabled.  No online banking services are available"));
+#endif
+}
+
+void KNewAccountDlg::adjustEditWidgets(kMyMoneyEdit* dst, kMyMoneyEdit* src, char mode, int corr)
+{
+  MyMoneyMoney factor(corr, 1);
+  if(m_account.accountGroup() == MyMoneyAccount::Asset)
+    factor = -factor;
+
+  switch(mode) {
+    case '<':
+      if(src->value()*factor < dst->value()*factor)
+        dst->setValue(src->value());
+      break;
+
+    case '>':
+      if(src->value()*factor > dst->value()*factor)
+        dst->setValue(src->value());
+      break;
+  }
+}
+
+void KNewAccountDlg::slotAdjustMinBalanceAbsoluteEdit(const QString&)
+{
+  adjustEditWidgets(m_minBalanceAbsoluteEdit, m_minBalanceEarlyEdit, '<', -1);
+}
+
+void KNewAccountDlg::slotAdjustMinBalanceEarlyEdit(const QString&)
+{
+  adjustEditWidgets(m_minBalanceEarlyEdit, m_minBalanceAbsoluteEdit, '>', -1);
+}
+
+void KNewAccountDlg::slotAdjustMaxCreditAbsoluteEdit(const QString&)
+{
+  adjustEditWidgets(m_maxCreditAbsoluteEdit, m_maxCreditEarlyEdit, '<', 1);
+}
+
+void KNewAccountDlg::slotAdjustMaxCreditEarlyEdit(const QString&)
+{
+  adjustEditWidgets(m_maxCreditEarlyEdit, m_maxCreditAbsoluteEdit, '>', 1);
+}
+
 #include "knewaccountdlg.moc"
+// vim:cin:si:ai:et:ts=2:sw=2:
