@@ -14,10 +14,13 @@
  *                                                                         *
  ***************************************************************************/
 
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+
 // ----------------------------------------------------------------------------
 // QT Includes
 
-#include <qlineedit.h>
 #include <qlabel.h>
 #include <qpixmap.h>
 
@@ -26,29 +29,24 @@
 
 #include <kglobal.h>
 #include <klocale.h>
-#if QT_VERSION > 300
 #include <kstandarddirs.h>
-#else
-#include <kstddirs.h>
-#endif
-
 #include <kmessagebox.h>
 #include <kpushbutton.h>
 #include <kiconloader.h>
 #include <kguiitem.h>
+#include <klineedit.h>
 
 // ----------------------------------------------------------------------------
 // Project Includes
 
+#include <kmymoney/mymoneyinstitution.h>
+#include <kmymoney/kguiutils.h>
+
 #include "knewbankdlg.h"
 
-KNewBankDlg::KNewBankDlg(MyMoneyInstitution& institution,  bool /*isEditing*/, QWidget *parent, const char *name)
+KNewBankDlg::KNewBankDlg(MyMoneyInstitution& institution, QWidget *parent, const char *name)
   : KNewBankDlgDecl(parent,name,true), m_institution(institution)
 {
-  QString filename = KGlobal::dirs()->findResource("appdata", "pics/dlg_new_institution.png");
-  QPixmap pm(filename);
-  m_qpixmaplabel->setPixmap(pm);
-
   KIconLoader* il = KGlobal::iconLoader();
   KGuiItem okButtenItem( i18n("&Ok" ),
                     QIconSet(il->loadIcon("button_ok", KIcon::Small, KIcon::SizeSmall)),
@@ -63,16 +61,27 @@ KNewBankDlg::KNewBankDlg(MyMoneyInstitution& institution,  bool /*isEditing*/, Q
   cancelBtn->setGuiItem(cancelButtenItem);
 
   nameEdit->setFocus();
-	nameEdit->setText(institution.name());
-	cityEdit->setText(institution.city());
-	streetEdit->setText(institution.street());
-	postcodeEdit->setText(institution.postcode());
-	telephoneEdit->setText(institution.telephone());
-	managerEdit->setText(institution.manager());
-	sortCodeEdit->setText(institution.sortcode());
+  nameEdit->setText(institution.name());
+  cityEdit->setText(institution.city());
+  streetEdit->setText(institution.street());
+  postcodeEdit->setText(institution.postcode());
+  telephoneEdit->setText(institution.telephone());
+  bicEdit->setText(institution.value("bic"));
+  sortCodeEdit->setText(institution.sortcode());
 
-	connect(okBtn, SIGNAL(clicked()), SLOT(okClicked()));
-	connect(cancelBtn, SIGNAL(clicked()), SLOT(reject()));
+  connect(okBtn, SIGNAL(clicked()), SLOT(okClicked()));
+  connect(cancelBtn, SIGNAL(clicked()), SLOT(reject()));
+  connect(nameEdit, SIGNAL(textChanged ( const QString & )), SLOT(institutionNameChanged( const QString &)));
+  institutionNameChanged( nameEdit->text());
+
+  kMandatoryFieldGroup* requiredFields = new kMandatoryFieldGroup (this);
+  requiredFields->setOkButton(okBtn); // button to be enabled when all fields present
+  requiredFields->add(nameEdit);
+}
+
+void KNewBankDlg::institutionNameChanged( const QString &_text)
+{
+  okBtn->setEnabled( !_text.isEmpty() );
 }
 
 KNewBankDlg::~KNewBankDlg()
@@ -87,19 +96,21 @@ void KNewBankDlg::okClicked()
     return;
   }
 
-	m_institution.setName(nameEdit->text());
-	m_institution.setTown(cityEdit->text());
-	m_institution.setStreet(streetEdit->text());
-	m_institution.setPostcode(postcodeEdit->text());
-	m_institution.setTelephone(telephoneEdit->text());
-	m_institution.setManager(managerEdit->text());
-	m_institution.setSortcode(sortCodeEdit->text());
+  m_institution.setName(nameEdit->text());
+  m_institution.setTown(cityEdit->text());
+  m_institution.setStreet(streetEdit->text());
+  m_institution.setPostcode(postcodeEdit->text());
+  m_institution.setTelephone(telephoneEdit->text());
+  m_institution.setValue("bic", bicEdit->text());
+  m_institution.setSortcode(sortCodeEdit->text());
 
   accept();
 }
 
-MyMoneyInstitution KNewBankDlg::institution(void)
+const MyMoneyInstitution& KNewBankDlg::institution(void)
 {
   return m_institution;
 }
 
+
+#include "knewbankdlg.moc"
