@@ -45,12 +45,11 @@
 #include "knewloanwizard.h"
 
 #include "../kmymoneyutils.h"
-#include "../widgets/kmymoneylineedit.h"
-#include "../widgets/kmymoneypayee.h"
-#include "../widgets/kmymoneydateinput.h"
+#include <kmymoney/kmymoneylineedit.h>
+#include <kmymoney/kmymoneydateinput.h>
+#include <kmymoney/kmymoneyedit.h>
+#include <kmymoney/kmymoneyaccountselector.h>
 #include "../widgets/kmymoneycombo.h"
-#include "../widgets/kmymoneyedit.h"
-#include "../widgets/kmymoneyaccountselector.h"
 
 #include "../dialogs/knewaccountdlg.h"
 #include "../dialogs/ksplittransactiondlg.h"
@@ -67,7 +66,8 @@ KNewLoanWizard::KNewLoanWizard(QWidget *parent, const char *name ) :
   connect(m_lendButton, SIGNAL(clicked()), this, SLOT(slotAssetLoan()));
 
   connect(m_nameEdit, SIGNAL(textChanged(const QString&)), this, SLOT(slotCheckPageFinished()));
-  connect(m_payeeEdit, SIGNAL(newPayee(const QString&)), this, SLOT(slotNewPayee(const QString&)));
+  // connect(m_payeeEdit, SIGNAL(newPayee(const QString&)), this, SLOT(slotNewPayee(const QString&)));
+  connect(m_payeeEdit, SIGNAL(createItem(const QString&, QCString&)), this, SIGNAL(createPayee(const QString&, QCString&)));
 
   connect(m_previousPaymentButton, SIGNAL(clicked()), this, SLOT(slotPaymentsMade()));
   connect(m_noPreviousPaymentButton, SIGNAL(clicked()), this, SLOT(slotNoPaymentsMade()));
@@ -90,10 +90,14 @@ KNewLoanWizard::KNewLoanWizard(QWidget *parent, const char *name ) :
   connect(m_assetAccountEdit,  SIGNAL(stateChanged()), this, SLOT(slotCheckPageFinished()));
   connect(m_dontCreatePayoutCheckBox,  SIGNAL(clicked()), this, SLOT(slotCheckPageFinished()));
 
+  connect(MyMoneyFile::instance(), SIGNAL(dataChanged()), this, SLOT(slotReloadEditWidgets()));
+
   loadComboBoxes();
 
   resetCalculator();
 
+  slotReloadEditWidgets();
+
   // As default we assume a liability loan, with fixed interest rate,
   // with a first payment due on the 30th of this month. All payments
   // should be recorded and none have been made so far.
@@ -105,10 +109,14 @@ KNewLoanWizard::KNewLoanWizard(QWidget *parent, const char *name ) :
   m_interestOnReceptionButton->animateClick();
 
   m_interestFrequencyAmountEdit->setValue(1);
-  m_interestFrequencyUnitEdit->setCurrentItem(i18n("Years"));
+  m_interestFrequencyUnitEdit->setCurrentItem(static_cast<int>(MyMoneyAccountLoan::changeYearly));
   m_paymentFrequencyUnitEdit->setCurrentItem(KMyMoneyUtils::occurenceToString(MyMoneySchedule::OCCUR_MONTHLY));
   m_firstDueDateEdit->loadDate(QDate(QDate::currentDate().year(),QDate::currentDate().month(),30));
 
+  m_paymentAccountEdit->removeButtons();
+  m_assetAccountEdit->removeButtons();
+  m_interestAccountEdit->removeButtons();
+
   // load button icons
   KIconLoader* il = KGlobal::iconLoader();
   KGuiItem createCategoryButtenItem( i18n( "&Create..." ),
@@ -151,10 +159,9 @@ KNewLoanWizard::KNewLoanWizard(QWidget *parent, const char *name ) :
 
   // setup a phony transaction for additional fee processing
   m_account = MyMoneyAccount(QCString("Phony-ID"), MyMoneyAccount());
-  MyMoneySplit split;
-  split.setAccountId(m_account.id());
-  split.setValue(0);
-  m_transaction.addSplit(split);
+  m_split.setAccountId(m_account.id());
+  m_split.setValue(0);
+  m_transaction.addSplit(m_split);
 }
 
 KNewLoanWizard::~KNewLoanWizard()
@@ -193,12 +200,7 @@ void KNewLoanWizard::resetCalculator(void)
   m_payment5->setText(QString());
   m_balloon5->setText(QString());
 
-  m_additionalCost->setText(MyMoneyMoney(0).formatMoney());
-}
-
-void KNewLoanWizard::slotNewPayee(const QString& payeeName)
-{
-  KMyMoneyUtils::newPayee(this, m_payeeEdit, payeeName);
+  m_additionalCost->setText(MyMoneyMoney(0).formatMoney(m_account.fraction(MyMoneyFile::instance()->security(m_account.currencyId()))));
 }
 
 void KNewLoanWizard::slotLiabilityLoan(void)
@@ -270,18 +272,18 @@ void KNewLoanWizard::slotCheckPageFinished(void)
     }
 
   } else if(currentPage() == m_interestCategoryPage) {
-    if(m_interestAccountEdit->selectedAccounts().count() > 0) {
+    if(m_interestAccountEdit->selectedItems().count() > 0) {
       nextButton()->setEnabled(true);
     }
 
   } else if(currentPage() == m_firstPaymentPage) {
-    if(m_firstDueDateEdit->getQDate().isValid())
+    if(m_firstDueDateEdit->date().isValid())
       nextButton()->setEnabled(true);
 
   } else if(currentPage() == m_schedulePage) {
-    if(m_nextDueDateEdit->getQDate().isValid()
-    && m_nextDueDateEdit->getQDate() >= m_firstDueDateEdit->getQDate()
-    && m_paymentAccountEdit->selectedAccounts().count() > 0)
+    if(m_nextDueDateEdit->date().isValid()
+    && m_nextDueDateEdit->date() >= m_firstDueDateEdit->date()
+    && m_paymentAccountEdit->selectedItems().count() > 0)
       nextButton()->setEnabled(true);
 
   } else if(currentPage() == m_assetAccountPage) {
@@ -294,8 +296,8 @@ void KNewLoanWizard::slotCheckPageFinished(void)
       m_assetAccountEdit->setEnabled(true);
       m_paymentDate->setEnabled(true);
       m_createNewAssetButton->setEnabled(true);
-      if(!m_assetAccountEdit->selectedAccounts().isEmpty()
-      && m_paymentDate->getQDate().isValid())
+      if(!m_assetAccountEdit->selectedItems().isEmpty()
+      && m_paymentDate->date().isValid())
         nextButton()->setEnabled(true);
     }
   } else
@@ -305,10 +307,10 @@ void KNewLoanWizard::slotCheckPageFinished(void)
 void KNewLoanWizard::updateLoanAmount(void)
 {
   QString txt;
-  if(m_loanAmountEdit->text().isEmpty()) {
+  if(m_loanAmountEdit->lineedit()->text().isEmpty()) {
     txt = QString("<") + i18n("calculate") + QString(">");
   } else {
-    txt = m_loanAmountEdit->value().formatMoney();
+    txt = m_loanAmountEdit->value().formatMoney(m_account.fraction(MyMoneyFile::instance()->security(m_account.currencyId())));
   }
   m_loanAmount1->setText(txt);
   m_loanAmount2->setText(txt);
@@ -320,7 +322,7 @@ void KNewLoanWizard::updateLoanAmount(void)
 void KNewLoanWizard::updateInterestRate(void)
 {
   QString txt;
-  if(m_interestRateEdit->text().isEmpty()) {
+  if(m_interestRateEdit->lineedit()->text().isEmpty()) {
     txt = QString("<") + i18n("calculate") + QString(">");
   } else {
     txt = m_interestRateEdit->value().formatMoney("", 3) + QString("%");
@@ -351,10 +353,10 @@ void KNewLoanWizard::updateDuration(void)
 void KNewLoanWizard::updatePayment(void)
 {
   QString txt;
-  if(m_paymentEdit->text().isEmpty()) {
+  if(m_paymentEdit->lineedit()->text().isEmpty()) {
     txt = QString("<") + i18n("calculate") + QString(">");
   } else {
-    txt = m_paymentEdit->value().formatMoney();
+    txt = m_paymentEdit->value().formatMoney(m_account.fraction(MyMoneyFile::instance()->security(m_account.currencyId())));
   }
   m_payment1->setText(txt);
   m_payment2->setText(txt);
@@ -367,10 +369,10 @@ void KNewLoanWizard::updatePayment(void)
 void KNewLoanWizard::updateFinalPayment(void)
 {
   QString txt;
-  if(m_finalPaymentEdit->text().isEmpty()) {
+  if(m_finalPaymentEdit->lineedit()->text().isEmpty()) {
     txt = QString("<") + i18n("calculate") + QString(">");
   } else {
-    txt = m_finalPaymentEdit->value().formatMoney();
+    txt = m_finalPaymentEdit->value().formatMoney(m_account.fraction(MyMoneyFile::instance()->security(m_account.currencyId())));
   }
   m_balloon1->setText(txt);
   m_balloon2->setText(txt);
@@ -390,13 +392,14 @@ void KNewLoanWizard::updateLoanInfo(void)
 
   QString txt;
 
-  m_loanAmount6->setText(m_loanAmountEdit->value().formatMoney());
+  int fraction = m_account.fraction(MyMoneyFile::instance()->security(m_account.currencyId()));
+  m_loanAmount6->setText(m_loanAmountEdit->value().formatMoney(fraction));
   m_interestRate6->setText(m_interestRateEdit->value().formatMoney("", 3) + QString("%"));
   txt = QString().sprintf("%d ", m_durationValueEdit->value())
         + m_durationUnitEdit->currentText();
   m_duration6->setText(txt);
-  m_payment6->setText(m_paymentEdit->value().formatMoney());
-  m_balloon6->setText(m_finalPaymentEdit->value().formatMoney());
+  m_payment6->setText(m_paymentEdit->value().formatMoney(fraction));
+  m_balloon6->setText(m_finalPaymentEdit->value().formatMoney(fraction));
 }
 
 void KNewLoanWizard::updatePeriodicPayment(void)
@@ -404,7 +407,7 @@ void KNewLoanWizard::updatePeriodicPayment(void)
   MyMoneyMoney base(m_basePayment->text());
   MyMoneyMoney add(m_additionalCost->text());
 
-  m_periodicPayment->setText((base + add).formatMoney());
+  m_periodicPayment->setText((base + add).formatMoney(m_account.fraction(MyMoneyFile::instance()->security(m_account.currencyId()))));
 }
 
 void KNewLoanWizard::updateSummary(void)
@@ -415,11 +418,11 @@ void KNewLoanWizard::updateSummary(void)
   else
     m_summaryLoanType->setText(i18n("lend"));
 
-  m_summaryFirstPayment->setText(KGlobal::locale()->formatDate(m_firstDueDateEdit->getQDate(), true));
-  if(m_payeeEdit->text().isEmpty()) {
+  m_summaryFirstPayment->setText(KGlobal::locale()->formatDate(m_firstDueDateEdit->date(), true));
+  if(m_payeeEdit->selectedItem().isEmpty()) {
     m_summaryPayee->setText(i18n("not assigned"));
   } else {
-    m_summaryPayee->setText(m_payeeEdit->text());
+    m_summaryPayee->setText(m_payeeEdit->currentText());
   }
 
   // Calculation
@@ -436,7 +439,7 @@ void KNewLoanWizard::updateSummary(void)
 
   // Payment
   try {
-    QCStringList sel = m_interestAccountEdit->selectedAccounts();
+    QCStringList sel = m_interestAccountEdit->selectedItems();
     if(sel.count() != 1)
       throw new MYMONEYEXCEPTION("Need a single selected interest category");
     MyMoneyAccount acc = MyMoneyFile::instance()->account(sel.first());
@@ -447,10 +450,10 @@ void KNewLoanWizard::updateSummary(void)
   }
   m_summaryAdditionalFees->setText(m_additionalCost->text());
   m_summaryTotalPeriodicPayment->setText(m_periodicPayment->text());
-  m_summaryNextPayment->setText(KGlobal::locale()->formatDate(m_nextDueDateEdit->getQDate(), true));
+  m_summaryNextPayment->setText(KGlobal::locale()->formatDate(m_nextDueDateEdit->date(), true));
 
   try {
-    QCStringList sel = m_paymentAccountEdit->selectedAccounts();
+    QCStringList sel = m_paymentAccountEdit->selectedItems();
     if(sel.count() != 1)
       throw new MYMONEYEXCEPTION("Need a single selected payment account");
     MyMoneyAccount acc = MyMoneyFile::instance()->account(sel.first());
@@ -493,7 +496,7 @@ void KNewLoanWizard::next()
   } else if(currentPage() == m_loanAmountPage) {
     m_interestRateEdit->setFocus();
     if(m_thisYearPaymentButton->isChecked()
-    && m_loanAmountEdit->text().isEmpty()) {
+    && m_loanAmountEdit->lineedit()->text().isEmpty()) {
       errMsg = i18n("You selected, that payments have already been made towards this loan. "
                     "This requires you to enter the loan amount exactly as found on your "
                     "last statement.");
@@ -504,16 +507,16 @@ void KNewLoanWizard::next()
 
   } else if(currentPage() == m_interestPage) {
 
-    if(m_loanAmountEdit->text().isEmpty()
-    && m_interestRateEdit->text().isEmpty()) {
+    if(m_loanAmountEdit->lineedit()->text().isEmpty()
+    && m_interestRateEdit->lineedit()->text().isEmpty()) {
       dontLeavePage = true;
       KMessageBox::error(0, errMsg.arg(i18n("interest rate")), i18n("Calculation error"));
     } else
       updateInterestRate();
 
   } else if(currentPage() == m_durationPage) {
-    if((m_loanAmountEdit->text().isEmpty()
-    || m_interestRateEdit->text().isEmpty())
+    if((m_loanAmountEdit->lineedit()->text().isEmpty()
+    || m_interestRateEdit->lineedit()->text().isEmpty())
     && m_durationValueEdit->value() == 0) {
       dontLeavePage = true;
       KMessageBox::error(0, errMsg.arg(i18n("term")), i18n("Calculation error"));
@@ -521,21 +524,21 @@ void KNewLoanWizard::next()
       updateDuration();
 
   } else if(currentPage() == m_paymentPage) {
-    if((m_loanAmountEdit->text().isEmpty()
-    || m_interestRateEdit->text().isEmpty()
+    if((m_loanAmountEdit->lineedit()->text().isEmpty()
+    || m_interestRateEdit->lineedit()->text().isEmpty()
     || m_durationValueEdit->value() == 0)
-    && m_paymentEdit->text().isEmpty()) {
+    && m_paymentEdit->lineedit()->text().isEmpty()) {
       dontLeavePage = true;
       KMessageBox::error(0, errMsg.arg(i18n("principal and interest")), i18n("Calculation error"));
     } else
       updatePayment();
 
   } else if(currentPage() == m_finalPaymentPage) {
-    if((m_loanAmountEdit->text().isEmpty()
-    || m_interestRateEdit->text().isEmpty()
+    if((m_loanAmountEdit->lineedit()->text().isEmpty()
+    || m_interestRateEdit->lineedit()->text().isEmpty()
     || m_durationValueEdit->value() == 0
-    || m_paymentEdit->text().isEmpty())
-    && m_finalPaymentEdit->text().isEmpty()) {
+    || m_paymentEdit->lineedit()->text().isEmpty())
+    && m_finalPaymentEdit->lineedit()->text().isEmpty()) {
       // if two fields are empty and one of them is the final payment
       // we assume the final payment to be 0 instead of presenting a
       m_finalPaymentEdit->setValue(MyMoneyMoney(0, 1));
@@ -549,17 +552,17 @@ void KNewLoanWizard::next()
   } else if(currentPage() == m_additionalFeesPage) {
     m_nextDueDateEdit->setEnabled(true);
     if(m_allPaymentsButton->isChecked() || m_noPreviousPaymentButton->isChecked()) {
-      m_nextDueDateEdit->setDate(m_firstDueDateEdit->getQDate());
+      m_nextDueDateEdit->setDate(m_firstDueDateEdit->date());
       m_nextDueDateEdit->setEnabled(false);
       setAppropriate(m_assetAccountPage, true);
     } else {
-      QDate nextPayment(QDate::currentDate().year(), 1, m_firstDueDateEdit->getQDate().day());
+      QDate nextPayment(QDate::currentDate().year(), 1, m_firstDueDateEdit->date().day());
       m_nextDueDateEdit->setDate(nextPayment);
       setAppropriate(m_assetAccountPage, false);
       m_assetAccountEdit->slotDeselectAllAccounts();
     }
-    if(m_nextDueDateEdit->getQDate() < m_firstDueDateEdit->getQDate()) {
-      m_nextDueDateEdit->setDate(m_firstDueDateEdit->getQDate());
+    if(m_nextDueDateEdit->date() < m_firstDueDateEdit->date()) {
+      m_nextDueDateEdit->setDate(m_firstDueDateEdit->date());
     }
 
   } else if(currentPage() == m_schedulePage) {
@@ -605,26 +608,29 @@ void KNewLoanWizard::next()
 
 void KNewLoanWizard::loadComboBoxes(void)
 {
-  m_interestFrequencyUnitEdit->insertItem(i18n("Days"));
-  m_interestFrequencyUnitEdit->insertItem(i18n("Weeks"));
-  m_interestFrequencyUnitEdit->insertItem(i18n("Months"));
-  m_interestFrequencyUnitEdit->insertItem(i18n("Years"));
+  m_interestFrequencyUnitEdit->insertItem(i18n("Days"), static_cast<int>(MyMoneyAccountLoan::changeDaily));
+  m_interestFrequencyUnitEdit->insertItem(i18n("Weeks"), static_cast<int>(MyMoneyAccountLoan::changeWeekly));
+  m_interestFrequencyUnitEdit->insertItem(i18n("Months"), static_cast<int>(MyMoneyAccountLoan::changeMonthly));
+  m_interestFrequencyUnitEdit->insertItem(i18n("Years"), static_cast<int>(MyMoneyAccountLoan::changeYearly));
 
   m_paymentFrequencyUnitEdit->insertItem(KMyMoneyUtils::occurenceToString(MyMoneySchedule::OCCUR_DAILY));
   m_paymentFrequencyUnitEdit->insertItem(KMyMoneyUtils::occurenceToString(MyMoneySchedule::OCCUR_WEEKLY));
   m_paymentFrequencyUnitEdit->insertItem(KMyMoneyUtils::occurenceToString(MyMoneySchedule::OCCUR_FORTNIGHTLY));
   m_paymentFrequencyUnitEdit->insertItem(KMyMoneyUtils::occurenceToString(MyMoneySchedule::OCCUR_EVERYOTHERWEEK));
+  m_paymentFrequencyUnitEdit->insertItem(KMyMoneyUtils::occurenceToString(MyMoneySchedule::OCCUR_EVERYTHREEWEEKS));
   m_paymentFrequencyUnitEdit->insertItem(KMyMoneyUtils::occurenceToString(MyMoneySchedule::OCCUR_EVERYFOURWEEKS));
+  m_paymentFrequencyUnitEdit->insertItem(KMyMoneyUtils::occurenceToString(MyMoneySchedule::OCCUR_EVERYTHIRTYDAYS));
   m_paymentFrequencyUnitEdit->insertItem(KMyMoneyUtils::occurenceToString(MyMoneySchedule::OCCUR_MONTHLY));
+  m_paymentFrequencyUnitEdit->insertItem(KMyMoneyUtils::occurenceToString(MyMoneySchedule::OCCUR_EVERYEIGHTWEEKS));
   m_paymentFrequencyUnitEdit->insertItem(KMyMoneyUtils::occurenceToString(MyMoneySchedule::OCCUR_EVERYOTHERMONTH));
   m_paymentFrequencyUnitEdit->insertItem(KMyMoneyUtils::occurenceToString(MyMoneySchedule::OCCUR_QUARTERLY));
   m_paymentFrequencyUnitEdit->insertItem(KMyMoneyUtils::occurenceToString(MyMoneySchedule::OCCUR_EVERYFOURMONTHS));
   m_paymentFrequencyUnitEdit->insertItem(KMyMoneyUtils::occurenceToString(MyMoneySchedule::OCCUR_TWICEYEARLY));
   m_paymentFrequencyUnitEdit->insertItem(KMyMoneyUtils::occurenceToString(MyMoneySchedule::OCCUR_YEARLY));
 
-  m_durationUnitEdit->insertItem(i18n("Months"));
-  m_durationUnitEdit->insertItem(i18n("Years"));
-  m_durationUnitEdit->insertItem(i18n("Payments"));
+  m_durationUnitEdit->insertItem(i18n("Months"), static_cast<int>(MyMoneySchedule::OCCUR_MONTHLY));
+  m_durationUnitEdit->insertItem(i18n("Years"), static_cast<int>(MyMoneySchedule::OCCUR_YEARLY));
+  m_durationUnitEdit->insertItem(i18n("Payments"), static_cast<int>(MyMoneySchedule::OCCUR_ONCE));
 
 }
 
@@ -657,12 +663,19 @@ int KNewLoanWizard::occurenceToPeriod(const MyMoneySchedule::occurenceE occurenc
     case MyMoneySchedule::OCCUR_EVERYOTHERWEEK:
       rc = 15;
       break;
+    case MyMoneySchedule::OCCUR_EVERYTHREEWEEKS:
+      rc = 21;
+      break;
     case MyMoneySchedule::OCCUR_EVERYFOURWEEKS:
       rc = 28;
       break;
     case MyMoneySchedule::OCCUR_MONTHLY:
+    case MyMoneySchedule::OCCUR_EVERYTHIRTYDAYS:
       rc = 30;
       break;
+    case MyMoneySchedule::OCCUR_EVERYEIGHTWEEKS:
+      rc = 56;
+      break;
     case MyMoneySchedule::OCCUR_EVERYOTHERMONTH:
       rc = 60;
       break;
@@ -702,12 +715,18 @@ int KNewLoanWizard::occurenceToFrequency(const MyMoneySchedule::occurenceE occur
     case MyMoneySchedule::OCCUR_EVERYOTHERWEEK:
       rc = 26;
       break;
+    case MyMoneySchedule::OCCUR_EVERYTHREEWEEKS:
+      rc = 17;
+      break;
     case MyMoneySchedule::OCCUR_EVERYFOURWEEKS:
       rc = 13;
       break;
     case MyMoneySchedule::OCCUR_MONTHLY:
       rc = 12;
       break;
+    case MyMoneySchedule::OCCUR_EVERYEIGHTWEEKS:
+      rc = 6;
+      break;
     case MyMoneySchedule::OCCUR_EVERYOTHERMONTH:
       rc = 6;
       break;
@@ -752,26 +771,26 @@ int KNewLoanWizard::calculateLoan(void)
   calc.setCF(PF);
 
 
-  if(!m_loanAmountEdit->text().isEmpty()) {
+  if(!m_loanAmountEdit->lineedit()->text().isEmpty()) {
     val = static_cast<long double> (m_loanAmountEdit->value().abs().toDouble());
     if(m_borrowButton->isChecked())
       val = -val;
     calc.setPv(val);
   }
 
-  if(!m_interestRateEdit->text().isEmpty()) {
+  if(!m_interestRateEdit->lineedit()->text().isEmpty()) {
     val = static_cast<long double> (m_interestRateEdit->value().abs().toDouble());
     calc.setIr(val);
   }
 
-  if(!m_paymentEdit->text().isEmpty()) {
+  if(!m_paymentEdit->lineedit()->text().isEmpty()) {
     val = static_cast<long double> (m_paymentEdit->value().abs().toDouble());
     if(m_lendButton->isChecked())
       val = -val;
     calc.setPmt(val);
   }
 
-  if(!m_finalPaymentEdit->text().isEmpty()) {
+  if(!m_finalPaymentEdit->lineedit()->text().isEmpty()) {
     val = static_cast<long double> (m_finalPaymentEdit->value().abs().toDouble());
     if(m_lendButton->isChecked())
       val = -val;
@@ -782,26 +801,27 @@ int KNewLoanWizard::calculateLoan(void)
     calc.setNpp(static_cast<long double>(term()));
   }
 
+  int fraction = m_account.fraction(MyMoneyFile::instance()->security(m_account.currencyId()));
   // setup of parameters is done, now do the calculation
   try {
-    if(m_loanAmountEdit->text().isEmpty()) {
+    if(m_loanAmountEdit->lineedit()->text().isEmpty()) {
       // calculate the amount of the loan out of the other information
       val = calc.presentValue();
-      m_loanAmountEdit->loadText(MyMoneyMoney(static_cast<double>(val)).abs().formatMoney());
+      m_loanAmountEdit->loadText(MyMoneyMoney(static_cast<double>(val)).abs().formatMoney(fraction));
       result = i18n("KMyMoney has calculated the amount of the loan as %1.")
-                        .arg(m_loanAmountEdit->text());
+                        .arg(m_loanAmountEdit->lineedit()->text());
 
-    } else if(m_interestRateEdit->text().isEmpty()) {
+    } else if(m_interestRateEdit->lineedit()->text().isEmpty()) {
       // calculate the interest rate out of the other information
       val = calc.interestRate();
       m_interestRateEdit->loadText(MyMoneyMoney(static_cast<double>(val)).abs().formatMoney("", 3));
       result = i18n("KMyMoney has calculated the interest rate to %1%.")
-                        .arg(m_interestRateEdit->text());
+                        .arg(m_interestRateEdit->lineedit()->text());
 
-    } else if(m_paymentEdit->text().isEmpty()) {
+    } else if(m_paymentEdit->lineedit()->text().isEmpty()) {
       // calculate the periodical amount of the payment out of the other information
       val = calc.payment();
-      m_paymentEdit->setText(MyMoneyMoney(static_cast<double>(val)).abs().formatMoney());
+      m_paymentEdit->setValue(MyMoneyMoney(static_cast<double>(val)).abs());
       // reset payment as it might have changed due to rounding
       val = static_cast<long double> (m_paymentEdit->value().abs().toDouble());
       if(m_lendButton->isChecked())
@@ -809,7 +829,7 @@ int KNewLoanWizard::calculateLoan(void)
       calc.setPmt(val);
 
       result = i18n("KMyMoney has calculated a periodic payment of %1 to cover principal and interest.")
-                      .arg(m_paymentEdit->text());
+                      .arg(m_paymentEdit->lineedit()->text());
 
       val = calc.futureValue();
       if((m_borrowButton->isChecked() && val < 0 && fabsl(val) >= fabsl(calc.payment()))
@@ -818,18 +838,18 @@ int KNewLoanWizard::calculateLoan(void)
         updateTermWidgets(calc.npp());
         val = calc.futureValue();
         MyMoneyMoney refVal(static_cast<double>(val));
-        m_finalPaymentEdit->loadText(refVal.abs().formatMoney());
+        m_finalPaymentEdit->loadText(refVal.abs().formatMoney(fraction));
         result += QString(" ");
         result += i18n("The number of payments has been decremented and the final payment has been modified to %1.")
-                      .arg(m_finalPaymentEdit->text());
+                      .arg(m_finalPaymentEdit->lineedit()->text());
       } else if((m_borrowButton->isChecked() && val < 0 && fabsl(val) < fabsl(calc.payment()))
              || (m_lendButton->isChecked() && val > 0 && fabs(val) < fabs(calc.payment()))) {
-        m_finalPaymentEdit->loadText(MyMoneyMoney(0,1).formatMoney());
+        m_finalPaymentEdit->loadText(MyMoneyMoney(0,1).formatMoney(fraction));
       } else {
         MyMoneyMoney refVal(static_cast<double>(val));
-        m_finalPaymentEdit->loadText(refVal.abs().formatMoney());
+        m_finalPaymentEdit->loadText(refVal.abs().formatMoney(fraction));
         result += i18n("The final payment has been modified to %1.")
-                      .arg(m_finalPaymentEdit->text());
+                      .arg(m_finalPaymentEdit->lineedit()->text());
       }
 
     } else if(m_durationValueEdit->value() == 0) {
@@ -847,9 +867,9 @@ int KNewLoanWizard::calculateLoan(void)
         calc.setNpp(floorl(val));
         val = calc.futureValue();
         MyMoneyMoney refVal(static_cast<double>(val));
-        m_finalPaymentEdit->loadText(refVal.abs().formatMoney());
+        m_finalPaymentEdit->loadText(refVal.abs().formatMoney(fraction));
         result += i18n("The final payment has been modified to %1.")
-                      .arg(m_finalPaymentEdit->text());
+                      .arg(m_finalPaymentEdit->lineedit()->text());
       }
 
     } else {
@@ -880,15 +900,15 @@ int KNewLoanWizard::calculateLoan(void)
 
       MyMoneyMoney refVal(static_cast<double>(val));
       result = i18n("KMyMoney has calculated a final payment of %1 for this loan.")
-                        .arg(refVal.abs().formatMoney());
+                        .arg(refVal.abs().formatMoney(fraction));
 
-      if(!m_finalPaymentEdit->text().isEmpty()) {
+      if(!m_finalPaymentEdit->lineedit()->text().isEmpty()) {
         if((m_finalPaymentEdit->value().abs() - refVal.abs()).abs().toDouble() > 1) {
           throw new MYMONEYEXCEPTION("incorrect fincancial calculation");
         }
         result = i18n("KMyMoney has successfully verified your loan information.");
       }
-      m_finalPaymentEdit->loadText(refVal.abs().formatMoney());
+      m_finalPaymentEdit->loadText(refVal.abs().formatMoney(fraction));
     }
 
   } catch (MyMoneyException *e) {
@@ -924,15 +944,15 @@ const QString KNewLoanWizard::updateTermWidgets(const long double val)
   switch(unit) {
     case MyMoneySchedule::OCCUR_MONTHLY:
       valString = i18n("one month", "%n months", vl);
-      m_durationUnitEdit->setCurrentItem(i18n("Months"));
+      m_durationUnitEdit->setCurrentItem(static_cast<int>(MyMoneySchedule::OCCUR_MONTHLY));
       break;
     case MyMoneySchedule::OCCUR_YEARLY:
       valString = i18n("one year", "%n years", vl);
-      m_durationUnitEdit->setCurrentItem(i18n("Years"));
+      m_durationUnitEdit->setCurrentItem(static_cast<int>(MyMoneySchedule::OCCUR_YEARLY));
       break;
     default:
       valString = i18n("one payment", "%n payments", vl);
-      m_durationUnitEdit->setCurrentItem(i18n("Payments"));
+      m_durationUnitEdit->setCurrentItem(static_cast<int>(MyMoneySchedule::OCCUR_ONCE));
       break;
   }
   m_durationValueEdit->setValue(vl);
@@ -957,19 +977,19 @@ void KNewLoanWizard::slotCreateCategory(void)
   if(dlg->exec() == QDialog::Accepted) {
     acc = dlg->account();
 
+    MyMoneyFileTransaction ft;
     try {
       QCString id;
       id = file->createCategory(base, acc.name());
       if(id.isEmpty())
         throw new MYMONEYEXCEPTION("failure while creating the account hierarchy");
 
-      loadAccountList();
+      ft.commit();
+
       m_interestAccountEdit->setSelected(id);
 
     } catch (MyMoneyException *e) {
-      QString message("Unable to add account: ");
-      message += e->what();
-      KMessageBox::information(this, message);
+      KMessageBox::information(this, i18n("Unable to add account: %1").arg(e->what()));
       delete e;
     }
   }
@@ -978,31 +998,25 @@ void KNewLoanWizard::slotCreateCategory(void)
 
 void KNewLoanWizard::loadAccountList(void)
 {
+  AccountSet interestSet, assetSet;
+
   if(m_borrowButton->isChecked()) {
-    m_interestAccountEdit->loadList(KMyMoneyUtils::expense);
+    interestSet.addAccountType(MyMoneyAccount::Expense);
   } else {
-    m_interestAccountEdit->loadList(KMyMoneyUtils::income);
+    interestSet.addAccountType(MyMoneyAccount::Income);
   }
+  interestSet.load(m_interestAccountEdit);
 
-  QValueList<int> typeList;
-  typeList << MyMoneyAccount::Checkings;
-  typeList << MyMoneyAccount::Savings;
-  typeList << MyMoneyAccount::Cash;
-  // typeList << MyMoneyAccount::AssetLoan;
-  // typeList << MyMoneyAccount::CertificateDep;
-  // typeList << MyMoneyAccount::Investment;
-  // typeList << MyMoneyAccount::MoneyMarket;
-  typeList << MyMoneyAccount::Asset;
-  typeList << MyMoneyAccount::Currency;
+  assetSet.addAccountType(MyMoneyAccount::Checkings);
+  assetSet.addAccountType(MyMoneyAccount::Savings);
+  assetSet.addAccountType(MyMoneyAccount::Cash);
+  assetSet.addAccountType(MyMoneyAccount::Asset);
+  assetSet.addAccountType(MyMoneyAccount::Currency);
+  assetSet.load(m_assetAccountEdit);
 
-  m_assetAccountEdit->loadList(typeList);
-
-  typeList << MyMoneyAccount::CreditCard;
-  // typeList << MyMoneyAccount::Loan;
-  typeList << MyMoneyAccount::Liability;
-  // typeList << MyMoneyAccount::Income;
-  // typeList << MyMoneyAccount::Expense;
-  m_paymentAccountEdit->loadList(typeList);
+  assetSet.addAccountType(MyMoneyAccount::CreditCard);
+  assetSet.addAccountType(MyMoneyAccount::Liability);
+  assetSet.load(m_paymentAccountEdit);
 }
 
 void KNewLoanWizard::slotAdditionalFees(void)
@@ -1010,7 +1024,9 @@ void KNewLoanWizard::slotAdditionalFees(void)
   // KMessageBox::information(0, QString("Not yet implemented ... if you want to help, contact kmymoney2-developer@lists.sourceforge.net"), QString("Development notice"));
   MyMoneyAccount account(QCString("Phony-ID"), MyMoneyAccount());
 
-  KSplitTransactionDlg* dlg = new KSplitTransactionDlg(m_transaction, account, false, !m_borrowButton->isChecked(), MyMoneyMoney(0));
+  QMap<QCString, MyMoneyMoney> priceInfo;
+  KSplitTransactionDlg* dlg = new KSplitTransactionDlg(m_transaction, m_split, account, false, !m_borrowButton->isChecked(), MyMoneyMoney(0), priceInfo);
+  connect(dlg, SIGNAL(newCategory(MyMoneyAccount&)), this, SIGNAL(newCategory(MyMoneyAccount&)));
 
   if(dlg->exec() == QDialog::Accepted) {
     m_transaction = dlg->transaction();
@@ -1023,7 +1039,7 @@ void KNewLoanWizard::slotAdditionalFees(void)
         fees += (*it).value();
       }
     }
-    m_additionalCost->setText(fees.abs().formatMoney());
+    m_additionalCost->setText(fees.abs().formatMoney(m_account.fraction(MyMoneyFile::instance()->security(m_account.currencyId()))));
   }
 
   delete dlg;
@@ -1037,9 +1053,9 @@ const MyMoneyTransaction KNewLoanWizard::transaction() const
   MyMoneySplit sPayment, sInterest, sAmortization;
   // setup accounts. at this point, we cannot fill in the id of the
   // account that the amortization will be performed on, because we
-  // create the is account. So the id is yet unknown.
-  sPayment.setAccountId(m_paymentAccountEdit->selectedAccounts().first());
-  sInterest.setAccountId(m_interestAccountEdit->selectedAccounts().first());
+  // create the account. So the id is yet unknown.
+  sPayment.setAccountId(m_paymentAccountEdit->selectedItems().first());
+  sInterest.setAccountId(m_interestAccountEdit->selectedItems().first());
 
   // values
   if(m_borrowButton->isChecked()) {
@@ -1049,6 +1065,14 @@ const MyMoneyTransaction KNewLoanWizard::transaction() const
   }
   sInterest.setValue(MyMoneyMoney::autoCalc);
   sAmortization.setValue(MyMoneyMoney::autoCalc);
+  // don't forget the shares
+  sPayment.setShares(sPayment.value());
+  sInterest.setShares(sInterest.value());
+  sAmortization.setShares(sAmortization.value());
+
+  // setup the commodity
+  MyMoneyAccount acc = MyMoneyFile::instance()->account(sPayment.accountId());
+  t.setCommodity(acc.currencyId());
 
   // actions
   sPayment.setAction(MyMoneySplit::ActionAmortization);
@@ -1056,16 +1080,12 @@ const MyMoneyTransaction KNewLoanWizard::transaction() const
   sInterest.setAction(MyMoneySplit::ActionInterest);
 
   // payee
-  if(!m_payeeEdit->text().isEmpty()) {
-    try {
-      QCString payeeId = MyMoneyFile::instance()->payeeByName(m_payeeEdit->text()).id();
-      sPayment.setPayeeId(payeeId);
-      sAmortization.setPayeeId(payeeId);
+  QCString payeeId = m_payeeEdit->selectedItem();
+  sPayment.setPayeeId(payeeId);
+  sAmortization.setPayeeId(payeeId);
 
-    } catch(MyMoneyException *e) {
-      delete e;
-    }
-  }
+  MyMoneyAccount account(QCString("Phony-ID"), MyMoneyAccount());
+  sAmortization.setAccountId(account.id());
 
   // IMPORTANT: Payment split must be the first one, because
   //            the schedule view expects it this way during display
@@ -1074,14 +1094,14 @@ const MyMoneyTransaction KNewLoanWizard::transaction() const
   t.addSplit(sInterest);
 
   // copy the splits from the other costs and update the payment split
-  MyMoneyAccount account(QCString("Phony-ID"), MyMoneyAccount());
   QValueList<MyMoneySplit>::ConstIterator it;
   for(it = m_transaction.splits().begin(); it != m_transaction.splits().end(); ++it) {
     if((*it).accountId() != account.id()) {
       MyMoneySplit sp = (*it);
-      sp.setId(QCString());
+      sp.clearId();
       t.addSplit(sp);
       sPayment.setValue(sPayment.value()-sp.value());
+      sPayment.setShares(sPayment.value());
       t.modifySplit(sPayment);
     }
   }
@@ -1094,86 +1114,31 @@ const MyMoneySchedule KNewLoanWizard::schedule() const
                         MyMoneySchedule::TYPE_LOANPAYMENT,
                         KMyMoneyUtils::stringToOccurence(m_paymentFrequencyUnitEdit->currentText()),
                         MyMoneySchedule::STYPE_OTHER,
-                        m_nextDueDateEdit->getQDate(),
+                        QDate(),
                         QDate(),
                         false,
                         false);
 
-  sched.setTransaction(transaction());
+  MyMoneyTransaction t = transaction();
+  t.setPostDate(m_nextDueDateEdit->date());
+  sched.setTransaction(t);
 
   return sched;
 }
 
-const MyMoneyAccount KNewLoanWizard::account(void) const
+void KNewLoanWizard::slotReloadEditWidgets(void)
 {
-  MyMoneyAccountLoan acc;
+  // load the various account widgets
+  loadAccountList();
 
-  acc.setName(m_nameEdit->text());
-  acc.setOpeningDate(m_firstDueDateEdit->getQDate());
-  if(m_allPaymentsButton->isChecked()) {
-    acc.setOpeningBalance(0);
-  } else {
-    acc.setOpeningBalance(m_loanAmountEdit->value());
+  // reload payee widget
+  QCString payeeId = m_payeeEdit->selectedItem();
+
+  m_payeeEdit->loadPayees(MyMoneyFile::instance()->payeeList());
+
+  if(!payeeId.isEmpty()) {
+    m_payeeEdit->setSelectedItem(payeeId);
   }
-  if(m_borrowButton->isChecked()) {
-    acc.setAccountType(MyMoneyAccount::Loan);
-    acc.setOpeningBalance(-acc.openingBalance());
-  } else {
-    acc.setAccountType(MyMoneyAccount::AssetLoan);
-  }
-  acc.setLoanAmount(m_loanAmountEdit->value());
-  acc.setInterestRate(m_firstDueDateEdit->getQDate(), m_interestRateEdit->value());
-  if(m_interestOnReceptionButton->isChecked())
-    acc.setInterestCalculation(MyMoneyAccountLoan::paymentReceived);
-  else
-    acc.setInterestCalculation(MyMoneyAccountLoan::paymentDue);
-
-  acc.setFixedInterestRate(m_fixedInterestButton->isChecked());
-  acc.setFinalPayment(MyMoneyMoney(m_finalPaymentEdit->text()));
-  acc.setTerm(term());
-  acc.setPeriodicPayment(m_paymentEdit->value());
-
-  if(!m_payeeEdit->text().isEmpty()) {
-    try {
-      acc.setPayee(MyMoneyFile::instance()->payeeByName(m_payeeEdit->text()).id());
-    } catch(MyMoneyException *e) {
-      delete e;
-      qWarning("%s(%d): Someone has removed the selected payee", __FILE__, __LINE__);
-      acc.setPayee(QCString());
-    }
-  } else
-    acc.setPayee(QCString());
-
-  if(m_variableInterestButton->isChecked()) {
-    acc.setNextInterestChange(m_interestChangeDateEdit->getQDate());
-    acc.setInterestChangeFrequency(m_interestFrequencyAmountEdit->value(),
-                                   m_interestFrequencyUnitEdit->currentItem());
-  }
-  return acc;
-}
-
-void KNewLoanWizard::loadWidgets(const MyMoneyAccount& account)
-{
-  bool borrowing = false;
-  MyMoneyAccountLoan acc(account);
-
-  m_nameEdit->loadText(acc.name());
-  if(acc.accountType() == MyMoneyAccount::Loan) {
-    m_borrowButton->animateClick();
-    borrowing = true;
-  } else
-    m_lendButton->animateClick();
-
-  if(!acc.openingBalance().isZero()) {
-    if(borrowing)
-      m_loanAmountEdit->loadText((-account.openingBalance()).formatMoney());
-    else
-      m_loanAmountEdit->loadText(account.openingBalance().formatMoney());
-  }
-
-
-  if(account.openingDate().isValid())
-    m_firstDueDateEdit->setDate(account.openingDate());
 }
 
 int KNewLoanWizard::term(void) const
@@ -1183,11 +1148,11 @@ int KNewLoanWizard::term(void) const
   if(m_durationValueEdit->value() != 0) {
     factor = 1;
     switch(m_durationUnitEdit->currentItem()) {
-      case 1: // years
+      case MyMoneySchedule::OCCUR_YEARLY: // years
         factor = 12;
         // tricky fall through here
 
-      case 0: // months
+      case MyMoneySchedule::OCCUR_MONTHLY: // months
         factor *= 30;
         factor *= m_durationValueEdit->value();
         // factor now is the duration in days. we divide this by the
@@ -1196,7 +1161,7 @@ int KNewLoanWizard::term(void) const
                             m_paymentFrequencyUnitEdit->currentText()));;
         break;
 
-      case 2: // payments
+      case MyMoneySchedule::OCCUR_ONCE: // payments
         factor = m_durationValueEdit->value();
         break;
     }
@@ -1209,7 +1174,7 @@ const QCString KNewLoanWizard::initialPaymentAccount(void) const
   if(m_dontCreatePayoutCheckBox->isChecked()) {
     return QCString();
   }
-  return m_assetAccountEdit->selectedAccounts().first();
+  return m_assetAccountEdit->selectedItems().first();
 }
 
 const QDate KNewLoanWizard::initialPaymentDate(void) const
@@ -1217,7 +1182,7 @@ const QDate KNewLoanWizard::initialPaymentDate(void) const
   if(m_dontCreatePayoutCheckBox->isChecked()) {
     return QDate();
   }
-  return m_paymentDate->getQDate();
+  return m_paymentDate->date();
 }
 
 #include "knewloanwizard.moc"
