@@ -35,9 +35,9 @@
 // Project Includes
 
 #include <../dialogs/knewloanwizarddecl.h>
-#include <../mymoney/mymoneyscheduled.h>
-#include <../widgets/kmymoneyaccountselector.h>
-#include <../widgets/kmymoneydateinput.h>
+#include <kmymoney/mymoneyscheduled.h>
+#include <kmymoney/kmymoneyaccountselector.h>
+#include <kmymoney/kmymoneydateinput.h>
 
 /**
   * @author Thomas Baumgart
@@ -58,15 +58,6 @@ public:
   KNewLoanWizard(QWidget *parent=0, const char *name=0);
   ~KNewLoanWizard();
 
-  /**
-    * This method returns a MyMoneyAccount object with all data
-    * filled out as provided by the wizard. The institution reference
-    * will be empty.
-    *
-    * @return MyMoneyAccount object to be used to create a new account
-    */
-  const MyMoneyAccount account(void) const;
-
   /**
     * This method returns the schedule for the payments. The account
     * where the amortization should be transferred to is the one
@@ -99,8 +90,6 @@ public:
     */
   const QDate initialPaymentDate(void) const;
 
-  void loadWidgets(const MyMoneyAccount& acc);
-
 protected:
   /**
     * This method returns the transaction that is stored within
@@ -125,7 +114,8 @@ protected slots:
   void slotInterestOnReception(void);
   void slotCreateCategory(void);
   virtual void slotAdditionalFees(void);
-  void slotNewPayee(const QString&);
+  // void slotNewPayee(const QString&);
+  void slotReloadEditWidgets(void);
 
 protected:
   void loadComboBoxes(void);
@@ -144,9 +134,35 @@ protected:
   int occurenceToPeriod(const MyMoneySchedule::occurenceE occurence) const;
   int term(void) const;
 
+signals:
+  /**
+    * This signal is emitted, when a new category name has been
+    * entered by the user and this name is not known as account
+    * by the MyMoneyFile object.
+    * Before the signal is emitted, a MyMoneyAccount is constructed
+    * by this object and filled with the desired name. All other members
+    * of MyMoneyAccount will remain in their default state. Upon return,
+    * the connected slot should have created the object in the MyMoneyFile
+    * engine and filled the member @p id.
+    *
+    * @param acc reference to MyMoneyAccount object that caries the name
+    *            and will return information about the created category.
+    */
+  void newCategory(MyMoneyAccount& acc);
+
+  /**
+    * This signal is sent out, when a new payee needs to be created
+    * @sa KMyMoneyCombo::createItem()
+    *
+    * @param txt The name of the payee to be created
+    * @param id A connected slot should store the id of the created object in this variable
+    */
+  void createPayee(const QString& txt, QCString& id);
+
 protected:
   MyMoneyAccountLoan  m_account;
   MyMoneyTransaction  m_transaction;
+  MyMoneySplit        m_split;
 };
 
 #endif
