@@ -18,10 +18,13 @@
 #ifdef HAVE_CONFIG_H
 #include "../../config.h"
 #endif
+#ifdef USE_OFX_DIRECTCONNECT
 
 // ----------------------------------------------------------------------------
 // System Includes
 
+#include <libofx/libofx.h>
+
 // ----------------------------------------------------------------------------
 // QT Includes
 
@@ -43,31 +46,34 @@
 #include "mymoneyofxconnector.h"
 
 MyMoneyOfxConnector::MyMoneyOfxConnector(const MyMoneyAccount& _account):
-  m_account(_account),
-  m_institution( MyMoneyFile::instance()->institution( _account.institutionId() ) )
+  m_account(_account)
 {
-  m_fiSettings = m_institution.ofxConnectionSettings();
+  m_fiSettings = m_account.onlineBankingSettings();
 }
 
-QString MyMoneyOfxConnector::iban(void) const { return m_fiSettings.value("iban"); }
-QString MyMoneyOfxConnector::fiorg(void) const { return m_fiSettings.value("fiorg"); }
-QString MyMoneyOfxConnector::fiid(void) const { return m_fiSettings.value("fiid"); }
-QString MyMoneyOfxConnector::username(void) const { return m_fiSettings.value("user"); }
+QString MyMoneyOfxConnector::iban(void) const { return m_fiSettings.value("bankid"); }
+QString MyMoneyOfxConnector::fiorg(void) const { return m_fiSettings.value("org"); }
+QString MyMoneyOfxConnector::fiid(void) const { return m_fiSettings.value("fid"); }
+QString MyMoneyOfxConnector::username(void) const { return m_fiSettings.value("username"); }
 QString MyMoneyOfxConnector::password(void) const { return m_fiSettings.value("password"); }
-QString MyMoneyOfxConnector::accountnum(void) const { return m_account.number(); }
+QString MyMoneyOfxConnector::accountnum(void) const { return m_fiSettings.value("accountid"); }
 QString MyMoneyOfxConnector::url(void) const { return m_fiSettings.value("url"); }
 
-QString MyMoneyOfxConnector::accounttype(void) const
+#if LIBOFX_IS_VERSION(0,9,0)
+OfxAccountData::AccountType MyMoneyOfxConnector::accounttype(void) const
 {
-  QString result = "BANK";
+  OfxAccountData::AccountType result = OfxAccountData::OFX_CHECKING;
 
   switch( m_account.accountType() )
   {
   case MyMoneyAccount::Investment:
-    result = "INV";
+    result = OfxAccountData::OFX_INVESTMENT;
     break;
   case MyMoneyAccount::CreditCard:
-    result = "CC";
+    result = OfxAccountData::OFX_CREDITCARD;
+    break;
+  case MyMoneyAccount::Savings:
+    result = OfxAccountData::OFX_SAVINGS;
     break;
   default:
     break;
@@ -79,36 +85,138 @@ QString MyMoneyOfxConnector::accounttype(void) const
   QRegExp rexp("OFXTYPE:([A-Z]*)");
   if ( rexp.search(m_account.description()) != -1 )
   {
-    result = rexp.cap(1);
+    QString override = rexp.cap(1);
     kdDebug(2) << "MyMoneyOfxConnector::accounttype() overriding to " << result << endl;
+
+    if ( override == "BANK" )
+    result = OfxAccountData::OFX_CHECKING;
+    else if ( override == "CC" )
+      result = OfxAccountData::OFX_CREDITCARD;
+    else if ( override == "INV" )
+      result = OfxAccountData::OFX_INVESTMENT;
   }
 
   return result;
 }
-
-const QByteArray MyMoneyOfxConnector::statementRequest(const QDate& _dtstart) const
+#else
+AccountType MyMoneyOfxConnector::accounttype(void) const
 {
-  QString request;
+  AccountType result = OFX_BANK_ACCOUNT;
 
-  if ( accounttype()=="CC" )
-    request = header() + Tag("OFX").subtag(signOn()).subtag(creditCardRequest(_dtstart));
-  else if ( accounttype()=="INV" )
-    request = header() + Tag("OFX").subtag(signOn()).subtag(investmentRequest(_dtstart));
-  else
-    request = header() + Tag("OFX").subtag(signOn()).subtag(bankStatementRequest(_dtstart));
+  switch( m_account.accountType() )
+  {
+  case MyMoneyAccount::Investment:
+    result = OFX_INVEST_ACCOUNT;
+    break;
+  case MyMoneyAccount::CreditCard:
+    result = OFX_CREDITCARD_ACCOUNT;
+    break;
+  default:
+    break;
+  }
 
-  // remove the trailing zero
-  QByteArray result = request.utf8();
-  result.truncate(result.size()-1);
+  // This is a bit of a personalized hack.  Sometimes we may want to override the
+  // ofx type for an account.  For now, I will stash it in the notes!
+
+  QRegExp rexp("OFXTYPE:([A-Z]*)");
+  if ( rexp.search(m_account.description()) != -1 )
+  {
+    QString override = rexp.cap(1);
+    kdDebug(2) << "MyMoneyOfxConnector::accounttype() overriding to " << result << endl;
+
+    if ( override == "BANK" )
+    result = OFX_BANK_ACCOUNT;
+    else if ( override == "CC" )
+      result = OFX_CREDITCARD_ACCOUNT;
+    else if ( override == "INV" )
+      result = OFX_INVEST_ACCOUNT;
+  }
 
   return result;
 }
+#endif
 
-QString MyMoneyOfxConnector::uuid(void)
+#if LIBOFX_IS_VERSION(0,9,0)
+const QByteArray MyMoneyOfxConnector::statementRequest(const QDate& _dtstart) const
 {
-  static int id = 1;
-  return QDateTime::currentDateTime().toString("yyyyMMdd-hhmmsszzz-") + QString::number(id++);
+  OfxFiLogin fi;
+  memset(&fi,0,sizeof(OfxFiLogin));
+  strncpy(fi.fid,fiid().latin1(),OFX_FID_LENGTH-1);
+  strncpy(fi.org,fiorg().latin1(),OFX_ORG_LENGTH-1);
+  strncpy(fi.userid,username().latin1(),OFX_USERID_LENGTH-1);
+  strncpy(fi.userpass,password().latin1(),OFX_USERPASS_LENGTH-1);
+
+  OfxAccountData account;
+  memset(&account,0,sizeof(OfxAccountData));
+
+  if(iban().latin1() != 0) {
+    strncpy(account.bank_id,iban().latin1(),OFX_BANKID_LENGTH-1);
+    strncpy(account.broker_id,iban().latin1(),OFX_BROKERID_LENGTH-1);
+  }
+  strncpy(account.account_number,accountnum().latin1(),OFX_ACCTID_LENGTH-1);
+  account.account_type = accounttype();
+
+  char* szrequest = libofx_request_statement( &fi, &account, QDateTime(_dtstart).toTime_t() );
+  QString request = szrequest;
+  // remove the trailing zero
+  QByteArray result = request.utf8();
+  result.truncate(result.size()-1);
+  free(szrequest);
+
+  QString msg(result);
+  return result;
 }
+#else
+const QByteArray MyMoneyOfxConnector::statementRequest(const QDate& _dtstart) const
+{
+  OfxFiLogin fi;
+  memset(&fi,0,sizeof(OfxFiLogin));
+  strncpy(fi.fid,fiid().latin1(),OFX_FID_LENGTH-1);
+  strncpy(fi.org,fiorg().latin1(),OFX_ORG_LENGTH-1);
+  strncpy(fi.userid,username().latin1(),OFX_USERID_LENGTH-1);
+  strncpy(fi.userpass,password().latin1(),OFX_USERPASS_LENGTH-1);
+
+  OfxAccountInfo account;
+  memset(&account,0,sizeof(OfxAccountInfo));
+
+  if(iban().latin1() != 0) {
+    strncpy(account.bankid,iban().latin1(),OFX_BANKID_LENGTH-1);
+    strncpy(account.brokerid,iban().latin1(),OFX_BROKERID_LENGTH-1);
+  }
+  strncpy(account.accountid,accountnum().latin1(),OFX_ACCOUNT_ID_LENGTH-1);
+  account.type = accounttype();
+
+  char* szrequest = libofx_request_statement( &fi, &account, QDateTime(_dtstart).toTime_t() );
+  QString request = szrequest;
+  // remove the trailing zero
+  QByteArray result = request.utf8();
+  result.truncate(result.size()-1);
+  free(szrequest);
+
+  QString msg(result);
+  return result;
+}
+#endif
+
+const QByteArray MyMoneyOfxConnector::accountInfoRequest(void) const
+{
+  OfxFiLogin fi;
+  memset(&fi,0,sizeof(OfxFiLogin));
+  strncpy(fi.fid,fiid().latin1(),OFX_FID_LENGTH-1);
+  strncpy(fi.org,fiorg().latin1(),OFX_ORG_LENGTH-1);
+  strncpy(fi.userid,username().latin1(),OFX_USERID_LENGTH-1);
+  strncpy(fi.userpass,password().latin1(),OFX_USERPASS_LENGTH-1);
+
+  char* szrequest = libofx_request_accountinfo( &fi );
+  QString request = szrequest;
+  // remove the trailing zero
+  QByteArray result = request.utf8();
+  result.truncate(result.size()-1);
+  free(szrequest);
+
+  return result;
+}
+#if 0
 
 MyMoneyOfxConnector::Tag MyMoneyOfxConnector::message(const QString& _msgType, const QString& _trnType, const Tag& _request)
 {
@@ -150,20 +258,6 @@ MyMoneyOfxConnector::Tag MyMoneyOfxConnector::creditCardRequest(const QDate& _dt
     .subtag(Tag("INCTRAN").element("DTSTART",dtstart_string).element("INCLUDE","Y")));
 }
 
-QString MyMoneyOfxConnector::header(void)
-{
-  return QString("OFXHEADER:100\r\n"
-                 "DATA:OFXSGML\r\n"
-                 "VERSION:102\r\n"
-                 "SECURITY:NONE\r\n"
-                 "ENCODING:USASCII\r\n"
-                 "CHARSET:1252\r\n"
-                 "COMPRESSION:NONE\r\n"
-                 "OLDFILEUID:NONE\r\n"
-                 "NEWFILEUID:%1\r\n"
-                 "\r\n").arg(uuid());
-}
-
 MyMoneyOfxConnector::Tag MyMoneyOfxConnector::signOn(void) const
 {
   QString dtnow_string = QDateTime::currentDateTime().toString(Qt::ISODate).remove(QRegExp("[^0-9]"));
@@ -184,6 +278,26 @@ MyMoneyOfxConnector::Tag MyMoneyOfxConnector::signOn(void) const
       .element("APPVER","1100"));
 }
 
+QString MyMoneyOfxConnector::header(void)
+{
+  return QString("OFXHEADER:100\r\n"
+                 "DATA:OFXSGML\r\n"
+                 "VERSION:102\r\n"
+                 "SECURITY:NONE\r\n"
+                 "ENCODING:USASCII\r\n"
+                 "CHARSET:1252\r\n"
+                 "COMPRESSION:NONE\r\n"
+                 "OLDFILEUID:NONE\r\n"
+                 "NEWFILEUID:%1\r\n"
+                 "\r\n").arg(uuid());
+}
+
+QString MyMoneyOfxConnector::uuid(void)
+{
+  static int id = 1;
+  return QDateTime::currentDateTime().toString("yyyyMMdd-hhmmsszzz-") + QString::number(id++);
+}
+
 //
 // Methods to provide RESPONSES to OFX requests.  This has no real use in
 // KMyMoney, but it's included for the purposes of unit testing.  This way, I
@@ -361,7 +475,7 @@ MyMoneyOfxConnector::Tag MyMoneyOfxConnector::transaction(const MyMoneyTransacti
   //Use this version for bank/cc transactions
   MyMoneySplit s = _t.splitByAccount( m_account.id(), true );
 
-  //TODO: Write "investmentTransaction()"...
+  //TODO (Ace) Write "investmentTransaction()"...
   //Use this version for inv transactions
   //MyMoneySplit s = _t.splitByAccount( m_account.accountList(), true );
 
@@ -481,3 +595,5 @@ MyMoneyOfxConnector::Tag MyMoneyOfxConnector::investmentTransaction(const MyMone
   //FIXME: Do something useful with these errors
   return Tag("ERROR").element("DETAILS","This transaction contains an unsupported action type");
 }
+#endif
+#endif // USE_OFX_DIRECTCONNECT
