@@ -21,6 +21,9 @@
 
 #ifdef HAVE_LIBCPPUNIT
 
+#include <kcmdlineargs.h>
+#include <kapplication.h>
+
 #include "cppunit/TextTestRunner.h"
 #include "cppunit/TextTestResult.h"
 #include "cppunit/TestSuite.h"
@@ -41,15 +44,17 @@
 #include "mymoney/mymoneyaccounttest.h"
 #include "mymoney/mymoneytransactiontest.h"
 #include "mymoney/storage/mymoneyseqaccessmgrtest.h"
-#include "mymoney/storage/mymoneystoragebintest.h"
 #include "mymoney/mymoneyfiletest.h"
 #include "mymoney/mymoneykeyvaluecontainertest.h"
 #include "mymoney/mymoneyscheduletest.h"
 #include "mymoney/mymoneyfinancialcalculatortest.h"
 #include "mymoney/mymoneysecuritytest.h"
 #include "mymoney/mymoneypricetest.h"
+#include "mymoney/mymoneyobjecttest.h"
+#include "mymoney/mymoneyforecasttest.h"
 
 #include "reports/pivottabletest.h"
+#include "reports/pivotgridtest.h"
 #include "reports/querytabletest.h"
 
 #include "converter/convertertest.h"
@@ -93,11 +98,15 @@ void unexpectedException(MyMoneyException *e)
 
 #endif // HAVE_LIBCPPUNIT
 
-int
-main(int /* argc */, char** /* argv */ )
+int main(int testargc, char** testargv)
 {
   int rc = 0;
+
 #ifdef HAVE_LIBCPPUNIT
+  // we seem to need a KApplication object to use KGlobal::locale()
+  KCmdLineArgs::init(testargc, testargv, testargv[0], "UNIT TESTS", "", "0.1");
+  KApplication::disableAutoDcopRegistration();
+  KApplication app(false, false);
 
 #ifdef _CHECK_MEMORY
   _CheckMemory_Init(0);
@@ -105,6 +114,7 @@ main(int /* argc */, char** /* argv */ )
 
   // mymoney tests
   CPPUNIT_TEST_SUITE_REGISTRATION(MyMoneyExceptionTest);
+  CPPUNIT_TEST_SUITE_REGISTRATION(MyMoneyObjectTest);
   CPPUNIT_TEST_SUITE_REGISTRATION(MyMoneyKeyValueContainerTest);
   CPPUNIT_TEST_SUITE_REGISTRATION(MyMoneyObserverTest);
   CPPUNIT_TEST_SUITE_REGISTRATION(MyMoneyMoneyTest);
@@ -114,19 +124,20 @@ main(int /* argc */, char** /* argv */ )
   CPPUNIT_TEST_SUITE_REGISTRATION(MyMoneyInstitutionTest);
   CPPUNIT_TEST_SUITE_REGISTRATION(MyMoneyAccountTest);
   CPPUNIT_TEST_SUITE_REGISTRATION(MyMoneySeqAccessMgrTest);
-  CPPUNIT_TEST_SUITE_REGISTRATION(MyMoneyStorageBinTest);
   CPPUNIT_TEST_SUITE_REGISTRATION(MyMoneyFileTest);
   CPPUNIT_TEST_SUITE_REGISTRATION(MyMoneyScheduleTest);
   CPPUNIT_TEST_SUITE_REGISTRATION(MyMoneySecurityTest);
   CPPUNIT_TEST_SUITE_REGISTRATION(MyMoneyPriceTest);
+  CPPUNIT_TEST_SUITE_REGISTRATION(MyMoneyForecastTest);
 
   // reports tests
   CPPUNIT_TEST_SUITE_REGISTRATION(PivotTableTest);
+  CPPUNIT_TEST_SUITE_REGISTRATION(PivotGridTest);
   CPPUNIT_TEST_SUITE_REGISTRATION(QueryTableTest);
 
   // converter tests
   CPPUNIT_TEST_SUITE_REGISTRATION(ConverterTest);
-  
+
   // off we go
   CppUnit::TestFactoryRegistry &registry =
     CppUnit::TestFactoryRegistry::getRegistry();
@@ -164,6 +175,12 @@ main(int /* argc */, char** /* argv */ )
   return rc;
 }
 
+// required for the testcases (mymoneystoragesql references it)
+void timetrace(const char *txt)
+{
+  Q_UNUSED(txt);
+}
+
 #if 0
 #ifdef HAVE_LIBOFX
 
