@@ -36,18 +36,12 @@
 // ----------------------------------------------------------------------------
 // Project Includes
 
+#include <kmymoney/mymoneyfile.h>
+#include <kmymoney/mymoneyfinancialcalculator.h>
+#include <kmymoney/kmymoneyglobalsettings.h>
+#include <kmymoney/kmymoneycombo.h>
+
 #include "kmymoneyutils.h"
-#include "mymoney/mymoneyfile.h"
-
-QColor KMyMoneyUtils::_backgroundColour;
-QColor KMyMoneyUtils::_listColour;
-QColor KMyMoneyUtils::_gridColour;
-QColor KMyMoneyUtils::_missingConversionColour;
-
-QFont  KMyMoneyUtils::_cellFont;
-QFont  KMyMoneyUtils::_headerFont;
-
-bool   KMyMoneyUtils::_expertMode;
 
 KMyMoneyUtils::KMyMoneyUtils()
 {
@@ -219,10 +213,16 @@ const MyMoneySchedule::occurenceE KMyMoneyUtils::stringToOccurence(const QString
     occurence = MyMoneySchedule::OCCUR_FORTNIGHTLY;
   else if(tmp == i18n("Every other week").lower())
     occurence = MyMoneySchedule::OCCUR_EVERYOTHERWEEK;
-  else if(tmp == i18n("Every four week").lower())
+  else if(tmp == i18n("Every three weeks").lower())
+    occurence = MyMoneySchedule::OCCUR_EVERYTHREEWEEKS;
+  else if(tmp == i18n("Every four weeks").lower())
     occurence = MyMoneySchedule::OCCUR_EVERYFOURWEEKS;
+  else if(tmp == i18n("Every thirty days").lower())
+    occurence = MyMoneySchedule::OCCUR_EVERYTHIRTYDAYS;
   else if(tmp == i18n("Monthly").lower())
     occurence = MyMoneySchedule::OCCUR_MONTHLY;
+  else if(tmp == i18n("Every eight weeks").lower())
+    occurence = MyMoneySchedule::OCCUR_EVERYEIGHTWEEKS;
   else if(tmp == i18n("Every two months").lower())
     occurence = MyMoneySchedule::OCCUR_EVERYOTHERMONTH;
   else if(tmp == i18n("Every three months").lower())
@@ -243,57 +243,9 @@ const MyMoneySchedule::occurenceE KMyMoneyUtils::stringToOccurence(const QString
 
 const QString KMyMoneyUtils::occurenceToString(const MyMoneySchedule::occurenceE occurence)
 {
-  QString text;
-
-  switch (occurence)
-  {
-    case MyMoneySchedule::OCCUR_ONCE:
-      text = i18n("Once");
-      break;
-    case MyMoneySchedule::OCCUR_DAILY:
-      text = i18n("Daily");
-      break;
-    case MyMoneySchedule::OCCUR_WEEKLY:
-      text = i18n("Weekly");
-      break;
-    case MyMoneySchedule::OCCUR_FORTNIGHTLY:
-      text = i18n("Fortnightly");
-      break;
-    case MyMoneySchedule::OCCUR_EVERYOTHERWEEK:
-      text = i18n("Every other week");
-      break;
-    case MyMoneySchedule::OCCUR_EVERYFOURWEEKS:
-      text = i18n("Every four weeks");
-      break;
-    case MyMoneySchedule::OCCUR_MONTHLY:
-      text = i18n("Monthly");
-      break;
-    case MyMoneySchedule::OCCUR_EVERYOTHERMONTH:
-      text = i18n("Every two months");
-      break;
-    case MyMoneySchedule::OCCUR_EVERYTHREEMONTHS:
-      text = i18n("Every three months");
-      break;
-    case MyMoneySchedule::OCCUR_QUARTERLY:
-      text = i18n("Quarterly");
-      break;
-    case MyMoneySchedule::OCCUR_EVERYFOURMONTHS:
-      text = i18n("Every four months");
-      break;
-    case MyMoneySchedule::OCCUR_TWICEYEARLY:
-      text = i18n("Twice yearly");
-      break;
-    case MyMoneySchedule::OCCUR_YEARLY:
-      text = i18n("Yearly");
-      break;
-    case MyMoneySchedule::OCCUR_EVERYOTHERYEAR:
-      text = i18n("Every other year");
-      break;
-    case MyMoneySchedule::OCCUR_ANY:
-      text = i18n("Any (Error)");
-      break;
-  }
-  return text;
+  KMyMoneyFrequencyCombo combo;
+  combo.setCurrentItem(occurence);
+  return combo.currentText();
 }
 
 const QString KMyMoneyUtils::paymentMethodToString(MyMoneySchedule::paymentTypeE paymentType)
@@ -317,12 +269,36 @@ const QString KMyMoneyUtils::paymentMethodToString(MyMoneySchedule::paymentTypeE
     case MyMoneySchedule::STYPE_WRITECHEQUE:
       text = i18n("Write check");
       break;
+    case MyMoneySchedule::STYPE_STANDINGORDER:
+      text = i18n("Standing order");
+      break;
+    case MyMoneySchedule::STYPE_BANKTRANSFER:
+      text = i18n("Bank transfer");
+      break;
     case MyMoneySchedule::STYPE_ANY:
       text = i18n("Any (Error)");
       break;
   }
   return text;
 }
+const QString KMyMoneyUtils::weekendOptionToString(MyMoneySchedule::weekendOptionE weekendOption)
+{
+  QString text;
+
+  switch (weekendOption)
+  {
+    case MyMoneySchedule::MoveFriday:
+      text = i18n("Change the date to the previous Friday");
+      break;
+    case MyMoneySchedule::MoveMonday:
+      text = i18n("Change the date to the next Monday");
+      break;
+    case MyMoneySchedule::MoveNothing:
+      text = i18n("Do Nothing");
+      break;
+  }
+  return text;
+}
 
 const QString KMyMoneyUtils::scheduleTypeToString(MyMoneySchedule::typeE type)
 {
@@ -349,18 +325,6 @@ const QString KMyMoneyUtils::scheduleTypeToString(MyMoneySchedule::typeE type)
   return text;
 }
 
-KGuiItem KMyMoneyUtils::splitGuiItem(void)
-{
-  KIconLoader *ic = KGlobal::iconLoader();
-
-  KGuiItem splitGuiItem(  i18n("&Split"),
-                          QIconSet(ic->loadIcon("split", KIcon::User, KIcon::SizeSmall)),
-                          i18n("Split the amount into different categories."),
-                          i18n("Split the amount into different categories."));
-
-  return splitGuiItem;
-}
-
 KGuiItem KMyMoneyUtils::scheduleNewGuiItem(void)
 {
   KIconLoader *ic = KGlobal::iconLoader();
@@ -409,26 +373,25 @@ QPixmap KMyMoneyUtils::scheduleIcon(int size)
   return ic->loadIcon("schedule", KIcon::User, size);
 }
 
+const char* homePageItems[] = {
+  I18N_NOOP("Payments"),
+  I18N_NOOP("Preferred accounts"),
+  I18N_NOOP("Payment accounts"),
+  I18N_NOOP("Favorite reports"),
+  I18N_NOOP("Forecast (schedule)"),
+  I18N_NOOP("Networth forecast"),
+  I18N_NOOP("Forecast (history)"),
+  I18N_NOOP("Summary"),
+  I18N_NOOP("Budget"),
+  // insert new items above this comment
+  0
+};
+
 const QString KMyMoneyUtils::homePageItemToString(const int idx)
 {
   QString rc;
-
-  switch(abs(idx)) {
-    case 1:
-      rc = i18n("Payments");
-      break;
-    case 2:
-      rc = i18n("Preferred accounts");
-      break;
-    case 3:
-      rc = i18n("Payment accounts");
-      break;
-    case 4:
-      rc = i18n("Favorite reports");
-      break;
-    default:
-      rc = "";
-      break;
+  if(abs(idx) > 0 && abs(idx) < static_cast<int>(sizeof(homePageItems)/sizeof(homePageItems[0]))) {
+    rc = i18n(homePageItems[abs(idx-1)]);
   }
   return rc;
 }
@@ -436,25 +399,11 @@ const QString KMyMoneyUtils::homePageItemToString(const int idx)
 const int KMyMoneyUtils::stringToHomePageItem(const QString& txt)
 {
   int idx = 0;
-  if(txt == i18n("Payments"))
-    idx = 1;
-  else if(txt == i18n("Preferred accounts"))
-    idx = 2;
-  else if(txt == i18n("Payment accounts"))
-    idx = 3;
-  else if(txt == i18n("Favorite reports"))
-    idx = 4;
-  return idx;
-}
-
-void KMyMoneyUtils::addDefaultHomePageItems(QStringList& list)
-{
-  for(int i = 1; i <= KMyMoneyUtils::maxHomePageItems; ++i) {
-    if(list.find(QString::number(i)) != list.end()
-    || list.find(QString::number(-i)) != list.end())
-      continue;
-    list.append(QString::number(i));
+  for(idx = 0; homePageItems[idx] != 0; ++idx) {
+    if(txt == i18n(homePageItems[idx]))
+      return idx+1;
   }
+  return 0;
 }
 
 bool KMyMoneyUtils::appendCorrectFileExt(QString& str, const QString& strExtToUse)
@@ -503,12 +452,19 @@ int KMyMoneyUtils::occurenceToFrequency(const MyMoneySchedule::occurenceE occure
     case MyMoneySchedule::OCCUR_EVERYOTHERWEEK:
       rc = 26;
       break;
+    case MyMoneySchedule::OCCUR_EVERYTHREEWEEKS:
+      rc = 17;
+      break;
     case MyMoneySchedule::OCCUR_EVERYFOURWEEKS:
       rc = 13;
       break;
     case MyMoneySchedule::OCCUR_MONTHLY:
+    case MyMoneySchedule::OCCUR_EVERYTHIRTYDAYS:
       rc = 12;
       break;
+    case MyMoneySchedule::OCCUR_EVERYEIGHTWEEKS:
+      rc = 6;
+      break;
     case MyMoneySchedule::OCCUR_EVERYOTHERMONTH:
       rc = 6;
       break;
@@ -531,77 +487,6 @@ int KMyMoneyUtils::occurenceToFrequency(const MyMoneySchedule::occurenceE occure
   return rc;
 }
 
-void KMyMoneyUtils::newPayee(QWidget* parent, kMyMoneyPayee* payeeEdit, const QString& payeeName)
-{
-  MyMoneyFile* file = MyMoneyFile::instance();
-  MyMoneyPayee payee;
-
-  if(!payeeEdit)
-    return;
-
-  // Ask the user if that is what he intended to do?
-  QString msg = i18n("Do you want to add '%1' as payee/receiver ?").arg(payeeName);
-
-  if(KMessageBox::questionYesNo(parent, msg, i18n("New payee/receiver"),KStdGuiItem::yes(),KStdGuiItem::no(),"NewPayee") == KMessageBox::Yes) {
-    // for now, we just add the payee to the pool. In the future,
-    // we could open a dialog and ask for all the other attributes
-    // of the payee.
-    payee.setName(payeeName);
-
-    try {
-      file->addPayee(payee);
-      payeeEdit->loadList();
-
-    } catch(MyMoneyException *e) {
-      KMessageBox::detailedSorry(0, i18n("Unable to add payee/receiver"),
-        (e->what() + " " + i18n("thrown in") + " " + e->file()+ ":%1").arg(e->line()));
-      delete e;
-
-      payeeEdit->resetText();
-    }
-  } else
-    payeeEdit->resetText();
-}
-
-void KMyMoneyUtils::updateSettings(void)
-{
-  KConfig *config = KGlobal::config();
-  config->setGroup("List Options");
-
-  bool useSystemColor = config->readBoolEntry("useSystemColor", true);
-  bool useSystemFont = config->readBoolEntry("useSystemFont", true);
-
-  _backgroundColour = KGlobalSettings::baseColor();
-  if(!useSystemColor)
-    _backgroundColour = config->readColorEntry("listBGColor", &_backgroundColour);
-
-  _listColour = KGlobalSettings::alternateBackgroundColor();
-  if(!_listColour.isValid())
-    _listColour = KGlobalSettings::baseColor();
-  if(!useSystemColor)
-    _listColour = config->readColorEntry("listColor", &_listColour);
-
-  _gridColour = QColor(154,154,154); // grey is the application default
-  if(!useSystemColor)
-    _gridColour = config->readColorEntry("listGridColor", &_gridColour);
-
-  _missingConversionColour = QColor(0, 0, 255); // blue is the application default
-  if(!useSystemColor)
-    _missingConversionColour = config->readColorEntry("listMissingConversionRate", &_missingConversionColour);
-
-  _cellFont = KGlobalSettings::generalFont();
-  if(!useSystemFont)
-    _cellFont = config->readFontEntry("listCellFont", &_cellFont);
-
-  _headerFont = KGlobalSettings::generalFont();
-  _headerFont.setBold(true);
-  if(!useSystemFont)
-    _headerFont = config->readFontEntry("listHeaderFont", &_headerFont);
-
-  config->setGroup("General Options");
-  _expertMode = config->readBoolEntry("ExpertMode", false);
-}
-
 void KMyMoneyUtils::checkConstants(void)
 {
   Q_ASSERT(static_cast<int>(KLocale::ParensAround) == static_cast<int>(MyMoneyMoney::ParensAround));
@@ -617,10 +502,10 @@ QString KMyMoneyUtils::variableCSS(void)
 
   QString css;
   css += "<style type=\"text/css\">\n<!--\n";
-
-  css += QString(".row-even { font-family: verdana, arial, helvetica; background-color: %1; color: %2; padding-top: 2px; padding-left: 20px; }\n").arg(backgroundColour().name()).arg(tcolor.name());
-  css += QString(".row-odd { font-family: verdana, arial, helvetica; background-color: %1; color: %2; padding-top: 2px; padding-left: 20px; }\n").arg(listColour().name()).arg(tcolor.name());
-
+  css += QString(".row-even, .item0 { background-color: %1; color: %2 }\n")
+    .arg((KMyMoneyGlobalSettings::listBGColor()).name()).arg(tcolor.name());
+  css += QString(".row-odd, .item1  { background-color: %1; color: %2 }\n")
+    .arg((KMyMoneyGlobalSettings::listColor()).name()).arg(tcolor.name());
   css += "-->\n</style>\n";
   return css;
 }
@@ -664,7 +549,7 @@ const MyMoneySplit KMyMoneyUtils::stockSplit(const MyMoneyTransaction& t)
   for(it_s = t.splits().begin(); it_s != t.splits().end(); ++it_s) {
     if(!(*it_s).accountId().isEmpty()) {
       MyMoneyAccount acc = MyMoneyFile::instance()->account((*it_s).accountId());
-      if(acc.accountType() == MyMoneyAccount::Stock) {
+      if(acc.isInvest()) {
         return *it_s;
       }
     }
@@ -700,3 +585,160 @@ const KMyMoneyUtils::transactionTypeE KMyMoneyUtils::transactionType(const MyMon
   return Normal;
 }
 
+void KMyMoneyUtils::calculateAutoLoan(const MyMoneySchedule& schedule, MyMoneyTransaction& transaction, const QMap<QCString, MyMoneyMoney>& balances)
+{
+  try {
+    if (schedule.type() == MyMoneySchedule::TYPE_LOANPAYMENT) {
+      MyMoneySplit interestSplit, amortizationSplit;
+      QValueList<MyMoneySplit>::ConstIterator it_s;
+      for(it_s = transaction.splits().begin(); it_s != transaction.splits().end(); ++it_s) {
+        if((*it_s).value() == MyMoneyMoney::autoCalc) {
+          if((*it_s).action() == MyMoneySplit::ActionAmortization) {
+            amortizationSplit = (*it_s);
+          } else if((*it_s).action() == MyMoneySplit::ActionInterest) {
+            interestSplit = (*it_s);
+          }
+        }
+      }
+
+      if(!amortizationSplit.id().isEmpty() && !interestSplit.id().isEmpty()) {
+        MyMoneyAccountLoan acc(MyMoneyFile::instance()->account(amortizationSplit.accountId()));
+        MyMoneyFinancialCalculator calc;
+        QDate dueDate;
+
+        // FIXME: setup dueDate according to when the interest should be calculated
+        // current implementation: take the date of the next payment according to
+        // the schedule. If the calculation is based on the payment reception, and
+        // the payment is overdue then take the current date
+        dueDate = schedule.nextDueDate();
+        if(acc.interestCalculation() == MyMoneyAccountLoan::paymentReceived) {
+          if(dueDate < QDate::currentDate())
+            dueDate = QDate::currentDate();
+        }
+
+        // we need to calculate the balance at the time the payment is due
+
+        MyMoneyMoney balance;
+        if(balances.count() == 0)
+          balance = MyMoneyFile::instance()->balance(acc.id(), dueDate.addDays(-1));
+        else
+          balance = balances[acc.id()];
+
+  /*
+        QValueList<MyMoneyTransaction> list;
+        QValueList<MyMoneyTransaction>::ConstIterator it;
+        MyMoneySplit split;
+        MyMoneyTransactionFilter filter(acc.id());
+
+        filter.setDateFilter(QDate(), dueDate.addDays(-1));
+        list = MyMoneyFile::instance()->transactionList(filter);
+
+        for(it = list.begin(); it != list.end(); ++it) {
+          try {
+            split = (*it).splitByAccount(acc.id());
+            balance += split.value();
+
+          } catch(MyMoneyException *e) {
+            // account is not referenced within this transaction
+            delete e;
+          }
+        }
+  */
+
+        // FIXME: for now, we only support interest calculation at the end of the period
+        calc.setBep();
+        // FIXME: for now, we only support periodic compounding
+        calc.setDisc();
+
+        calc.setPF(occurenceToFrequency(schedule.occurence()));
+        MyMoneySchedule::occurenceE compoundingOccurence = static_cast<MyMoneySchedule::occurenceE>(acc.interestCompounding());
+        if(compoundingOccurence == MyMoneySchedule::OCCUR_ANY)
+          compoundingOccurence = schedule.occurence();
+
+        calc.setCF(occurenceToFrequency(compoundingOccurence));
+
+        calc.setPv(balance.toDouble());
+        calc.setIr(static_cast<FCALC_DOUBLE> (acc.interestRate(dueDate).abs().toDouble()));
+        calc.setPmt(acc.periodicPayment().toDouble());
+
+        MyMoneyMoney interest(calc.interestDue()), amortization;
+        interest = interest.abs();    // make sure it's positive for now
+        amortization = acc.periodicPayment() - interest;
+
+        if(acc.accountType() == MyMoneyAccount::AssetLoan) {
+          interest = -interest;
+          amortization = -amortization;
+        }
+        amortizationSplit.setValue(amortization);
+        interestSplit.setValue(interest);
+
+        // FIXME: for now we only assume loans to be in the base currency
+        amortizationSplit.setShares(amortization);
+        interestSplit.setShares(interest);
+
+
+        transaction.modifySplit(amortizationSplit);
+        transaction.modifySplit(interestSplit);
+      }
+    }
+
+  } catch (MyMoneyException* e) {
+    KMessageBox::detailedError(0, i18n("Unable to load schedule details"), e->what());
+    delete e;
+  }
+}
+
+QString KMyMoneyUtils::nextCheckNumber(const MyMoneyAccount& acc)
+{
+  // determine next check number
+  QString number;
+  QRegExp exp(QString("(.*\\D)?(\\d+)(\\D.*)?"));
+  if(exp.search(acc.value("lastNumberUsed")) != -1) {
+    number = QString("%1%2%3").arg(exp.cap(1)).arg(exp.cap(2).toULongLong() + 1).arg(exp.cap(3));
+  } else {
+    number = "1";
+  }
+  return number;
+}
+
+QString KMyMoneyUtils::reconcileStateToString(MyMoneySplit::reconcileFlagE flag, bool text)
+{
+  QString txt;
+  if(text) {
+    switch(flag) {
+      case MyMoneySplit::NotReconciled:
+        txt = i18n("Reconcile state 'Not reconciled'", "Not reconciled");
+        break;
+      case MyMoneySplit::Cleared:
+        txt = i18n("Reconcile state 'Cleared'", "Cleared");
+        break;
+      case MyMoneySplit::Reconciled:
+        txt = i18n("Reconcile state 'Reconciled'", "Reconciled");
+        break;
+      case MyMoneySplit::Frozen:
+        txt = i18n("Reconcile state 'Frozen'", "Frozen");
+        break;
+      default:
+        txt = i18n("Unknown");
+        break;
+    }
+  } else {
+    switch(flag) {
+      case MyMoneySplit::NotReconciled:
+        break;
+      case MyMoneySplit::Cleared:
+        txt = i18n("Reconcile flag C", "C");
+        break;
+      case MyMoneySplit::Reconciled:
+        txt = i18n("Reconcile flag R", "R");
+        break;
+      case MyMoneySplit::Frozen:
+        txt = i18n("Reconcile flag F", "F");
+        break;
+      default:
+        txt = i18n("Flag for unknown reconciliation state", "?");
+        break;
+    }
+  }
+  return txt;
+}
