@@ -22,12 +22,15 @@
 
 // ----------------------------------------------------------------------------
 // QT Includes
+
 #include <qwidgetlist.h>
 #include <qdatetime.h>
 #include <qstringlist.h>
+#include <qeventloop.h>
 
 // ----------------------------------------------------------------------------
 // KDE Includes
+
 #include <kcmdlineargs.h>
 #include <kaboutdata.h>
 #include <klocale.h>
@@ -37,11 +40,12 @@
 
 // ----------------------------------------------------------------------------
 // Project Includes
+
 #include "mymoney/mymoneyfile.h"
-#include "views/kbanklistitem.h"
 #include "kmymoney2.h"
 #include "kstartuplogo.h"
 #include "kmymoneyutils.h"
+#include "kmymoneyglobalsettings.h"
 
 static const char *description =
   I18N_NOOP("\nKMyMoney, the Personal Finance Manager for KDE.\n\nPlease consider contributing to this project with code and/or suggestions.");
@@ -49,8 +53,14 @@ static const char *description =
 static KCmdLineOptions options[] =
 {
   { "lang <lang-code>", I18N_NOOP("language to be used"), 0 },
-  { "trace", I18N_NOOP("turn on program traces"), 0},
   { "n", I18N_NOOP("don't open last used file"), 0},
+  { "timers", I18N_NOOP("enable performance timers"), 0},
+
+#if KMM_DEBUG
+  // The following options are only available when compiled in debug mode
+  { "trace", I18N_NOOP("turn on program traces"), 0},
+  { "dump-actions", I18N_NOOP("dump the names of all defined KAction objects to stdout and quit"), 0},
+#endif
 
   // INSERT YOUR COMMANDLINE OPTIONS HERE
   { "+[File]", I18N_NOOP("file to open"), 0 },
@@ -58,9 +68,12 @@ static KCmdLineOptions options[] =
 };
 
 QTime timer;
+bool timersOn = false;
 
 KMyMoney2App* kmymoney2;
 
+static KCmdLineArgs* args = 0;
+
 static void _cleanup(void)
 {
 #ifdef _CHECK_MEMORY
@@ -69,9 +82,10 @@ static void _cleanup(void)
 #endif
 }
 
-
 int main(int argc, char *argv[])
 {
+  timer.start();
+
   QString feature;
 
 #ifdef _CHECK_MEMORY
@@ -98,6 +112,7 @@ int main(int argc, char *argv[])
   aboutData.addCredit("Tony Bloomfield", I18N_NOOP("GnuCash Importer"), "tonybloom@users.sourceforge.net");
   aboutData.addCredit("Robert Wadley", I18N_NOOP("Icons & splash screen"), "rob@robntina.fastmail.us");
   aboutData.addCredit("Laurent Montel", I18N_NOOP("Patches"), "montel@kde.org");
+  aboutData.addCredit("Alvaro Soliverez", I18N_NOOP("Forecast view"), "asoliverez@gmail.com");
 
   KCmdLineArgs::init( argc, argv, &aboutData );
   KCmdLineArgs::addCmdLineOptions( options ); // Add our own options.
@@ -113,13 +128,10 @@ int main(int argc, char *argv[])
 
   KMyMoneyUtils::checkConstants();
 
-  timer.start();
-
   KApplication* a = new KApplication();
 
-  if(KGlobal::locale()->monetaryThousandsSeparator().isEmpty()
-  || KGlobal::locale()->monetaryDecimalSymbol().isEmpty()) {
-    KMessageBox::error(0, i18n("Either the monetary decimal symbol or the monetary thousands separator are not correctly set in the KDE Control Center's Country/Region & Language settings. Please set to reasonable values and start KMyMoney again."), i18n("Invalid settings"));
+  if(KGlobal::locale()->monetaryDecimalSymbol().isEmpty()) {
+    KMessageBox::error(0, i18n("The monetary decimal symbol is not correctly set in the KDE Control Center's Country/Region & Language settings. Please set it to a reasonable value and start KMyMoney again."), i18n("Invalid settings"));
     delete a;
     exit(1);
   }
@@ -128,7 +140,7 @@ int main(int argc, char *argv[])
   KStartupLogo* splash = new KStartupLogo();
   a->processEvents();
 
-  KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
+  args = KCmdLineArgs::parsedArgs();
 
   // setup the MyMoneyMoney locale settings according to the KDE settings
   MyMoneyMoney::setThousandSeparator(*(KGlobal::locale()->monetaryThousandsSeparator().latin1()));
@@ -145,12 +157,32 @@ int main(int argc, char *argv[])
     }
   }
 
+#if KMM_DEBUG
   if(args->isSet("trace"))
     MyMoneyTracer::on();
+  timersOn = args->isSet("timers");
+#endif
 
+  kmymoney2 = 0;
   kmymoney2 = new KMyMoney2App();
   a->setMainWidget( kmymoney2 );
 
+#if KMM_DEBUG
+  if(args->isSet("dump-actions")) {
+    kmymoney2->dumpActions();
+
+    // Before we delete the application, we make sure that we destroy all
+    // widgets by running the event loop for some time to catch all those
+    // widgets that are requested to be destroyed using the deleteLater() method.
+    QApplication::eventLoop()->processEvents(QEventLoop::ExcludeUserInput, 10);
+
+    delete kmymoney2;
+    delete splash;
+    delete a;
+    exit(0);
+  }
+#endif
+
   int rc = 0;
 
   do {
@@ -180,6 +212,11 @@ int main(int argc, char *argv[])
             if (!client->send(primary, "kmymoney2app", "webConnect(QString,QCString)",data))
               qDebug("Unable to launch WebConnect via DCOP.");
 
+            // Before we delete the application, we make sure that we destroy all
+            // widgets by running the event loop for some time to catch all those
+            // widgets that are requested to be destroyed using the deleteLater() method.
+            QApplication::eventLoop()->processEvents(QEventLoop::ExcludeUserInput, 10);
+
             delete kmymoney2;
             delete splash;
             break;
@@ -231,7 +268,10 @@ int main(int argc, char *argv[])
     KTipDialog::showTip(kmymoney2, "", false);
     if(url.isValid() && !args->isSet("n")) {
       kmymoney2->slotFileOpenRecent(url);
+    } else if(KMyMoneyGlobalSettings::firstTimeRun()) {
+      kmymoney2->slotFileNew();
     }
+    KMyMoneyGlobalSettings::setFirstTimeRun(false);
 
     if ( ! importfile.isEmpty() )
       kmymoney2->webConnect( importfile, kapp->startupId() );
@@ -249,10 +289,20 @@ int main(int argc, char *argv[])
   return rc;
 }
 
-
-void timetrace(char *txt)
+void timestamp(char *txt)
 {
-  qDebug("Timer(%s): %d elapsed", txt, timer.elapsed());
-  timer.restart();
+  if(timersOn)
+  {
+    qDebug("Time(%s): %d", txt, timer.elapsed());
+  }
 }
 
+void timetrace(const char *txt)
+{
+  if(timersOn)
+  {
+    qDebug("Timer(%s): %d elapsed", txt, timer.elapsed());
+    timer.restart();
+  }
+}
+// vim:cin:si:ai:et:ts=2:sw=2:
