@@ -29,19 +29,21 @@
 #include <qdatetime.h>
 #include <qvaluelist.h>
 #include <qstringlist.h>
+#include <qdom.h>
 
 // ----------------------------------------------------------------------------
 // Project Includes
 
-#include "mymoneymoney.h"
-#include "mymoneyutils.h"
-#include "mymoneykeyvaluecontainer.h"
+#include <kmymoney/mymoneymoney.h>
+#include <kmymoney/mymoneyobject.h>
+#include <kmymoney/mymoneykeyvaluecontainer.h>
 #include <kmymoney/mymoneysecurity.h>
 #include <kmymoney/export.h>
+#include "mymoneyutils.h"
 class MyMoneyTransaction;
 class MyMoneyInstitution;
-class MyMoneyFile;
-
+class MyMoneySplit;
+class MyMoneyObjectContainer;
 
 /**
   * A representation of an account.
@@ -75,28 +77,11 @@ class MyMoneyFile;
   * @author Thomas Baumgart 2002
   *
 **/
-class KMYMONEY_EXPORT MyMoneyAccount : public MyMoneyKeyValueContainer
+class KMYMONEY_EXPORT MyMoneyAccount : public MyMoneyObject, public MyMoneyKeyValueContainer
 {
+  friend class MyMoneyObjectContainer;
 public:
 
-/*
-  class Transaction
-  {
-  public:
-    Transaction() {};
-    Transaction(const QCString& transaction, const MyMoneyMoney& balance) {
-      m_transaction = transaction;
-      m_balance = balance;
-    };
-    const QCString& transactionID(void) const { return m_transaction; };
-    const MyMoneyMoney& balance(void) const { return m_balance; };
-
-  private:
-    QCString m_transaction;
-    MyMoneyMoney m_balance;
-  };
-*/
-
   /**
     * Account types currently supported.
     */
@@ -139,6 +124,15 @@ public:
     */
   MyMoneyAccount(const QCString& id, const MyMoneyAccount& right);
 
+  /**
+    * This is the constructor for an account that is described by a
+    * QDomElement (e.g. from a file).
+    *
+    * @param el const reference to the QDomElement from which to
+    *           create the object
+    */
+  MyMoneyAccount(const QDomElement& el);
+
   /**
     * This is the destructor for any MyMoneyAccount object
     */
@@ -164,6 +158,8 @@ public:
     *   - Investment
     *   - MoneyMarket
     *   - CertificateDep
+    *   - AssetLoan
+    *   - Stock
     *
     * - Liability
     *   - Liability
@@ -183,15 +179,6 @@ public:
 
   const MyMoneyAccount::accountTypeE accountGroup(void) const;
 
-  /**
-    * This method returns the ID of the account under which it is known
-    * inside the MyMoneyFile.
-    *
-    * @return ID as QString. If the ID is unknown, an empty QString is returned.
-    * @see setID()
-    */
-  const QCString id(void) const { return m_id; }
-
   /**
     * This method returns the id of the MyMoneyInstitution object this account
     * belongs to.
@@ -199,125 +186,62 @@ public:
     *         an internal account
     * @see setInstitution
     */
-  const QCString institutionId(void) const { return m_institution; }
+  const QCString& institutionId(void) const { return m_institution; }
 
   /**
     * This method returns the name of the account
     * @return name of account
     * @see setName()
     */
-  const QString name(void) const { return m_name; }
+  const QString& name(void) const { return m_name; }
 
   /**
     * This method returns the number of the account at the institution
     * @return number of account at the institution
     * @see setNumber
     */
-  const QString number(void) const { return m_number; }
+  const QString& number(void) const { return m_number; }
 
   /**
     * This method returns the descriptive text of the account.
     * @return description of account
     * @see setDescription
     */
-  const QString description(void) const { return m_description; }
+  const QString& description(void) const { return m_description; }
 
   /**
     * This method returns the opening date of this account
     * @return date of opening of this account as const QDate value
     * @see setOpeningDate()
     */
-  const QDate openingDate(void) const { return m_openingDate; }
-
-  /**
-    * This method is used to return the opening balance of an account
-    *
-    * @return MyMoneyMoney value of opening balance
-    */
-  const MyMoneyMoney openingBalance(void) const { return m_openingBalance; };
+  const QDate& openingDate(void) const { return m_openingDate; }
 
   /**
     * This method returns the date of the last reconciliation of this account
     * @return date of last reconciliation as const QDate value
     * @see setLastReconciliationDate
     */
-  const QDate lastReconciliationDate(void) const { return m_lastReconciliationDate; }
+  const QDate& lastReconciliationDate(void) const { return m_lastReconciliationDate; }
 
   /**
     * This method returns the date the account was last modified
     * @return date of last modification as const QDate value
     * @see setLastModified
     */
-  const QDate lastModified(void) const { return m_lastModified; }
-
-  /**
-    * This method is used to get the number of transactions
-    * in this account.
-    *
-    * @return number of transactions in the account
-    */
-/*
-  const int transactionCount(void) const { return m_transactionList.count(); }
-*/
-
-  /**
-    * This method returns the current total balance of this account's
-    * transactions. Sub-Accounts are not included.
-    * @return const reference to MyMoneyMoney balance of split values
-    * @see balanceTotal()
-    */
-/*
-  const MyMoneyMoney balance(void) const;
-*/
+  const QDate& lastModified(void) const { return m_lastModified; }
 
   /**
     * This method is used to return the ID of the parent account
     * @return QCString with the ID of the parent of this account
     */
-  const QCString parentAccountId(void) const { return m_parentAccount; };
-
-  /**
-    * This method is used to return the transaction id and balance
-    * at the given index. If the index is invalid or larger than
-    * the count of transactions in the list, an exception will be
-    * thrown.
-    *
-    * @param idx index of transaction in list
-    * @return id and balance of transaction
-    */
-/*
-  const MyMoneyAccount::Transaction& transaction(const int idx) const;
-*/
-  /**
-    * This method returns the list of transactions and balances
-    * @return const QValueList<MyMoneyAccount::Transaction>& transactionList
-    */
-/*
-  const QValueList<MyMoneyAccount::Transaction>& transactionList(void) const;
-*/
-
-  /**
-    * This method is used by the mymoney engine to add a transaction
-    * of the journal to this account
-    * @param transaction reference to MyMoneyAccount::Transaction
-    */
-/*
-  void addTransaction(const MyMoneyAccount::Transaction& val);
-*/
-
-  /**
-    * This method is used to clear the list of transactions
-    */
-/*
-  void clearTransactions(void);
-*/
+  const QCString& parentAccountId(void) const { return m_parentAccount; };
 
   /**
     * This method returns the list of the account id's of
     * subordinate accounts
     * @return QStringList account ids
     */
-  const QCStringList accountList(void) const { return m_accountList; };
+  const QCStringList& accountList(void) const { return m_accountList; };
 
   /**
     * This method returns the number of entries in the m_accountList
@@ -338,6 +262,12 @@ public:
     */
   void removeAccountId(const QCString& account);
 
+  /**
+    * This method is used to remove all accounts from the list of
+    * sub-ordinate accounts.
+    */
+  void removeAccountIds(void);
+
   /**
     * This method is used to modify the date of the last
     * modification access.
@@ -386,16 +316,6 @@ public:
     */
   void setOpeningDate(const QDate& date);
 
-  /**
-    * This method is used to set the opening balance information of an
-    * account.
-    *
-    * @param balance MyMoneyMoney value of opening balance
-    *
-    * @see openingBalance
-    */
-  void setOpeningBalance(const MyMoneyMoney& balance);
-
   /**
     * This method is used to set the date of the last reconciliation
     * of an account.
@@ -411,27 +331,12 @@ public:
     */
   void setAccountType(const accountTypeE type);
 
-  /**
-    * This method returns the account type based on the string that is read from
-    * the XML file.
-    *
-    * @param QCString reference to type from XML file.
-    */
-  // void setAccountTypeByName(const QCString& strType);
-
   /**
     * This method is used to set a new parent account id
     * @param parent QString reference to new parent account
     */
   void setParentAccountId(const QCString& parent);
 
-  /**
-    * This method is used to set a new account id
-    * @param id QString reference to new id.
-    */
-  void setAccountId(const QCString& id);
-
-
   /**
     * This method is used to update m_lastModified to the current date
     */
@@ -440,7 +345,7 @@ public:
   /**
     * This method returns the type of the account.
     */
-  const accountTypeE accountType(void) const {return m_accountType; };
+  accountTypeE accountType(void) const { return m_accountType; }
 
   /**
     * This method retrieves the id of the currency used with this account.
@@ -448,7 +353,7 @@ public:
     *
     * @return id of currency
     */
-  const QCString currencyId(void) const { return m_currencyId; };
+  const QCString& currencyId(void) const { return m_currencyId; };
 
   /**
     * This method sets the id of the currency used with this account.
@@ -457,17 +362,160 @@ public:
     */
   void setCurrencyId(const QCString& id);
 
+  void writeXML(QDomDocument& document, QDomElement& parent) const;
+
+  /**
+    * This method checks if a reference to the given object exists. It returns,
+    * a @p true if the object is referencing the one requested by the
+    * parameter @p id. If it does not, this method returns @p false.
+    *
+    * @param id id of the object to be checked for references
+    * @retval true This object references object with id @p id.
+    * @retval false This object does not reference the object with id @p id.
+    */
+  virtual bool hasReferenceTo(const QCString& id) const;
+
+  /**
+    * This member returns the balance of this account based on
+    * all transactions stored in the journal.
+    */
+  const MyMoneyMoney& balance(void) const { return m_balance; }
+
+  /**
+    * This method adjusts the balance of this account
+    * according to the difference contained in the split @p s.
+    * If the s.action() is MyMoneySplit::ActionSplitShares then
+    * the balance will be adjusted accordingly.
+    *
+    * @param s const reference to MyMoneySplit object containing the
+    *             value to be added/subtracted to/from the balance
+    * @param reverse add (false) or subtract (true) the shares contained in the split.
+    *          It also affects the balance for share splits in the opposite direction.
+    */
+  void adjustBalance(const MyMoneySplit& s, bool reverse = false);
+
+  /**
+    * This method sets the balance of this account
+    * according to the value provided by @p val.
+    *
+    * @param val const reference to MyMoneyMoney object containing the
+    *             value to be assigned to the balance
+    */
+  void setBalance(const MyMoneyMoney& val) { m_balance = val; }
+
+  /**
+    * This method sets the kvp's for online banking with this account
+    *
+    * @param values The container of kvp's needed when connecting to this account
+    */
+  void setOnlineBankingSettings(const MyMoneyKeyValueContainer& values);
+
+  /**
+    * This method retrieves the kvp's for online banking with this account
+    *
+    * @return The container of kvp's needed when connecting to this account
+    */
+  const MyMoneyKeyValueContainer& onlineBankingSettings(void) const;
+
+  /**
+    * This method sets the closed flag for the account. This is just
+    * an informational flag for the application. It has no other influence
+    * on the behaviour of the account object. The default for
+    * new objects @p open.
+    *
+    * @param isClosed mark the account closed (@p true) or open (@p false).
+    */
+  void setClosed(bool isClosed);
+
+  /**
+    * Return the closed flag for the account.
+    *
+    * @retval false account is marked open (the default for new accounts)
+    * @retval true account is marked closed
+    */
+  bool isClosed(void) const;
+
   /**
     * returns the applicable smallest fraction for this account
-    * for the given security based on the account type.
+    * for the given security based on the account type. At the same
+    * time, m_fraction is updated to the value returned.
     *
     * @param sec const reference to currency (security)
     *
     * @retval sec.smallestCashFraction() for account type Cash
     * @retval sec.smallestAccountFraction() for all other account types
     */
+  int fraction(const MyMoneySecurity& sec);
+
+  /**
+   * Same as the above method, but does not modify m_fraction.
+   */
   int fraction(const MyMoneySecurity& sec) const;
 
+  /**
+    * This method returns the stored value for the fraction of this
+    * account or -1 if not initialized. It can be initialized by
+    * calling fraction(const MyMoneySecurity& sec) once.
+    *
+    * @note Don't use this method outside of KMyMoney application context (eg. testcases).
+    * Use the above method instead.
+    */
+  int fraction(void) const;
+
+  /**
+    * This method returns @a true if the account type is
+    * either Income or Expense
+    *
+    * @retval true account is of type income or expense
+    * @retval false for all other account types
+    *
+    * @deprecated use isIncomeExpense() instead
+    */
+  bool isCategory(void) const __attribute__ ((deprecated));
+
+  /**
+    * This method returns @a true if the account type is
+    * either Income or Expense
+    *
+    * @retval true account is of type income or expense
+    * @retval false for all other account types
+    */
+  bool isIncomeExpense(void) const;
+
+  /**
+    * This method returns @a true if the account type is
+    * either Asset or Liability
+    *
+    * @retval true account is of type asset or liability
+    * @retval false for all other account types
+    */
+  bool isAssetLiability(void) const;
+
+  /**
+    * This method returns @a true if the account type is
+    * either AssetLoan or Loan
+    *
+    * @retval true account is of type Loan or AssetLoan
+    * @retval false for all other account types
+    */
+  bool isLoan(void) const;
+
+  /**
+    * This method returns @a true if the account type is
+    * Stock
+    *
+    * @retval true account is of type Stock
+    * @retval false for all other account types
+    */
+  bool isInvest(void) const;
+
+  /**
+   * This method returns a name that has a brokerage suffix of
+   * the current name. It only works on investment accounts and
+   * returns the name for all other cases.
+   */
+  QString brokerageName(void) const;
+
 private:
   /**
     * This member variable identifies the type of account
@@ -480,12 +528,6 @@ private:
     */
   QCString m_institution;
 
-  /**
-    * This member variable keeps the ID of the institution under which it
-    * is known inside the MyMoneyFile.
-    */
-  QCString  m_id;
-
   /**
     * This member variable keeps the name of the account
     * It is solely for documentation purposes and it's contents is not
@@ -519,27 +561,12 @@ private:
     */
   QDate m_openingDate;
 
-  /**
-    * This member holds the opening balance of this account
-    */
-  MyMoneyMoney m_openingBalance;
-
   /**
     * This member variable keeps the date of the last
     * reconciliation of this account
     */
   QDate m_lastReconciliationDate;
 
-  /**
-    * This member variable keeps a sorted list of transactions keys
-    * of those transactions that have a split that references
-    * this account and the balance of the account after processing of
-    * the corresponding split.
-    */
-/*
-  QValueList<MyMoneyAccount::Transaction> m_transactionList;
-*/
-
   /**
     * This member holds the ID's of all sub-ordinate accounts
     */
@@ -554,6 +581,25 @@ private:
     * This member contains the ID of the currency associated with this account
     */
   QCString m_currencyId;
+
+  /**
+    * This member holds the balance of all transactions stored in the journal
+    * for this account.
+    */
+  MyMoneyMoney    m_balance;
+
+  /**
+    * This member variable keeps the set of kvp's needed to establish
+    * online banking sessions to this account.
+    */
+  MyMoneyKeyValueContainer m_onlineBankingSettings;
+
+  /**
+    * This member keeps the fraction for the account. It is filled by MyMoneyFile
+    * when set to -1. See also @sa fraction(const MyMoneySecurity&).
+    */
+  int             m_fraction;
+
 };
 
 /**
@@ -571,9 +617,16 @@ public:
     paymentReceived
   };
 
-  MyMoneyAccountLoan() {};
+  enum interestChangeUnitE {
+    changeDaily = 0,
+    changeWeekly,
+    changeMonthly,
+    changeYearly
+  };
+
+  MyMoneyAccountLoan() {}
   MyMoneyAccountLoan(const MyMoneyAccount&);
-  ~MyMoneyAccountLoan() {};
+  ~MyMoneyAccountLoan() {}
 
   const MyMoneyMoney loanAmount(void) const;
   void setLoanAmount(const MyMoneyMoney& amount);
@@ -595,10 +648,24 @@ public:
   void setInterestChangeFrequency(const int amount, const int unit);
   const MyMoneyMoney periodicPayment(void) const;
   void setPeriodicPayment(const MyMoneyMoney& payment);
+  int interestCompounding(void) const;
+  void setInterestCompounding(int frequency);
   const QCString payee(void) const;
   void setPayee(const QCString& payee);
   const QCString interestAccountId(void) const;
   void setInterestAccountId(const QCString& id);
+
+  /**
+    * This method checks if a reference to the given object exists. It returns,
+    * a @p true if the object is referencing the one requested by the
+    * parameter @p id. If it does not, this method returns @p false.
+    *
+    * @param id id of the object to be checked for references
+    * @retval true This object references object with id @p id.
+    * @retval false This object does not reference the object with id @p id.
+    */
+  virtual bool hasReferenceTo(const QCString& id) const;
+
 };
 
 #endif
