@@ -1,7 +1,7 @@
 /***************************************************************************
                           mymoneyfile.h
                              -------------------
-    copyright            : (C) 2002 by Thomas Baumgart
+    copyright            : (C) 2002, 2007 by Thomas Baumgart
     email                : ipwizard@users.sourceforge.net
  ***************************************************************************/
 
@@ -24,6 +24,7 @@
 // ----------------------------------------------------------------------------
 // QT Includes
 
+#include <qobject.h>
 #include <qstring.h>
 #include <qmap.h>
 #include <qvaluelist.h>
@@ -38,12 +39,11 @@
 #include <kmymoney/mymoneyaccount.h>
 #include <kmymoney/mymoneytransaction.h>
 #include <kmymoney/mymoneypayee.h>
-#include <kmymoney/mymoneyobserver.h>
-#include <kmymoney/mymoneysubject.h>
 #include <kmymoney/mymoneykeyvaluecontainer.h>
 #include <kmymoney/mymoneysecurity.h>
 #include <kmymoney/mymoneyprice.h>
 #include <kmymoney/mymoneyreport.h>
+#include <kmymoney/mymoneybudget.h>
 #include <kmymoney/mymoneyscheduled.h>
 #include <kmymoney/export.h>
 
@@ -53,6 +53,7 @@
 
 class IMyMoneyStorage;
 class MyMoneyTransactionFilter;
+class MyMoneyFilePrivate;
 
 /**
   * This class represents the interface to the MyMoney engine.
@@ -121,9 +122,8 @@ class MyMoneyTransactionFilter;
   * given accountId references one of the or not. setAccountName() is used
   * to specify a name for the standard accounts from the GUI.
   *
-  * The methods attach() and detach() provide the necessary functions
-  * for an external observer to be attached and detached to and from
-  * an object of the engine.
+  * The MyMoneyFile object emits the dataChanged() signal when data
+  * has been changed.
   *
   * For abritrary values that have to be stored with the storage object
   * but are of importance to the application only, the object is derived
@@ -134,18 +134,11 @@ class MyMoneyTransactionFilter;
   * while the engine code is running. The MyMoneyException:: object
   * describes the problem.
   */
-class KMYMONEY_EXPORT MyMoneyFile
+class KMYMONEY_EXPORT MyMoneyFile : public QObject
 {
+  Q_OBJECT
 public:
 
-  class MyMoneyFileSubject : public MyMoneySubject
-  {
-  public:
-    MyMoneyFileSubject() {};
-    ~MyMoneyFileSubject() {};
-  };
-
-
   class MyMoneyNotifier
   {
   public:
@@ -160,9 +153,8 @@ public:
   /**
     * This is the function to access the MyMoneyFile object.
     * It returns a pointer to the single instance of the object.
-    * If no instance exists, it will be created.
     */
-  static MyMoneyFile* const instance();
+  static inline MyMoneyFile* const instance() { return &file; }
 
   /**
     * This is the destructor for any MyMoneyFile object
@@ -179,54 +171,41 @@ public:
   MyMoneyFile(IMyMoneyStorage *storage);
 
   // general get functions
-  const QString userName(void) const;
-  const QString userStreet(void) const;
-  const QString userTown(void) const;
-  const QString userCounty(void) const;
-  const QString userPostcode(void) const;
-  const QString userTelephone(void) const;
-  const QString userEmail(void) const;
+  const MyMoneyPayee& user(void) const;
 
   // general set functions
-  void setUserName(const QString& val);
-  void setUserStreet(const QString& val);
-  void setUserTown(const QString& val);
-  void setUserCounty(const QString& val);
-  void setUserPostcode(const QString& val);
-  void setUserTelephone(const QString& val);
-  void setUserEmail(const QString& val);
+  void setUser(const MyMoneyPayee& user);
 
   /**
-    * This method can be used to turn on/off the notifications
-    * of the engine. During bulk-updates it's sometimes useful
-    * to turn off notifications for an improved performance.
-    *
-    * Engine notifications that are generated while the notifications
-    * are suspended will be postponed until the notifications are
-    * re-enabled.
-    *
-    * @param state if @p true, no notifications will be send out,
-    *              if @p false, notifications will be send out.
-    */
-  void suspendNotify(const bool state);
-
-  /**
-    * This method is used to attach a storage to the MyMoneyFile object
+    * This method is used to attach a storage object to the MyMoneyFile object
     * Without an attached storage object, the MyMoneyFile object is
-    * of no use. In case of an error condition, an exception is thrown.
+    * of no use.
+    *
+    * After successful completion, the dataChanged() signal is emitted.
+    *
+    * In case of an error condition, an exception is thrown.
+    * The following error conditions are checked:
+    *
+    * - @a storage is not equal to 0
+    * - there is no other @a storage object attached (use detachStorage()
+    *   to revert the attachStorage() operation.
     *
     * @param storage pointer to object that implements the IMyMoneyStorage
     *                interface.
+    *
+    * @sa detachStorage()
     */
   void attachStorage(IMyMoneyStorage* const storage);
 
   /**
     * This method is used to detach a previously attached storage
-    * object from the MyMoneyFile object. In case of an error
-    * condition an exception will be thrown.
+    * object from the MyMoneyFile object. If no storage object
+    * is attached to the engine, this is a NOP.
     *
     * @param storage pointer to object that implements the IMyMoneyStorage
     *                interface.
+    *
+    * @sa attachStorage()
     */
   void detachStorage(IMyMoneyStorage* const storage = 0);
 
@@ -246,35 +225,61 @@ public:
     */
   IMyMoneyStorage* const storage(void) const { return m_storage; };
 
+  /**
+    * This method must be called before any single change or a series of changes
+    * in the underlying storage area is performed.
+    * Once all changes are complete (i.e. the transaction is completed),
+    * commitTransaction() must be called to finalize all changes. If an error occurs
+    * during the processing of the changes call rollbackTransaction() to undo the
+    * changes done so far.
+    */
+  void startTransaction(void);
+
+  /**
+    * This method returns whether a transaction has been started (@a true)
+    * or not (@a false).
+    */
+  bool hasTransaction(void) const;
+
+  /**
+    * @sa startTransaction()
+    */
+  void commitTransaction(void);
+
+  /**
+    * @sa startTransaction()
+    */
+  void rollbackTransaction(void);
+
   /**
     * This method is used to return the standard liability account
     * @return MyMoneyAccount liability account(group)
     */
-  const MyMoneyAccount liability(void) const;
+  const MyMoneyAccount& liability(void) const;
 
   /**
     * This method is used to return the standard asset account
     * @return MyMoneyAccount asset account(group)
     */
-  const MyMoneyAccount asset(void) const;
+  const MyMoneyAccount& asset(void) const;
 
   /**
     * This method is used to return the standard expense account
     * @return MyMoneyAccount expense account(group)
     */
-  const MyMoneyAccount expense(void) const;
+  const MyMoneyAccount& expense(void) const;
 
   /**
     * This method is used to return the standard income account
     * @return MyMoneyAccount income account(group)
     */
-  const MyMoneyAccount income(void) const;
+  const MyMoneyAccount& income(void) const;
 
   /**
     * This method is used to return the standard equity account
     * @return MyMoneyAccount equity account(group)
     */
-  const MyMoneyAccount equity(void) const;
+  const MyMoneyAccount& equity(void) const;
 
   /**
     * This method returns the account information for the opening
@@ -379,9 +384,9 @@ public:
   void removeInstitution(const MyMoneyInstitution& institution);
 
   /**
-    * Adds an account to the file-global institution pool. A respective
+    * Adds an account to the file-global account pool. A respective
     * account-ID will be generated within this record. The modified
-    * members of account will be updated.
+    * members of @a account will be updated.
     *
     * A few parameters of the account to be added are checked against
     * the following conditions. If they do not match, an exception is
@@ -394,7 +399,7 @@ public:
     *   c) the transaction list must be empty
     *   d) the account must not have any sub-ordinate accounts
     *   e) the account must have no parent account
-    *   f) the account must not have reference to a MyMoneyFile object
+    *   f) the account must not have any reference to a MyMoneyFile object
     *
     * An exception will be thrown upon error conditions.
     *
@@ -422,14 +427,6 @@ public:
     */
   void reparentAccount(MyMoneyAccount &account, MyMoneyAccount& parent);
 
-  /**
-    * This converts the account type into one of the four
-    * major account types liability, asset, expense or income
-    * @param type actual account type
-    * @return accountTypeE of major account type
-    */
-  const MyMoneyAccount::accountTypeE accountGroup(MyMoneyAccount::accountTypeE type) const;
-
   /**
     * moves splits from one account to another
     *
@@ -460,6 +457,13 @@ public:
     */
   const bool isStandardAccount(const QCString& id) const;
 
+  /**
+    * Returns @a true, if transaction @t is a transfer transaction.
+    * A transfer transaction has two splits, both referencing either
+    * an asset, a liability or an equity account.
+    */
+  bool isTransfer(const MyMoneyTransaction& t) const;
+
   /**
     * This method is used to set the name for the specified standard account
     * within the storage area. An exception will be thrown, if an error
@@ -482,6 +486,39 @@ public:
     */
   void removeAccount(const MyMoneyAccount& account);
 
+  /**
+    * Deletes existing accounts and their subaccounts recursivly
+    * from the global account pool.
+    * This method expects that all accounts and their subaccounts
+    * are no longer assigned to any transactions or splits.
+    * An exception is thrown in case of a problem deleting an account.
+    *
+    * The optional parameter level is used to keep track of the recursion level.
+    * If the recursion level exceeds 100 (some arbitrary number which seems a good
+    * maximum), an exception is thrown.
+    *
+    * @param account_list Reference to a list of account IDs to be deleted.
+    * @param level Parameter to keep track of recursion level (do not pass a value here).
+    */
+  void removeAccountList(const QCStringList& account_list, unsigned int level = 0);
+
+  /**
+    * This member function checks all accounts identified by account_list
+    * and their subaccounts wether they are assigned to transactions/splits or not.
+    * The function calls itself recursively with the list of sub-accounts of
+    * the currently processed account.
+    *
+    * The optional parameter level is used to keep track of the recursion level.
+    * If the recursion level exceeds 100 (some arbitrary number which seems a good
+    * maximum), an exception is thrown.
+    *
+    * @param account_list  A QCStringList with account IDs that need to be checked.
+    * @param level         (optional) Optional parameter to indicate recursion level.
+    * @return              Returns 'false' if at least one account has been found that
+    *                      is still referenced by a transaction.
+    */
+  bool hasOnlyUnusedAccounts(const QCStringList& account_list, unsigned int level = 0);
+
   /**
     * Adds a transaction to the file-global transaction pool. A respective
     * transaction-ID will be generated for this object. The ID is stored
@@ -540,7 +577,11 @@ public:
     *
     * @return set of transactions in form of a QValueList<MyMoneyTransaction>
     */
-  const QValueList<MyMoneyTransaction> transactionList(MyMoneyTransactionFilter& filter) const;
+  QValueList<MyMoneyTransaction> transactionList(MyMoneyTransactionFilter& filter) const;
+
+  void transactionList(QValueList<MyMoneyTransaction>& list, MyMoneyTransactionFilter& filter) const;
+
+  void transactionList(QValueList<QPair<MyMoneyTransaction, MyMoneySplit> >& list, MyMoneyTransactionFilter& filter) const;
 
   /**
     * This method is used to remove a transaction from the transaction
@@ -576,76 +617,6 @@ public:
     */
   const MyMoneyMoney totalBalance(const QCString& id, const QDate& date = QDate()) const;
 
-  /**
-    * This method is used to return the actual value of an account
-    * including it's sub-ordinate accounts. A value of an account
-    * is calculated by multiplying the balance with the currencies
-    * price for today. The price for accounts held in the baseCurrency()
-    * is 1. The same applies for prices that are not available (no entry
-    * in price list for the current date).
-    *
-    * See totalValueValid(const QCString&) for information about the
-    * validity of the returned value.
-    *
-    * @param id account id of the account in question
-    * @param date return value for specific date (default = QDate())
-    * @return value of the account as MyMoneyMoney object
-    */
-  const MyMoneyMoney totalValue(const QCString& id, const QDate& date = QDate()) const;
-
-  /**
-    * This method is used to return the actual value of an account
-    * excluding it's sub-ordinate accounts. A value of an account
-    * is calculated by multiplying the balance with the security's
-    * price for today. If the security's trading currency differs
-    * from baseCurrency() then the result of this multiplication
-    * is multiplied again by the trading currency's price for today.
-    * If a @p date is presented,
-    * the value at the beginning of this date (not including any
-    * transaction on this date) is returned. Otherwise all recorded
-    * transactions are included in the value.
-    *
-    * The price for accounts held in the baseCurrency()
-    * is 1. The same applies for prices that are not available (no entry
-    * in price list for the current date).
-    *
-    * See accountValueValid(const QCString&) for information about the
-    * validity of the returned value.
-    *
-    * @param id account id of the account in question
-    * @param date return value for specific date (default = QDate())
-    * @return value of the account as MyMoneyMoney object
-    */
-  const MyMoneyMoney accountValue(const QCString& id, const QDate& date = QDate()) const;
-
-  /**
-    * This method returns if the value returned by totalValue(const QCString&)
-    * is valid or not. The value is considered valid, if all conversion rates (prices)
-    * are available to convert the account's balance to the base currency.
-    * The same applies for any of the subaccounts of @p id.
-    *
-    * If for any of the accounts a conversion rate for the current date cannot
-    * be found, false is returned. If for all such currencies a conversion rate
-    * is available, true is returned.
-    *
-    * @param id account id of the account in question
-    * @retval false value returned by totalValue(const QCString&) is not
-    *               completely correct (missing conversion rates are assumed to be 1)
-    * @retval true value returned by totalValue(const QCString&) is correct
-    */
-  const bool totalValueValid(const QCString& id) const;
-
-  /**
-    * This method returns if the value returned by accountValue(const QCString&)
-    * is valid or not. The value is considered valid, if a conversion rate (price)
-    * is available for the currency used for the account referenced by @p id.
-    *
-    * @param id account id of the account in question
-    * @retval false value returned by accountValue(const QCString&) is not correct
-    * @retval true value returned by accountValue(const QCString&) is correct
-    */
-  const bool accountValueValid(const QCString& id) const;
-
   /**
     * This method returns the number of transactions currently known to file
     * in the range 0..MAXUINT
@@ -699,18 +670,46 @@ public:
     * This method returns a list of the institutions
     * inside a MyMoneyFile object
     *
+    * @param list reference to the list. It will be cleared by this method first
+    */
+  void institutionList(QValueList<MyMoneyInstitution>& list) const;
+
+  /**
+    * This method returns a list of the institutions
+    * inside a MyMoneyFile object. This is a convenience method
+    * to the one above
+    *
     * @return QValueList containing the institution objects
     */
   const QValueList<MyMoneyInstitution> institutionList(void) const;
 
   /**
-    * Returns the account addressed by it's id.
+    * Returns the account addressed by its id.
     *
     * @param id id of the account to locate.
     * @return MyMoneyAccount object carrying the @p id. An exception is thrown
     *         if the id is unknown
     */
-  const MyMoneyAccount account(const QCString& id) const;
+  const MyMoneyAccount& account(const QCString& id) const;
+
+  /**
+   * Returns the account addressed by its name.
+   *
+   * @param name  name of the account to locate.
+   * @return First MyMoneyAccount object found carrying the @p name.
+   * An empty MyMoneyAccount object will be returned if the name is not found.
+   */
+  const MyMoneyAccount& accountByName(const QString& name) const;
+
+  /**
+   * Returns the sub-account addressed by its name.
+   *
+   * @param acc account to search in
+   * @param name  name of the account to locate.
+   * @return First MyMoneyAccount object found carrying the @p name.
+   * An empty MyMoneyAccount object will be returned if the name is not found.
+   */
+  const MyMoneyAccount& subAccountByName(const MyMoneyAccount& acc, const QString& name) const;
 
   /**
     * This method returns a list of accounts inside a MyMoneyFile object.
@@ -718,15 +717,14 @@ public:
     * the returned list contains all accounts, otherwise only those referenced
     * in the id-list.
     *
+    * @param list reference to QValueList receiving the account objects
     * @param idlist QCStringList of account ids of those accounts that
     *        should be returned. If this list is empty, all accounts
     *        currently known will be returned.
     *
     * @param recursive if @p true, then recurse in all found accounts. The default is @p false
-    *
-    * @return QValueList containing the account objects
     */
-  const QValueList<MyMoneyAccount> accountList(const QCStringList& idlist = QCStringList(), const bool recursive = false) const;
+  void accountList(QValueList<MyMoneyAccount>& list, const QCStringList& idlist = QCStringList(), const bool recursive = false) const;
 
   /**
     * This method is used to convert an account id to a string representation
@@ -735,10 +733,12 @@ public:
     * the single account names seperated by MyMoneyAccount::AccountSeperator.
     *
     * @param accountId const QCString reference of the account's id
+    * @param includeStandardAccounts if true, the standard top account will be part
+    *                   of the name, otherwise it will not be included (default is @c false)
     *
     * @return QString of the constructed name.
     */
-  const QString accountToCategory(const QCString& accountId) const;
+  const QString accountToCategory(const QCString& accountId, bool includeStandardAccounts = false) const;
 
   /**
     * This method is used to convert a string representing a category to
@@ -751,7 +751,7 @@ public:
     * @return QCString of the corresponding account. If account was not found
     *         the return value will be an empty string.
     */
-  const QCString categoryToAccount(const QString& category) const;
+  const QCString categoryToAccount(const QString& category, MyMoneyAccount::accountTypeE type = MyMoneyAccount::UnknownAccountType) const;
 
   /**
     * This method is used to convert a string representing an asset or
@@ -775,39 +775,6 @@ public:
     */
   const QString parentName(const QString& name) const;
 
-  /**
-    * This method is used to attach an observer to a subject
-    * represented by it's id. Whenever the object represented
-    * by the id changes it's state, the observers method
-    * ::update(const QCString& id) is called with id set to
-    * the value passed as argument to attach. This allows an
-    * object of the GUI to observe multiple objects with the
-    * ability to know which object changed. If it observes
-    * multiple objects, it's update(const QCString& id) method
-    * is called multiple times if more than one object is changed
-    * during a single operation.
-    *
-    * @param id reference to id of the subject from which
-    *           notifications should be activated
-    * @param observer pointer to object observing the subject
-    *
-    * @see detach
-    */
-  void attach(const QCString& id, MyMoneyObserver* observer);
-
-  /**
-    * This method is used to detach an observer from a subject
-    * represented by it's id. If an object observes more than one
-    * subject, it must call this method multiple times.
-    *
-    * @param id reference to id of the subject from which
-    *           notifications were activated
-    * @param observer pointer to object observing the subject
-    *
-    * @see attach
-    */
-  void detach(const QCString& id, MyMoneyObserver* observer);
-
   /**
     * This method is used to create a new payee
     *
@@ -825,7 +792,7 @@ public:
     *
     * @return MyMoneyPayee object of payee
     */
-  const MyMoneyPayee payee(const QCString& id) const;
+  const MyMoneyPayee& payee(const QCString& id) const;
 
   /**
     * This method is used to retrieve the id to a corresponding
@@ -836,7 +803,7 @@ public:
     *
     * @return MyMoneyPayee object of payee
     */
-  const MyMoneyPayee payeeByName(const QString& payee) const;
+  const MyMoneyPayee& payeeByName(const QString& payee) const;
 
   /**
     * This method is used to modify an existing payee
@@ -975,91 +942,6 @@ public:
 
   const QStringList consistencyCheck(void);
 
-  /**
-    * MyMoneyFile::NotifyClassAccount
-    * is a special id that will be notified whenever any account is changed.
-    * The typical usage is in the account tree, that must be notified about
-    * all changes of any account.
-    *
-    * @code
-    * MyAccountTreeClass tree;  // MyAccountTreeClass is derived from MyMoneyObserver
-    * MyMoneyFile *file = MyMoneyFile::instance();
-    *
-    * file->attach(MyMoneyFile::NotifyClassAccount, &tree);
-    * // from now on, the tree.update() method will be called, whenever an
-    * // account in the storage area is changed.
-    *
-    * file->detach(MyMoneyFile::NotifyClassAccount, &tree);
-    * // the change notification will not be performed anymore.
-    * @endcode
-    */
-  static const QCString NotifyClassAccount;
-
-  /**
-    * MyMoneyFile::NotifyClassAccount
-    * is a special id that will be notified whenever the accounthierarchy is
-    * changed.
-    * The typical usage is in the account tree, that must be reconstructed
-    * upon this type of change.
-    */
-  static const QCString NotifyClassAccountHierarchy;
-
-  /**
-    * MyMoneyFile::NotifyClassPayee
-    * is a special id that will be notified whenever any account is add,
-    * changed or removed from the engine
-    */
-  static const QCString NotifyClassPayee;
-
-  /**
-    * MyMoneyFile::NotifyClassPayeeSet
-    * is a special id that will be notified whenever any account is add
-    * or removed from the engine
-    */
-  static const QCString NotifyClassPayeeSet;
-
-  /**
-    * MyMoneyFile::NotifyClassPayee
-    * is a special id that will be notified whenever any account is changed
-    */
-  static const QCString NotifyClassInstitution;
-
-  /**
-    * MyMoneyFile::NotifyClassSchedule
-    * is a special id that will be notified whenever any schedule is changed
-    */
-  static const QCString NotifyClassSchedule;
-
-  /**
-    * MyMoneyFile::NotifyClassAnyChange
-    * is a special id that will be notified whenever any object of the engine changes
-    */
-  static const QCString NotifyClassAnyChange;
-
-  /**
-    * MyMoneyFile::NotifyClassCurrency
-    * is a special id that will be notified whenever any currency is changed
-    */
-  static const QCString NotifyClassCurrency;
-
-  /**
-    * MyMoneyFile::NotifyClassSecurity
-    * is a special id that will be notified whenever any security is changed
-    */
-  static const QCString NotifyClassSecurity;
-
-  /**
-    * MyMoneyFile::NotifyClassPrice
-    * is a special id that will be notified whenever a price is changed
-    */
-  static const QCString NotifyClassPrice;
-
-  /**
-    * MyMoneyFile::NotifyClassReport
-    * is a special id that will be notified whenever any report is changed
-    */
-  static const QCString NotifyClassReport;
-
   /**
     * MyMoneyFile::OpeningBalancesPrefix is a special string used
     * to generate the name for opening balances accounts. See openingBalanceAccount()
@@ -1067,6 +949,12 @@ public:
     */
   static const QString OpeningBalancesPrefix;
 
+  /**
+    * MyMoneyFile::AccountSeperator is used as the seperator
+    * between account names to form a hierarchy.
+    */
+  static const QString AccountSeperator;
+
   /**
     * createCategory creates a category from a text name.
     *
@@ -1183,7 +1071,7 @@ public:
     * @param id QCString containing the id of the MyMoneySchedule object
     * @return MyMoneySchedule object
     */
-  const MyMoneySecurity currency(const QCString& id) const;
+  const MyMoneySecurity& currency(const QCString& id) const;
 
   /**
     * This method is used to retrieve the list of all currencies
@@ -1206,7 +1094,14 @@ public:
     *
     * @return MyMoneySecurity describing base currency
     */
-  const MyMoneySecurity baseCurrency(void) const;
+  const MyMoneySecurity& baseCurrency(void) const;
+
+  /**
+    * This method returns the foreign currency of the given two
+    * currency ids. If second is the base currency id then @a first
+    * is returned otherwise @a second is returned.
+    */
+  const QCString& foreignCurrency(const QCString& first, const QCString& second) const;
 
   /**
     * This method allows to select the base currency. It does
@@ -1327,48 +1222,92 @@ public:
   void removeReport(const MyMoneyReport& report);
 
   /**
-    * This method checks, if the given @p security is referenced
-    * by another engine object.
+    * This method is used to retrieve the list of all budgets
+    * known to the engine.
     *
-    * @param security const reference to object to be checked
+    * An exception will be thrown upon erronous situations.
     *
-    * @retval false @p security is not used
-    * @retval true @p security is used
+    * @return QValueList of all MyMoneyBudget objects.
     */
-  const bool isReferenced(const MyMoneySecurity& security) const;
+  const QValueList<MyMoneyBudget> budgetList( void ) const;
 
   /**
-    * This method checks, if the given @p account is referenced
-    * by another engine object.
+    * Adds a budget to the file-global institution pool. A
+    * respective budget-ID will be generated for this object.
+    * The ID is stored as QString in the object passed as argument.
     *
-    * @param account const reference to object to be checked
+    * An exception will be thrown upon error conditions.
     *
-    * @retval false @p account is not used
-    * @retval true @p account is used
+    * @param budget The complete budget information in a
+    *        MyMoneyBudget object
     */
-  const bool isReferenced(const MyMoneyAccount& account) const;
+  void addBudget( MyMoneyBudget& budget );
+
 
   /**
-    * This method checks, if the given @p institution is referenced
-    * by another engine object.
+    * This method is used to retrieve the id to a corresponding
+    * name of a budget.
+    * An exception will be thrown upon error conditions.
     *
-    * @param institution const reference to object to be checked
+    * @param budget QString reference to name of budget
     *
-    * @retval false @p institution is not used
-    * @retval true @p institution is used
+    * @return MyMoneyBudget refernce to object of budget
     */
-  const bool isReferenced(const MyMoneyInstitution& institution) const;
+  const MyMoneyBudget& budgetByName(const QString& budget) const;
+
 
   /**
-    * This method checks, if the given @p payee is referenced
+    * Modifies an already existing budget in the file global
+    * budget pool.
+    *
+    * An exception will be thrown upon error conditions.
+    *
+    * @param budget The complete new budget information
+    */
+  void modifyBudget( const MyMoneyBudget& budget );
+
+  /**
+    * This method returns the number of budgets currently known to file
+    * in the range 0..MAXUINT
+    *
+    * @return number of budgets known to file
+    */
+  unsigned countBudgets( void ) const;
+
+  /**
+    * This method is used to retrieve a single MyMoneyBudget object.
+    * The id of the object must be supplied in the parameter @p id.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param id QCString containing the id of the MyMoneyBudget object
+    * @return MyMoneyBudget object
+    */
+  MyMoneyBudget budget( const QCString& id ) const;
+
+  /**
+    * This method is used to remove an existing MyMoneyBudget object
+    * from the engine. The id attribute of the object must be set.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param budget const reference to the MyMoneyBudget object to be updated
+    */
+  void removeBudget(const MyMoneyBudget& budget);
+
+
+  /**
+    * This method checks, if the given @p object is referenced
     * by another engine object.
     *
-    * @param payee const reference to object to be checked
+    * @param obj const reference to object to be checked
+    * @param skipCheck MyMoneyFileBitArray with ReferenceCheckBits set for which
+    *                  the check should be skipped
     *
-    * @retval false @p payee is not used
-    * @retval true @p payee is used
+    * @retval false @p object is not referenced
+    * @retval true @p institution is referenced
     */
-  const bool isReferenced(const MyMoneyPayee& payee) const;
+  bool isReferenced(const MyMoneyObject& obj, const MyMoneyFileBitArray& skipCheck = MyMoneyFileBitArray()) const;
 
   /**
     * This method checks if the given check no &p no is used in
@@ -1391,13 +1330,34 @@ public:
     */
   QString highestCheckNo(const QCString& accId) const;
 
+  /**
+    * Clear all internal caches (used internally for performance measurements)
+    */
+  void clearCache(void);
+
+  void forceDataChanged(void) { emit dataChanged(); }
+
+  void preloadCache(void);
+
 protected:
   /**
     * This is the constructor for a new empty file description
     */
   MyMoneyFile();
 
+signals:
+  /**
+    * This signal is emitted whenever any data has been changed in the engine
+    * via any of the methods of this object
+    */
+  void dataChanged(void);
+
 private:
+  static MyMoneyFile file;
+
+  MyMoneyFile& operator=(MyMoneyFile&); // not allowed for singleton
+  MyMoneyFile(const MyMoneyFile&);      // not allowed for singleton
+
   const QCString locateSubAccount(const MyMoneyAccount& base, const QString& category) const;
 
   void ensureDefaultCurrency(MyMoneyAccount& acc) const;
@@ -1419,12 +1379,13 @@ private:
 private:
   /**
     * This method is used to add an id to the list of objects
-    * to be notified. If id is empty, then nothing is added to the list.
+    * to be removed from the cache. If id is empty, then nothing is added to the list.
     *
     * @param id id of object to be notified
+    * @param reload reload the object (@c true) or not (@c false). The default is @c true
     * @see attach, detach
     */
-  void addNotification(const QCString& id);
+  void addNotification(const QCString& id, bool reload = true);
 
   /**
     * This method is used to clear the notification list
@@ -1432,25 +1393,11 @@ private:
   void clearNotification(void);
 
   /**
-    * This method is used to notify the objects observing
-    * a specific id
-    */
-  void notify(const QCString& id);
-
-  /**
-    * This method is used to notify all objects observing
-    * any object contained in m_notificationList.
+    * This method is used to clear all
+    * objects mentioned in m_notificationList from the cache.
     */
   void notify(void);
 
-  /**
-    * This method is used to add the account with id 'id'
-    * and all it's parent account's ids to the notification list.
-    *
-    * @param id account's id
-    */
-  void notifyAccountTree(const QCString& id);
-
   /**
     * This method checks if a storage object is attached and
     * throws and exception if not.
@@ -1460,6 +1407,13 @@ private:
       throw new MYMONEYEXCEPTION("No storage object attached to MyMoneyFile");
   }
 
+  /**
+    * This method checks that a transaction has been started with
+    * startTransaction() and throws an exception otherwise. Calls
+    * checkStorage() to make sure a storage object is present and attached.
+    */
+  void checkTransaction(const char* txt) const;
+
 private:
   /**
     * This member points to the storage strategy
@@ -1467,22 +1421,26 @@ private:
   IMyMoneyStorage *m_storage;
 
   /**
-    * This member variable keeps a set of all subjects known to
-    * the object (accounts, institutions, etc)
+    * This member points to the private data
     */
-  QMap<QCString, MyMoneyFileSubject> m_subjects;
-
-  /**
-    * This member keeps a list of ids to notify after an
-    * operation is completed.
-    */
-  QMap<QCString, bool> m_notificationList;
-
-  /**
-    */
-  bool m_suspendNotify;
+  MyMoneyFilePrivate* const d;
 
   static MyMoneyFile* _instance;
 };
+
+class KMYMONEY_EXPORT MyMoneyFileTransaction
+{
+public:
+  MyMoneyFileTransaction();
+  ~MyMoneyFileTransaction();
+  void commit(void);
+  void rollback(void);
+  void restart(void);
+
+private:
+  bool m_isNested;
+  bool m_needRollback;
+};
+
 #endif
 
