@@ -15,6 +15,7 @@
  ***************************************************************************/
 
 #include "mymoneyfiletest.h"
+#include <iostream>
 
 #include <memory>
 #include <unistd.h>
@@ -25,8 +26,6 @@ MyMoneyFileTest:: MyMoneyFileTest () {};
 
 
 void MyMoneyFileTest::setUp () {
-	observer = new TestObserverSet;
-	hierarchyObserver = new TestObserverSet;
 	storage = new MyMoneySeqAccessMgr;
 	m = MyMoneyFile::instance();
 	m->attachStorage(storage);
@@ -34,20 +33,19 @@ void MyMoneyFileTest::setUp () {
 
 void MyMoneyFileTest::tearDown () {
 	m->detachStorage(storage);
-	delete m;
 	delete storage;
-	delete observer;
-	delete hierarchyObserver;
 }
 
 void MyMoneyFileTest::testEmptyConstructor() {
-	CPPUNIT_ASSERT(m->userName().isEmpty());
-	CPPUNIT_ASSERT(m->userStreet().isEmpty());
-	CPPUNIT_ASSERT(m->userTown().isEmpty());
-	CPPUNIT_ASSERT(m->userCounty().isEmpty());
-	CPPUNIT_ASSERT(m->userPostcode().isEmpty());
-	CPPUNIT_ASSERT(m->userTelephone().isEmpty());
-	CPPUNIT_ASSERT(m->userEmail().isEmpty());
+	MyMoneyPayee user = m->user();
+
+	CPPUNIT_ASSERT(user.name().isEmpty());
+	CPPUNIT_ASSERT(user.address().isEmpty());
+	CPPUNIT_ASSERT(user.city().isEmpty());
+	CPPUNIT_ASSERT(user.state().isEmpty());
+	CPPUNIT_ASSERT(user.postcode().isEmpty());
+	CPPUNIT_ASSERT(user.telephone().isEmpty());
+	CPPUNIT_ASSERT(user.email().isEmpty());
 
 	CPPUNIT_ASSERT(m->institutionCount() == 0);
 	CPPUNIT_ASSERT(m->dirty() == false);
@@ -75,8 +73,10 @@ void MyMoneyFileTest::testAddOneInstitution() {
 	CPPUNIT_ASSERT(m->institutionCount() == 0);
 	storage->m_dirty = false;
 
+	MyMoneyFileTransaction ft;
 	try {
 		m->addInstitution(institution);
+		ft.commit();
 		CPPUNIT_ASSERT(institution.id() == "I000001");
 		CPPUNIT_ASSERT(m->institutionCount() == 1);
 		CPPUNIT_ASSERT(m->dirty() == true);
@@ -85,18 +85,22 @@ void MyMoneyFileTest::testAddOneInstitution() {
 		delete e;
 	}
 
+	ft.restart();
 	try {
 		m->addInstitution(institution_id);
 		CPPUNIT_FAIL("Missing expected exception");
 	} catch(MyMoneyException *e) {
+		ft.commit();
 		CPPUNIT_ASSERT(m->institutionCount() == 1);
 		delete e;
 	}
 
+	ft.restart();
 	try {
 		m->addInstitution(institution_noname);
 		CPPUNIT_FAIL("Missing expected exception");
 	} catch(MyMoneyException *e) {
+		ft.commit();
 		CPPUNIT_ASSERT(m->institutionCount() == 1);
 		delete e;
 	}
@@ -116,8 +120,10 @@ void MyMoneyFileTest::testAddTwoInstitutions() {
 	QString id;
 
 	storage->m_dirty = false;
+	MyMoneyFileTransaction ft;
 	try {
 		m->addInstitution(institution);
+		ft.commit();
 
 		CPPUNIT_ASSERT(institution.id() == "I000002");
 		CPPUNIT_ASSERT(m->institutionCount() == 2);
@@ -157,9 +163,10 @@ void MyMoneyFileTest::testRemoveInstitution() {
 	CPPUNIT_ASSERT(i.accountCount() == 0);
 
 	storage->m_dirty = false;
-
+	MyMoneyFileTransaction ft;
 	try {
 		m->removeInstitution(i);
+		ft.commit();
 		CPPUNIT_ASSERT(m->institutionCount() == 1);
 		CPPUNIT_ASSERT(m->dirty() == true);
 	} catch (MyMoneyException *e) {
@@ -178,10 +185,12 @@ void MyMoneyFileTest::testRemoveInstitution() {
 		delete e;
 	}
 
+	ft.restart();
 	try {
 		m->removeInstitution(i);
 		CPPUNIT_FAIL("Missing expected exception");
 	} catch(MyMoneyException *e) {
+		ft.commit();
 		CPPUNIT_ASSERT(m->institutionCount() == 1);
 		CPPUNIT_ASSERT(m->dirty() == false);
 		delete e;
@@ -258,8 +267,10 @@ void MyMoneyFileTest::testInstitutionModify() {
 
 	storage->m_dirty = false;
 
+	MyMoneyFileTransaction ft;
 	try {
 		m->modifyInstitution(institution);
+		ft.commit();
 		CPPUNIT_ASSERT(institution.id() == "I000001");
 		CPPUNIT_ASSERT(m->institutionCount() == 2);
 		CPPUNIT_ASSERT(m->dirty() == true);
@@ -282,11 +293,13 @@ void MyMoneyFileTest::testInstitutionModify() {
 
 	storage->m_dirty = false;
 
+	ft.restart();
 	MyMoneyInstitution failInstitution2("I000003", newInstitution);
 	try {
 		m->modifyInstitution(failInstitution2);
 		CPPUNIT_FAIL("Exception expected");
 	} catch(MyMoneyException *e) {
+		ft.commit();
 		delete e;
 		CPPUNIT_ASSERT(failInstitution2.id() == "I000003");
 		CPPUNIT_ASSERT(m->institutionCount() == 2);
@@ -295,44 +308,56 @@ void MyMoneyFileTest::testInstitutionModify() {
 }
 
 void MyMoneyFileTest::testSetFunctions() {
-	CPPUNIT_ASSERT(m->userName().isEmpty());
-	CPPUNIT_ASSERT(m->userStreet().isEmpty());
-	CPPUNIT_ASSERT(m->userTown().isEmpty());
-	CPPUNIT_ASSERT(m->userCounty().isEmpty());
-	CPPUNIT_ASSERT(m->userPostcode().isEmpty());
-	CPPUNIT_ASSERT(m->userTelephone().isEmpty());
-	CPPUNIT_ASSERT(m->userEmail().isEmpty());
-	
+	MyMoneyPayee user = m->user();
+
+	CPPUNIT_ASSERT(user.name().isEmpty());
+	CPPUNIT_ASSERT(user.address().isEmpty());
+	CPPUNIT_ASSERT(user.city().isEmpty());
+	CPPUNIT_ASSERT(user.state().isEmpty());
+	CPPUNIT_ASSERT(user.postcode().isEmpty());
+	CPPUNIT_ASSERT(user.telephone().isEmpty());
+	CPPUNIT_ASSERT(user.email().isEmpty());
+
+	MyMoneyFileTransaction ft;
 	storage->m_dirty = false;
-	m->setUserName("Name");
+	user.setName("Name");
+	m->setUser(user);
 	CPPUNIT_ASSERT(m->dirty() == true);
 	storage->m_dirty = false;
-	m->setUserStreet("Street");
+	user.setAddress("Street");
+	m->setUser(user);
 	CPPUNIT_ASSERT(m->dirty() == true);
 	storage->m_dirty = false;
-	m->setUserTown("Town");
+	user.setCity("Town");
+	m->setUser(user);
 	CPPUNIT_ASSERT(m->dirty() == true);
 	storage->m_dirty = false;
-	m->setUserCounty("County");
+	user.setState("County");
+	m->setUser(user);
 	CPPUNIT_ASSERT(m->dirty() == true);
 	storage->m_dirty = false;
-	m->setUserPostcode("Postcode");
+	user.setPostcode("Postcode");
+	m->setUser(user);
 	CPPUNIT_ASSERT(m->dirty() == true);
 	storage->m_dirty = false;
-	m->setUserTelephone("Telephone");
+	user.setTelephone("Telephone");
+	m->setUser(user);
 	CPPUNIT_ASSERT(m->dirty() == true);
 	storage->m_dirty = false;
-	m->setUserEmail("Email");
+	user.setEmail("Email");
+	m->setUser(user);
 	CPPUNIT_ASSERT(m->dirty() == true);
 	storage->m_dirty = false;
 
-	CPPUNIT_ASSERT(m->userName() == "Name");
-	CPPUNIT_ASSERT(m->userStreet() == "Street");
-	CPPUNIT_ASSERT(m->userTown() == "Town");
-	CPPUNIT_ASSERT(m->userCounty() == "County");
-	CPPUNIT_ASSERT(m->userPostcode() == "Postcode");
-	CPPUNIT_ASSERT(m->userTelephone() == "Telephone");
-	CPPUNIT_ASSERT(m->userEmail() == "Email");
+	ft.commit();
+	user = m->user();
+	CPPUNIT_ASSERT(user.name() == "Name");
+	CPPUNIT_ASSERT(user.address() == "Street");
+	CPPUNIT_ASSERT(user.city() == "Town");
+	CPPUNIT_ASSERT(user.state() == "County");
+	CPPUNIT_ASSERT(user.postcode() == "Postcode");
+	CPPUNIT_ASSERT(user.telephone() == "Telephone");
+	CPPUNIT_ASSERT(user.email() == "Email");
 }
 
 void MyMoneyFileTest::testAddAccounts() {
@@ -350,19 +375,14 @@ void MyMoneyFileTest::testAddAccounts() {
 	institution = m->institution("I000001");
 	CPPUNIT_ASSERT(institution.id() == "I000001");
 
-	m->attach("I000001", observer);
-	m->attach("I000002", observer);
-	m->attach(MyMoneyFile::NotifyClassAccount, observer);
-	m->attach(MyMoneyFile::NotifyClassAccountHierarchy, hierarchyObserver);
-
 	a.setName("Account1");
 	a.setInstitutionId(institution.id());
 
+	MyMoneyFileTransaction ft;
 	try {
 		MyMoneyAccount parent = m->asset();
-		observer->reset();
-		hierarchyObserver->reset();
 		m->addAccount(a, parent);
+		ft.commit();
 		CPPUNIT_ASSERT(m->accountCount() == 6);
 		CPPUNIT_ASSERT(a.parentAccountId() == "AStd::Asset");
 		CPPUNIT_ASSERT(a.id() == "A000001");
@@ -375,30 +395,22 @@ void MyMoneyFileTest::testAddAccounts() {
 		CPPUNIT_ASSERT(institution.accountCount() == 1);
 		CPPUNIT_ASSERT(institution.accountList()[0] == "A000001");
 
-		CPPUNIT_ASSERT(observer->updated().count() == 2);
-		CPPUNIT_ASSERT(observer->updated().contains("I000001") == 1);
-		CPPUNIT_ASSERT(observer->updated().contains(MyMoneyFile::NotifyClassAccount) == 1);
-
-		CPPUNIT_ASSERT(hierarchyObserver->updated().count() == 1);
-		CPPUNIT_ASSERT(hierarchyObserver->updated().contains(MyMoneyFile::NotifyClassAccountHierarchy) == 1);
-		observer->reset();
-		hierarchyObserver->reset();
 	} catch(MyMoneyException *e) {
 		delete e;
 		CPPUNIT_FAIL("Unexpected exception!");
 	}
 
 	// try to add this account again, should not work
+	ft.restart();
 	try {
 		MyMoneyAccount parent = m->asset();
 		m->addAccount(a, parent);
 		CPPUNIT_FAIL("Expecting exception!");
 	} catch(MyMoneyException *e) {
+		ft.commit();
 		delete e;
 	}
 
-	CPPUNIT_ASSERT(observer->updated().count() == 0);
-
 	// check that we can modify the local object and
 	// reload it from the file
 	a.setName("AccountX");
@@ -415,15 +427,16 @@ void MyMoneyFileTest::testAddAccounts() {
 	CPPUNIT_ASSERT(c.institutionId() == "I000001");
 
 	CPPUNIT_ASSERT(m->dirty() == false);
-	CPPUNIT_ASSERT(observer->updated().count() == 0);
 
 	// add a second account
 	institution = m->institution("I000002");
 	b.setName("Account2");
 	b.setInstitutionId(institution.id());
+	ft.restart();
 	try {
 		MyMoneyAccount parent = m->asset();
 		m->addAccount(b, parent);
+		ft.commit();
 		CPPUNIT_ASSERT(m->dirty() == true);
 		CPPUNIT_ASSERT(b.id() == "A000002");
 		CPPUNIT_ASSERT(b.parentAccountId() == "AStd::Asset");
@@ -441,10 +454,6 @@ void MyMoneyFileTest::testAddAccounts() {
 		CPPUNIT_ASSERT(m->asset().accountList()[0] == "A000001");
 		CPPUNIT_ASSERT(m->asset().accountList()[1] == "A000002");
 
-		CPPUNIT_ASSERT(observer->updated().count() == 2);
-		CPPUNIT_ASSERT(observer->updated().contains("I000002") == 1);
-		CPPUNIT_ASSERT(observer->updated().contains(MyMoneyFile::NotifyClassAccount) == 1);
-		observer->reset();
 	} catch(MyMoneyException *e) {
 		delete e;
 		CPPUNIT_FAIL("Unexpected exception!");
@@ -463,9 +472,6 @@ void MyMoneyFileTest::testModifyAccount() {
 	testAddAccounts();
 	storage->m_dirty = false;
 
-	observer->reset();
-	CPPUNIT_ASSERT(observer->updated().count() == 0);
-
 	MyMoneyAccount p = m->account("A000001");
 	MyMoneyInstitution institution;
 
@@ -473,17 +479,16 @@ void MyMoneyFileTest::testModifyAccount() {
 	CPPUNIT_ASSERT(p.name() == "Account1");
 
 	p.setName("New account name");
+	MyMoneyFileTransaction ft;
 	try {
 		m->modifyAccount(p);
+		ft.commit();
 
 		CPPUNIT_ASSERT(m->dirty() == true);
 		CPPUNIT_ASSERT(m->accountCount() == 7);
 		CPPUNIT_ASSERT(p.accountType() == MyMoneyAccount::Checkings);
 		CPPUNIT_ASSERT(p.name() == "New account name");
 
-		CPPUNIT_ASSERT(observer->updated().count() == 2);
-		CPPUNIT_ASSERT(observer->updated().contains(MyMoneyFile::NotifyClassAccount) == 1);
-		observer->reset();
 	} catch(MyMoneyException *e) {
 		delete e;
 		CPPUNIT_FAIL("Unexpected exception!");
@@ -492,8 +497,10 @@ void MyMoneyFileTest::testModifyAccount() {
 
 	// try to move account to new institution
 	p.setInstitutionId("I000002");
+	ft.restart();
 	try {
 		m->modifyAccount(p);
+		ft.commit();
 
 		CPPUNIT_ASSERT(m->dirty() == true);
 		CPPUNIT_ASSERT(m->accountCount() == 7);
@@ -509,9 +516,6 @@ void MyMoneyFileTest::testModifyAccount() {
 		CPPUNIT_ASSERT(institution.accountList()[0] == "A000002");
 		CPPUNIT_ASSERT(institution.accountList()[1] == "A000001");
 
-		CPPUNIT_ASSERT(observer->updated().count() == 3);
-		CPPUNIT_ASSERT(observer->updated().contains(MyMoneyFile::NotifyClassAccount) == 1);
-		observer->reset();
 	} catch(MyMoneyException *e) {
 		delete e;
 		CPPUNIT_FAIL("Unexpected exception!");
@@ -520,10 +524,12 @@ void MyMoneyFileTest::testModifyAccount() {
 
 	// try to fool engine a bit
 	p.setParentAccountId("A000001");
+	ft.restart();
 	try {
 		m->modifyAccount(p);
 		CPPUNIT_FAIL("Expecting exception!");
 	} catch(MyMoneyException *e) {
+		ft.commit();
 		delete e;
 	}
 }
@@ -532,22 +538,18 @@ void MyMoneyFileTest::testReparentAccount() {
 	testAddAccounts();
 	storage->m_dirty = false;
 
-	observer->reset();
-	CPPUNIT_ASSERT(observer->updated().count() == 0);
-
-	hierarchyObserver->reset();
-	CPPUNIT_ASSERT(hierarchyObserver->updated().count() == 0);
-
 	MyMoneyAccount p = m->account("A000001");
 	MyMoneyAccount q = m->account("A000002");
 	MyMoneyAccount o = m->account(p.parentAccountId());
 
 	// make A000001 a child of A000002
+	MyMoneyFileTransaction ft;
 	try {
 		CPPUNIT_ASSERT(p.parentAccountId() != q.id());
 		CPPUNIT_ASSERT(o.accountCount() == 2);
 		CPPUNIT_ASSERT(q.accountCount() == 0);
 		m->reparentAccount(p, q);
+		ft.commit();
 		CPPUNIT_ASSERT(m->dirty() == true);
 		CPPUNIT_ASSERT(p.parentAccountId() == q.id());
 		CPPUNIT_ASSERT(q.accountCount() == 1);
@@ -555,14 +557,6 @@ void MyMoneyFileTest::testReparentAccount() {
 		CPPUNIT_ASSERT(p.id() == "A000001");
 		CPPUNIT_ASSERT(q.accountList()[0] == p.id());
 		
-		CPPUNIT_ASSERT(observer->updated().count() == 3);
-		CPPUNIT_ASSERT(observer->updated().contains(MyMoneyFile::NotifyClassAccount) == 1);
-
-		CPPUNIT_ASSERT(hierarchyObserver->updated().count() == 1);
-		CPPUNIT_ASSERT(hierarchyObserver->updated().contains(MyMoneyFile::NotifyClassAccountHierarchy) == 1);
-		observer->reset();
-		hierarchyObserver->reset();
-
 		o = m->account(o.id());
 		CPPUNIT_ASSERT(o.accountCount() == 1);
 	} catch(MyMoneyException *e) {
@@ -574,10 +568,12 @@ void MyMoneyFileTest::testReparentAccount() {
 void MyMoneyFileTest::testRemoveStdAccount(const MyMoneyAccount& acc) {
 	QString txt("Exception expected while removing account ");
 	txt += acc.id();
+	MyMoneyFileTransaction ft;
 	try {
 		m->removeAccount(acc);
 		CPPUNIT_FAIL(txt.latin1());
 	} catch(MyMoneyException *e) {
+		ft.commit();
 		delete e;
 	}
 }
@@ -586,27 +582,26 @@ void MyMoneyFileTest::testRemoveAccount() {
 	MyMoneyInstitution institution;
 
 	testAddAccounts();
+	CPPUNIT_ASSERT(m->accountCount() == 7);
 	storage->m_dirty = false;
 
-	observer->reset();
-	CPPUNIT_ASSERT(observer->updated().count() == 0);
-
-	hierarchyObserver->reset();
-	CPPUNIT_ASSERT(hierarchyObserver->updated().count() == 0);
-
 	QString id;
 	MyMoneyAccount p = m->account("A000001");
 
+	MyMoneyFileTransaction ft;
 	try {
 		MyMoneyAccount q("Ainvalid", p);
 		m->removeAccount(q);
 		CPPUNIT_FAIL("Exception expected!");
 	} catch(MyMoneyException *e) {
+		ft.commit();
 		delete e;
 	}
 
+	ft.restart();
 	try {
 		m->removeAccount(p);
+		ft.commit();
 		CPPUNIT_ASSERT(m->dirty() == true);
 		CPPUNIT_ASSERT(m->accountCount() == 6);
 		institution = m->institution("I000001");
@@ -616,13 +611,6 @@ void MyMoneyFileTest::testRemoveAccount() {
 		institution = m->institution("I000002");
 		CPPUNIT_ASSERT(institution.accountCount() == 1);
 
-		CPPUNIT_ASSERT(observer->updated().count() == 1);
-		CPPUNIT_ASSERT(observer->updated().contains(MyMoneyFile::NotifyClassAccount) == 1);
-
-		CPPUNIT_ASSERT(hierarchyObserver->updated().count() == 1);
-		CPPUNIT_ASSERT(hierarchyObserver->updated().contains(MyMoneyFile::NotifyClassAccountHierarchy) == 1);
-		observer->reset();
-		hierarchyObserver->reset();
 	} catch(MyMoneyException *e) {
 		delete e;
 		CPPUNIT_FAIL("Unexpected exception!");
@@ -639,9 +627,11 @@ void MyMoneyFileTest::testRemoveAccountTree() {
 	testReparentAccount();
 	MyMoneyAccount a = m->account("A000002");
 
+	MyMoneyFileTransaction ft;
 	// remove the account
 	try {
 		m->removeAccount(a);
+		ft.commit();
 	} catch(MyMoneyException *e) {
 		delete e;
 		CPPUNIT_FAIL("Unexpected exception!");
@@ -671,14 +661,15 @@ void MyMoneyFileTest::testAccountListRetrieval () {
 	QValueList<MyMoneyAccount> list;
 
 	storage->m_dirty = false;
-	list = m->accountList();
+	m->accountList(list);
 	CPPUNIT_ASSERT(m->dirty() == false);
 	CPPUNIT_ASSERT(list.count() == 0);
 
 	testAddAccounts();
 
 	storage->m_dirty = false;
-	list = m->accountList();
+	list.clear();
+	m->accountList(list);
 	CPPUNIT_ASSERT(m->dirty() == false);
 	CPPUNIT_ASSERT(list.count() == 2);
 
@@ -697,10 +688,12 @@ void MyMoneyFileTest::testAddTransaction () {
 	exp2.setAccountType(MyMoneyAccount::Expense);
 	exp2.setName("Expense2");
 
+	MyMoneyFileTransaction ft;
 	try {
 		MyMoneyAccount parent = m->expense();
 		m->addAccount(exp1, parent);
 		m->addAccount(exp2, parent);
+		ft.commit();
 	} catch(MyMoneyException *e) {
 		delete e;
 		CPPUNIT_FAIL("Unexpected exception!");
@@ -710,12 +703,15 @@ void MyMoneyFileTest::testAddTransaction () {
 	// date is updated when we add the transaction
 	MyMoneyAccount a = m->account("A000001");
 	a.setLastModified(QDate(1,2,3));
+	ft.restart();
 	try {
 		m->modifyAccount(a);
+		ft.commit();
 	} catch(MyMoneyException *e) {
 		delete e;
 		CPPUNIT_FAIL("Unexpected exception!");
 	}
+	ft.restart();
 
 	CPPUNIT_ASSERT(m->accountCount() == 9);
 	a = m->account("A000001");
@@ -753,19 +749,15 @@ void MyMoneyFileTest::testAddTransaction () {
 */
 	storage->m_dirty = false;
 
-	observer->reset();
-	CPPUNIT_ASSERT(observer->updated().count() == 0);
-
+	ft.restart();
 	try {
 		m->addTransaction(t);
-
-		CPPUNIT_ASSERT(observer->updated().count() == 2);
-		CPPUNIT_ASSERT(observer->updated().contains(MyMoneyFile::NotifyClassAccount) == 1);
-		observer->reset();
+		ft.commit();
 	} catch(MyMoneyException *e) {
 		delete e;
 		CPPUNIT_FAIL("Unexpected exception!");
 	}
+	ft.restart();
 
 	CPPUNIT_ASSERT(t.id() == "T000000000000000001");
 	CPPUNIT_ASSERT(t.postDate() == QDate(2002,2,1));
@@ -775,15 +767,11 @@ void MyMoneyFileTest::testAddTransaction () {
 	// check the balance of the accounts
 	a = m->account("A000001");
 	CPPUNIT_ASSERT(a.lastModified() == QDate::currentDate());
-/* removed with MyMoneyAccount::Transaction
-	CPPUNIT_ASSERT(a.balance() == -1000);
+	CPPUNIT_ASSERT(a.balance() == MyMoneyMoney(-1000));
 
-*/
 	MyMoneyAccount b = m->account("A000003");
 	CPPUNIT_ASSERT(b.lastModified() == QDate::currentDate());
-/* removed with MyMoneyAccount::Transaction
-	CPPUNIT_ASSERT(b.balance() == 1000);
-*/
+	CPPUNIT_ASSERT(b.balance() == MyMoneyMoney(1000));
 
 	storage->m_dirty = false;
 
@@ -795,7 +783,6 @@ void MyMoneyFileTest::testAddTransaction () {
 		CPPUNIT_ASSERT(p.memo() == "Memotext");
 		CPPUNIT_ASSERT(p.splits()[0].accountId() == "A000001");
 		CPPUNIT_ASSERT(p.splits()[1].accountId() == "A000003");
-		CPPUNIT_ASSERT(observer->updated().count() == 0);
 	} catch(MyMoneyException *e) {
 		delete e;
 		CPPUNIT_FAIL("Unexpected exception!");
@@ -810,7 +797,6 @@ void MyMoneyFileTest::testAddTransaction () {
 		CPPUNIT_ASSERT(p.memo() == "Memotext");
 		CPPUNIT_ASSERT(p.splits()[0].accountId() == "A000001");
 		CPPUNIT_ASSERT(p.splits()[1].accountId() == "A000003");
-		CPPUNIT_ASSERT(observer->updated().count() == 0);
 	} catch(MyMoneyException *e) {
 		delete e;
 		CPPUNIT_FAIL("Unexpected exception!");
@@ -823,7 +809,6 @@ void MyMoneyFileTest::testAddTransaction () {
 		CPPUNIT_ASSERT(p.memo() == "Memotext");
 		CPPUNIT_ASSERT(p.splits()[0].accountId() == "A000001");
 		CPPUNIT_ASSERT(p.splits()[1].accountId() == "A000003");
-		CPPUNIT_ASSERT(observer->updated().count() == 0);
 	} catch(MyMoneyException *e) {
 		delete e;
 		CPPUNIT_FAIL("Unexpected exception!");
@@ -854,18 +839,14 @@ void MyMoneyFileTest::testModifyTransactionSimple() {
 	t.setMemo("New Memotext");
 	storage->m_dirty = false;
 
-	observer->reset();
-	CPPUNIT_ASSERT(observer->updated().count() == 0);
-
+	MyMoneyFileTransaction ft;
 	try {
 		m->modifyTransaction(t);
+		ft.commit();
 		t = m->transaction("T000000000000000001");
 		CPPUNIT_ASSERT(t.memo() == "New Memotext");
 		CPPUNIT_ASSERT(m->dirty() == true);
 
-		CPPUNIT_ASSERT(observer->updated().count() == 2);
-		CPPUNIT_ASSERT(observer->updated().contains(MyMoneyFile::NotifyClassAccount) == 1);
-		observer->reset();
 	} catch(MyMoneyException *e) {
 		delete e;
 		CPPUNIT_FAIL("Unexpected exception!");
@@ -880,20 +861,17 @@ void MyMoneyFileTest::testModifyTransactionNewPostDate() {
 	MyMoneyTransaction t = m->transaction("T000000000000000001");
 	t.setPostDate(QDate(2004,2,1));
 	storage->m_dirty = false;
-	observer->reset();
-	CPPUNIT_ASSERT(observer->updated().count() == 0);
 
+	MyMoneyFileTransaction ft;
 	try {
 		m->modifyTransaction(t);
+		ft.commit();
 		t = m->transaction("T000000000000000001");
 		CPPUNIT_ASSERT(t.postDate() == QDate(2004,2,1));
 		t = m->transaction("A000001", 0);
 		CPPUNIT_ASSERT(t.id() == "T000000000000000001");
 		CPPUNIT_ASSERT(m->dirty() == true);
 
-		CPPUNIT_ASSERT(observer->updated().count() == 2);
-		CPPUNIT_ASSERT(observer->updated().contains(MyMoneyFile::NotifyClassAccount) == 1);
-		observer->reset();
 	} catch(MyMoneyException *e) {
 		delete e;
 		CPPUNIT_FAIL("Unexpected exception!");
@@ -912,8 +890,7 @@ void MyMoneyFileTest::testModifyTransactionNewAccount() {
 	t.modifySplit(s);
 
 	storage->m_dirty = false;
-	observer->reset();
-	CPPUNIT_ASSERT(observer->updated().count() == 0);
+	MyMoneyFileTransaction ft;
 	try {
 /* removed with MyMoneyAccount::Transaction
 		CPPUNIT_ASSERT(m->account("A000001").transactionCount() == 1);
@@ -928,6 +905,7 @@ void MyMoneyFileTest::testModifyTransactionNewAccount() {
 		CPPUNIT_ASSERT(m->transactionList(f3).count() == 1);
 
 		m->modifyTransaction(t);
+		ft.commit();
 		t = m->transaction("T000000000000000001");
 		CPPUNIT_ASSERT(t.postDate() == QDate(2002,2,1));
 		t = m->transaction("A000002", 0);
@@ -941,9 +919,6 @@ void MyMoneyFileTest::testModifyTransactionNewAccount() {
 		CPPUNIT_ASSERT(m->transactionList(f2).count() == 1);
 		CPPUNIT_ASSERT(m->transactionList(f3).count() == 1);
 
-		CPPUNIT_ASSERT(observer->updated().count() == 3);
-		CPPUNIT_ASSERT(observer->updated().contains(MyMoneyFile::NotifyClassAccount) == 1);
-		observer->reset();
 	} catch(MyMoneyException *e) {
 		delete e;
 		CPPUNIT_FAIL("Unexpected exception!");
@@ -957,10 +932,10 @@ void MyMoneyFileTest::testRemoveTransaction () {
 	t = m->transaction("T000000000000000001");
 
 	storage->m_dirty = false;
-	observer->reset();
-	CPPUNIT_ASSERT(observer->updated().count() == 0);
+	MyMoneyFileTransaction ft;
 	try {
 		m->removeTransaction(t);
+		ft.commit();
 		CPPUNIT_ASSERT(m->dirty() == true);
 		CPPUNIT_ASSERT(m->transactionCount() == 0);
 /* removed with MyMoneyAccount::Transaction
@@ -975,9 +950,6 @@ void MyMoneyFileTest::testRemoveTransaction () {
 		CPPUNIT_ASSERT(m->transactionList(f2).count() == 0);
 		CPPUNIT_ASSERT(m->transactionList(f3).count() == 0);
 
-		CPPUNIT_ASSERT(observer->updated().count() == 2);
-		CPPUNIT_ASSERT(observer->updated().contains(MyMoneyFile::NotifyClassAccount) == 1);
-		observer->reset();
 	} catch(MyMoneyException *e) {
 		delete e;
 		CPPUNIT_FAIL("Unexpected exception!");
@@ -1020,6 +992,7 @@ void MyMoneyFileTest::testBalanceTotal() {
 	MyMoneySplit split1;
 	MyMoneySplit split2;
 
+	MyMoneyFileTransaction ft;
 	try {
 		split1.setAccountId("A000002");
 		split1.setShares(-1000);
@@ -1030,6 +1003,8 @@ void MyMoneyFileTest::testBalanceTotal() {
 		t.addSplit(split1);
 		t.addSplit(split2);
 		m->addTransaction(t);
+		ft.commit();
+		ft.restart();
 		CPPUNIT_ASSERT(t.id() == "T000000000000000002");
 		CPPUNIT_ASSERT(m->totalBalance("A000001") == MyMoneyMoney(-1000));
 		CPPUNIT_ASSERT(m->totalBalance("A000002") == MyMoneyMoney(-1000));
@@ -1037,6 +1012,7 @@ void MyMoneyFileTest::testBalanceTotal() {
 		MyMoneyAccount p = m->account("A000001");
 		MyMoneyAccount q = m->account("A000002");
 		m->reparentAccount(p, q);
+		ft.commit();
 		CPPUNIT_ASSERT(m->totalBalance("A000001") == MyMoneyMoney(-1000));
 		CPPUNIT_ASSERT(m->totalBalance("A000002") == MyMoneyMoney(-2000));
 	} catch(MyMoneyException *e) {
@@ -1046,30 +1022,39 @@ void MyMoneyFileTest::testBalanceTotal() {
 }
 
 void MyMoneyFileTest::testSetAccountName() {
+	MyMoneyFileTransaction ft;
 	try {
 		m->setAccountName(STD_ACC_LIABILITY, "Verbindlichkeiten");
+		ft.commit();
 	} catch (MyMoneyException *e) {
 		delete e;
 		CPPUNIT_FAIL("Unexpected exception");
 	}
+	ft.restart();
 	try {
 		m->setAccountName(STD_ACC_ASSET, "Vermgen");
+		ft.commit();
 	} catch (MyMoneyException *e) {
 		delete e;
 		CPPUNIT_FAIL("Unexpected exception");
 	}
+	ft.restart();
 	try {
 		m->setAccountName(STD_ACC_EXPENSE, "Ausgaben");
+		ft.commit();
 	} catch (MyMoneyException *e) {
 		delete e;
 		CPPUNIT_FAIL("Unexpected exception");
 	}
+	ft.restart();
 	try {
 		m->setAccountName(STD_ACC_INCOME, "Einnahmen");
+		ft.commit();
 	} catch (MyMoneyException *e) {
 		delete e;
 		CPPUNIT_FAIL("Unexpected exception");
 	}
+	ft.restart();
 
 	CPPUNIT_ASSERT(m->liability().name() == "Verbindlichkeiten");
 	CPPUNIT_ASSERT(m->asset().name() == "Vermgen");
@@ -1078,6 +1063,7 @@ void MyMoneyFileTest::testSetAccountName() {
 
 	try {
 		m->setAccountName("A000001", "New account name");
+		ft.commit();
 		CPPUNIT_FAIL("Exception expected");
 	} catch (MyMoneyException *e) {
 		delete e;
@@ -1087,20 +1073,15 @@ void MyMoneyFileTest::testSetAccountName() {
 void MyMoneyFileTest::testAddPayee() {
 	MyMoneyPayee p;
 
-	m->attach(MyMoneyFile::NotifyClassPayee, observer);
-	observer->reset();
-	CPPUNIT_ASSERT(observer->updated().count() == 0);
-
 	p.setName("THB");
 	CPPUNIT_ASSERT(m->dirty() == false);
+	MyMoneyFileTransaction ft;
 	try {
 		m->addPayee(p);
+		ft.commit();
 		CPPUNIT_ASSERT(m->dirty() == true);
 		CPPUNIT_ASSERT(p.id() == "P000001");
 
-		CPPUNIT_ASSERT(observer->updated().count() == 1);
-		CPPUNIT_ASSERT(observer->updated().contains(MyMoneyFile::NotifyClassPayee) == 1);
-		observer->reset();
 	} catch (MyMoneyException *e) {
 		delete e;
 		CPPUNIT_FAIL("Unexpected exception");
@@ -1111,18 +1092,16 @@ void MyMoneyFileTest::testModifyPayee() {
 	MyMoneyPayee p;
 
 	testAddPayee();
-	CPPUNIT_ASSERT(observer->updated().count() == 0);
 
 	p = m->payee("P000001");
 	p.setName("New name");
+	MyMoneyFileTransaction ft;
 	try {
 		m->modifyPayee(p);
+		ft.commit();
 		p = m->payee("P000001");
 		CPPUNIT_ASSERT(p.name() == "New name");
 
-		CPPUNIT_ASSERT(observer->updated().count() == 1);
-		CPPUNIT_ASSERT(observer->updated().contains(MyMoneyFile::NotifyClassPayee) == 1);
-		observer->reset();
 	} catch (MyMoneyException *e) {
 		delete e;
 		CPPUNIT_FAIL("Unexpected exception");
@@ -1134,16 +1113,14 @@ void MyMoneyFileTest::testRemovePayee() {
 
 	testAddPayee();
 	CPPUNIT_ASSERT(m->payeeList().count() == 1);
-	CPPUNIT_ASSERT(observer->updated().count() == 0);
 
 	p = m->payee("P000001");
+	MyMoneyFileTransaction ft;
 	try {
 		m->removePayee(p);
+		ft.commit();
 		CPPUNIT_ASSERT(m->payeeList().count() == 0);
 
-		CPPUNIT_ASSERT(observer->updated().count() == 1);
-		CPPUNIT_ASSERT(observer->updated().contains(MyMoneyFile::NotifyClassPayee) == 1);
-		observer->reset();
 	} catch (MyMoneyException *e) {
 		delete e;
 		CPPUNIT_FAIL("Unexpected exception");
@@ -1188,46 +1165,17 @@ void MyMoneyFileTest::testAddTransactionStd() {
 	t.setState(MyMoneyCheckingTransaction::Cleared);
 */
 	storage->m_dirty = false;
-	observer->reset();
-	CPPUNIT_ASSERT(observer->updated().count() == 0);
 
+	MyMoneyFileTransaction ft;
 	try {
 		m->addTransaction(t);
-
-		CPPUNIT_ASSERT(observer->updated().count() == 2);
-		CPPUNIT_ASSERT(observer->updated().contains(MyMoneyFile::NotifyClassAccount) == 1);
-		observer->reset();
+		ft.commit();
+		CPPUNIT_FAIL("Missing expected exception!");
 	} catch(MyMoneyException *e) {
 		delete e;
-		CPPUNIT_FAIL("Unexpected exception!");
 	}
 
-	CPPUNIT_ASSERT(t.id() == "T000000000000000001");
-	CPPUNIT_ASSERT(t.postDate() == QDate(2002,2,1));
-	CPPUNIT_ASSERT(t.entryDate() == QDate::currentDate());
-	CPPUNIT_ASSERT(m->dirty() == true);
-
-	// check the balance of the accounts
-	a = m->account("A000001");
-	CPPUNIT_ASSERT(a.lastModified() == QDate::currentDate());
-
-	MyMoneyAccount b = m->account(STD_ACC_EXPENSE);
-	CPPUNIT_ASSERT(b.lastModified() == QDate::currentDate());
-
-	storage->m_dirty = false;
-
-	// locate transaction in MyMoneyFile via id
-
-	try {
-		p = m->transaction("T000000000000000001");
-		CPPUNIT_ASSERT(p.splitCount() == 2);
-		CPPUNIT_ASSERT(p.memo() == "Memotext");
-		CPPUNIT_ASSERT(p.splits()[0].accountId() == "A000001");
-		CPPUNIT_ASSERT(p.splits()[1].accountId() == STD_ACC_EXPENSE);
-	} catch(MyMoneyException *e) {
-		delete e;
-		CPPUNIT_FAIL("Unexpected exception!");
-	}
+	CPPUNIT_ASSERT(m->dirty() == false);
 }
 
 void MyMoneyFileTest::testAttachStorage() {
@@ -1269,6 +1217,7 @@ void MyMoneyFileTest::testAttachStorage() {
 	delete file;
 }
 
+
 void MyMoneyFileTest::testAccount2Category() {
 	testReparentAccount();
 	CPPUNIT_ASSERT(m->accountToCategory("A000001") == "Account2:Account1");
@@ -1280,8 +1229,10 @@ void MyMoneyFileTest::testCategory2Account() {
 	MyMoneyAccount a = m->account("A000003");
 	MyMoneyAccount b = m->account("A000004");
 
+	MyMoneyFileTransaction ft;
 	try {
 		m->reparentAccount(b, a);
+		ft.commit();
 		CPPUNIT_ASSERT(m->categoryToAccount("Expense1") == "A000003");
 		CPPUNIT_ASSERT(m->categoryToAccount("Expense1:Expense2") == "A000004");
 		CPPUNIT_ASSERT(m->categoryToAccount("Acc2").isEmpty());
@@ -1309,8 +1260,10 @@ void MyMoneyFileTest::testHasAccount() {
 	a.setAccountType(MyMoneyAccount::Checkings);
 	a.setName("Account3");
 	b = m->account("A000001");
+	MyMoneyFileTransaction ft;
 	try {
 		m->addAccount(a, b);
+		ft.commit();
 		CPPUNIT_ASSERT(m->accountCount() == 8);
 		CPPUNIT_ASSERT(a.parentAccountId() == "A000001");
 		CPPUNIT_ASSERT(m->hasAccount("A000001", "Account3") == true);
@@ -1326,14 +1279,16 @@ void MyMoneyFileTest::testAddEquityAccount() {
 	i.setName("Investment");
 	i.setAccountType(MyMoneyAccount::Investment);
 
+	MyMoneyFileTransaction ft;
 	try {
 		MyMoneyAccount parent = m->asset();
-		observer->reset();
-		hierarchyObserver->reset();
 		m->addAccount(i, parent);
+		ft.commit();
 	} catch(MyMoneyException *e) {
 		unexpectedException(e);
 	}
+	// keep a copy for later use
+	m_inv = i;
 
 	// make sure, that only equity accounts can be children to it
 	MyMoneyAccount a;
@@ -1357,22 +1312,23 @@ void MyMoneyFileTest::testAddEquityAccount() {
 	QValueList<MyMoneyAccount::accountTypeE>::Iterator it;
 	for(it = list.begin(); it != list.end(); ++it) {
 		a.setAccountType(*it);
+		ft.restart();
 		try {
 			char	msg[100];
-			observer->reset();
-			hierarchyObserver->reset();
 			m->addAccount(a, i);
 			sprintf(msg, "Can add non-equity type %d to investment", *it);
 			CPPUNIT_FAIL(msg);
 		} catch(MyMoneyException *e) {
+			ft.commit();
 			delete e;
 		}
 	}
+	ft.restart();
 	try {
-		observer->reset();
-		hierarchyObserver->reset();
+		a.setName("Teststock");
 		a.setAccountType(MyMoneyAccount::Stock);
 		m->addAccount(a,i);
+		ft.commit();
 	} catch(MyMoneyException *e) {
 		unexpectedException(e);
 	}
@@ -1415,11 +1371,11 @@ void MyMoneyFileTest::testReparentEquity() {
 
 	// now check the good case
 	MyMoneyAccount stock = m->account("A000002");
-	MyMoneyAccount inv = m->account("A000003");
+	MyMoneyAccount inv = m->account(m_inv.id());
+	MyMoneyFileTransaction ft;
 	try {
-		observer->reset();
-		hierarchyObserver->reset();
 		m->reparentAccount(stock, inv);
+		ft.commit();
 	} catch(MyMoneyException *e) {
 		unexpectedException(e);
 	}
@@ -1428,23 +1384,24 @@ void MyMoneyFileTest::testReparentEquity() {
 void MyMoneyFileTest::testReparentEquity(QValueList<MyMoneyAccount::accountTypeE>& list, MyMoneyAccount& parent)
 {
 	MyMoneyAccount a;
-	a.setName("Testaccount");
 	MyMoneyAccount stock = m->account("A000002");
 
 	QValueList<MyMoneyAccount::accountTypeE>::Iterator it;
+	MyMoneyFileTransaction ft;
 	for(it = list.begin(); it != list.end(); ++it) {
+		a.setName(QString("Testaccount %1").arg(*it));
 		a.setAccountType(*it);
 		try {
 			m->addAccount(a, parent);
 			char	msg[100];
-			observer->reset();
-			hierarchyObserver->reset();
 			m->reparentAccount(stock, a);
 			sprintf(msg, "Can reparent stock to non-investment type %d account", *it);
 			CPPUNIT_FAIL(msg);
 		} catch(MyMoneyException *e) {
+			ft.commit();
 			delete e;
 		}
+		ft.restart();
 	}
 }
 
@@ -1469,13 +1426,16 @@ void MyMoneyFileTest::testBaseCurrency(void)
 		delete e;
 	}
 
+	MyMoneyFileTransaction ft;
 	// add the currency and try again
 	try {
 		m->addCurrency(base);
 		m->setBaseCurrency(base);
+		ft.commit();
 	} catch(MyMoneyException *e) {
 		unexpectedException(e);
 	}
+	ft.restart();
 
 	// make sure, the base currency is set
 	try {
@@ -1486,6 +1446,24 @@ void MyMoneyFileTest::testBaseCurrency(void)
 	} catch(MyMoneyException *e) {
 		unexpectedException(e);
 	}
+
+	// check if it gets reset when attaching a new storage
+	m->detachStorage(storage);
+
+	MyMoneySeqAccessMgr* newStorage = new MyMoneySeqAccessMgr;
+	m->attachStorage(newStorage);
+
+	ref = m->baseCurrency();
+	CPPUNIT_ASSERT(ref.id().isEmpty());
+
+	m->detachStorage(newStorage);
+	delete newStorage;
+
+	m->attachStorage(storage);
+	ref = m->baseCurrency();
+	CPPUNIT_ASSERT(ref.id() == "EUR");
+	CPPUNIT_ASSERT(ref.name() == "Euro");
+	CPPUNIT_ASSERT(ref.tradingSymbol() == QChar(0x20ac));
 }
 
 void MyMoneyFileTest::testOpeningBalanceNoBase(void)
@@ -1512,15 +1490,16 @@ void MyMoneyFileTest::testOpeningBalance(void)
 		openingAcc = m->openingBalanceAccount(m->baseCurrency());
 		CPPUNIT_ASSERT(openingAcc.parentAccountId() == m->equity().id());
 		CPPUNIT_ASSERT(openingAcc.name() == MyMoneyFile::OpeningBalancesPrefix);
-		CPPUNIT_ASSERT(openingAcc.openingBalance() == MyMoneyMoney(0,1));
 		CPPUNIT_ASSERT(openingAcc.openingDate() == QDate::currentDate());
 	} catch(MyMoneyException *e) {
 		unexpectedException(e);
 	}
 
 	// add a second currency
+	MyMoneyFileTransaction ft;
 	try {
 		m->addCurrency(second);
+		ft.commit();
 	} catch(MyMoneyException *e) {
 		unexpectedException(e);
 	}
@@ -1530,10 +1509,42 @@ void MyMoneyFileTest::testOpeningBalance(void)
 		openingAcc = m->openingBalanceAccount(second);
 		CPPUNIT_ASSERT(openingAcc.parentAccountId() == m->equity().id());
 		CPPUNIT_ASSERT(openingAcc.name() == refName);
-		CPPUNIT_ASSERT(openingAcc.openingBalance() == MyMoneyMoney(0,1));
 		CPPUNIT_ASSERT(openingAcc.openingDate() == QDate::currentDate());
 	} catch(MyMoneyException *e) {
 		unexpectedException(e);
 	}
 }
 
+void MyMoneyFileTest::testModifyStdAccount() {
+	CPPUNIT_ASSERT(m->asset().currencyId().isEmpty());
+	CPPUNIT_ASSERT(m->asset().name() == "Asset");
+	testBaseCurrency();
+	CPPUNIT_ASSERT(m->asset().currencyId().isEmpty());
+	CPPUNIT_ASSERT(!m->baseCurrency().id().isEmpty());
+
+	MyMoneyFileTransaction ft;
+	try {
+		MyMoneyAccount acc = m->asset();
+		acc.setName("Anlagen");
+		acc.setCurrencyId(m->baseCurrency().id());
+		m->modifyAccount(acc);
+		ft.commit();
+
+		CPPUNIT_ASSERT(m->asset().name() == "Anlagen");
+		CPPUNIT_ASSERT(m->asset().currencyId() == m->baseCurrency().id());
+	} catch(MyMoneyException *e) {
+		unexpectedException(e);
+	}
+
+	ft.restart();
+	try {
+		MyMoneyAccount acc = m->asset();
+		acc.setNumber("Test");
+		m->modifyAccount(acc);
+		CPPUNIT_FAIL("Missing expected exception");
+	} catch(MyMoneyException *e) {
+		ft.rollback();
+		delete e;
+	}
+	
+}
