@@ -2,13 +2,8 @@
                           mymoneykeyvaluecontainer.cpp
                              -------------------
     begin                : Sun Nov 10 2002
-    copyright            : (C) 2000-2002 by Michael Edwardes
-    email                : mte@users.sourceforge.net
-                           Javier Campos Morales <javi_c@users.sourceforge.net>
-                           Felix Rodriguez <frodriguez@users.sourceforge.net>
-                           John C <thetacoturtle@users.sourceforge.net>
-                           Thomas Baumgart <ipwizard@users.sourceforge.net>
-                           Kevin Tambascio <ktambascio@users.sourceforge.net>
+    copyright            : (C) 2002-2005 by Thomas Baumgart
+    email                : Thomas Baumgart <ipwizard@users.sourceforge.net>
  ***************************************************************************/
 
 /***************************************************************************
@@ -20,24 +15,47 @@
  *                                                                         *
  ***************************************************************************/
 
-#include "mymoneykeyvaluecontainer.h"
+// ----------------------------------------------------------------------------
+// QT Includes
+
+// ----------------------------------------------------------------------------
+// Project Includes
+
+#include <kmymoney/mymoneykeyvaluecontainer.h>
+#include <kmymoney/mymoneyexception.h>
 
 MyMoneyKeyValueContainer::MyMoneyKeyValueContainer()
 {
 }
 
+MyMoneyKeyValueContainer::MyMoneyKeyValueContainer(const QDomElement& node)
+{
+  if(!node.isNull()) {
+    if("KEYVALUEPAIRS" != node.tagName())
+      throw new MYMONEYEXCEPTION("Node was not KEYVALUEPAIRS");
+
+    m_kvp.clear();
+
+    QDomNodeList nodeList = node.elementsByTagName("PAIR");
+    for(unsigned int i = 0; i < nodeList.count(); ++i) {
+      const QDomElement& el(nodeList.item(i).toElement());
+      m_kvp[QCString(el.attribute("key"))] = el.attribute("value");
+    }
+  }
+}
+
 MyMoneyKeyValueContainer::~MyMoneyKeyValueContainer()
 {
 }
 
-const QString MyMoneyKeyValueContainer::value(const QCString& key) const
+const QString& MyMoneyKeyValueContainer::value(const QCString& key) const
 {
   QMap<QCString, QString>::ConstIterator it;
 
   it = m_kvp.find(key);
   if(it != m_kvp.end())
     return (*it);
-  return QString();
+  return QString::null;
 }
 
 void MyMoneyKeyValueContainer::setValue(const QCString& key, const QString& value)
@@ -60,6 +78,11 @@ void MyMoneyKeyValueContainer::deletePair(const QCString& key)
     m_kvp.remove(it);
 }
 
+void MyMoneyKeyValueContainer::clear(void)
+{
+  m_kvp.clear();
+}
+
 const bool MyMoneyKeyValueContainer::operator == (const MyMoneyKeyValueContainer& right) const
 {
   QMap<QCString, QString>::ConstIterator it_a, it_b;
@@ -77,3 +100,19 @@ const bool MyMoneyKeyValueContainer::operator == (const MyMoneyKeyValueContainer
 
   return (it_a == m_kvp.end() && it_b == right.m_kvp.end());
 }
+
+void MyMoneyKeyValueContainer::writeXML(QDomDocument& document, QDomElement& parent) const
+{
+  QDomElement el = document.createElement("KEYVALUEPAIRS");
+
+  QMap<QCString, QString>::ConstIterator it;
+  for(it = m_kvp.begin(); it != m_kvp.end(); ++it)
+  {
+    QDomElement pair = document.createElement("PAIR");
+    pair.setAttribute("key", it.key());
+    pair.setAttribute("value", it.data());
+    el.appendChild(pair);
+  }
+
+  parent.appendChild(el);
+}
