@@ -2,13 +2,8 @@
                           mymoneykeyvaluecontainer.h
                              -------------------
     begin                : Sun Nov 10 2002
-    copyright            : (C) 2000-2002 by Michael Edwardes
-    email                : mte@users.sourceforge.net
-                           Javier Campos Morales <javi_c@users.sourceforge.net>
-                           Felix Rodriguez <frodriguez@users.sourceforge.net>
-                           John C <thetacoturtle@users.sourceforge.net>
-                           Thomas Baumgart <ipwizard@users.sourceforge.net>
-                           Kevin Tambascio <ktambascio@users.sourceforge.net>
+    copyright            : (C) 2000-2005 by Thomas Baumgart
+    email                : Thomas Baumgart <ipwizard@users.sourceforge.net>
  ***************************************************************************/
 
 /***************************************************************************
@@ -25,7 +20,7 @@
 
 
 /**
-  *@author Thomas Baumgart
+  * @author Thomas Baumgart
   */
 
 #ifdef HAVE_CONFIG_H
@@ -38,7 +33,9 @@
 #include <qstring.h>
 #include <qcstring.h>
 #include <qmap.h>
+#include <qdom.h>
 #include <kmymoney/export.h>
+
 // ----------------------------------------------------------------------------
 // Project Includes
 
@@ -55,19 +52,21 @@
   */
 class KMYMONEY_EXPORT MyMoneyKeyValueContainer
 {
-public: 
-	MyMoneyKeyValueContainer();
-	~MyMoneyKeyValueContainer();
+public:
+  MyMoneyKeyValueContainer();
+  MyMoneyKeyValueContainer(const QDomElement& node);
+
+  ~MyMoneyKeyValueContainer();
 
   /**
     * This method can be used to retrieve the value for a specific @p key.
     * If the key is unknown in this container, an empty string will be returned.
     *
     * @param key const reference to QCString with the key to search for
-    * @return QCString value of this key. If the key does not exist,
+    * @return reference to value of this key. If the key does not exist,
     *         an emtpy string is returned.
     */
-  const QString value(const QCString& key) const;
+  const QString& value(const QCString& key) const;
 
   /**
     * This method is used to add a key/value pair to the container or
@@ -86,6 +85,11 @@ public:
     */
   void deletePair(const QCString& key);
 
+  /**
+    * This method clears all pairs currently in the container.
+    */
+  void clear(void);
+
   /**
     * This method is used to retrieve the whole set of key/value pairs
     * from the container. It is meant to be used for permanent storage
@@ -94,7 +98,7 @@ public:
     * @return QMap<QCString, QString> containing all key/value pairs of
     *         this container.
     */
-  const QMap<QCString, QString> pairs(void) const { return m_kvp; };
+  const QMap<QCString, QString>& pairs(void) const { return m_kvp; };
 
   /**
     * This method is used to initially store a set of key/value pairs
@@ -113,6 +117,15 @@ public:
     */
   const bool operator == (const MyMoneyKeyValueContainer &) const;
 
+  /**
+    * This method creates a QDomElement for the @p document
+    * under the parent node @p parent.
+    *
+    * @param document reference to QDomDocument
+    * @param parent reference to QDomElement parent node
+    */
+  void writeXML(QDomDocument& document, QDomElement& parent) const;
+
 private:
   /**
     * This member variable represents the container of key/value pairs.
