@@ -34,6 +34,8 @@
 // Project Includes
 
 #include "mymoneymoney.h"
+#include "mymoneyaccount.h"
+#include "mymoneysecurity.h"
 
 unsigned char MyMoneyMoney::_thousandSeparator = ',';
 unsigned char MyMoneyMoney::_decimalSeparator = '.';
@@ -171,16 +173,32 @@ MyMoneyMoney::MyMoneyMoney(const QString& pszAmount)
     m_num = -m_num;
 }
 
-const QString MyMoneyMoney::formatMoney(const QString currency, const int prec) const
+const QString MyMoneyMoney::formatMoney(int denom, bool showThousandSeparator) const
+{
+  return formatMoney("", denomToPrec(denom), showThousandSeparator);
+}
+
+const QString MyMoneyMoney::formatMoney(const MyMoneyAccount& acc, const MyMoneySecurity& sec, bool showThousandSeparator) const
+{
+  return formatMoney(sec.tradingSymbol(), denomToPrec(acc.fraction()), showThousandSeparator);
+}
+
+const QString MyMoneyMoney::formatMoney(const QString& currency, const int prec, bool showThousandSeparator) const
 {
   QString res;
   QString tmpCurrency = currency;
-  unsigned int tmpPrec = prec;
+  int tmpPrec = prec;
   signed64 denom = 1;
   signed64 m_64Value;
 
-  while(tmpPrec--) {
-    denom *= 10;
+  // if prec == -1 we want the maximum possible but w/o trailing zeroes
+  if(tmpPrec > -1) {
+    while(tmpPrec--) {
+      denom *= 10;
+    }
+  } else {
+    // fix it to a max of 8 digits on the right side for now
+    denom = 100000000;
   }
 
   m_64Value = convert(denom).m_num;
@@ -190,6 +208,7 @@ const QString MyMoneyMoney::formatMoney(const QString currency, const int prec)
   bool bNegative = false;
   signed64 left = m_64Value / denom;
   signed64 right = m_64Value % denom;
+
   if (right < 0){
     right = -right;
     bNegative = true;
@@ -212,11 +231,13 @@ const QString MyMoneyMoney::formatMoney(const QString currency, const int prec)
   } else
     res = QString("%1").arg((long)left);
 
-  int pos = res.length();
-  while((0 < (pos -= 3)) && thousandSeparator())
-    res.insert(pos, thousandSeparator());
+  if(showThousandSeparator) {
+    int pos = res.length();
+    while((0 < (pos -= 3)) && thousandSeparator())
+      res.insert(pos, thousandSeparator());
+  }
 
-  if(prec > 0) {
+  if(prec > 0 || (prec == -1 && right != 0)) {
     if(decimalSeparator())
       res += decimalSeparator();
 
@@ -229,7 +250,15 @@ const QString MyMoneyMoney::formatMoney(const QString currency, const int prec)
     // res being "2," the result wasn't "2,6666000000" as expected, but rather
     // "2,0666600000" which was not usable. The code below works for me.
     QString rs  = QString("%1").arg(right);
-    rs = rs.rightJustify(prec, '0', true);
+    if(prec != -1)
+      rs = rs.rightJustify(prec, '0', true);
+    else {
+      // no trailing zeroes or decimal separators
+      while(rs.endsWith("0"))
+        rs.truncate(rs.length()-1);
+      while(rs.endsWith(QChar(decimalSeparator())))
+        rs.truncate(rs.length()-1);
+    }
     res += rs;
   }
 
@@ -543,7 +572,7 @@ const signed64 MyMoneyMoney::getLcd(const MyMoneyMoney& b) const
    * I.e. LCM 100,96875 == 2*2*5*5,31*5*5*5*5 = 2*2,31*5*5
    *      answer: multiply 100 by 31*5*5 == 387500
    */
-  while(current_divisor * current_divisor <= max_square) {
+  while((current_divisor * current_divisor) <= max_square) {
     if(((small_denom % current_divisor) == 0) &&
        ((big_denom % current_divisor) == 0)) {
       big_denom = big_denom / current_divisor;
