@@ -48,21 +48,27 @@ MyMoneyPrice::MyMoneyPrice(const QCString& from, const QCString& to, const QDate
   m_rate(rate),
   m_source(source)
 {
+  if(!m_rate.isZero())
+    m_invRate = MyMoneyMoney(1,1) / m_rate;
+  else
+    qDebug("Price with zero value created");
 }
 
 MyMoneyPrice::~MyMoneyPrice()
 {
 }
 
-const MyMoneyMoney MyMoneyPrice::rate(const QCString& id) const
+const MyMoneyMoney& MyMoneyPrice::rate(const QCString& id) const
 {
+  static MyMoneyMoney dummyPrice(1,1);
+
   if(!isValid())
-    return MyMoneyMoney(1,1);
+    return dummyPrice;
 
   if(id.isEmpty() || id == m_toSecurity)
     return m_rate;
   if(id == m_fromSecurity)
-    return MyMoneyMoney(1,1) / m_rate;
+    return m_invRate;
 
   QString msg = QString("Unknown security id %1 for price info %2/%3.").arg(id).arg(m_fromSecurity).arg(m_toSecurity);
   throw new MYMONEYEXCEPTION(msg);
@@ -82,3 +88,8 @@ const bool MyMoneyPrice::operator == (const MyMoneyPrice &right) const
       ((m_toSecurity.length() == 0 && right.m_toSecurity.length() == 0) || (m_toSecurity == right.m_toSecurity)) &&
       ((m_source.length() == 0 && right.m_source.length() == 0) || (m_source == right.m_source)));
 }
+
+bool MyMoneyPrice::hasReferenceTo(const QCString& id) const
+{
+  return (id == m_fromSecurity) || (id == m_toSecurity);
+}
