@@ -44,6 +44,7 @@
 
 #include <kmymoney/mymoneymoney.h>
 #include <kmymoney/export.h>
+
 /**
   * @author Thomas Baumgart
   */
@@ -81,7 +82,6 @@ public:
     *
     * @param id return price to be the factor to be used to convert a value into
     *           the correcponding value in security @p id.
-    *           Defaults to @p toSecurity.
     *
     * @return returns the exchange rate (price) as MyMoneyMoney object.
     *
@@ -106,12 +106,12 @@ public:
     * @p 3/1 even though the price information kept with the object was @p 1/3, but based on the other
     * conversion direction (from ADF to GBP).
     */
-  const MyMoneyMoney rate(const QCString& id = QCString()) const;
+  const MyMoneyMoney& rate(const QCString& id) const;
 
-  const QDate date(void) const { return m_date; };
-  const QString source(void) const { return m_source; };
-  const QCString from(void) const { return m_fromSecurity; };
-  const QCString to(void) const { return m_toSecurity; };
+  const QDate& date(void) const { return m_date; };
+  const QString& source(void) const { return m_source; };
+  const QCString& from(void) const { return m_fromSecurity; };
+  const QCString& to(void) const { return m_toSecurity; };
 
   /**
     * Check whether the object is valid or not. A MyMoneyPrice object
@@ -129,11 +129,23 @@ public:
   // Inequality operator
   const bool operator != (const MyMoneyPrice &right) const { return !(operator == (right)); };
 
+  /**
+    * This method checks if a reference to the given object exists. It returns,
+    * a @p true if the object is referencing the one requested by the
+    * parameter @p id. If it does not, this method returns @p false.
+    *
+    * @param id id of the object to be checked for references
+    * @retval true This object references object with id @p id.
+    * @retval false This object does not reference the object with id @p id.
+    */
+  bool hasReferenceTo(const QCString& id) const;
+
 private:
   QCString      m_fromSecurity;
   QCString      m_toSecurity;
   QDate         m_date;
   MyMoneyMoney  m_rate;
+  MyMoneyMoney  m_invRate;
   QString       m_source;
 };
 
