@@ -15,7 +15,9 @@
  *                                                                         *
  ***************************************************************************/
 
-#include "config.h"
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 // ----------------------------------------------------------------------------
 // QT Includes
@@ -32,47 +34,80 @@
 #include "mymoneyfile.h"
 #include "mymoneyreport.h"
 
-//ReportContainer gContainer;
+const QStringList MyMoneyReport::kRowTypeText = QStringList::split(",","none,assetliability,expenseincome,category,topcategory,account,payee,month,week,topaccount,topaccount-account,equitytype,accounttype,institution,budget,budgetactual",true);
+const QStringList MyMoneyReport::kColumnTypeText = QStringList::split(",","none,months,bimonths,quarters,4,5,6,weeks,8,9,10,11,years",true);
 
-const QStringList MyMoneyReport::kRowTypeText = QStringList::split(",","none,assetliability,expenseincome,category,topcategory,account,payee,month,week,topaccount,topaccount-account,equitytype,accounttype,institution",true);
-const QStringList MyMoneyReport::kColumnTypeText = QStringList::split(",","none,months,bimonths,quarters,,,,,,,,,years",true);
-const QStringList MyMoneyReport::kQueryColumnsText = QStringList::split(",","none,number,payee,category,memo,account,reconcileflag,action,shares,price,performance",true);
-const MyMoneyReport::EReportType MyMoneyReport::kTypeArray[] = { eNoReport, ePivotTable, ePivotTable, eQueryTable, eQueryTable, eQueryTable, eQueryTable, eQueryTable, eQueryTable, eQueryTable, eQueryTable, eQueryTable, eQueryTable, eQueryTable, eQueryTable, eNoReport };
+// if you add names here, don't forget to update the bitmap for EQueryColumns
+// and shift the bit for eQCend one position to the left
+const QStringList MyMoneyReport::kQueryColumnsText = QStringList::split(",","none,number,payee,category,memo,account,reconcileflag,action,shares,price,performance,loan,balance",true);
+
+const MyMoneyReport::EReportType MyMoneyReport::kTypeArray[] = { eNoReport, ePivotTable, ePivotTable, eQueryTable, eQueryTable, eQueryTable, eQueryTable, eQueryTable, eQueryTable, eQueryTable, eQueryTable, eQueryTable, eQueryTable, eQueryTable, ePivotTable, ePivotTable, eNoReport };
+const QStringList MyMoneyReport::kDetailLevelText = QStringList::split(",","none,all,top,group,total,invalid",true);
+const QStringList MyMoneyReport::kChartTypeText = QStringList::split(",","none,line,bar,pie,ring,stackedbar,invalid",true);
 
 // This should live in mymoney/mymoneytransactionfilter.h
 static const QStringList kTypeText = QStringList::split(",","all,payments,deposits,transfers,none");
 static const QStringList kStateText = QStringList::split(",","all,notreconciled,cleared,reconciled,frozen,none");
-static const QStringList kDateLockText = QStringList::split(",", "alldates,untiltoday,currentmonth,currentyear,monthtodate,yeartodate,lastmonth,lastyear,last30days,last3months,last6months,last12months,next30days,next3months,next6months,next12months,userdefined");
+static const QStringList kDateLockText = QStringList::split(",", "alldates,untiltoday,currentmonth,currentyear,monthtodate,yeartodate,yeartomonth,lastmonth,lastyear,last7days,last30days,last3months,last6months,last12months,next7days,next30days,next3months,next6months,next12months,userdefined,last3tonext3months");
 static const QStringList kAccountTypeText = QStringList::split(",","unknown,checkings,savings,cash,creditcard,loan,certificatedep,investment,moneymarket,asset,liability,currency,income,expense,assetloan,stock,equity,invalid");
 
-MyMoneyReport::MyMoneyReport(void):
+MyMoneyReport::MyMoneyReport() :
     m_name("Unconfigured Pivot Table Report"),
-    m_showSubAccounts(false),
+    m_detailLevel(eDetailNone),
     m_convertCurrency(true),
     m_favorite(false),
     m_tax(false),
     m_investments(false),
+    m_loans(false),
     m_reportType(kTypeArray[eExpenseIncome]),
     m_rowType(eExpenseIncome),
     m_columnType(eMonths),
+    m_columnsAreDays(false),
     m_queryColumns(eQCnone),
     m_dateLock(userDefined),
-    m_accountGroupFilter(false)
+    m_accountGroupFilter(false),
+    m_chartType(eChartLine),
+    m_chartDataLabels(true),
+    m_chartGridLines(true),
+    m_chartByDefault(false),
+    m_includeSchedules(false),
+    m_includeTransfers(false),
+    m_includeBudgetActuals(false),
+    m_includeUnusedAccounts(false),
+    m_showRowTotals(false)
 {
 }
 
-MyMoneyReport::MyMoneyReport(ERowType _rt, unsigned _ct, unsigned _dl, bool _ss, const QString& _name, const QString& _comment ):
+MyMoneyReport::MyMoneyReport(const QCString& id, const MyMoneyReport& right) :
+  MyMoneyObject(id)
+{
+  *this = right;
+  setId(id);
+}
+
+MyMoneyReport::MyMoneyReport(ERowType _rt, unsigned _ct, dateOptionE _dl, bool _ss, const QString& _name, const QString& _comment ):
     m_name(_name),
     m_comment(_comment),
-    m_showSubAccounts(_ss),
+    m_detailLevel(_ss?eDetailAll:eDetailTop),
     m_convertCurrency(true),
     m_favorite(false),
     m_tax(false),
     m_investments(false),
+    m_loans(false),
     m_reportType(kTypeArray[_rt]),
     m_rowType(_rt),
+    m_columnsAreDays(false),
     m_dateLock(_dl),
-    m_accountGroupFilter(false)
+    m_accountGroupFilter(false),
+    m_chartType(eChartLine),
+    m_chartDataLabels(true),
+    m_chartGridLines(true),
+    m_chartByDefault(false),
+    m_includeSchedules(false),
+    m_includeTransfers(false),
+    m_includeBudgetActuals(false),
+    m_includeUnusedAccounts(false),
+    m_showRowTotals(false)
 {
   if ( m_reportType == ePivotTable )
     m_columnType = static_cast<EColumnType>(_ct);
@@ -80,19 +115,35 @@ MyMoneyReport::MyMoneyReport(ERowType _rt, unsigned _ct, unsigned _dl, bool _ss,
     m_queryColumns = static_cast<EQueryColumns>(_ct);
   setDateFilter(_dl);
 
-  if ( static_cast<size_t>(_rt) > sizeof(kTypeArray)/sizeof(EReportType) || m_reportType == eNoReport )
+  if ( (_rt > static_cast<ERowType>(sizeof(kTypeArray)/sizeof(kTypeArray[0])))
+  || (m_reportType == eNoReport) )
     throw new MYMONEYEXCEPTION("Invalid report type");
 
   if ( _rt == MyMoneyReport::eAssetLiability )
   {
     addAccountGroup(MyMoneyAccount::Asset);
     addAccountGroup(MyMoneyAccount::Liability);
+    m_showRowTotals = true;
   }
   if ( _rt == MyMoneyReport::eExpenseIncome )
   {
     addAccountGroup(MyMoneyAccount::Expense);
     addAccountGroup(MyMoneyAccount::Income);
+    m_showRowTotals = true;
   }
+  //FIXME take this out once we have sorted out all issues regarding budget of assets and liabilities -- asoliverez@gmail.com
+  if ( _rt == MyMoneyReport::eBudget || _rt == MyMoneyReport::eBudgetActual )
+  {
+    addAccountGroup(MyMoneyAccount::Expense);
+    addAccountGroup(MyMoneyAccount::Income);
+  }
+}
+
+MyMoneyReport::MyMoneyReport(const QDomElement& node) :
+  MyMoneyObject(node)
+{
+  if(!read(node))
+    clearId();
 }
 
 void MyMoneyReport::clear(void)
@@ -188,7 +239,9 @@ void MyMoneyReport::addAccountGroup(MyMoneyAccount::accountTypeE type)
 
 const bool MyMoneyReport::includesAccountGroup( MyMoneyAccount::accountTypeE type ) const
 {
-  bool result = (! m_accountGroupFilter) || m_accountGroups.contains( type );
+  bool result = (! m_accountGroupFilter)
+                || ( isIncludingTransfers() && m_rowType == MyMoneyReport::eExpenseIncome )
+                || m_accountGroups.contains( type );
 
   return result;
 }
@@ -210,8 +263,14 @@ const bool MyMoneyReport::includes( const MyMoneyAccount& acc ) const
       break;
     case MyMoneyAccount::Asset:
     case MyMoneyAccount::Liability:
-      if ( isInvestmentsOnly() )
-        result = (acc.accountType() == MyMoneyAccount::Stock) && includesAccount( acc.id() );
+      if ( isLoansOnly() )
+        result = acc.isLoan() && includesAccount( acc.id() );
+      else if ( isInvestmentsOnly() )
+        result = acc.isInvest() && includesAccount( acc.id() );
+      else if ( isIncludingTransfers() && m_rowType == MyMoneyReport::eExpenseIncome )
+        // If transfers are included, ONLY include this account if it is NOT
+        // included in the report itself!!
+        result = ! includesAccount( acc.id() );
       else
         result = includesAccount( acc.id() );
       break;
@@ -229,52 +288,49 @@ void MyMoneyReport::write(QDomElement& e, QDomDocument *doc, bool anonymous) con
   // older versions of the program as new features are added to the reports.
   // Feel free to change the minor type every time a change is made here.
 
+  writeBaseXML(*doc, e);
+
+  if ( anonymous )
+  {
+    e.setAttribute("name", m_id);
+    e.setAttribute("comment", QString(m_comment).fill('x'));
+  }
+  else
+  {
+    e.setAttribute("name", m_name);
+    e.setAttribute("comment", m_comment);
+  }
+  e.setAttribute("group", m_group);
+  e.setAttribute("convertcurrency", m_convertCurrency);
+  e.setAttribute("favorite", m_favorite);
+  e.setAttribute("tax", m_tax);
+  e.setAttribute("investments", m_investments);
+  e.setAttribute("loans", m_loans);
+  e.setAttribute("rowtype", kRowTypeText[m_rowType]);
+  e.setAttribute("datelock", kDateLockText[m_dateLock]);
+  e.setAttribute("includeschedules",m_includeSchedules);
+  e.setAttribute("columnsaredays",m_columnsAreDays);
+  e.setAttribute("includestransfers",m_includeTransfers);
+  if(!m_budgetId.isEmpty())
+    e.setAttribute("budget",m_budgetId);
+  e.setAttribute("includesactuals",m_includeBudgetActuals);
+  e.setAttribute("includeunused", m_includeUnusedAccounts);
+
+  e.setAttribute("charttype",kChartTypeText[m_chartType]);
+  e.setAttribute("chartdatalabels",m_chartDataLabels);
+  e.setAttribute("chartgridlines",m_chartGridLines);
+  e.setAttribute("chartbydefault",m_chartByDefault);
+
   if ( m_reportType == ePivotTable )
   {
-    e.setAttribute("type","pivottable 1.7");
-
-    if ( anonymous )
-    {
-      e.setAttribute("name", m_id);
-      e.setAttribute("comment", QString(m_comment).fill('x'));
-    }
-    else
-    {
-      e.setAttribute("name", m_name);
-      e.setAttribute("comment", m_comment);
-    }
-    e.setAttribute("group", m_group);
-    e.setAttribute("showsubaccounts", m_showSubAccounts);
-    e.setAttribute("convertcurrency", m_convertCurrency);
-    e.setAttribute("favorite", m_favorite);
-    e.setAttribute("tax", m_tax);
-    e.setAttribute("investments", m_investments);
-    e.setAttribute("rowtype", kRowTypeText[m_rowType]);
+    e.setAttribute("type","pivottable 1.15");
+    e.setAttribute("detail", kDetailLevelText[m_detailLevel]);
     e.setAttribute("columntype", kColumnTypeText[m_columnType]);
-    e.setAttribute("id", m_id);
-    e.setAttribute("datelock", kDateLockText[m_dateLock]);
+    e.setAttribute("showrowtotals", m_showRowTotals);
   }
   else if ( m_reportType == eQueryTable )
   {
-    e.setAttribute("type","querytable 1.7");
-    if ( anonymous )
-    {
-      e.setAttribute("name", m_id);
-      e.setAttribute("comment", QString(m_comment).fill('x'));
-    }
-    else
-    {
-      e.setAttribute("name", m_name);
-      e.setAttribute("comment", m_comment);
-    }
-    e.setAttribute("group", m_group);
-    e.setAttribute("convertcurrency", m_convertCurrency);
-    e.setAttribute("favorite", m_favorite);
-    e.setAttribute("tax", m_tax);
-    e.setAttribute("investments", m_investments);
-    e.setAttribute("rowtype", kRowTypeText[m_rowType]);
-    e.setAttribute("id", m_id);
-    e.setAttribute("datelock", kDateLockText[m_dateLock]);
+    e.setAttribute("type","querytable 1.14");
 
     QStringList columns;
     unsigned qc = m_queryColumns;
@@ -475,12 +531,12 @@ bool MyMoneyReport::read(const QDomElement& e)
   bool result = false;
 
   if (
-    QString("REPORT") == e.tagName()
+    "REPORT" == e.tagName()
     &&
     (
-      (  e.attribute("type").find(QString("pivottable 1.")) == 0 )
+      (  e.attribute("type").find("pivottable 1.") == 0 )
       ||
-      (  e.attribute("type").find(QString("querytable 1.")) == 0 )
+      (  e.attribute("type").find("querytable 1.") == 0 )
     )
   )
   {
@@ -494,14 +550,36 @@ bool MyMoneyReport::read(const QDomElement& e)
     // Removed the line that screened out loading reports that are called
     // "Default Report".  It's possible for the user to change the comment
     // to this, and we'd hate for it to break as a result.
-
     m_group = e.attribute("group");
     m_id = e.attribute("id");
-    m_showSubAccounts = e.attribute("showsubaccounts","0").toUInt();
+    if ( e.hasAttribute("detail") )
+    {
+      i = kDetailLevelText.findIndex(e.attribute("detail","all"));
+      if ( i != -1 )
+        m_detailLevel = static_cast<EDetailLevel>(i);
+    }
+    else
+      setShowSubAccounts( e.attribute("showsubaccounts","0").toUInt() );
     m_convertCurrency = e.attribute("convertcurrency","1").toUInt();
     m_favorite = e.attribute("favorite","0").toUInt();
     m_tax = e.attribute("tax","0").toUInt();
     m_investments = e.attribute("investments","0").toUInt();
+    m_loans = e.attribute("loans","0").toUInt();
+    m_includeSchedules = e.attribute("includeschedules","0").toUInt();
+    m_columnsAreDays = e.attribute("columnsaredays","0").toUInt();
+    m_includeTransfers = e.attribute("includestransfers","0").toUInt();
+    if(e.hasAttribute("budget"))
+      m_budgetId = e.attribute("budget");
+    m_includeBudgetActuals = e.attribute("includesactuals","0").toUInt();
+    m_includeUnusedAccounts = e.attribute("includeunused", "0").toUInt();
+
+    i = kChartTypeText.findIndex(e.attribute("charttype"));
+    if ( i != -1 )
+      m_chartType = static_cast<EChartType>(i);
+
+    m_chartDataLabels = e.attribute("chartdatalabels","1").toUInt();
+    m_chartGridLines = e.attribute("chartgridlines","1").toUInt();
+    m_chartByDefault = e.attribute("chartbydefault","0").toUInt();
 
     QString datelockstr = e.attribute("datelock","userdefined");
     // Handle the pivot 1.2/query 1.1 case where the values were saved as
@@ -514,13 +592,21 @@ bool MyMoneyReport::read(const QDomElement& e)
       if ( i == -1 )
         i = userDefined;
     }
-    setDateFilter( i );
+    setDateFilter( static_cast<dateOptionE>(i) );
 
     i = kRowTypeText.findIndex(e.attribute("rowtype","expenseincome"));
     if ( i != -1 )
     {
       setRowType( static_cast<ERowType>(i) );
+      // recent versions of KMyMoney always showed a total column for
+      // income/expense reports. We turn it on for backward compatability
+      // here. If the total column is turned off, the flag will be reset
+      // in the next step
+      if(i == eExpenseIncome)
+        m_showRowTotals = true;
     }
+    if(e.hasAttribute("showrowtotals"))
+      m_showRowTotals = e.attribute("showrowtotals").toUInt();
 
     i = kColumnTypeText.findIndex(e.attribute("columntype","months"));
     if ( i != -1 )
@@ -543,31 +629,31 @@ bool MyMoneyReport::read(const QDomElement& e)
     while(!child.isNull() && child.isElement())
     {
       QDomElement c = child.toElement();
-      if(QString("TEXT") == c.tagName() && c.hasAttribute("pattern"))
+      if("TEXT" == c.tagName() && c.hasAttribute("pattern"))
       {
         setTextFilter(QRegExp(c.attribute("pattern"),c.attribute("casesensitive","1").toUInt(),!c.attribute("regex","1").toUInt()));
       }
-      if(QString("TYPE") == c.tagName() && c.hasAttribute("type"))
+      if("TYPE" == c.tagName() && c.hasAttribute("type"))
       {
         i = kTypeText.findIndex(c.attribute("type"));
         if ( i != -1 )
           addType(i);
       }
-      if(QString("STATE") == c.tagName() && c.hasAttribute("state"))
+      if("STATE" == c.tagName() && c.hasAttribute("state"))
       {
         i = kStateText.findIndex(c.attribute("state"));
         if ( i != -1 )
           addState(i);
       }
-      if(QString("NUMBER") == c.tagName())
+      if("NUMBER" == c.tagName())
       {
         setNumberFilter(c.attribute("from"),c.attribute("to"));
       }
-      if(QString("AMOUNT") == c.tagName())
+      if("AMOUNT" == c.tagName())
       {
         setAmountFilter(MyMoneyMoney(c.attribute("from","0/100")),MyMoneyMoney(c.attribute("to","0/100")));
       }
-      if(QString("DATES") == c.tagName())
+      if("DATES" == c.tagName())
       {
         QDate from, to;
         if ( c.hasAttribute("from") )
@@ -576,19 +662,19 @@ bool MyMoneyReport::read(const QDomElement& e)
           to = QDate::fromString(c.attribute("to"),Qt::ISODate);
         MyMoneyTransactionFilter::setDateFilter(from,to);
       }
-      if(QString("PAYEE") == c.tagName() )
+      if("PAYEE" == c.tagName() )
       {
         addPayee(c.attribute("id").latin1());
       }
-      if(QString("CATEGORY") == c.tagName() && c.hasAttribute("id"))
+      if("CATEGORY" == c.tagName() && c.hasAttribute("id"))
       {
         addCategory(c.attribute("id").latin1());
       }
-      if(QString("ACCOUNT") == c.tagName() && c.hasAttribute("id"))
+      if("ACCOUNT" == c.tagName() && c.hasAttribute("id"))
       {
         addAccount(c.attribute("id").latin1());
       }
-      if(QString("ACCOUNTGROUP") == c.tagName() && c.hasAttribute("group"))
+      if("ACCOUNTGROUP" == c.tagName() && c.hasAttribute("group"))
       {
         i = kAccountTypeText.findIndex(c.attribute("group"));
         if ( i != -1 )
@@ -600,11 +686,23 @@ bool MyMoneyReport::read(const QDomElement& e)
   return result;
 }
 
-void MyMoneyReport::removeAccount( const QCString& id)
+void MyMoneyReport::writeXML(QDomDocument& document, QDomElement& parent) const
 {
-  if(m_accounts.find(id) != 0) {
-    m_accounts.remove(id);
-  }
+  QDomElement el = document.createElement("REPORT");
+  write(el,&document,false);
+  parent.appendChild(el);
+}
+
+bool MyMoneyReport::hasReferenceTo(const QCString& id) const
+{
+  QCStringList list;
+
+  // collect all ids
+  accounts(list);
+  categories(list);
+  payees(list);
+
+  return (list.contains(id) > 0);
 }
 
 // vim:cin:si:ai:et:ts=2:sw=2:
