@@ -32,9 +32,11 @@ class QDomDocument;
 
 // ----------------------------------------------------------------------------
 // Project Includes
+#include <kmymoney/mymoneyobject.h>
 #include <kmymoney/mymoneyaccount.h>
 #include <kmymoney/mymoneytransactionfilter.h>
 #include <kmymoney/export.h>
+
 /**
   * This class defines a report within the MyMoneyEngine.  The report class
   * contains all the configuration parameters needed to run a report, plus
@@ -54,28 +56,44 @@ class QDomDocument;
   * @author Ace Jones <acejones@users.sourceforge.net>
   */
 
-class KMYMONEY_EXPORT MyMoneyReport: public MyMoneyTransactionFilter
+class KMYMONEY_EXPORT MyMoneyReport: public MyMoneyObject, public MyMoneyTransactionFilter
 {
 public:
   // When adding a new row type, be sure to add a corresponding entry in kTypeArray
-  enum ERowType { eNoRows = 0, eAssetLiability, eExpenseIncome, eCategory, eTopCategory, eAccount, ePayee, eMonth, eWeek, eTopAccount, eAccountByTopAccount, eEquityType, eAccountType, eInstitution };
+  enum ERowType { eNoRows = 0, eAssetLiability, eExpenseIncome, eCategory, eTopCategory, eAccount, ePayee, eMonth, eWeek, eTopAccount, eAccountByTopAccount, eEquityType, eAccountType, eInstitution, eBudget, eBudgetActual};
   enum EReportType { eNoReport = 0, ePivotTable, eQueryTable };
-  enum EColumnType { eNoColumns = 0, eMonths = 1, eBiMonths = 2, eQuarters = 3, eYears = 12 };
-  enum EQueryColumns { eQCnone = 0x0, eQCbegin = 0x1, eQCnumber = 0x1, eQCpayee = 0x2, eQCcategory = 0x4, eQCmemo = 0x8, eQCaccount = 0x10, eQCreconciled=0x20, eQCaction=0x40, eQCshares=0x80, eQCprice=0x100, eQCperformance=0x200, eQCend=0x400 };
+  enum EColumnType { eNoColumns = 0, eDays = 1, eMonths = 1, eBiMonths = 2, eQuarters = 3, eWeeks = 7, eYears = 12 };
+
+  // if you add bits to this bitmask, start with the value currently assigned to eQCend and update its value afterwards
+  // also don't forget to add column names to kQueryColumnsText in mymoneyreport.cpp
+  enum EQueryColumns { eQCnone = 0x0, eQCbegin = 0x1, eQCnumber = 0x1, eQCpayee = 0x2, eQCcategory = 0x4, eQCmemo = 0x8, eQCaccount = 0x10, eQCreconciled = 0x20, eQCaction = 0x40, eQCshares = 0x80, eQCprice = 0x100, eQCperformance = 0x200, eQCloan = 0x400, eQCbalance = 0x800, eQCend = 0x1000 };
+
+  enum EDetailLevel { eDetailNone = 0, eDetailAll, eDetailTop, eDetailGroup, eDetailTotal, eDetailEnd };
+  enum EChartType { eChartNone = 0, eChartLine, eChartBar, eChartPie, eChartRing, eChartStackedBar, eChartEnd };
 
   static const QStringList kRowTypeText;
   static const QStringList kColumnTypeText;
   static const QStringList kQueryColumnsText;
+  static const QStringList kDetailLevelText;
+  static const QStringList kChartTypeText;
   static const EReportType kTypeArray[];
 
 public:
   MyMoneyReport(void);
-  MyMoneyReport(ERowType _rt, unsigned _ct, unsigned _dl, bool _ss, const QString& _name, const QString& _comment );
+  MyMoneyReport(ERowType _rt, unsigned _ct, dateOptionE _dl, bool _ss, const QString& _name, const QString& _comment );
+  MyMoneyReport(const QCString& id, const MyMoneyReport& right);
+
+  /**
+    * This constructor creates an object based on the data found in the
+    * QDomElement referenced by @p node. If problems arise, the @p id of
+    * the object is cleared (see MyMoneyObject::clearId()).
+    */
+  MyMoneyReport(const QDomElement& node);
 
   // Simple get operations
   const QString& name(void) const { return m_name; }
-  bool isShowingSubAccounts(void) const { return m_showSubAccounts; }
-  bool isShowingRowTotals(void) const { return (m_rowType==eExpenseIncome); }
+  bool isShowingSubAccounts(void) const { return (m_detailLevel==eDetailAll); }
+  bool isShowingRowTotals(void) const { return (m_showRowTotals); }
   EReportType reportType(void) const { return m_reportType; }
   ERowType rowType(void) const { return m_rowType; }
   EColumnType columnType(void) const { return m_columnType; }
@@ -83,17 +101,29 @@ public:
   bool isConvertCurrency(void) const { return m_convertCurrency; }
   unsigned columnPitch(void) const { return static_cast<unsigned>(m_columnType); }
   bool isShowingColumnTotals(void) const { return m_convertCurrency; }
-  QCString id(void) const { return m_id; }
   const QString& comment( void ) const { return m_comment; }
   EQueryColumns queryColumns(void) const { return m_queryColumns; }
   const QString& group( void ) const { return m_group; }
   bool isFavorite(void) const { return m_favorite; }
   bool isTax(void) const { return m_tax; }
   bool isInvestmentsOnly(void) const { return m_investments; }
+  bool isLoansOnly(void) const { return m_loans; }
+  EDetailLevel detailLevel(void) const { return m_detailLevel; }
+  EChartType chartType(void) const { return m_chartType; }
+  bool isChartDataLabels(void) const { return m_chartDataLabels; }
+  bool isChartGridLines(void) const { return m_chartGridLines; }
+  bool isChartByDefault(void) const { return m_chartByDefault; }
+  bool isIncludingSchedules(void) const { return m_includeSchedules; }
+  bool isColumnsAreDays(void) const { return m_columnsAreDays; }
+  bool isIncludingTransfers(void) const { return m_includeTransfers; }
+  bool isIncludingUnusedAccounts(void) const { return m_includeUnusedAccounts; }
+  bool hasBudget(void) const { return !m_budgetId.isEmpty(); }
+  const QCString& budget(void) const { return m_budgetId; }
+  bool isIncludingBudgetActuals(void) const { return m_includeBudgetActuals; }
 
   // Simple set operations
   void setName(const QString& _s) { m_name = _s; }
-  void setShowSubAccounts(bool _f) { m_showSubAccounts = _f; }
+  void setShowSubAccounts(bool _f) { m_detailLevel = _f?eDetailAll:eDetailTop; }
   void setConvertCurrency(bool _f) { m_convertCurrency = _f; }
   void setRowType(ERowType _rt);
   void setColumnType(EColumnType _ct) { m_columnType = _ct; }
@@ -101,9 +131,32 @@ public:
   void setGroup( const QString& _group ) { m_group = _group; }
   void setFavorite(bool _f) { m_favorite = _f; }
   void setQueryColumns( EQueryColumns _qc ) { m_queryColumns = _qc; }
-  void setId( const QCString& _id ) { m_id = _id; }
   void setTax(bool _f) { m_tax = _f; }
-  void setInvestmentsOnly(bool _f) { m_investments = _f; }
+  void setInvestmentsOnly(bool _f) { m_investments = _f; if (_f) m_loans = false; }
+  void setLoansOnly(bool _f) { m_loans = _f; if (_f) m_investments = false; }
+  void setDetailLevel( EDetailLevel _detail ) { m_detailLevel = _detail; }
+  void setChartType ( EChartType _type ) { m_chartType = _type; }
+  void setChartDataLabels ( bool _f ) { m_chartDataLabels = _f; }
+  void setChartGridLines ( bool _f ) { m_chartGridLines = _f; }
+  void setChartByDefault ( bool _f ) { m_chartByDefault = _f; }
+  void setIncludingSchedules( bool _f ) { m_includeSchedules = _f; }
+  void setColumnsAreDays( bool _f ) { m_columnsAreDays = _f; }
+  void setIncludingTransfers( bool _f ) { m_includeTransfers = _f; }
+  void setIncludingUnusedAccounts( bool _f ) { m_includeUnusedAccounts = _f; }
+  void setShowingRowTotals( bool _f ) { m_showRowTotals = _f; }
+
+  /**
+    * Sets the budget used for this report
+    *
+    * @param _budget The ID of the budget to use, or an empty string
+    * to indicate a budget is NOT included
+    * @param _fa Whether to display actual data alongside the budget.
+    * Setting to false means the report displays ONLY the budget itself.
+    * @warning For now, the budget ID is ignored.  The budget id is
+    * simply checked for any non-empty string, and if so, hasBudget()
+    * will return true.
+    */
+  void setBudget( const QCString& _budget, bool _fa = true ) { m_budgetId = _budget; m_includeBudgetActuals=_fa; }
 
   /**
     * This method allows you to clear the underlying transaction filter
@@ -130,7 +183,12 @@ public:
     *          which this report should be locked to.
     */
 
-  void setDateFilter(unsigned _u) { m_dateLock = _u; if (_u != userDefined) MyMoneyTransactionFilter::setDateFilter( _u ); }
+  void setDateFilter(dateOptionE _u)
+    {
+      m_dateLock = _u;
+      if (_u != userDefined)
+        MyMoneyTransactionFilter::setDateFilter( _u );
+    }
 
   /**
     * Set the underlying date filter using the start and end dates provided.
@@ -243,18 +301,27 @@ public:
   bool read(const QDomElement& e);
 
   /**
-    * Remove an account from the list of accounts. This should only
-    * be used to remove accounts that don't exist anymore.
+    * This method creates a QDomElement for the @p document
+    * under the parent node @p parent.  (This version overwrites the
+    * MMObject base class.)
     *
-    * @param id ID of the account to be removed
+    * @param document reference to QDomDocument
+    * @param parent reference to QDomElement parent node
     */
-  void removeAccount(const QCString& id);
+  virtual void writeXML(QDomDocument& document, QDomElement& parent) const;
+
+  /**
+    * This method checks if a reference to the given object exists. It returns,
+    * a @p true if the object is referencing the one requested by the
+    * parameter @p id. If it does not, this method returns @p false.
+    *
+    * @param id id of the object to be checked for references
+    * @retval true This object references object with id @p id.
+    * @retval false This object does not reference the object with id @p id.
+    */
+  virtual bool hasReferenceTo(const QCString& id) const;
 
 private:
-  /**
-    * The engine-assigned ID of the report.  Do not set this yourself!!
-    */
-  QCString m_id;
   /**
     * The user-assigned name of the report
     */
@@ -270,9 +337,9 @@ private:
     */
   QString m_group;
   /**
-    * Whether to show all aub-accounts (true) or only top-level accounts (false)
+    * How much detail to show in the accounts
     */
-  bool m_showSubAccounts;
+  enum EDetailLevel m_detailLevel;
   /**
     * Whether to convert all currencies to the base currency of the file (true).
     * If this is false, it's up to the report generator to decide how to handle
@@ -291,6 +358,12 @@ private:
     * Whether this report should only include investment accounts
     */
   bool m_investments;
+  /**
+    * Whether this report should only include loan accounts
+    * Applies only to querytable reports.  Mutually exclusive with
+    * m_investments.
+    */
+  bool m_loans;
   /**
     * What sort of algorithm should be used to run the report
     */
@@ -301,14 +374,22 @@ private:
   enum ERowType m_rowType;
   /**
     * What sort of values should show up on the COLUMNS of this report,
-    * in the case of a 'PivotTable' report
+    * in the case of a 'PivotTable' report.  Really this is used more as a
+    * QUANTITY of months or days.  Whether it's months or days is determiend
+    * by m_columnsAreDays.
     */
   enum EColumnType m_columnType;
-  /**
+   /**
+    * Whether the base unit of columns of this report is days.  Only applies to
+    * 'PivotTable' reports.  If false, then columns are months or multiples thereof.
+    */
+  bool m_columnsAreDays;
+ /**
     * What sort of values should show up on the COLUMNS of this report,
     * in the case of a 'QueryTable' report
     */
   enum EQueryColumns m_queryColumns;
+
   /**
     * The plain-language description of what the date range should be locked
     * to.  'userDefined' means NO locking, in any other case, the report
@@ -318,7 +399,7 @@ private:
     * automatically when the report is loaded, and should also be done
     * manually by calling updateDateFilter() before generating the report
     */
-  unsigned m_dateLock;
+  dateOptionE m_dateLock;
   /**
     * Which account groups should be included in the report.  This filter
     * is applied to the individual splits AFTER a transaction has been
@@ -329,7 +410,49 @@ private:
     * Whether an account group filter has been set (see m_accountGroups)
     */
   bool m_accountGroupFilter;
-
+  /**
+    * What format should be used to draw this report as a chart
+    */
+  enum EChartType m_chartType;
+  /**
+    * Whether the value of individual data points should be drawn on the chart
+    */
+  bool m_chartDataLabels;
+  /**
+    * Whether grid lines should be drawn on the chart
+    */
+  bool m_chartGridLines;
+  /**
+    * Whether this report should be shown as a chart by default (otherwise it
+    * should be shown as a textual report)
+    */
+  bool m_chartByDefault;
+  /**
+    * Whether to include scheduled transactions
+    */
+  bool m_includeSchedules;
+  /**
+    * Whether to include transfers.  Only applies to Income/Expense reports
+    */
+  bool m_includeTransfers;
+  /**
+    * The id of the budget associated with this report.
+    */
+  QCString m_budgetId;
+  /**
+    * Whether this report should print the actual data to go along with
+    * the budget.  This is only valid if the report has a budget.
+    */
+  bool m_includeBudgetActuals;
+  /**
+    * Whether this report should include all accounts and not only
+    * accounts with transactions.
+    */
+  bool m_includeUnusedAccounts;
+  /**
+   * Whether this report should include columns for row totals
+   */
+  bool m_showRowTotals;
 };
 
 #endif // MYMONEYREPORT_H
