@@ -2,12 +2,9 @@
                           mymoneyscheduled.cpp
                              -------------------
     copyright            : (C) 2000-2002 by Michael Edwardes
+                           (C) 2007 by Thomas Baumgart
     email                : mte@users.sourceforge.net
-                           Javier Campos Morales <javi_c@users.sourceforge.net>
-                           Felix Rodriguez <frodriguez@users.sourceforge.net>
-                           John C <thetacoturtle@users.sourceforge.net>
                            Thomas Baumgart <ipwizard@users.sourceforge.net>
-                           Kevin Tambascio <ktambascio@users.sourceforge.net>
  ***************************************************************************/
 
 /***************************************************************************
@@ -31,7 +28,8 @@
 #include "mymoneyexception.h"
 #include "mymoneyfile.h"
 
-MyMoneySchedule::MyMoneySchedule()
+MyMoneySchedule::MyMoneySchedule() :
+  MyMoneyObject()
 {
   // Set up the default values
   m_occurence = OCCUR_ANY;
@@ -47,9 +45,11 @@ MyMoneySchedule::MyMoneySchedule()
 
 MyMoneySchedule::MyMoneySchedule(const QString& name, typeE type,
                                  occurenceE occurence, paymentTypeE paymentType,
-                                 const QDate& startDate,
+                                 const QDate& /* startDate */,
                                  const QDate& endDate,
-                                 bool fixed, bool autoEnter) {
+                                 bool fixed, bool autoEnter) :
+  MyMoneyObject()
+{
   // Set up the default values
   m_name = name;
   m_occurence = occurence;
@@ -57,12 +57,83 @@ MyMoneySchedule::MyMoneySchedule(const QString& name, typeE type,
   m_paymentType = paymentType;
   m_fixed = fixed;
   m_autoEnter = autoEnter;
-  m_startDate = startDate;
+  m_startDate = QDate();
   m_endDate = endDate;
   m_lastPayment = QDate();
   m_weekendOption = MoveNothing;
 }
 
+MyMoneySchedule::MyMoneySchedule(const QDomElement& node) :
+  MyMoneyObject(node)
+{
+  if("SCHEDULED_TX" != node.tagName())
+    throw new MYMONEYEXCEPTION("Node was not SCHEDULED_TX");
+
+  m_name = node.attribute("name");
+  m_startDate = stringToDate(node.attribute("startDate"));
+  m_endDate = stringToDate(node.attribute("endDate"));
+  m_lastPayment = stringToDate(node.attribute("lastPayment"));
+
+  m_type = static_cast<MyMoneySchedule::typeE>(node.attribute("type").toInt());
+  m_paymentType = static_cast<MyMoneySchedule::paymentTypeE>(node.attribute("paymentType").toInt());
+  m_occurence = static_cast<MyMoneySchedule::occurenceE>(node.attribute("occurence").toInt());
+
+  m_autoEnter = static_cast<bool>(node.attribute("autoEnter").toInt());
+  m_fixed = static_cast<bool>(node.attribute("fixed").toInt());
+  m_weekendOption = static_cast<MyMoneySchedule::weekendOptionE>(node.attribute("weekendOption").toInt());
+
+  // read in the associated transaction
+  QDomNodeList nodeList = node.elementsByTagName("TRANSACTION");
+  if(nodeList.count() == 0)
+    throw new MYMONEYEXCEPTION("SCHEDULED_TX has no TRANSACTION node");
+
+  m_transaction = MyMoneyTransaction(nodeList.item(0).toElement(), false);
+
+  // some old versions did not remove the entry date and post date fields
+  // in the schedule. So if this is the case, we deal with a very old transaction
+  // and can't use the post date field as next due date. Hence, we wipe it out here
+  if(m_transaction.entryDate().isValid()) {
+    m_transaction.setPostDate(QDate());
+    m_transaction.setEntryDate(QDate());
+  }
+
+  // readin the recorded payments
+  nodeList = node.elementsByTagName("PAYMENTS");
+  if(nodeList.count() > 0) {
+    nodeList = nodeList.item(0).toElement().elementsByTagName("PAYMENT");
+    for(unsigned int i = 0; i < nodeList.count(); ++i) {
+      m_recordedPayments << stringToDate(nodeList.item(i).toElement().attribute("date"));
+    }
+  }
+
+  // if the next due date is not set (comes from old version)
+  // then set it up the old way
+  if(!nextDueDate().isValid() && !m_lastPayment.isValid()) {
+    m_transaction.setPostDate(m_startDate);
+    // clear it, because the schedule has never been used
+    m_startDate = QDate();
+  }
+
+  // There are reports that lastPayment and nextDueDate are identical or
+  // that nextDueDate is older than lastPayment. This could
+  // be caused by older versions of the application. In this case, we just
+  // clear out the nextDueDate and let it calculate from the lastPayment.
+  if(nextDueDate().isValid() && nextDueDate() <= m_lastPayment) {
+    m_transaction.setPostDate(QDate());
+  }
+
+  if(!nextDueDate().isValid()) {
+    m_transaction.setPostDate(m_startDate);
+    m_transaction.setPostDate(nextPayment(m_lastPayment.addDays(1)));
+  }
+}
+
+MyMoneySchedule::MyMoneySchedule(const QCString& id, const MyMoneySchedule& right) :
+  MyMoneyObject(id)
+{
+  *this = right;
+  setId(id);
+}
 
 void MyMoneySchedule::setStartDate(const QDate& date)
 {
@@ -81,10 +152,21 @@ void MyMoneySchedule::setFixed(bool fixed)
 
 void MyMoneySchedule::setTransaction(const MyMoneyTransaction& transaction)
 {
-  m_transaction = transaction;
+  MyMoneyTransaction t = transaction;
+  // don't allow a transaction that has no due date
+  // if we get something like that, then we use the
+  // the current next due date. If that is also invalid
+  // we can't help it.
+  if(!t.postDate().isValid()) {
+    t.setPostDate(m_transaction.postDate());
+  }
+  if(!t.postDate().isValid())
+    return;
+
+  m_transaction = t;
   // make sure that the transaction does not have an id so that we can enter
   // it into the engine
-  m_transaction.setId(QCString());
+  m_transaction.clearId();
 }
 
 void MyMoneySchedule::setEndDate(const QDate& date)
@@ -97,6 +179,51 @@ void MyMoneySchedule::setAutoEnter(bool autoenter)
   m_autoEnter = autoenter;
 }
 
+const QDate& MyMoneySchedule::startDate(void) const
+{
+  if(m_startDate.isValid())
+    return m_startDate;
+  return nextDueDate();
+}
+
+const QDate& MyMoneySchedule::nextDueDate(void) const
+{
+  return m_transaction.postDate();
+}
+
+QDate MyMoneySchedule::adjustedNextDueDate(void) const
+{
+  QDate date(nextDueDate());
+
+  if(isFinished())
+    return QDate();
+
+  if(weekendOption() != MyMoneySchedule::MoveNothing) {
+    int dayOfWeek = date.dayOfWeek();
+    if (dayOfWeek >= 6) {
+      if (weekendOption() == MyMoneySchedule::MoveFriday) {
+        if (dayOfWeek == 7)
+          date = date.addDays(-2);
+        else
+          date = date.addDays(-1);
+      } else {
+        if (dayOfWeek == 6)
+          date = date.addDays(2);
+        else
+          date = date.addDays(1);
+      }
+    }
+  }
+  return date;
+}
+
+void MyMoneySchedule::setNextDueDate(const QDate& date)
+{
+  m_transaction.setPostDate(date);
+  if(date.isValid())
+    m_startDate = date;
+}
+
 void MyMoneySchedule::setLastPayment(const QDate& date)
 {
   // Delete all payments older than date
@@ -115,6 +242,8 @@ void MyMoneySchedule::setLastPayment(const QDate& date)
   }
 
   m_lastPayment = date;
+  if(!m_startDate.isValid())
+    m_startDate = date;
 }
 
 void MyMoneySchedule::setName(const QString& nm)
@@ -154,8 +283,8 @@ void MyMoneySchedule::validate(bool id_check) const
   if(m_type == TYPE_ANY)
     throw new MYMONEYEXCEPTION("Invalid type for schedule");
 
-  if(!m_startDate.isValid())
-    throw new MYMONEYEXCEPTION("Invalid start date for schedule");
+  if(!nextDueDate().isValid())
+    throw new MYMONEYEXCEPTION("Invalid next due date for schedule");
 
   if(m_paymentType == STYPE_ANY)
     throw new MYMONEYEXCEPTION("Invalid payment type for schedule");
@@ -192,6 +321,7 @@ void MyMoneySchedule::validate(bool id_check) const
 
 QDate MyMoneySchedule::nextPayment(const QDate& refDate) const
 {
+#if 0
   QDate paymentDate(m_lastPayment);
 
   // if there never was a payment, then the next payment date must
@@ -215,101 +345,146 @@ QDate MyMoneySchedule::nextPayment(const QDate& refDate) const
         return paymentDate;
     }
   }
+#endif
 
   // if the enddate is valid and it is before the reference date,
   // then there will be no more payments.
   if(m_endDate.isValid() && m_endDate < refDate) {
-
     return QDate();
   }
 
-  switch (m_occurence)
-  {
-    case OCCUR_ONCE:
-      // if the lastPayment is already set, then there will be no more payments
-      // otherwise, the start date is the payment date
-      if(m_lastPayment.isValid())
-        return QDate();
-      // if the only payment should have been prior to the reference date,
-      // then don't show it
-      if(m_startDate < refDate)
-        return QDate();
-      paymentDate = m_startDate;
-      break;
+  QDate paymentDate(nextDueDate());
 
-    case OCCUR_DAILY:
-      paymentDate = refDate.addDays(1);
-      break;
+  if(refDate >= paymentDate) {
+    switch (m_occurence)
+    {
+      case OCCUR_ONCE:
+        // if the lastPayment is already set, then there will be no more payments
+        // otherwise, the start date is the payment date
+        if(m_lastPayment.isValid())
+          return QDate();
+        // if the only payment should have been prior to the reference date,
+        // then don't show it
+        if(nextDueDate() < refDate)
+          return QDate();
+        paymentDate = nextDueDate();
+        break;
 
-    case OCCUR_WEEKLY:
-      while (paymentDate <= refDate)
-        paymentDate = paymentDate.addDays(7);
-      break;
+      case OCCUR_DAILY:
+        if (paymentDate <= refDate)
+          paymentDate = refDate.addDays(1);
+        else
+          paymentDate = paymentDate.addDays(1);
+        break;
 
-    case OCCUR_FORTNIGHTLY:
-    case OCCUR_EVERYOTHERWEEK:
-      while (paymentDate <= refDate)
-        paymentDate = paymentDate.addDays(14);
-      break;
+      case OCCUR_WEEKLY:
+        do {
+          paymentDate = paymentDate.addDays(7);
+        }
+        while (paymentDate <= refDate);
+        break;
 
-    case OCCUR_EVERYFOURWEEKS:
-      while (paymentDate <= refDate)
-        paymentDate = paymentDate.addDays(28);
-      break;
+      case OCCUR_FORTNIGHTLY:
+      case OCCUR_EVERYOTHERWEEK:
+        do
+        {
+          paymentDate = paymentDate.addDays(14);
+        }
+        while (paymentDate <= refDate);
+        break;
 
-    case OCCUR_MONTHLY:
-      while (paymentDate <= refDate) {
-        paymentDate = paymentDate.addMonths(1);
-        fixDate(paymentDate);
-      }
-      break;
+      case OCCUR_EVERYTHREEWEEKS:
+	do {
+          paymentDate = paymentDate.addDays(21);
+	}
+	while (paymentDate <= refDate);
+	break;
 
-    case OCCUR_EVERYOTHERMONTH:
-      while (paymentDate <= refDate) {
-        paymentDate = paymentDate.addMonths(2);
-        fixDate(paymentDate);
-      }
-      break;
+        case OCCUR_EVERYTHIRTYDAYS:
+        do
+        {
+          paymentDate = paymentDate.addDays(30);
+        }
+        while (paymentDate <= refDate);
+        break;
 
-    case OCCUR_QUARTERLY:
-    case OCCUR_EVERYTHREEMONTHS:
-      while (paymentDate <= refDate) {
-        paymentDate = paymentDate.addMonths(3);
-        fixDate(paymentDate);
-      }
-      break;
+      case OCCUR_EVERYFOURWEEKS:
+        do
+        {
+          paymentDate = paymentDate.addDays(28);
+        }
+        while (paymentDate <= refDate);
+        break;
 
-    case OCCUR_EVERYFOURMONTHS:
-      while (paymentDate <= refDate) {
-        paymentDate = paymentDate.addMonths(4);
-        fixDate(paymentDate);
-      }
-      break;
+      case OCCUR_MONTHLY:
+        do {
+          paymentDate = paymentDate.addMonths(1);
+          fixDate(paymentDate);
+        }
+        while (paymentDate <= refDate);
+        break;
 
-    case OCCUR_TWICEYEARLY:
-      while (paymentDate <= refDate) {
-        paymentDate = paymentDate.addMonths(6);
-        fixDate(paymentDate);
-      }
-      break;
+      case OCCUR_EVERYEIGHTWEEKS:
+        do
+        {
+          paymentDate = paymentDate.addDays(56);
+        }
+        while (paymentDate <= refDate);
+        break;
 
-    case OCCUR_YEARLY:
-      while (paymentDate <= refDate) {
-        paymentDate = paymentDate.addYears(1);
-        fixDate(paymentDate);
-      }
-      break;
+      case OCCUR_EVERYOTHERMONTH:
+        do {
+          paymentDate = paymentDate.addMonths(2);
+          fixDate(paymentDate);
+        }
+        while (paymentDate <= refDate);
+        break;
 
-    case OCCUR_EVERYOTHERYEAR:
-      while (paymentDate <= refDate) {
-        paymentDate = paymentDate.addYears(2);
-        fixDate(paymentDate);
-      }
-      break;
+      case OCCUR_QUARTERLY:
+      case OCCUR_EVERYTHREEMONTHS:
+        do {
+          paymentDate = paymentDate.addMonths(3);
+          fixDate(paymentDate);
+        }
+        while (paymentDate <= refDate);
+        break;
 
-    case OCCUR_ANY:
-      paymentDate = QDate();
-      break;
+      case OCCUR_EVERYFOURMONTHS:
+        do {
+          paymentDate = paymentDate.addMonths(4);
+          fixDate(paymentDate);
+        }
+        while (paymentDate <= refDate);
+        break;
+
+      case OCCUR_TWICEYEARLY:
+        do {
+          paymentDate = paymentDate.addMonths(6);
+          fixDate(paymentDate);
+        }
+        while (paymentDate <= refDate);
+        break;
+
+      case OCCUR_YEARLY:
+        do {
+          paymentDate = paymentDate.addYears(1);
+          fixDate(paymentDate);
+        }
+        while (paymentDate <= refDate);
+        break;
+
+      case OCCUR_EVERYOTHERYEAR:
+        do {
+          paymentDate = paymentDate.addYears(2);
+          fixDate(paymentDate);
+        }
+        while (paymentDate <= refDate);
+        break;
+
+      case OCCUR_ANY:
+        paymentDate = QDate();
+        break;
+    }
   }
   if(paymentDate.isValid()) {
     if(m_endDate.isValid() && paymentDate > m_endDate)
@@ -322,141 +497,191 @@ QDate MyMoneySchedule::nextPayment(const QDate& refDate) const
   return paymentDate;
 }
 
-QValueList<QDate> MyMoneySchedule::paymentDates(const QDate& startDate, const QDate& endDate) const
+QValueList<QDate> MyMoneySchedule::paymentDates(const QDate& _startDate, const QDate& _endDate) const
 {
-  QDate paymentDate(m_startDate);
+  QDate paymentDate(nextDueDate());
   QValueList<QDate> theDates;
 
+  QDate endDate(_endDate);
+  if ( willEnd() && m_endDate < endDate )
+    endDate = m_endDate;
+
   // if the period specified by the parameters and the period
-  // defined for this schedule, then the list remains empty
-  if ((willEnd() && m_endDate < startDate)
-  || m_startDate > endDate)
+  // defined for this schedule don't overlap, then the list remains empty
+  if ((willEnd() && m_endDate < _startDate)
+  || startDate() > endDate)
     return theDates;
 
   switch (m_occurence)
   {
     case OCCUR_ONCE:
-      if (m_startDate >= startDate && m_startDate <= endDate)
-        theDates.append(m_startDate);
+      if (startDate() >= _startDate && startDate() <= endDate)
+        theDates.append(startDate());
       break;
 
     case OCCUR_DAILY:
+      if (paymentDate < _startDate)
+        paymentDate = _startDate;
       while (paymentDate <= endDate)
       {
-        paymentDate = paymentDate.addDays(1);
         theDates.append(paymentDate);
+        paymentDate = paymentDate.addDays(1);
       }
       break;
 
     case OCCUR_WEEKLY:
-      while (paymentDate < startDate)
+      while (paymentDate < _startDate)
         paymentDate = paymentDate.addDays(7);
-
       while (paymentDate <= endDate)
       {
-        paymentDate = paymentDate.addDays(7);
         theDates.append(paymentDate);
+        paymentDate = paymentDate.addDays(7);
       }
       break;
 
     case OCCUR_FORTNIGHTLY:
     case OCCUR_EVERYOTHERWEEK:
-      while (paymentDate < startDate)
+      while (paymentDate < _startDate)
         paymentDate = paymentDate.addDays(14);
       while (paymentDate <= endDate)
       {
-        paymentDate = paymentDate.addDays(14);
         theDates.append(paymentDate);
+        paymentDate = paymentDate.addDays(14);
+      }
+      break;
+
+    case OCCUR_EVERYTHREEWEEKS:
+      while (paymentDate < _startDate)
+        paymentDate = paymentDate.addDays(21);
+      while (paymentDate <= endDate)
+      {
+        theDates.append(paymentDate);
+        paymentDate = paymentDate.addDays(21);
+      }
+      break;
+
+    case OCCUR_EVERYTHIRTYDAYS:
+      while (paymentDate < _startDate)
+        paymentDate = paymentDate.addDays(30);
+      while (paymentDate <= endDate)
+      {
+        theDates.append(paymentDate);
+        paymentDate = paymentDate.addDays(30);
       }
       break;
 
     case OCCUR_EVERYFOURWEEKS:
-      while (paymentDate < startDate)
+      while (paymentDate < _startDate)
         paymentDate = paymentDate.addDays(28);
       while (paymentDate <= endDate)
       {
-        paymentDate = paymentDate.addDays(28);
         theDates.append(paymentDate);
+        paymentDate = paymentDate.addDays(28);
       }
       break;
 
     case OCCUR_MONTHLY:
-      while (paymentDate < startDate)
-        paymentDate = paymentDate.addMonths(1);
-      while (paymentDate <= endDate)
-      {
+      while (paymentDate < _startDate) {
         paymentDate = paymentDate.addMonths(1);
         fixDate(paymentDate);
+      }
+      while (paymentDate <= endDate)
+      {
         theDates.append(paymentDate);
+        paymentDate = paymentDate.addMonths(1);
+        fixDate(paymentDate);
+      }
+      break;
+
+    case OCCUR_EVERYEIGHTWEEKS:
+      while (paymentDate < _startDate)
+        paymentDate = paymentDate.addDays(56);
+      while (paymentDate <= endDate)
+      {
+        theDates.append(paymentDate);
+        paymentDate = paymentDate.addDays(56);
       }
       break;
 
     case OCCUR_EVERYOTHERMONTH:
-      while (paymentDate < startDate)
-        paymentDate = paymentDate.addMonths(2);
-      while (paymentDate <= endDate)
-      {
+      while (paymentDate < _startDate) {
         paymentDate = paymentDate.addMonths(2);
         fixDate(paymentDate);
+      }
+      while (paymentDate <= endDate)
+      {
         theDates.append(paymentDate);
+        paymentDate = paymentDate.addMonths(2);
+        fixDate(paymentDate);
       }
       break;
 
     case OCCUR_QUARTERLY:
     case OCCUR_EVERYTHREEMONTHS:
-      while (paymentDate < startDate)
-        paymentDate = paymentDate.addMonths(3);
-      while (paymentDate <= endDate)
-      {
+      while (paymentDate < _startDate) {
         paymentDate = paymentDate.addMonths(3);
         fixDate(paymentDate);
+      }
+      while (paymentDate <= endDate)
+      {
         theDates.append(paymentDate);
+        paymentDate = paymentDate.addMonths(3);
+        fixDate(paymentDate);
       }
       break;
 
     case OCCUR_EVERYFOURMONTHS:
-      while (paymentDate < startDate)
-        paymentDate = paymentDate.addMonths(4);
-      while (paymentDate <= endDate)
-      {
+      while (paymentDate < _startDate) {
         paymentDate = paymentDate.addMonths(4);
         fixDate(paymentDate);
+      }
+      while (paymentDate <= endDate)
+      {
         theDates.append(paymentDate);
+        paymentDate = paymentDate.addMonths(4);
+        fixDate(paymentDate);
       }
       break;
 
     case OCCUR_TWICEYEARLY:
-      while (paymentDate < startDate)
-        paymentDate = paymentDate.addMonths(6);
-      while (paymentDate <= endDate)
-      {
+      while (paymentDate < _startDate) {
         paymentDate = paymentDate.addMonths(6);
         fixDate(paymentDate);
+      }
+      while (paymentDate <= endDate)
+      {
         theDates.append(paymentDate);
+        paymentDate = paymentDate.addMonths(6);
+        fixDate(paymentDate);
       }
       break;
 
     case OCCUR_YEARLY:
-      while (paymentDate < startDate)
-        paymentDate = paymentDate.addYears(1);
-      while (paymentDate <= endDate)
-      {
+      while (paymentDate < _startDate) {
         paymentDate = paymentDate.addYears(1);
         fixDate(paymentDate);
+      }
+      while (paymentDate <= endDate)
+      {
         theDates.append(paymentDate);
+        paymentDate = paymentDate.addYears(1);
+        fixDate(paymentDate);
       }
       break;
 
     case OCCUR_EVERYOTHERYEAR:
-      while (paymentDate < startDate)
-        paymentDate = paymentDate.addYears(2);
-      while (paymentDate <= endDate)
-      {
+      while (paymentDate < _startDate) {
         paymentDate = paymentDate.addYears(2);
         fixDate(paymentDate);
+      }
+      while (paymentDate <= endDate)
+      {
         theDates.append(paymentDate);
+        paymentDate = paymentDate.addYears(2);
+        fixDate(paymentDate);
       }
       break;
+
     case OCCUR_ANY:
       break;
   }
@@ -467,12 +692,13 @@ QValueList<QDate> MyMoneySchedule::paymentDates(const QDate& startDate, const QD
 
 bool MyMoneySchedule::operator <(const MyMoneySchedule& right)
 {
-  return nextPayment(m_lastPayment) < right.nextPayment(right.m_lastPayment);
+  return nextDueDate() < right.nextDueDate();
 }
 
 bool MyMoneySchedule::operator ==(const MyMoneySchedule& right)
 {
-  if (  m_occurence == right.m_occurence &&
+  if (  MyMoneyObject::operator==(right) &&
+        m_occurence == right.m_occurence &&
         m_type == right.m_type &&
         m_startDate == right.m_startDate &&
         m_paymentType == right.m_paymentType &&
@@ -480,7 +706,6 @@ bool MyMoneySchedule::operator ==(const MyMoneySchedule& right)
         m_transaction == right.m_transaction &&
         m_endDate == right.m_endDate &&
         m_autoEnter == right.m_autoEnter &&
-        m_id == right.m_id &&
         m_lastPayment == right.m_lastPayment &&
         ((m_name.length() == 0 && right.m_name.length() == 0) || (m_name == right.m_name)))
     return true;
@@ -500,7 +725,7 @@ int MyMoneySchedule::transactionsRemaining(void) const
   return counter;
 }
 
-const MyMoneyAccount MyMoneySchedule::account(int cnt) const
+MyMoneyAccount MyMoneySchedule::account(int cnt) const
 {
   QValueList<MyMoneySplit> splits = m_transaction.splits();
   QValueList<MyMoneySplit>::ConstIterator it;
@@ -509,18 +734,18 @@ const MyMoneyAccount MyMoneySchedule::account(int cnt) const
 
   // search the first asset or liability account
   for(it = splits.begin(); it != splits.end() && (acc.id().isEmpty() || cnt); ++it) {
-    acc = file->account((*it).accountId());
-    switch(file->accountGroup(acc.accountType())) {
-      case MyMoneyAccount::Liability:
-      case MyMoneyAccount::Asset:
-        --cnt;
-        break;
+    try {
+      acc = file->account((*it).accountId());
+      if(acc.isAssetLiability())
+          --cnt;
 
-      default:
-        break;
+      if(!cnt)
+        return acc;
+    } catch(MyMoneyException *e) {
+      qWarning("Schedule '%s' references unknown account '%s'", id().data(),   (*it).accountId().data());
+      delete e;
+      return MyMoneyAccount();
     }
-    if(!cnt)
-      return acc;
   }
 
   return MyMoneyAccount();
@@ -529,7 +754,7 @@ const MyMoneyAccount MyMoneySchedule::account(int cnt) const
 QDate MyMoneySchedule::dateAfter(int transactions) const
 {
   int counter=1;
-  QDate paymentDate(m_startDate);
+  QDate paymentDate(startDate());
 
   if (transactions<=0)
     return paymentDate;
@@ -555,6 +780,16 @@ QDate MyMoneySchedule::dateAfter(int transactions) const
         paymentDate = paymentDate.addDays(14);
       break;
 
+    case OCCUR_EVERYTHREEWEEKS:
+      while (counter++ < transactions)
+        paymentDate = paymentDate.addDays(21);
+      break;
+
+    case OCCUR_EVERYTHIRTYDAYS:
+      while (counter++ < transactions)
+        paymentDate = paymentDate.addDays(30);
+      break;
+
     case OCCUR_EVERYFOURWEEKS:
       while (counter++ < transactions)
         paymentDate = paymentDate.addDays(28);
@@ -565,6 +800,11 @@ QDate MyMoneySchedule::dateAfter(int transactions) const
         paymentDate = paymentDate.addMonths(1);
       break;
 
+    case OCCUR_EVERYEIGHTWEEKS:
+      while (counter++ < transactions)
+        paymentDate = paymentDate.addDays(56);
+      break;
+
     case OCCUR_EVERYOTHERMONTH:
       while (counter++ < transactions)
         paymentDate = paymentDate.addMonths(2);
@@ -607,6 +847,12 @@ bool MyMoneySchedule::isOverdue() const
   if (isFinished())
     return false;
 
+  if(nextDueDate() >= QDate::currentDate())
+    return false;
+
+  return true;
+
+#if 0
   bool bOverdue = true;
 
   // Check the payment dates first
@@ -655,23 +901,24 @@ bool MyMoneySchedule::isOverdue() const
     if (datesBeforeToday.count() == 0)
       bOverdue = false;
   }
-
   return bOverdue;
+#endif
 }
 
 bool MyMoneySchedule::isFinished() const
 {
-  if (m_endDate.isValid()) {
-    if(m_lastPayment >= m_endDate)
-      return true;
+  if(!m_lastPayment.isValid())
+    return false;
 
-    QDate next = nextPayment(m_lastPayment);
-    if(!next.isValid() || next > m_endDate)
+  if (m_endDate.isValid()) {
+    if(m_lastPayment >= m_endDate
+    || !nextDueDate().isValid()
+    || nextDueDate() > m_endDate)
       return true;
   }
 
   // Check to see if its a once off payment
-  if (m_occurence == MyMoneySchedule::OCCUR_ONCE && m_lastPayment.isValid())
+  if (m_occurence == MyMoneySchedule::OCCUR_ONCE)
     return true;
 
   return false;
@@ -711,8 +958,49 @@ void MyMoneySchedule::setWeekendOption(const weekendOptionE option)
 
 void MyMoneySchedule::fixDate(QDate& date) const
 {
-  if(date.day() != m_startDate.day()
-  && QDate::isValid(date.year(), date.month(), m_startDate.day())) {
-    date.setYMD(date.year(), date.month(), m_startDate.day());
+  QDate fixDate(m_startDate);
+  if(fixDate.isValid()
+  && date.day() != fixDate.day()
+  && QDate::isValid(date.year(), date.month(), fixDate.day())) {
+    date.setYMD(date.year(), date.month(), fixDate.day());
   }
 }
+
+void MyMoneySchedule::writeXML(QDomDocument& document, QDomElement& parent) const
+{
+  QDomElement el = document.createElement("SCHEDULED_TX");
+
+  writeBaseXML(document, el);
+
+  el.setAttribute("name", m_name);
+  el.setAttribute("type", m_type);
+  el.setAttribute("occurence", m_occurence);
+  el.setAttribute("paymentType", m_paymentType);
+  el.setAttribute("startDate", dateToString(m_startDate));
+  el.setAttribute("endDate", dateToString(m_endDate));
+  el.setAttribute("fixed", m_fixed);
+  el.setAttribute("autoEnter", m_autoEnter);
+  el.setAttribute("lastPayment", dateToString(m_lastPayment));
+  el.setAttribute("weekendOption", m_weekendOption);
+
+  //store the payment history for this scheduled task.
+  QValueList<QDate> payments = recordedPayments();
+  QValueList<QDate>::ConstIterator it;
+  QDomElement paymentsElement = document.createElement("PAYMENTS");
+  for (it=payments.begin(); it!=payments.end(); ++it) {
+    QDomElement paymentEntry = document.createElement("PAYMENT");
+    paymentEntry.setAttribute("date", dateToString(*it));
+    paymentsElement.appendChild(paymentEntry);
+  }
+  el.appendChild(paymentsElement);
+
+  //store the transaction data for this task.
+  m_transaction.writeXML(document, el);
+
+  parent.appendChild(el);
+}
+
+bool MyMoneySchedule::hasReferenceTo(const QCString& id) const
+{
+  return m_transaction.hasReferenceTo(id);
+}
