@@ -2,12 +2,9 @@
                           mymoneyscheduled.h
                              -------------------
     copyright            : (C) 2000-2002 by Michael Edwardes
+                           (C) 2007 by Thomas Baumgart
     email                : mte@users.sourceforge.net
-                           Javier Campos Morales <javi_c@users.sourceforge.net>
-                           Felix Rodriguez <frodriguez@users.sourceforge.net>
-                           John C <thetacoturtle@users.sourceforge.net>
                            Thomas Baumgart <ipwizard@users.sourceforge.net>
-                           Kevin Tambascio <ktambascio@users.sourceforge.net>
  ***************************************************************************/
 
 /***************************************************************************
@@ -40,6 +37,7 @@
 #include "mymoneytransaction.h"
 #include "mymoneyaccount.h"
 #include <kmymoney/export.h>
+#include <kmymoney/mymoneyobject.h>
 
 /**
   * @author Michael Edwardes
@@ -52,14 +50,19 @@
   * @short A class to represent a schedule.
   * @see MyMoneyScheduled
   */
-class KMYMONEY_EXPORT MyMoneySchedule {
+class KMYMONEY_EXPORT MyMoneySchedule : public MyMoneyObject
+{
 public:
   /**
     * This enum is used to describe all the possible schedule frequencies.
     * The special entry, OCCUR_ANY, is used to combine all the other types.
     */
   enum occurenceE { OCCUR_ANY=0, OCCUR_ONCE=1, OCCUR_DAILY=2, OCCUR_WEEKLY=4, OCCUR_FORTNIGHTLY=8,
-                    OCCUR_EVERYOTHERWEEK=16, OCCUR_MONTHLY=32, OCCUR_EVERYFOURWEEKS=64,
+                    OCCUR_EVERYOTHERWEEK=16,
+                    OCCUR_EVERYTHREEWEEKS=20,
+                    OCCUR_EVERYTHIRTYDAYS=30,
+                    OCCUR_MONTHLY=32, OCCUR_EVERYFOURWEEKS=64,
+                    OCCUR_EVERYEIGHTWEEKS=126,
                     OCCUR_EVERYOTHERMONTH=128, OCCUR_EVERYTHREEMONTHS=256,
                     OCCUR_TWICEYEARLY=1024, OCCUR_EVERYOTHERYEAR=2048, OCCUR_QUARTERLY=4096,
                     OCCUR_EVERYFOURMONTHS=8192, OCCUR_YEARLY=16384
@@ -75,7 +78,9 @@ public:
     */
   enum paymentTypeE { STYPE_ANY=0, STYPE_DIRECTDEBIT=1, STYPE_DIRECTDEPOSIT=2,
                       STYPE_MANUALDEPOSIT=4, STYPE_OTHER=8,
-                      STYPE_WRITECHEQUE=16 };
+                      STYPE_WRITECHEQUE=16,
+                      STYPE_STANDINGORDER=32,
+                      STYPE_BANKTRANSFER=64 };
 
   /**
     * This enum is used by the auto-commit functionality.
@@ -95,15 +100,20 @@ public:
     *
     * Please note that the optional fields are not set and the transaction
     * MUST be set before it can be used.
+    *
+    * @a startDate is not used anymore and internally set to QDate()
     */
   MyMoneySchedule(const QString& name, typeE type, occurenceE occurence, paymentTypeE paymentType,
-        const QDate& startDate, const QDate& endDate, bool fixed, bool autoEnter);
+          const QDate& startDate, const QDate& endDate, bool fixed, bool autoEnter);
+
+  MyMoneySchedule(const QDomElement& node);
+
+  MyMoneySchedule(const QCString& id, const MyMoneySchedule& right);
 
   /**
     * Standard destructor
     */
-  ~MyMoneySchedule() {
-  }
+  ~MyMoneySchedule() {}
 
   /**
     * Simple get method that returns the occurence frequency.
@@ -120,11 +130,14 @@ public:
   typeE type(void) const { return m_type; }
 
   /**
-    * Simple get method that returns the schedule startDate.
+    * Simple get method that returns the schedule startDate. If
+    * the schedule has been executed once, the date of the first
+    * execution is returned. Otherwise, the next due date is
+    * returned.
     *
-    * @return QDate The instance startDate.
+    * @return reference to QDate containing the start date.
     */
-  QDate startDate(void) const { return m_startDate; }
+  const QDate& startDate(void) const;
 
   /**
     * Simple get method that returns the schedule paymentType.
@@ -160,7 +173,7 @@ public:
     *
     * @return QDate The end date for the instance.
     */
-  const QDate endDate(void) const { return m_endDate; }
+  const QDate& endDate(void) const { return m_endDate; }
 
   /**
     * Simple get method that returns true if the transaction should be
@@ -168,7 +181,7 @@ public:
     *
     * @return bool Indicates whether the instance will be automatically entered.
     */
-  const bool autoEnter(void) const { return m_autoEnter; }
+  bool autoEnter(void) const { return m_autoEnter; }
 
   /**
     * Simple get method that returns the transaction data for the schedule.
@@ -178,18 +191,35 @@ public:
   const MyMoneyTransaction& transaction(void) const { return m_transaction; }
 
   /**
-    * Simple method that returns the schedule id.
+    * Simple method that returns the schedules last payment. If the
+    * schedule has never been executed, QDate() will be returned.
     *
-    * @return QString The instances id.
+    * @return QDate The last payment for the schedule.
     */
-  const QCString id(void) const { return m_id; }
+  const QDate& lastPayment(void) const { return m_lastPayment; }
 
   /**
-    * Simple method that returns the schedule last payment.
+    * Simple method that returns the next due date for the schedule.
     *
-    * @return QDate The last payment for the instance.
+    * @return reference to QDate containing the next due date.
+    *
+    * @note The date returned can represent a value that is past
+    *       a possible end of the schedule. Make sure to consider
+    *       the return value of isFinished() when using the value returned.
     */
-  const QDate lastPayment(void) const { return m_lastPayment; }
+  const QDate& nextDueDate(void) const;
+
+  /**
+    * This method adjusts returns the next due date adjusted
+    * according to the rules specified by the schedule's weekend option.
+    *
+    * @return QDate containing the adjusted next due date. If the
+    *         schedule is finished (@sa isFinished()) then the method
+    *         returns an invalid QDate.
+    *
+    * @sa weekendOption()
+    */
+  QDate adjustedNextDueDate(void) const;
 
   /**
     * Get the weekendOption that determines how the schedule check code
@@ -241,6 +271,8 @@ public:
 
   /**
     * Simple method that sets the transaction for the schedule.
+    * The transaction must have a valid postDate set, otherwise
+    * it will not be accepted.
     *
     * @param transaction The new transaction.
     * @return none
@@ -266,20 +298,21 @@ public:
   void setAutoEnter(bool autoenter);
 
   /**
-    * Simple set method to set the schedule's id.  DO NOT USE, EVER!
+    * Simple set method to set the schedule's next payment date.
     *
-    * @param id The instances id.
+    * @param date The next payment date.
     * @return none
-    *
-    * @note DO NOT USE, EVER IN APPLICATION CODE! This method is for test purposes only!
     */
-  void setId(const QCString& id)
-    { m_id = id; }
+  void setNextDueDate(const QDate& date);
 
   /**
-    * Simple set method to set the schedule's last payment.
+    * Simple set method to set the schedule's last payment. If
+    * this method is called for the first time on the object,
+    * the @a m_startDate member will be set to @a date as well.
     *
-    * @param date The instances last payment date.
+    * This method should be called whenever a schedule is entered or skipped.
+    *
+    * @param date The last payment date.
     * @return none
     */
   void setLastPayment(const QDate& date);
@@ -348,7 +381,7 @@ public:
     *
     * @return The name
     */
-  const QString name(void) const { return m_name; }
+  const QString& name(void) const { return m_name; }
 
   /**
     * Changes the instance name
@@ -362,8 +395,8 @@ public:
 
   bool operator <(const MyMoneySchedule& right);
 
-  const MyMoneyAccount account(int cnt = 1) const;
-  const MyMoneyAccount transferAccount(void) const { return account(2); };
+  MyMoneyAccount account(int cnt = 1) const;
+  MyMoneyAccount transferAccount(void) const { return account(2); };
   QDate dateAfter(int transactions) const;
 
   bool isOverdue() const;
@@ -372,6 +405,19 @@ public:
   void recordPayment(const QDate&);
   QValueList<QDate> recordedPayments(void) const { return m_recordedPayments; }
 
+  void writeXML(QDomDocument& document, QDomElement& parent) const;
+
+  /**
+    * This method checks if a reference to the given object exists. It returns,
+    * a @p true if the object is referencing the one requested by the
+    * parameter @p id. If it does not, this method returns @p false.
+    *
+    * @param id id of the object to be checked for references
+    * @retval true This object references object with id @p id.
+    * @retval false This object does not reference the object with id @p id.
+    */
+  virtual bool hasReferenceTo(const QCString& id) const;
+
 private:
   /**
     * This method forces the day of the passed @p date to
@@ -408,9 +454,6 @@ private:
   /// Enter the transaction into the register automatically
   bool m_autoEnter;
 
-  /// The internal id.
-  QCString m_id;
-
   /// Internal date used for calculations
   QDate m_lastPayment;
 
