@@ -43,7 +43,7 @@ void MyMoneyStatement::write(QDomElement& _root,QDomDocument* _doc) const
   QDomElement e = _doc->createElement("STATEMENT");
   _root.appendChild(e);
 
-  e.setAttribute("verson","1.1");
+  e.setAttribute("version","1.1");
   e.setAttribute("accountname", m_strAccountName);
   e.setAttribute("accountnumber", m_strAccountNumber);
   e.setAttribute("currency", m_strCurrency);
@@ -87,7 +87,21 @@ void MyMoneyStatement::write(QDomElement& _root,QDomDocument* _doc) const
 
     e.appendChild(p);
 
-    ++it_t;
+    ++it_p;
+  }
+
+  // iterate over securities, and add each one
+  QValueList<Security>::const_iterator it_s = m_listSecurities.begin();
+  while ( it_s != m_listSecurities.end() )
+  {
+    QDomElement p = _doc->createElement("SECURITY");
+    p.setAttribute("name", (*it_s).m_strName);
+    p.setAttribute("symbol", (*it_s).m_strSymbol);
+    p.setAttribute("id", (*it_s).m_strId);
+
+    e.appendChild(p);
+
+    ++it_s;
   }
 
 }
@@ -137,7 +151,6 @@ bool MyMoneyStatement::read(const QDomElement& _e)
         }
 
         m_listTransactions += t;
-        child = child.nextSibling();
       }
       else if ( c.tagName() == "PRICE" )
       {
@@ -148,8 +161,18 @@ bool MyMoneyStatement::read(const QDomElement& _e)
         p.m_amount = MyMoneyMoney(c.attribute("amount"));
 
         m_listPrices += p;
-        child = child.nextSibling();
       }
+      else if ( c.tagName() == "SECURITY" )
+      {
+        MyMoneyStatement::Security s;
+
+        s.m_strName = c.attribute("name");
+        s.m_strSymbol = c.attribute("symbol");
+        s.m_strId = c.attribute("id");
+
+        m_listSecurities += s;
+      }
+      child = child.nextSibling();
     }
   }
 
@@ -199,14 +222,8 @@ void MyMoneyStatement::writeXMLFile( const MyMoneyStatement& _s, const QString&
   g.open( IO_WriteOnly );
 
   QTextStream stream(&g);
-#if KDE_IS_VERSION(3,2,0)
   stream.setEncoding(QTextStream::UnicodeUTF8);
   stream << doc->toString();
-#else
-  //stream.setEncoding(QTextStream::Locale);
-  QCString temp = doc->toCString();
-  stream << temp.data();
-#endif
   g.close();
 
   delete doc;
@@ -238,3 +255,4 @@ bool MyMoneyStatement::readXMLFile( MyMoneyStatement& _s, const QString& _filena
 
   return result;
 }
+// vim:cin:si:ai:et:ts=2:sw=2:
