@@ -213,21 +213,63 @@ void MyMoneyTransactionFilter::setConsiderCategory(const bool check)
   m_considerCategory = check;
 }
 
-const QValueList<MyMoneySplit> MyMoneyTransactionFilter::matchingSplits(void) const
+const QValueList<MyMoneySplit>& MyMoneyTransactionFilter::matchingSplits(void) const
 {
   return m_matchingSplits;
 }
 
+bool MyMoneyTransactionFilter::match(const MyMoneySplit * const sp, const IMyMoneyStorage* const storage) const
+{
+  bool rc = true;
+
+  // check if the text is contained in one of the fields
+  // memo, value, number, payee, account, date
+  if(m_filterSet.singleFilter.textFilter) {
+    bool textMatch = false;
+    const MyMoneyAccount& acc = storage->account(sp->accountId());
+    const MyMoneySecurity& sec = storage->security(acc.currencyId());
+    if(sp->memo().contains(m_text)
+    || sp->value().formatMoney(acc.fraction(sec)).contains(m_text)
+    || sp->number().contains(m_text))
+      textMatch = true;
+
+    if(!textMatch && acc.name().contains(m_text))
+      textMatch = true;
+
+    if(!textMatch && !sp->payeeId().isEmpty()) {
+      const MyMoneyPayee& payee = storage->payee(sp->payeeId());
+      if(payee.name().contains(m_text))
+        textMatch = true;
+    }
+
+    if(!textMatch)
+      rc = false;
+  }
+
+  if(rc && m_filterSet.singleFilter.amountFilter) {
+    if(((sp->value().abs() < m_fromAmount) || sp->value().abs() > m_toAmount)
+    && ((sp->shares().abs() < m_fromAmount) || sp->shares().abs() > m_toAmount))
+      rc = false;
+  }
+
+  return rc;
+}
+
 const bool MyMoneyTransactionFilter::match(const MyMoneyTransaction& transaction, const IMyMoneyStorage* const storage)
 {
-  QValueList<MyMoneySplit>::Iterator it;
+  QValueList<MyMoneySplit>::ConstIterator it;
 
   m_matchingSplits.clear();
 
   // qDebug("T: %s", transaction.id().data());
   // if no filter is set, we can savely return a match
-  if(!m_filterSet.allFilter)
+  // if we should report all splits, then we collect them
+  if(!m_filterSet.allFilter) {
+    if(m_reportAllSplits) {
+      m_matchingSplits = transaction.splits();
+    }
     return true;
+  }
 
   // perform checks on the MyMoneyTransaction object first
 
@@ -244,9 +286,13 @@ const bool MyMoneyTransactionFilter::match(const MyMoneyTransaction& transaction
     }
   }
 
-  // construct a local copy of all splits and
-  // remove all the splits that do not match account and/or categories.
-  m_matchingSplits = transaction.splits();
+  // construct a local list of pointers to all splits and
+  // remove the ones that do not match account and/or categories.
+
+  QPtrList<MyMoneySplit> matchingSplits;
+  for(it = transaction.splits().begin(); it != transaction.splits().end(); ++it) {
+    matchingSplits.append(&(*it));
+  }
 
   bool categoryMatched = !m_filterSet.singleFilter.categoryFilter;
   bool accountMatched = !m_filterSet.singleFilter.accountFilter;
@@ -260,11 +306,13 @@ const bool MyMoneyTransactionFilter::match(const MyMoneyTransaction& transaction
     }
   }
 
+  MyMoneySplit* sp;
+
   if(m_filterSet.singleFilter.accountFilter == 1
   || m_filterSet.singleFilter.categoryFilter == 1) {
-    for(it = m_matchingSplits.begin(); it != m_matchingSplits.end(); ) {
-      bool removeSplit = true;
-      MyMoneyAccount acc = storage->account((*it).accountId());
+    for(sp = matchingSplits.first(); sp != 0; ) {
+      MyMoneySplit* removeSplit = sp;
+      const MyMoneyAccount& acc = storage->account(sp->accountId());
       if(m_considerCategory) {
         switch(acc.accountGroup()) {
           case MyMoneyAccount::Income:
@@ -273,9 +321,9 @@ const bool MyMoneyTransactionFilter::match(const MyMoneyTransaction& transaction
             // check if the split references one of the categories in the list
             if(m_filterSet.singleFilter.categoryFilter) {
               if(m_categories.count() > 0) {
-                if(m_categories.find((*it).accountId())) {
+                if(m_categories.find(sp->accountId())) {
                   categoryMatched = true;
-                  removeSplit = false;
+                  removeSplit = 0;
                 }
               } else {
                 // we're looking for transactions with 'no' categories
@@ -288,13 +336,13 @@ const bool MyMoneyTransactionFilter::match(const MyMoneyTransaction& transaction
             // check if the split references one of the accounts in the list
             if(m_filterSet.singleFilter.accountFilter) {
               if(m_accounts.count() > 0) {
-                if(m_accounts.find((*it).accountId())) {
+                if(m_accounts.find(sp->accountId())) {
                   accountMatched = true;
-                  removeSplit = false;
+                  removeSplit = 0;
                 }
               }
             } else
-              removeSplit = false;
+              removeSplit = 0;
 
             break;
         }
@@ -302,20 +350,19 @@ const bool MyMoneyTransactionFilter::match(const MyMoneyTransaction& transaction
       } else {
         if(m_filterSet.singleFilter.accountFilter) {
           if(m_accounts.count() > 0) {
-            if(m_accounts.find((*it).accountId())) {
+            if(m_accounts.find(sp->accountId())) {
               accountMatched = true;
-              removeSplit = false;
+              removeSplit = 0;
             }
           }
         } else
-          removeSplit = false;
+          removeSplit = 0;
       }
 
+      sp = matchingSplits.next();
       if(removeSplit) {
         // qDebug(" S: %s", (*it).id().data());
-        it = m_matchingSplits.remove(it);
-      } else {
-        ++it;
+        matchingSplits.remove(removeSplit);
       }
     }
   }
@@ -331,7 +378,7 @@ const bool MyMoneyTransactionFilter::match(const MyMoneyTransaction& transaction
   if(!categoryMatched && !m_filterSet.singleFilter.categoryFilter)
     categoryMatched = isTransfer;
 
-  if(m_matchingSplits.count() == 0
+  if(matchingSplits.count() == 0
   || !(accountMatched && categoryMatched))
     return false;
 
@@ -342,9 +389,24 @@ const bool MyMoneyTransactionFilter::match(const MyMoneyTransaction& transaction
 
   // check if we still have something to do
   if(filterSet.allFilter != 0) {
-    for(it = m_matchingSplits.begin(); it != m_matchingSplits.end();) {
-      bool removeSplit = false;
-      MyMoneyAccount acc = storage->account((*it).accountId());
+    for(sp = matchingSplits.first(); sp != 0;) {
+      MyMoneySplit* removeSplit = match(sp, storage) ? 0 : sp;
+
+      const MyMoneyAccount& acc = storage->account(sp->accountId());
+
+      // Determine if this account is a category or an account
+      bool isCategory = false;
+      switch(acc.accountGroup()) {
+        case MyMoneyAccount::Income:
+        case MyMoneyAccount::Expense:
+          isCategory = true;
+        default:
+          break;
+      }
+
+#if 0
+      // TODO code moved to match() and should be removed here
+      const MyMoneyAccount& acc = storage->account(sp->accountId());
 
       // Determine if this account is a category or an account
       bool isCategory = false;
@@ -360,108 +422,116 @@ const bool MyMoneyTransactionFilter::match(const MyMoneyTransaction& transaction
       // memo, value, number, payee, account, date
       if(m_filterSet.singleFilter.textFilter) {
         bool textMatch = false;
-        if((*it).memo().contains(m_text)
-        || (*it).value().formatMoney().contains(m_text)
-        || (*it).number().contains(m_text))
+        if(sp->memo().contains(m_text)
+        || sp->value().formatMoney().contains(m_text)
+        || sp->number().contains(m_text))
           textMatch = true;
 
         if(!textMatch && acc.name().contains(m_text))
           textMatch = true;
 
-        if(!textMatch && !(*it).payeeId().isEmpty()) {
-          MyMoneyPayee payee;
-          payee = storage->payee((*it).payeeId());
+        if(!textMatch && !sp->payeeId().isEmpty()) {
+          const MyMoneyPayee& payee = storage->payee(sp->payeeId());
           if(payee.name().contains(m_text))
             textMatch = true;
         }
 
         if(!textMatch)
-          removeSplit = true;
+          removeSplit = sp;
       }
 
       if(!removeSplit && m_filterSet.singleFilter.amountFilter) {
-        if((*it).value().abs() < m_fromAmount)
-          removeSplit = true;
-        if((*it).value().abs() > m_toAmount)
-          removeSplit = true;
+        if(sp->value().abs() < m_fromAmount)
+          removeSplit = sp;
+        if(sp->value().abs() > m_toAmount)
+          removeSplit = sp;
       }
+#endif
 
       if(!isCategory && !removeSplit) {
         // check the payee list
         if(!removeSplit && m_filterSet.singleFilter.payeeFilter) {
           if(m_payees.count() > 0) {
-            if((*it).payeeId().isEmpty() || !m_payees.find((*it).payeeId()))
-              removeSplit = true;
-          } else if(!(*it).payeeId().isEmpty())
-              removeSplit = true;
+            if(sp->payeeId().isEmpty() || !m_payees.find(sp->payeeId()))
+              removeSplit = sp;
+          } else if(!sp->payeeId().isEmpty())
+              removeSplit = sp;
         }
 
         // check the type list
         if(!removeSplit && m_filterSet.singleFilter.typeFilter) {
           if(m_types.count() > 0) {
-            if(!m_types.find(splitType(storage, transaction, *it)))
-              removeSplit = true;
+            if(!m_types.find(splitType(storage, transaction, *sp)))
+              removeSplit = sp;
           }
         }
 
         // check the state list
         if(!removeSplit && m_filterSet.singleFilter.stateFilter) {
           if(m_states.count() > 0) {
-            if(!m_states.find(splitState(*it)))
-              removeSplit = true;
+            if(!m_states.find(splitState(*sp)))
+              removeSplit = sp;
           }
         }
 
         if(!removeSplit && m_filterSet.singleFilter.nrFilter) {
           if(!m_fromNr.isEmpty()) {
-            if((*it).number() < m_fromNr)
-              removeSplit = true;
+            if(sp->number() < m_fromNr)
+              removeSplit = sp;
           }
           if(!m_toNr.isEmpty()) {
-            if((*it).number() > m_toNr)
-              removeSplit = true;
+            if(sp->number() > m_toNr)
+              removeSplit = sp;
           }
         }
       } else if(m_filterSet.singleFilter.payeeFilter
       || m_filterSet.singleFilter.typeFilter
       || m_filterSet.singleFilter.stateFilter
       || m_filterSet.singleFilter.nrFilter)
-        removeSplit = true;
+        removeSplit = sp;
 
+      sp = matchingSplits.next();
       if(removeSplit) {
         // qDebug(" S: %s", (*it).id().data());
-        it = m_matchingSplits.remove(it);
-      } else {
-        ++it;
+        matchingSplits.remove(removeSplit);
       }
     }
   }
 
-  if(m_reportAllSplits == false && m_matchingSplits.count() != 0) {
-    m_matchingSplits.clear();
+  if(m_reportAllSplits == false && matchingSplits.count() != 0) {
     m_matchingSplits.append(transaction.splits()[0]);
+  } else {
+    for(sp = matchingSplits.first(); sp != 0; sp = matchingSplits.next()) {
+      m_matchingSplits.append(*sp);
+    }
   }
   // all filters passed, I guess we have a match
   // qDebug("  C: %d", m_matchingSplits.count());
-  return m_matchingSplits.count() != 0;
+  return matchingSplits.count() != 0;
 }
 
 const int MyMoneyTransactionFilter::splitState(const MyMoneySplit& split) const
 {
+  int rc = notReconciled;
+
   switch(split.reconcileFlag()) {
+    default:
     case MyMoneySplit::NotReconciled:
-      return notReconciled;
+      break;;
 
     case MyMoneySplit::Cleared:
-      return cleared;
+      rc = cleared;
+      break;
 
     case MyMoneySplit::Reconciled:
-      return reconciled;
+      rc = reconciled;
+      break;
 
     case MyMoneySplit::Frozen:
-      return frozen;
+      rc = frozen;
+      break;
   }
-  return notReconciled;
+  return rc;
 }
 
 const int MyMoneyTransactionFilter::splitType(const IMyMoneyStorage* const storage, const MyMoneyTransaction& t, const MyMoneySplit& split) const
@@ -652,94 +722,170 @@ const bool MyMoneyTransactionFilter::textFilter(QRegExp& exp) const
   return m_filterSet.singleFilter.textFilter == 1;
 }
 
-void MyMoneyTransactionFilter::setDateFilter(unsigned range)
+void MyMoneyTransactionFilter::setDateFilter(dateOptionE range)
 {
   QDate from, to;
   if ( translateDateRange(range,from,to) )
     setDateFilter(from,to);
 }
 
-const bool MyMoneyTransactionFilter::translateDateRange(int range, QDate& start, QDate& end)
+static int fiscalYearStartMonth = 1;
+static int fiscalYearStartDay = 1;
+
+void MyMoneyTransactionFilter::setFiscalYearStart(int firstMonth, int firstDay)
 {
-  int result = true;
+  fiscalYearStartMonth = firstMonth;
+  fiscalYearStartDay = firstDay;
+}
+
+bool MyMoneyTransactionFilter::translateDateRange(dateOptionE id, QDate& start, QDate& end)
+{
+  bool rc = true;
   int yr, mon, day;
   yr = QDate::currentDate().year();
   mon = QDate::currentDate().month();
   day = QDate::currentDate().day();
+  QDate tmp;
 
-  switch(range)
-  {
-    case allDates:
+  switch(id) {
+    case MyMoneyTransactionFilter::allDates:
       start = QDate();
       end = QDate();
       break;
-    case untilToday:
+    case MyMoneyTransactionFilter::untilToday:
       start = QDate();
       end =  QDate::currentDate();
       break;
-    case currentMonth:
+    case MyMoneyTransactionFilter::currentMonth:
       start = QDate(yr,mon,1);
       end = QDate(yr,mon,1).addMonths(1).addDays(-1);
       break;
-    case currentYear:
+    case MyMoneyTransactionFilter::currentYear:
       start = QDate(yr,1,1);
       end = QDate(yr,12,31);
       break;
-    case monthToDate:
+    case MyMoneyTransactionFilter::monthToDate:
       start = QDate(yr,mon,1);
       end = QDate::currentDate();
       break;
-    case yearToDate:
+    case MyMoneyTransactionFilter::yearToDate:
       start = QDate(yr,1,1);
       end = QDate::currentDate();
       break;
-    case lastMonth:
+    case MyMoneyTransactionFilter::yearToMonth:
+      start = QDate(yr,1,1);
+      end = QDate(yr,mon,1).addDays(-1);
+      break;
+    case MyMoneyTransactionFilter::lastMonth:
       start = QDate(yr,mon,1).addMonths(-1);
       end = QDate(yr,mon,1).addDays(-1);
       break;
-    case lastYear:
+    case MyMoneyTransactionFilter::lastYear:
       start = QDate(yr,1,1).addYears(-1);
       end = QDate(yr,12,31).addYears(-1);
       break;
-    case last30Days:
+    case MyMoneyTransactionFilter::last7Days:
+      start = QDate::currentDate().addDays(-7);
+      end = QDate::currentDate();
+      break;
+    case MyMoneyTransactionFilter::last30Days:
       start = QDate::currentDate().addDays(-30);
       end = QDate::currentDate();
       break;
-    case last3Months:
+    case MyMoneyTransactionFilter::last3Months:
       start = QDate::currentDate().addMonths(-3);
       end = QDate::currentDate();
       break;
-    case last6Months:
+    case MyMoneyTransactionFilter::last6Months:
       start = QDate::currentDate().addMonths(-6);
       end = QDate::currentDate();
       break;
-    case last12Months:
+    case MyMoneyTransactionFilter::last11Months:
+      start = QDate(yr,mon,1).addMonths(-12);
+      end = QDate(yr,mon,1).addDays(-1);
+      break;
+    case MyMoneyTransactionFilter::last12Months:
       start = QDate::currentDate().addMonths(-12);
       end = QDate::currentDate();
       break;
-    case next30Days:
+    case MyMoneyTransactionFilter::next7Days:
+      start = QDate::currentDate();
+      end = QDate::currentDate().addDays(7);
+      break;
+    case MyMoneyTransactionFilter::next30Days:
       start = QDate::currentDate();
       end = QDate::currentDate().addDays(30);
       break;
-    case next3Months:
+    case MyMoneyTransactionFilter::next3Months:
       start = QDate::currentDate();
       end = QDate::currentDate().addMonths(3);
       break;
-    case next6Months:
+    case MyMoneyTransactionFilter::next6Months:
       start = QDate::currentDate();
       end = QDate::currentDate().addMonths(6);
       break;
-    case next12Months:
+    case MyMoneyTransactionFilter::next12Months:
       start = QDate::currentDate();
       end = QDate::currentDate().addMonths(12);
       break;
-    case userDefined:
+    case MyMoneyTransactionFilter::userDefined:
       start = QDate();
       end = QDate();
       break;
+    case MyMoneyTransactionFilter::last3ToNext3Months:
+      start = QDate::currentDate().addMonths(-3);
+      end = QDate::currentDate().addMonths(3);
+      break;
+    case MyMoneyTransactionFilter::currentQuarter:
+      start = QDate(yr, mon - ((mon-1) % 3), 1);
+      end = start.addMonths(3).addDays(-1);
+      break;
+    case MyMoneyTransactionFilter::lastQuarter:
+      start = QDate(yr, mon - ((mon-1) % 3), 1).addMonths(-3);
+      end = start.addMonths(3).addDays(-1);
+      break;
+    case MyMoneyTransactionFilter::nextQuarter:
+      start = QDate(yr, mon - ((mon-1) % 3), 1).addMonths(3);
+      end = start.addMonths(3).addDays(-1);
+      break;
+    case MyMoneyTransactionFilter::currentFiscalYear:
+      start = QDate(QDate::currentDate().year(), fiscalYearStartMonth, fiscalYearStartDay);
+      if(QDate::currentDate() < start)
+        start = start.addYears(-1);
+      end = start.addYears(1).addDays(-1);
+      break;
+    case MyMoneyTransactionFilter::lastFiscalYear:
+      start = QDate(QDate::currentDate().year(), fiscalYearStartMonth, fiscalYearStartDay);
+      if(QDate::currentDate() < start)
+       start = start.addYears(-1);
+      start = start.addYears(-1);
+      end = start.addYears(1).addDays(-1);
+      break;
+    case MyMoneyTransactionFilter::today:
+      start = QDate::currentDate();
+      end =  QDate::currentDate();
+      break;
     default:
-      result = false;
+      qFatal("Unknown date identifier %d in MyMoneyTransactionFilter::translateDateRange()", id);
+      rc = false;
+      break;
   }
-  return result;
+  return rc;
 }
+
+void MyMoneyTransactionFilter::removeReference(const QCString& id)
+{
+  if(m_accounts.find(id)) {
+    qDebug("%s", (QString("Remove account '%1' from report").arg(id)).data());
+    m_accounts.remove(id);
+  } else if(m_categories.find(id)) {
+    qDebug("%s", (QString("Remove category '%1' from report").arg(id)).data());
+    m_categories.remove(id);
+  } else if(m_payees.find(id)) {
+    qDebug("%s", (QString("Remove payee '%1' from report").arg(id)).data());
+    m_payees.remove(id);
+  }
+}
+
+
 // vim:cin:si:ai:et:ts=2:sw=2:
