@@ -97,21 +97,48 @@ public:
     currentYear,
     monthToDate,
     yearToDate,
+    yearToMonth,
     lastMonth,
     lastYear,
+    last7Days,
     last30Days,
     last3Months,
     last6Months,
     last12Months,
+    next7Days,
     next30Days,
     next3Months,
     next6Months,
     next12Months,
     userDefined,
+    last3ToNext3Months,
+    last11Months,
+    currentQuarter,
+    lastQuarter,
+    nextQuarter,
+    currentFiscalYear,
+    lastFiscalYear,
+    today,
     // insert new constants above of this line
     dateOptionCount
   };
 
+  typedef union {
+    unsigned  allFilter;
+    struct {
+      unsigned textFilter       : 1;
+      unsigned accountFilter    : 1;
+      unsigned payeeFilter      : 1;
+      unsigned categoryFilter   : 1;
+      unsigned nrFilter         : 1;
+      unsigned dateFilter       : 1;
+      unsigned amountFilter     : 1;
+      unsigned typeFilter       : 1;
+      unsigned stateFilter      : 1;
+      unsigned validityFilter   : 1;
+    } singleFilter;
+  } FilterSet;
+
   /**
     * This is the standard constructor for a transaction filter.
     * It creates the object and calls setReportAllSplits() to
@@ -203,7 +230,7 @@ public:
     */
   void setDateFilter(const QDate& from, const QDate& to);
 
-  void setDateFilter(unsigned range);
+  void setDateFilter(dateOptionE range);
 
   /**
     * This method sets the amount filter to match only transactions with
@@ -259,11 +286,26 @@ public:
     * @param storage pointer to object of IMyMoneyStorage class
     *
     * @retval true The transaction matches the filter set
-    * @retval false The transaction does not match at least of of
+    * @retval false The transaction does not match at least one of
     *               the filters in the filter set
     */
   const bool match(const MyMoneyTransaction& transaction, const IMyMoneyStorage* const storage);
 
+  /**
+    * This method is used to check a specific split against the
+    * text and amount filter. The split will match if all specified and
+    * checked filters match. If the filter is cleared using the clear()
+    * method, any split matches.
+    *
+    * @param sp pointer to the split to be checked
+    * @param storage pointer to object of IMyMoneyStorage class
+    *
+    * @retval true The split matches the filter set
+    * @retval false The split does not match at least one of
+    *               the filters in the filter set
+    */
+  bool match(const MyMoneySplit * const sp, const IMyMoneyStorage* const storage) const;
+
   /**
     * This method is used to switch the amount of splits reported
     * by matchingSplits(). If the argument @p report is @p true (the default
@@ -279,16 +321,16 @@ public:
   void setConsiderCategory(const bool check = true);
 
   /**
-    * This method returns the id of the matching splits for the filter.
+    * This method returns a list of the matching splits for the filter.
     * If m_reportAllSplits is set to false, then only the very first
     * split will be returned. Use setReportAllSplits() to change the
     * behaviour.
     *
-    * @return reference to QCString object containing the id of the
-    *         matching split. If multiple split match, only the first
+    * @return reference list of MyMoneySplit objects containing the
+    *         matching splits. If multiple splits match, only the first
     *         one will be returned.
     *
-    * @note an empty id will be returned, if the filter only required
+    * @note an empty list will be returned, if the filter only required
     *       to check the data contained in the MyMoneyTransaction
     *       object (e.g. posting-date, state, etc.).
     *
@@ -296,7 +338,7 @@ public:
     *       see the documentation of the constructors MyMoneyTransactionFilter()
     *       and MyMoneyTransactionFilter(const QCString&) for details.
     */
-  const QValueList<MyMoneySplit> matchingSplits(void) const;
+  const QValueList<MyMoneySplit>& matchingSplits(void) const;
 
   /**
     * This method returns the from date set in the filter. If
@@ -445,7 +487,17 @@ public:
     * @param end QDate will be set to corresponding to the last date in @p range
     * @return return true if a range was successfully set, or false if @p range was invalid
     */
-  static const bool translateDateRange(int range, QDate& start, QDate& end);
+  static bool translateDateRange(dateOptionE range, QDate& start, QDate& end);
+
+  static void setFiscalYearStart(int firstMonth, int firstDay);
+
+  FilterSet filterSet(void) const { return m_filterSet; };
+
+  /**
+    * This member removes all references to object identified by @p id. Used
+    * to remove objects which are about to be removed from the engine.
+    */
+  void removeReference(const QCString& id);
 
 private:
   /**
@@ -481,21 +533,7 @@ private:
   const validityOptionE validTransaction(const MyMoneyTransaction& transaction) const;
 
 protected:
-  union FilterSet {
-    unsigned  allFilter;
-    struct {
-      unsigned textFilter       : 1;
-      unsigned accountFilter    : 1;
-      unsigned payeeFilter      : 1;
-      unsigned categoryFilter   : 1;
-      unsigned nrFilter         : 1;
-      unsigned dateFilter       : 1;
-      unsigned amountFilter     : 1;
-      unsigned typeFilter       : 1;
-      unsigned stateFilter      : 1;
-      unsigned validityFilter   : 1;
-    } singleFilter;
-  }                   m_filterSet;
+  FilterSet           m_filterSet;
   bool                m_reportAllSplits;
   bool                m_considerCategory;
 
