@@ -42,6 +42,8 @@
 #include <kmymoney/mymoneysecurity.h>
 #include <kmymoney/mymoneyprice.h>
 #include <kmymoney/mymoneyreport.h>
+#include <kmymoney/mymoneybudget.h>
+#include "mymoneystoragesql.h"
 
 /**
   * @author Thomas Baumgart
@@ -56,25 +58,27 @@ public:
   virtual ~IMyMoneySerialize();
 
   // general get functions
-  virtual const QString& userName(void) const = 0;
-  virtual const QString& userStreet(void) const = 0;
-  virtual const QString& userTown(void) const = 0;
-  virtual const QString& userCounty(void) const = 0;
-  virtual const QString& userPostcode(void) const = 0;
-  virtual const QString& userTelephone(void) const = 0;
-  virtual const QString& userEmail(void) const = 0;
+  virtual const MyMoneyPayee& user(void) const = 0;
   virtual const QDate& creationDate(void) const = 0;
   virtual const QDate& lastModificationDate(void) const = 0;
+  virtual const unsigned int currentFixVersion(void) const = 0;
+  virtual const unsigned int fileFixVersion(void) const = 0;
 
   // general set functions
-  virtual void setUserName(const QString& val) = 0;
-  virtual void setUserStreet(const QString& val) = 0;
-  virtual void setUserTown(const QString& val) = 0;
-  virtual void setUserCounty(const QString& val) = 0;
-  virtual void setUserPostcode(const QString& val) = 0;
-  virtual void setUserTelephone(const QString& val) = 0;
-  virtual void setUserEmail(const QString& val) = 0;
+  virtual void setUser(const MyMoneyPayee& val) = 0;
   virtual void setCreationDate(const QDate& val) = 0;
+  virtual void setFileFixVersion(const unsigned int v) = 0;
+  /**
+   * This method is used to get a SQL reader for subsequent database access
+   */
+  virtual MyMoneyStorageSql *connectToDatabase
+      (const KURL& url) = 0;
+  /**
+    * This method is used when a database file is open, and the data is to
+    * be saved in a different file or format. It will ensure that all data
+    * from the database is available in memory to enable it to be written.
+    */
+  virtual void fillStorage() = 0;
 
   /**
     * This method is used to set the last modification date of
@@ -87,20 +91,13 @@ public:
   virtual void setLastModificationDate(const QDate& val) = 0;
 
   /**
-    * This method returns a list of accounts inside a MyMoneyFile object.
-    * An optional parameter is a list of id's. If this list is emtpy (the default)
-    * the returned list contains all accounts, otherwise only those referenced
-    * in the id-list.
+    * This method returns a list of accounts inside the storage object.
     *
-    * @param idlist QCStringList of account ids of those accounts that
-    *        should be returned. If this list is empty, all accounts
-    *        currently known will be returned.
+    * @param list reference to QValueList receiving the account objects
     *
-    * @param recursive if @p true, then recurse in all found accounts. The default is @p false
-    *
-    * @return QValueList containing the account objects
+    * @note The standard accounts will not be returned
     */
-  virtual const QValueList<MyMoneyAccount> accountList(const QCStringList& idlist = QCStringList(), const bool recursive = false) const = 0;
+  virtual void accountList(QValueList<MyMoneyAccount>& list) = 0;
 
   /**
     * This method returns a list of the institutions
@@ -116,11 +113,11 @@ public:
     * that match the filter passed as argument. If the filter is empty,
     * the whole journal will be returned.
     *
+    * @param list reference to QValueList<MyMoneyTransaction> receiving
+    *             the set of transactions
     * @param filter MyMoneyTransactionFilter object with the match criteria
-    *
-    * @return set of transactions in form of a QValueList<MyMoneyTransaction>
     */
-  virtual const QValueList<MyMoneyTransaction> transactionList(MyMoneyTransactionFilter& filter) const = 0;
+  virtual void transactionList(QValueList<MyMoneyTransaction>& list, MyMoneyTransactionFilter& filter) const = 0;
 
   /**
     * This method is used to pull a list of transactions from the file
@@ -137,6 +134,11 @@ public:
     */
   // virtual const QValueList<MyMoneyTransaction> transactionList(const QCString& account = "") const = 0;
 
+  /**
+   * This method returns whether a given transaction is already in memory, to avoid
+   * reloading it from the database
+   */
+  virtual bool isDuplicateTransaction(const QCString&) const = 0;
   /**
     * This method returns a list of the payees
     * inside a MyMoneyStorage object
@@ -169,31 +171,31 @@ public:
     * This method is used to return the standard liability account
     * @return MyMoneyAccount liability account(group)
     */
-  virtual const MyMoneyAccount liability(void) const = 0;
+  virtual const MyMoneyAccount& liability(void) const = 0;
 
   /**
     * This method is used to return the standard asset account
     * @return MyMoneyAccount asset account(group)
     */
-  virtual const MyMoneyAccount asset(void) const = 0;
+  virtual const MyMoneyAccount& asset(void) const = 0;
 
   /**
     * This method is used to return the standard expense account
     * @return MyMoneyAccount expense account(group)
     */
-  virtual const MyMoneyAccount expense(void) const = 0;
+  virtual const MyMoneyAccount& expense(void) const = 0;
 
   /**
     * This method is used to return the standard income account
     * @return MyMoneyAccount income account(group)
     */
-  virtual const MyMoneyAccount income(void) const = 0;
+  virtual const MyMoneyAccount& income(void) const = 0;
 
   /**
     * This method is used to return the standard equity account
     * @return MyMoneyAccount equity account(group)
     */
-  virtual const MyMoneyAccount equity(void) const = 0;
+  virtual const MyMoneyAccount& equity(void) const = 0;
 
   /**
     * This method is used to create a new account
@@ -236,25 +238,6 @@ public:
     */
   virtual void addPayee(MyMoneyPayee& payee) = 0;
 
-  /**
-    * This method is used to add an account to an institution. The account
-    * data will be updated to contain the correct id of the referenced institution
-    * and the account will be added to the list of accounts held at the
-    * referenced institution. The objects passed as arguments will be updated
-    * accordingly.
-    *
-    * An exception will be thrown upon error conditions.
-    *
-    * @param institution MyMoneyInstitution the account should be added to
-    * @param account MyMoneyAccount to be added
-    *
-    * @deprecated This method is only provided as long as we provide
-    *             the version 0.4 binary reader. As soon as we deprecate
-    *             this compatability mode this method will disappear from
-    *             this interface!
-    */
-  virtual void addAccount(MyMoneyInstitution& institution, MyMoneyAccount& account) = 0;
-
   /**
     * Adds an institution to the storage. A
     * respective institution-ID will be generated within this record.
@@ -293,14 +276,15 @@ public:
     */
   virtual void addTransaction(MyMoneyTransaction& transaction, const bool skipAccountUpdate = false) = 0;
 
-  virtual void loadAccount(const MyMoneyAccount& acc) = 0;
-  virtual void loadTransaction(const MyMoneyTransaction& tr) = 0;
-  virtual void loadInstitution(const MyMoneyInstitution& inst) = 0;
-  virtual void loadPayee(const MyMoneyPayee& payee) = 0;
-  virtual void loadSchedule(const MyMoneySchedule& sched) = 0;
-  virtual void loadSecurity(const MyMoneySecurity& security) = 0;
-  virtual void loadCurrency(const MyMoneySecurity& currency) = 0;
-  virtual void loadReport( const MyMoneyReport& report ) = 0;
+  virtual void loadAccounts(const QMap<QCString, MyMoneyAccount>& map) = 0;
+  virtual void loadTransactions(const QMap<QCString, MyMoneyTransaction>& map) = 0;
+  virtual void loadInstitutions(const QMap<QCString, MyMoneyInstitution>& map) = 0;
+  virtual void loadPayees(const QMap<QCString, MyMoneyPayee>& map) = 0;
+  virtual void loadSchedules(const QMap<QCString, MyMoneySchedule>& map) = 0;
+  virtual void loadSecurities(const QMap<QCString, MyMoneySecurity>& map) = 0;
+  virtual void loadCurrencies(const QMap<QCString, MyMoneySecurity>& map) = 0;
+  virtual void loadReports( const QMap<QCString, MyMoneyReport>& reports ) = 0;
+  virtual void loadBudgets( const QMap<QCString, MyMoneyBudget>& budgets ) = 0;
 
   virtual const unsigned long accountId(void) = 0;
   virtual const unsigned long transactionId(void) = 0;
@@ -309,6 +293,7 @@ public:
   virtual const unsigned long scheduleId(void) = 0;
   virtual const unsigned long securityId(void) = 0;
   virtual const unsigned long reportId(void) = 0;
+  virtual const unsigned long budgetId(void) = 0;
 
   virtual void loadAccountId(const unsigned long id) = 0;
   virtual void loadTransactionId(const unsigned long id) = 0;
@@ -317,6 +302,7 @@ public:
   virtual void loadScheduleId(const unsigned long id) = 0;
   virtual void loadSecurityId(const unsigned long id) = 0;
   virtual void loadReportId(const unsigned long id) = 0;
+  virtual void loadBudgetId(const unsigned long id) = 0;
 
   /**
     * This method is used to retrieve the whole set of key/value pairs
@@ -367,6 +353,17 @@ public:
     */
   virtual const QValueList<MyMoneyReport> reportList( void ) const = 0;
 
+  /**
+    * This method is used to retrieve the list of all budgets
+    * known to the engine.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @return QValueList of all MyMoneyBudget objects.
+    */
+  virtual const QValueList<MyMoneyBudget> budgetList( void ) const = 0;
+
+
   /**
     * This method adds a price entry to the price list.
     */
@@ -379,6 +376,12 @@ public:
     */
   virtual const MyMoneyPriceList& priceList(void) const = 0;
 
+  /**
+    * This method recalculates the balances of all accounts
+    * based on the transactions stored in the engine.
+    */
+  virtual void rebuildAccountBalances(void) = 0;
+
 };
 
 #endif
