@@ -27,6 +27,7 @@
 // QT Includes
 
 #include <qstring.h>
+#include <qbitarray.h>
 
 // ----------------------------------------------------------------------------
 // Project Includes
@@ -42,6 +43,25 @@
 #include <kmymoney/mymoneysecurity.h>
 #include <kmymoney/mymoneyprice.h>
 #include <kmymoney/mymoneyreport.h>
+#include <kmymoney/mymoneybudget.h>
+
+/**
+  * @author Thomas Baumgart
+  *
+  * A simple replacement for QBitArray that does not bark if testBit()
+  * is called with an index out of bounds. It silently returns false
+  * in that case, otherwise calls the base classes implementation.
+  */
+class MyMoneyFileBitArray : public QBitArray
+{
+public:
+  MyMoneyFileBitArray() : QBitArray() {}
+  MyMoneyFileBitArray(int size) : QBitArray(size) {}
+  bool testBit(uint index) const;
+  bool operator[](int index) const { return testBit(index); }
+  bool at(uint index) const { return testBit(index); }
+};
+
 
 /**
   * @author Thomas Baumgart
@@ -55,28 +75,35 @@
   */
 class IMyMoneyStorage {
 public:
+
+  typedef enum {
+    RefCheckAccount = 0,
+    RefCheckInstitution,
+    RefCheckPayee,
+    RefCheckTransaction,
+    RefCheckReport,
+    RefCheckBudget,
+    RefCheckSchedule,
+    RefCheckSecurity,
+    RefCheckCurrency,
+    RefCheckPrice,
+    // insert new entries above this line
+    MaxRefCheckBits
+  } ReferenceCheckBits;
+
   IMyMoneyStorage();
   virtual ~IMyMoneyStorage();
 
   // general get functions
-  virtual const QString& userName(void) const = 0;
-  virtual const QString& userStreet(void) const = 0;
-  virtual const QString& userTown(void) const = 0;
-  virtual const QString& userCounty(void) const = 0;
-  virtual const QString& userPostcode(void) const = 0;
-  virtual const QString& userTelephone(void) const = 0;
-  virtual const QString& userEmail(void) const = 0;
+  virtual const MyMoneyPayee& user(void) const = 0;
   virtual const QDate& creationDate(void) const = 0;
   virtual const QDate& lastModificationDate(void) const = 0;
+  virtual const unsigned int currentFixVersion(void) const = 0;
+  virtual const unsigned int fileFixVersion(void) const = 0;
 
   // general set functions
-  virtual void setUserName(const QString& val) = 0;
-  virtual void setUserStreet(const QString& val) = 0;
-  virtual void setUserTown(const QString& val) = 0;
-  virtual void setUserCounty(const QString& val) = 0;
-  virtual void setUserPostcode(const QString& val) = 0;
-  virtual void setUserTelephone(const QString& val) = 0;
-  virtual void setUserEmail(const QString& val) = 0;
+  virtual void setUser(const MyMoneyPayee& user) = 0;
+  virtual void setFileFixVersion(const unsigned int v) = 0;
 
   // methods provided by MyMoneyKeyValueContainer
   virtual void setValue(const QCString& key, const QString& value) = 0;
@@ -128,7 +155,7 @@ public:
     *
     * @return MyMoneyPayee object of payee
     */
-  virtual const MyMoneyPayee payee(const QCString& id) const = 0;
+  virtual const MyMoneyPayee& payee(const QCString& id) const = 0;
 
   /**
     * This method is used to retrieve the id to a corresponding
@@ -139,7 +166,7 @@ public:
     *
     * @return MyMoneyPayee object of payee
     */
-  virtual const MyMoneyPayee payeeByName(const QString& payee) const = 0;
+  virtual const MyMoneyPayee& payeeByName(const QString& payee) const = 0;
 
   /**
     * This method is used to modify an existing payee
@@ -167,20 +194,6 @@ public:
     */
   virtual const QValueList<MyMoneyPayee> payeeList(void) const = 0;
 
-  /**
-    * This method is used to add an account to an institution. The account
-    * data will be updated to contain the correct id of the referenced institution
-    * and the account will be added to the list of accounts held at the
-    * referenced institution. The objects passed as arguments will be updated
-    * accordingly.
-    *
-    * An exception will be thrown upon error conditions.
-    *
-    * @param institution MyMoneyInstitution the account should be added to
-    * @param account MyMoneyAccount to be added
-    */
-  virtual void addAccount(MyMoneyInstitution& institution, MyMoneyAccount& account) = 0;
-
   /**
     * Returns the account addressed by it's id.
     *
@@ -190,7 +203,7 @@ public:
     * @return reference to MyMoneyAccount object. An exception is thrown
     *         if the id is unknown
     */
-  virtual const MyMoneyAccount account(const QCString id) const = 0;
+  virtual const MyMoneyAccount& account(const QCString& id) const = 0;
 
   /**
     * This method is used to check whether a given
@@ -406,22 +419,11 @@ public:
     *
     * @return set of transactions in form of a QValueList<MyMoneyTransaction>
     */
-  virtual const QValueList<MyMoneyTransaction> transactionList(MyMoneyTransactionFilter& filter) const = 0;
+  virtual QValueList<MyMoneyTransaction> transactionList(MyMoneyTransactionFilter& filter) const = 0;
 
-  /**
-    * This method is used to pull a list of transactions from the file
-    * global transaction pool. It returns either the whole journal or
-    * the set of transaction referenced by a specific account depending
-    * on the argument given.
-    *
-    * @param account QCString reference to account id. If account is empty all
-    +                transactions (the journal) is returned. If account
-    *                is not empty it returns the set of transactions
-    *                that have splits in this account.
-    *
-    * @return set of transactions in form of a QValueList<MyMoneyTransaction>
-    */
-  // virtual const QValueList<MyMoneyTransaction> transactionList(const QCString& account = "") const = 0;
+  virtual void transactionList(QValueList<MyMoneyTransaction>& list, MyMoneyTransactionFilter& filter) const = 0;
+
+  virtual void transactionList(QValueList<QPair<MyMoneyTransaction, MyMoneySplit> >& list, MyMoneyTransactionFilter& filter) const = 0;
 
   /**
     * Deletes an existing account from the file global account pool
@@ -474,44 +476,43 @@ public:
   virtual const unsigned int institutionCount(void) const = 0;
 
   /**
-    * This method returns a list of accounts inside a MyMoneyFile object.
-    * An optional parameter is a list of id's. If this list is emtpy (the default)
-    * the returned list contains all accounts, otherwise only those referenced
-    * in the id-list.
+    * This method returns a list of accounts inside the storage object.
     *
-    * @param idlist QCStringList of account ids of those accounts that
-    *        should be returned. If this list is empty, all accounts
-    *        currently known will be returned.
+    * @param list reference to QValueList receiving the account objects
     *
-    * @param recursive if @p true, then recurse in all found accounts. The default is @p false
-    *
-    * @return QValueList containing the account objects
+    * @note The standard accounts will not be returned
     */
-  virtual const QValueList<MyMoneyAccount> accountList(const QCStringList& idlist = QCStringList(), const bool recursive = false) const = 0;
+  virtual void accountList(QValueList<MyMoneyAccount>& list) = 0;
 
   /**
     * This method is used to return the standard liability account
     * @return MyMoneyAccount liability account(group)
     */
-  virtual const MyMoneyAccount liability(void) const = 0;
+  virtual const MyMoneyAccount& liability(void) const = 0;
 
   /**
     * This method is used to return the standard asset account
     * @return MyMoneyAccount asset account(group)
     */
-  virtual const MyMoneyAccount asset(void) const = 0;
+  virtual const MyMoneyAccount& asset(void) const = 0;
 
   /**
     * This method is used to return the standard expense account
     * @return MyMoneyAccount expense account(group)
     */
-  virtual const MyMoneyAccount expense(void) const = 0;
+  virtual const MyMoneyAccount& expense(void) const = 0;
 
   /**
     * This method is used to return the standard income account
     * @return MyMoneyAccount income account(group)
     */
-  virtual const MyMoneyAccount income(void) const = 0;
+  virtual const MyMoneyAccount& income(void) const = 0;
+
+  /**
+    * This method is used to return the standard equity account
+    * @return MyMoneyAccount equity account(group)
+    */
+  virtual const MyMoneyAccount& equity(void) const = 0;
 
   /**
     * This method is used to create a new security object.  The ID will be created
@@ -574,12 +575,6 @@ public:
     */
   virtual const MyMoneyPriceList& priceList(void) const = 0;
 
-  /**
-    * This method is used to return the standard equity account
-    * @return MyMoneyAccount equity account(group)
-    */
-  virtual const MyMoneyAccount equity(void) const = 0;
-
   /**
     * This method is used to add a scheduled transaction to the engine.
     * It must be sure, that the id of the object is not filled. When the
@@ -703,7 +698,7 @@ public:
     * @param id QCString containing the id of the MyMoneySecurity object
     * @return MyMoneyCurrency object
     */
-  virtual const MyMoneySecurity currency(const QCString& id) const = 0;
+  virtual const MyMoneySecurity& currency(const QCString& id) const = 0;
 
   /**
     * This method is used to retrieve the list of all currencies
@@ -776,6 +771,109 @@ public:
     */
   virtual void removeReport(const MyMoneyReport& report) = 0;
 
+  /**
+    * This method is used to retrieve the list of all budgets
+    * known to the engine.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @return QValueList of all MyMoneyBudget objects.
+    */
+  virtual const QValueList<MyMoneyBudget> budgetList( void ) const = 0;
+
+  /**
+    * This method is used to add a new budget to the engine.
+    * It must be sure, that the id of the object is not filled. When the
+    * method returns to the caller, the id will be filled with the
+    * newly created object id value.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param budget reference to the MyMoneyBudget object
+    */
+  virtual void addBudget( MyMoneyBudget& budget ) = 0;
+
+  /**
+    * This method is used to retrieve the id to a corresponding
+    * name of a budget
+    * An exception will be thrown upon error conditions.
+    *
+    * @param budget QString reference to name of budget
+    *
+    * @return MyMoneyBudget object of budget
+    */
+  virtual const MyMoneyBudget& budgetByName(const QString& budget) const = 0;
+
+  /**
+    * This method is used to modify an existing MyMoneyBudget
+    * object. Therefor, the id attribute of the object must be set.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param budget const reference to the MyMoneyBudget object to be updated
+    */
+  virtual void modifyBudget( const MyMoneyBudget& budget ) = 0;
+
+  /**
+    * This method returns the number of budgets currently known to file
+    * in the range 0..MAXUINT
+    *
+    * @return number of budgets known to file
+    */
+  virtual unsigned countBudgets( void ) const = 0;
+
+  /**
+    * This method is used to retrieve a single MyMoneyBudget object.
+    * The id of the object must be supplied in the parameter @p id.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param id QCString containing the id of the MyMoneyBudget object
+    * @return MyMoneyBudget object
+    */
+  virtual MyMoneyBudget budget( const QCString& id ) const = 0;
+
+  /**
+    * This method is used to remove an existing MyMoneyBudget object
+    * from the engine. The id attribute of the object must be set.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param budget const reference to the MyMoneyBudget object to be updated
+    */
+  virtual void removeBudget(const MyMoneyBudget& budget) = 0;
+
+
+
+  /**
+    * Clear all internal caches (used internally for performance measurements)
+    */
+  virtual void clearCache(void) = 0;
+
+  /**
+    * This method checks, if the given @p object is referenced
+    * by another engine object.
+    *
+    * @param obj const reference to object to be checked
+    * @param skipCheck MyMoneyFileBitArray with ReferenceCheckBits set for which
+    *                  the check should be skipped
+    *
+    * @retval false @p object is not referenced
+    * @retval true @p institution is referenced
+    */
+  virtual bool isReferenced(const MyMoneyObject& obj, const MyMoneyFileBitArray& skipCheck = MyMoneyFileBitArray()) const = 0;
+
+  /**
+    * This method is provided to allow closing of the database before logoff
+    */
+  virtual void close(void) = 0;
+
+  /**
+    * These methods have to be provided to allow transaction safe data handling.
+    */
+  virtual void startTransaction(void) = 0;
+  virtual void commitTransaction(void) = 0;
+  virtual void rollbackTransaction(void) = 0;
 };
 
 #endif
