@@ -31,6 +31,8 @@
 
 #include "imymoneystorage.h"
 #include "imymoneyserialize.h"
+#include "mymoneystoragesql.h"
+#include "mymoneymap.h"
 
 /**
   * @author Thomas Baumgart
@@ -90,26 +92,29 @@ public:
   ~MyMoneySeqAccessMgr();
 
   // general get functions
-  const QString& userName(void) const { return m_userName; };
-  const QString& userStreet(void) const { return m_userStreet; };
-  const QString& userTown(void) const { return m_userTown; };
-  const QString& userCounty(void) const { return m_userCounty; };
-  const QString& userPostcode(void) const { return m_userPostcode; };
-  const QString& userTelephone(void) const { return m_userTelephone; };
-  const QString& userEmail(void) const { return m_userEmail; };
+  const MyMoneyPayee& user(void) const { return m_user; };
   const QDate& creationDate(void) const { return m_creationDate; };
   const QDate& lastModificationDate(void) const { return m_lastModificationDate; };
+  const unsigned int currentFixVersion(void) const { return m_currentFixVersion; };
+  const unsigned int fileFixVersion(void) const { return m_fileFixVersion; };
+
 
   // general set functions
-  void setUserName(const QString& val) { m_userName = val; touch(); };
-  void setUserStreet(const QString& val) { m_userStreet = val; touch(); };
-  void setUserTown(const QString& val) { m_userTown = val; touch(); };
-  void setUserCounty(const QString& val) { m_userCounty = val; touch(); };
-  void setUserPostcode(const QString& val) { m_userPostcode = val; touch(); };
-  void setUserTelephone(const QString& val) { m_userTelephone = val; touch(); };
-  void setUserEmail(const QString& val) { m_userEmail = val; touch(); };
+  void setUser(const MyMoneyPayee& user) { m_user = user;
+        touch(); if (m_sql != 0) m_sql->modifyUserInfo(user);};
   void setCreationDate(const QDate& val) { m_creationDate = val; touch(); };
   void setLastModificationDate(const QDate& val) { m_lastModificationDate = val; m_dirty = false; };
+  void setFileFixVersion(const unsigned int v) { m_fileFixVersion = v; };
+  /**
+    * This method is used to get a SQL reader for subsequent database access
+    */
+  MyMoneyStorageSql *connectToDatabase (const KURL& url);
+   /**
+   * This method is used when a database file is open, and the data is to
+   * be saved in a different file or format. It will ensure that all data
+   * from the database is available in memory to enable it to be written.
+   */
+  virtual void fillStorage() {m_sql->fillStorage();};
 
   /**
     * This method is used to duplicate the MyMoneySeqAccessMgr object and return
@@ -125,7 +130,7 @@ public:
     * @return reference to MyMoneyAccount object. An exception is thrown
     *         if the id is unknown
     */
-  const MyMoneyAccount account(const QCString id) const;
+  const MyMoneyAccount& account(const QCString& id) const;
 
   /**
     * This method is used to check whether a given
@@ -181,7 +186,7 @@ public:
     *
     * @return MyMoneyPayee object of payee
     */
-  const MyMoneyPayee payee(const QCString& id) const;
+  const MyMoneyPayee& payee(const QCString& id) const;
 
   /**
     * This method is used to retrieve the id to a corresponding
@@ -190,9 +195,9 @@ public:
     *
     * @param payee QString reference to name of payee
     *
-    * @return MyMoneyPayee object of payee
+    * @return MyMoneyPayee reference to object of payee
     */
-  const MyMoneyPayee payeeByName(const QString& payee) const;
+  const MyMoneyPayee& payeeByName(const QString& payee) const;
 
   /**
     * This method is used to modify an existing payee
@@ -230,20 +235,6 @@ public:
     */
   void addAccount(MyMoneyAccount& parent, MyMoneyAccount& account);
 
-  /**
-    * This method is used to add an account to an institution. The account
-    * data will be updated to contain the correct id of the referenced institution
-    * and the account will be added to the list of accounts held at the
-    * referenced institution. The objects passed as arguments will be updated
-    * accordingly.
-    *
-    * An exception will be thrown upon error conditions.
-    *
-    * @param institution MyMoneyInstitution the account should be added to
-    * @param account MyMoneyAccount to be added
-    */
-  void addAccount(MyMoneyInstitution& institution, MyMoneyAccount& account);
-
   /**
     * Adds an institution to the storage. A
     * respective institution-ID will be generated within this record.
@@ -429,20 +420,13 @@ public:
   const QValueList<MyMoneyInstitution> institutionList(void) const;
 
   /**
-    * This method returns a list of accounts inside a MyMoneyFile object.
-    * An optional parameter is a list of id's. If this list is emtpy (the default)
-    * the returned list contains all accounts, otherwise only those referenced
-    * in the id-list.
+    * This method returns a list of accounts inside the storage object.
     *
-    * @param idlist QCStringList of account ids of those accounts that
-    *        should be returned. If this list is empty, all accounts
-    *        currently known will be returned.
+    * @param list reference to QValueList receiving the account objects
     *
-    * @param recursive if @p true, then recurse in all found accounts. The default is @p false
-    *
-    * @return QValueList containing the account objects
+    * @note The standard accounts will not be returned
     */
-  const QValueList<MyMoneyAccount> accountList(const QCStringList& idlist = QCStringList(), const bool recursive = false) const;
+  void accountList(QValueList<MyMoneyAccount>& list);
 
   /**
     * This method is used to pull a list of transactions from the file
@@ -453,29 +437,43 @@ public:
     * If more than one transaction exists for the same date, the order among
     * them is undefined.
     *
+    * The @p list will be cleared by this method.
+    *
+    * @param list reference to list
     * @param filter MyMoneyTransactionFilter object with the match criteria
     *
     * @return set of transactions in form of a QValueList<MyMoneyTransaction>
     */
-  const QValueList<MyMoneyTransaction> transactionList(MyMoneyTransactionFilter& filter) const;
+  void transactionList(QValueList<MyMoneyTransaction>& list, MyMoneyTransactionFilter& filter) const;
 
   /**
     * This method is used to pull a list of transactions from the file
-    * global transaction pool. It returns either the whole journal or
-    * the set of transaction referenced by a specific account depending
-    * on the argument given.
+    * global transaction pool. It returns all those transactions
+    * that match the filter passed as argument. If the filter is empty,
+    * the whole journal will be returned.
+    * The list returned is sorted according to the transactions posting date.
+    * If more than one transaction exists for the same date, the order among
+    * them is undefined.
     *
-    * @param account QCString reference to account id. If account is empty
-    +                all transactions (the journal) is returned. If account
-    *                is not empty it returns the set of transactions
-    *                that have splits in this account.
+    * The @p list will be cleared by this method.
     *
-    * @return set of transactions in form of a QValueList<MyMoneyTransaction>
+    * @param list reference to list
+    * @param filter MyMoneyTransactionFilter object with the match criteria
     *
-    * @note This function is provided for convenience and historic reasons. It
-    *       makes use of the above mentioned method.
+    * @return set of transactions in form of a QValueList<QPair<MyMoneyTransaction,MyMoneySplit> >
     */
-  // const QValueList<MyMoneyTransaction> transactionList(const QCString& account = "") const;
+  void transactionList(QValueList< QPair<MyMoneyTransaction, MyMoneySplit> >& list, MyMoneyTransactionFilter& filter) const;
+
+  /**
+    * Compatibility interface for the previous method.
+    */
+  QValueList<MyMoneyTransaction> transactionList(MyMoneyTransactionFilter& filter) const;
+
+  /**
+    * This method returns whether a given transaction is already in memory, to avoid
+    * reloading it from the database
+    */
+  bool isDuplicateTransaction(const QCString& id) const { return m_transactionKeys.contains(id); }
 
   /**
     * This method returns the number of transactions currently known to file
@@ -512,39 +510,39 @@ public:
     * This method is used to return the standard liability account
     * @return MyMoneyAccount liability account(group)
     */
-  const MyMoneyAccount liability(void) const { return account(STD_ACC_LIABILITY); };
+  const MyMoneyAccount& liability(void) const { return account(STD_ACC_LIABILITY); };
 
   /**
     * This method is used to return the standard asset account
     * @return MyMoneyAccount asset account(group)
     */
-  const MyMoneyAccount asset(void) const { return account(STD_ACC_ASSET); };
+  const MyMoneyAccount& asset(void) const { return account(STD_ACC_ASSET); };
 
   /**
     * This method is used to return the standard expense account
     * @return MyMoneyAccount expense account(group)
     */
-  const MyMoneyAccount expense(void) const { return account(STD_ACC_EXPENSE); };
+  const MyMoneyAccount& expense(void) const { return account(STD_ACC_EXPENSE); };
 
   /**
     * This method is used to return the standard income account
     * @return MyMoneyAccount income account(group)
     */
-  const MyMoneyAccount income(void) const { return account(STD_ACC_INCOME); };
+  const MyMoneyAccount& income(void) const { return account(STD_ACC_INCOME); };
 
   /**
     * This method is used to return the standard equity account
     * @return MyMoneyAccount equity account(group)
     */
-  const MyMoneyAccount equity(void) const { return account(STD_ACC_EQUITY); };
+  const MyMoneyAccount& equity(void) const { return account(STD_ACC_EQUITY); };
 
-  virtual void loadAccount(const MyMoneyAccount& acc);
-  virtual void loadTransaction(const MyMoneyTransaction& tr);
-  virtual void loadInstitution(const MyMoneyInstitution& inst);
-  virtual void loadPayee(const MyMoneyPayee& payee);
-  virtual void loadSchedule(const MyMoneySchedule& sched);
-  virtual void loadSecurity(const MyMoneySecurity& security);
-  virtual void loadCurrency(const MyMoneySecurity& currency);
+  virtual void loadAccounts(const QMap<QCString, MyMoneyAccount>& acc);
+  virtual void loadTransactions(const QMap<QCString, MyMoneyTransaction>& map);
+  virtual void loadInstitutions(const QMap<QCString, MyMoneyInstitution>& map);
+  virtual void loadPayees(const QMap<QCString, MyMoneyPayee>& map);
+  virtual void loadSchedules(const QMap<QCString, MyMoneySchedule>& map);
+  virtual void loadSecurities(const QMap<QCString, MyMoneySecurity>& map);
+  virtual void loadCurrencies(const QMap<QCString, MyMoneySecurity>& map);
 
   virtual void loadAccountId(const unsigned long id);
   virtual void loadTransactionId(const unsigned long id);
@@ -553,6 +551,7 @@ public:
   virtual void loadScheduleId(const unsigned long id);
   virtual void loadSecurityId(const unsigned long id);
   virtual void loadReportId(const unsigned long id);
+  virtual void loadBudgetId(const unsigned long id);
 
   virtual const unsigned long accountId(void) { return m_nextAccountID; };
   virtual const unsigned long transactionId(void) { return m_nextTransactionID; };
@@ -561,6 +560,7 @@ public:
   virtual const unsigned long scheduleId(void) { return m_nextScheduleID; };
   virtual const unsigned long securityId(void) { return m_nextSecurityID; };
   virtual const unsigned long reportId(void) { return m_nextReportID; };
+  virtual const unsigned long budgetId(void) { return m_nextBudgetID; };
 
 
   /**
@@ -650,19 +650,6 @@ public:
     */
   virtual void addSecurity(MyMoneySecurity& security);
 
-#if 0
-  /**
-    * This method is used to add a new security object to the engine.
-    * The ID of the object is the trading symbol, so there is no need for an additional
-    * ID since the symbol is guaranteed to be unique.
-    *
-    * An exception will be thrown upon erronous situations.
-    *
-    * @param security reference to the MyMoneySecurity object
-    */
-  void addSecurity(MyMoneySecurity& security);
-#endif
-
   /**
     * This method is used to modify an existing MyMoneySchedule
     * object.
@@ -741,7 +728,7 @@ public:
     * @param id QCString containing the id of the MyMoneySchedule object
     * @return MyMoneySchedule object
     */
-  const MyMoneySecurity currency(const QCString& id) const;
+  const MyMoneySecurity& currency(const QCString& id) const;
 
   /**
     * This method is used to retrieve the list of all currencies
@@ -816,16 +803,14 @@ public:
   void addReport( MyMoneyReport& report );
 
   /**
-    * This method is used to load a report into the engine.  This is
-    * used when loading from storage, and an ID is already known.  It
-    * is similiar to addReport, however loadReport will not assign
-    * a new ID.
+    * This method is used to load a set of reports into the engine.  This is
+    * used when loading from storage, and an ID is already known.
     *
     * An exception will be thrown upon erronous situations.
     *
-    * @param report reference to the MyMoneyReport object
+    * @param reports reference to the map of MyMoneyReport objects
     */
-  void loadReport( const MyMoneyReport& report );
+  void loadReports( const QMap<QCString, MyMoneyReport>& reports );
 
   /**
     * This method is used to modify an existing MyMoneyReport
@@ -866,6 +851,89 @@ public:
     */
   void removeReport(const MyMoneyReport& report);
 
+  /**
+    * This method is used to retrieve the list of all budgets
+    * known to the engine.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @return QValueList of all MyMoneyBudget objects.
+    */
+  const QValueList<MyMoneyBudget> budgetList( void ) const;
+
+  /**
+    * This method is used to add a new budget to the engine.
+    * It must be sure, that the id of the object is not filled. When the
+    * method returns to the caller, the id will be filled with the
+    * newly created object id value.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param budget reference to the MyMoneyBudget object
+    */
+  void addBudget( MyMoneyBudget& budget );
+
+  /**
+    * This method is used to load a set of budgets into the engine.  This is
+    * used when loading from storage, and an ID is already known.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param budgets reference to the map of MyMoneyBudget object
+    */
+  void loadBudgets(const QMap<QCString, MyMoneyBudget>& budgets);
+
+  /**
+    * This method is used to retrieve the id to a corresponding
+    * name of a budget
+    * An exception will be thrown upon error conditions.
+    *
+    * @param budget QString reference to name of budget
+    *
+    * @return MyMoneyBudget reference to object of budget
+    */
+  const MyMoneyBudget& budgetByName(const QString& budget) const;
+
+  /**
+    * This method is used to modify an existing MyMoneyBudget
+    * object. Therefore, the id attribute of the object must be set.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param budget const reference to the MyMoneyBudget object to be updated
+    */
+  void modifyBudget( const MyMoneyBudget& budget );
+
+  /**
+    * This method returns the number of budgets currently known to file
+    * in the range 0..MAXUINT
+    *
+    * @return number of budgets known to file
+    */
+  unsigned countBudgets(void) const;
+
+  /**
+    * This method is used to retrieve a single MyMoneyBudget object.
+    * The id of the object must be supplied in the parameter @p id.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param id QCString containing the id of the MyMoneyBudget object
+    * @return MyMoneyBudget object
+    */
+  MyMoneyBudget budget( const QCString& id ) const;
+
+  /**
+    * This method is used to remove an existing MyMoneyBudget object
+    * from the engine. The id attribute of the object must be set.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param budget const reference to the MyMoneyBudget object to be updated
+    */
+  void removeBudget(const MyMoneyBudget& budget);
+
+
   /**
     * This method adds/replaces a price to/from the price list
     */
@@ -877,7 +945,7 @@ public:
   void removePrice(const MyMoneyPrice& price);
 
   /**
-    * This method retrieves a price from the price list
+    * This method retrieves a price from the price list.
     * If @p date is inValid, QDate::currentDate() is assumed.
     */
   MyMoneyPrice price(const QCString& fromId, const QCString& toId, const QDate& date, const bool exactDate) const;
@@ -887,6 +955,37 @@ public:
     */
   const MyMoneyPriceList& priceList(void) const;
 
+  /**
+    * Clear all internal caches (used internally for performance measurements)
+    */
+  void clearCache(void);
+
+  /**
+    * This method checks, if the given @p object is referenced
+    * by another engine object.
+    *
+    * @param obj const reference to object to be checked
+    * @param skipCheck QBitArray with ReferenceCheckBits set for which
+    *                  the check should be skipped
+    *
+    * @retval false @p object is not referenced
+    * @retval true @p institution is referenced
+    */
+  bool isReferenced(const MyMoneyObject& obj, const MyMoneyFileBitArray& skipCheck = MyMoneyFileBitArray()) const;
+
+  /**
+    * This method recalculates the balances of all accounts
+    * based on the transactions stored in the engine.
+    */
+  void rebuildAccountBalances(void);
+
+  virtual void startTransaction(void);
+  virtual void commitTransaction(void);
+  virtual void rollbackTransaction(void);
+
+protected:
+  void removeReferences(const QCString& id);
+
 private:
 
   static const int INSTITUTION_ID_SIZE = 6;
@@ -896,18 +995,7 @@ private:
   static const int SCHEDULE_ID_SIZE = 6;
   static const int SECURITY_ID_SIZE = 6;
   static const int REPORT_ID_SIZE = 6;
-
-  static const int YEAR_SIZE = 4;
-  static const int MONTH_SIZE = 2;
-  static const int DAY_SIZE = 2;
-
-  /**
-    * This method is used to generate the sort-key for a transaction
-    *
-    * @param transaction reference to the transaction
-    * @return key as QString
-    */
-  const QCString transactionKey(const MyMoneyTransaction& transaction) const;
+  static const int BUDGET_ID_SIZE = 6;
 
   /**
     * This method is used to set the dirty flag and update the
@@ -924,46 +1012,10 @@ private:
   void invalidateBalanceCache(const QCString& id);
 
   /**
-    * This member variable keeps the name of the User
-    * @see userName(), setUserName()
+    * This member variable keeps the User information.
+    * @see setUser()
     */
-  QString m_userName;
-
-  /**
-    * This member variable keeps the street of the users address
-    * @see userStreet(), setUserStreet()
-    */
-  QString m_userStreet;
-
-  /**
-    * This member variable keeps the name of the town the user resides in
-    * @see userTown(), setUserTown()
-    */
-  QString m_userTown;
-
-  /**
-    * This member variable keeps the name of the county for the users address
-    * @see userCounty(), setUserCounty()
-    */
-  QString m_userCounty;
-
-  /**
-    * This member variable keeps the zip-code of the users address
-    * @see userPostcode(), setUserPostcode()
-    */
-  QString m_userPostcode;
-
-  /**
-    * This member variable keeps the telephone number of the user
-    * @see userTelephone(), setUserTelephone()
-    */
-  QString m_userTelephone;
-
-  /**
-    * This member variable keeps the users e-mail address
-    * @see userEmail(), setUserEmail()
-    */
-  QString m_userEmail;
+  MyMoneyPayee m_user;
 
   /**
     * The member variable m_nextInstitutionID keeps the number that will be
@@ -1009,17 +1061,24 @@ private:
 
   unsigned long m_nextReportID;
 
+  /**
+    * The member variable m_nextBudgetID keeps the number that will be
+    * assigned to the next budget object created.  It is maintained by
+    * nextBudgetID()
+    */
+  unsigned long m_nextBudgetID;
+
   /**
     * The member variable m_institutionList is the container for the
     * institutions known within this file.
     */
-  QMap<QCString, MyMoneyInstitution> m_institutionList;
+  MyMoneyMap<QCString, MyMoneyInstitution> m_institutionList;
 
   /**
     * The member variable m_accountList is the container for the accounts
     * known within this file.
     */
-  QMap<QCString, MyMoneyAccount> m_accountList;
+  MyMoneyMap<QCString, MyMoneyAccount> m_accountList;
 
   /**
     * The member variable m_balanceCache is the container for the
@@ -1027,29 +1086,36 @@ private:
     */
   QMap<QCString, MyMoneyBalanceCacheItem> m_balanceCache;
 
+  /**
+    * This member keeps the date for which the m_balanceCache member
+    * is valid. In case the whole cache is invalid it is set to
+    * QDate().
+    */
+  QDate          m_balanceCacheDate;
+
   /**
     * The member variable m_transactionList is the container for all
     * transactions within this file.
     * @see m_transactionKeys
     */
-  QMap<QCString, MyMoneyTransaction> m_transactionList;
+  MyMoneyMap<QCString, MyMoneyTransaction> m_transactionList;
 
   /**
     * The member variable m_transactionKeys is used to convert
     * transaction id's into the corresponding key used in m_transactionList.
     * @see m_transactionList;
     */
-  QMap<QCString, QCString> m_transactionKeys;
+  MyMoneyMap<QCString, QCString> m_transactionKeys;
 
   /**
     * A list containing all the payees that have been used
     */
-  QMap<QCString, MyMoneyPayee> m_payeeList;
+  MyMoneyMap<QCString, MyMoneyPayee> m_payeeList;
 
   /**
     * A list containing all the scheduled transactions
     */
-  QMap<QCString, MyMoneySchedule> m_scheduleList;
+  MyMoneyMap<QCString, MyMoneySchedule> m_scheduleList;
 
   /**
     * A list containing all the security information objects.  Each object
@@ -1058,14 +1124,19 @@ private:
     * to determine the cost basis for sales, as well as the source of
     * information for reports in a security account.
     */
-  QMap<QCString, MyMoneySecurity> m_securitiesList;
+  MyMoneyMap<QCString, MyMoneySecurity> m_securitiesList;
 
   /**
     * A list containing all the currency information objects.
     */
-  QMap<QCString, MyMoneySecurity> m_currencyList;
+  MyMoneyMap<QCString, MyMoneySecurity> m_currencyList;
 
-  QMap<QCString, MyMoneyReport> m_reportList;
+  MyMoneyMap<QCString, MyMoneyReport> m_reportList;
+
+  /**
+    * A list containing all the budget information objects.
+    */
+  MyMoneyMap<QCString, MyMoneyBudget> m_budgetList;
 
   MyMoneyPriceList              m_priceList;
 
@@ -1087,6 +1158,16 @@ private:
     */
   QDate m_lastModificationDate;
 
+  /**
+    * This member variable contains the current fix level of application
+    * data files. (see kmymoneyview.cpp)
+    */
+  unsigned int m_currentFixVersion;
+  /**
+   * This member variable contains the current fix level of the
+   *  presently open data file. (see kmymoneyview.cpp)
+   */
+  unsigned int m_fileFixVersion;
   /**
     * This method is used to get the next valid ID for a institution
     * @return id for a institution
@@ -1125,6 +1206,13 @@ private:
 
   const QCString nextReportID(void);
 
+  /**
+    * This method is used to get the next valid ID for a budget object.
+    * @return id for an budget object
+    */
+  const QCString nextBudgetID(void);
+
+
   /**
     * This method re-parents an existing account
     *
@@ -1136,6 +1224,19 @@ private:
     *                of all modified objects are send
     */
   void reparentAccount(MyMoneyAccount &account, MyMoneyAccount& parent, const bool sendNotification);
+  /**
+   * This method will close a database and log the use roff
+   */
+  void close(void) {if (m_sql != 0) m_sql->close();};
+  /**
+    * This contains the interface with SQL reader for database access
+    */
+  MyMoneyStorageSql* m_sql;
+  /**
+    * This member variable is set when all transactions have been read from the database.
+    * This is would be probably the case when doing, for e.g., a full report,
+    * or after some types of transaction search which cannot be easily implemented in SQL
+    */
+  bool m_transactionListFull;
 };
-
 #endif
