@@ -25,29 +25,34 @@ MyMoneySeqAccessMgrTest::MyMoneySeqAccessMgrTest()
 void MyMoneySeqAccessMgrTest::setUp()
 {
 	m = new MyMoneySeqAccessMgr;
+	m->startTransaction();
 }
 
 void MyMoneySeqAccessMgrTest::tearDown()
 {
+	m->commitTransaction();
 	delete m;
 }
 
 void MyMoneySeqAccessMgrTest::testEmptyConstructor()
 {
-	CPPUNIT_ASSERT(m->m_userName.isEmpty());
-	CPPUNIT_ASSERT(m->m_userStreet.isEmpty());
-	CPPUNIT_ASSERT(m->m_userTown.isEmpty());
-	CPPUNIT_ASSERT(m->m_userCounty.isEmpty());
-	CPPUNIT_ASSERT(m->m_userPostcode.isEmpty());
-	CPPUNIT_ASSERT(m->m_userTelephone.isEmpty());
-	CPPUNIT_ASSERT(m->m_userEmail.isEmpty());
+	MyMoneyPayee user = m->user();
+
+	CPPUNIT_ASSERT(user.name().isEmpty());
+	CPPUNIT_ASSERT(user.address().isEmpty());
+	CPPUNIT_ASSERT(user.city().isEmpty());
+	CPPUNIT_ASSERT(user.state().isEmpty());
+	CPPUNIT_ASSERT(user.postcode().isEmpty());
+	CPPUNIT_ASSERT(user.telephone().isEmpty());
+	CPPUNIT_ASSERT(user.email().isEmpty());
 	CPPUNIT_ASSERT(m->m_nextInstitutionID == 0);
 	CPPUNIT_ASSERT(m->m_nextAccountID == 0);
 	CPPUNIT_ASSERT(m->m_nextTransactionID == 0);
 	CPPUNIT_ASSERT(m->m_nextPayeeID == 0);
 	CPPUNIT_ASSERT(m->m_nextScheduleID == 0);
+	CPPUNIT_ASSERT(m->m_nextReportID == 0);
 	CPPUNIT_ASSERT(m->m_institutionList.count() == 0);
-	CPPUNIT_ASSERT(m->m_accountList.count() == 4);
+	CPPUNIT_ASSERT(m->m_accountList.count() == 5);
 	CPPUNIT_ASSERT(m->m_transactionList.count() == 0);
 	CPPUNIT_ASSERT(m->m_transactionKeys.count() == 0);
 	CPPUNIT_ASSERT(m->m_payeeList.count() == 0);
@@ -60,40 +65,57 @@ void MyMoneySeqAccessMgrTest::testEmptyConstructor()
 	CPPUNIT_ASSERT(m->asset().name() == "Asset");
 	CPPUNIT_ASSERT(m->expense().name() == "Expense");
 	CPPUNIT_ASSERT(m->income().name() == "Income");
+	CPPUNIT_ASSERT(m->equity().name() == "Equity");
 }
 
 void MyMoneySeqAccessMgrTest::testSetFunctions() {
-	
-	m->m_dirty = false;
-	m->setUserName("Name");
-	CPPUNIT_ASSERT(m->dirty() == true);
-	m->m_dirty = false;
-	m->setUserStreet("Street");
-	CPPUNIT_ASSERT(m->dirty() == true);
-	m->m_dirty = false;
-	m->setUserTown("Town");
-	CPPUNIT_ASSERT(m->dirty() == true);
-	m->m_dirty = false;
-	m->setUserCounty("County");
-	CPPUNIT_ASSERT(m->dirty() == true);
-	m->m_dirty = false;
-	m->setUserPostcode("Postcode");
-	CPPUNIT_ASSERT(m->dirty() == true);
-	m->m_dirty = false;
-	m->setUserTelephone("Telephone");
-	CPPUNIT_ASSERT(m->dirty() == true);
-	m->m_dirty = false;
-	m->setUserEmail("Email");
-	CPPUNIT_ASSERT(m->dirty() == true);
-	m->m_dirty = false;
+	MyMoneyPayee user = m->user();
 
-	CPPUNIT_ASSERT(m->userName() == "Name");
-	CPPUNIT_ASSERT(m->userStreet() == "Street");
-	CPPUNIT_ASSERT(m->userTown() == "Town");
-	CPPUNIT_ASSERT(m->userCounty() == "County");
-	CPPUNIT_ASSERT(m->userPostcode() == "Postcode");
-	CPPUNIT_ASSERT(m->userTelephone() == "Telephone");
-	CPPUNIT_ASSERT(m->userEmail() == "Email");
+	m->m_dirty = false;
+	user.setName("Name");
+	m->setUser(user);
+	CPPUNIT_ASSERT(m->dirty() == true);
+	m->m_dirty = false;
+	user.setAddress("Street");
+	m->setUser(user);
+	CPPUNIT_ASSERT(m->dirty() == true);
+	m->m_dirty = false;
+	user.setCity("Town");
+	m->setUser(user);
+	CPPUNIT_ASSERT(m->dirty() == true);
+	m->m_dirty = false;
+	user.setState("County");
+	m->setUser(user);
+	CPPUNIT_ASSERT(m->dirty() == true);
+	m->m_dirty = false;
+	user.setPostcode("Postcode");
+	m->setUser(user);
+	CPPUNIT_ASSERT(m->dirty() == true);
+	m->m_dirty = false;
+	user.setTelephone("Telephone");
+	m->setUser(user);
+	CPPUNIT_ASSERT(m->dirty() == true);
+	m->m_dirty = false;
+	user.setEmail("Email");
+	m->setUser(user);
+	CPPUNIT_ASSERT(m->dirty() == true);
+	m->m_dirty = false;
+	m->setValue("key", "value");
+	CPPUNIT_ASSERT(m->dirty() == true);
+
+	user = m->user();
+	CPPUNIT_ASSERT(user.name() == "Name");
+	CPPUNIT_ASSERT(user.address() == "Street");
+	CPPUNIT_ASSERT(user.city() == "Town");
+	CPPUNIT_ASSERT(user.state() == "County");
+	CPPUNIT_ASSERT(user.postcode() == "Postcode");
+	CPPUNIT_ASSERT(user.telephone() == "Telephone");
+	CPPUNIT_ASSERT(user.email() == "Email");
+	CPPUNIT_ASSERT(m->value("key") == "value");
+
+	m->m_dirty = false;
+	m->deletePair("key");
+	CPPUNIT_ASSERT(m->dirty() == true);
 }
 
 void MyMoneySeqAccessMgrTest::testSupportFunctions()
@@ -108,6 +130,8 @@ void MyMoneySeqAccessMgrTest::testSupportFunctions()
 	CPPUNIT_ASSERT(m->m_nextPayeeID == 1);
 	CPPUNIT_ASSERT(m->nextScheduleID() == "SCH000001");
 	CPPUNIT_ASSERT(m->m_nextScheduleID == 1);
+	CPPUNIT_ASSERT(m->nextReportID() == "R000001");
+	CPPUNIT_ASSERT(m->m_nextReportID == 1);
 }
 
 void MyMoneySeqAccessMgrTest::testIsStandardAccount()
@@ -125,11 +149,11 @@ void MyMoneySeqAccessMgrTest::testNewAccount() {
 	a.setName("AccountName");
 	a.setNumber("AccountNumber");
 
-	m->newAccount(a);
+	m->addAccount(a);
 
 	CPPUNIT_ASSERT(m->m_nextAccountID == 1);
 	CPPUNIT_ASSERT(m->dirty() == true);
-	CPPUNIT_ASSERT(m->m_accountList.count() == 5);
+	CPPUNIT_ASSERT(m->m_accountList.count() == 6);
 	CPPUNIT_ASSERT(m->m_accountList["A000001"].name() == "AccountName");
 }
 
@@ -166,12 +190,12 @@ void MyMoneySeqAccessMgrTest::testAddNewAccount() {
 	MyMoneyAccount a,b;
 	b.setName("Account2");
 	b.setNumber("Acc2");
-	m->newAccount(b);
+	m->addAccount(b);
 
 	m->m_dirty = false;
 
 	CPPUNIT_ASSERT(m->m_nextAccountID == 2);
-	CPPUNIT_ASSERT(m->m_accountList.count() == 6);
+	CPPUNIT_ASSERT(m->m_accountList.count() == 7);
 
 	// try to add account to undefined account
 	try {
@@ -255,8 +279,9 @@ void MyMoneySeqAccessMgrTest::testAccount2Institution() {
 
 	// try to add to a false institution
 	MyMoneyInstitution fake("Unknown ID", i);
+	a.setInstitutionId(fake.id());
 	try {
-		m->addAccount(fake, a);
+		m->modifyAccount(a);
 		CPPUNIT_FAIL("Exception expected");
 	} catch (MyMoneyException *e) {
 		delete e;
@@ -266,12 +291,13 @@ void MyMoneySeqAccessMgrTest::testAccount2Institution() {
 	// now try to do it with a real institution
 	try {
 		CPPUNIT_ASSERT(i.accountList().count() == 0);
-		m->addAccount(i, a);
+		a.setInstitutionId(i.id());
+		m->modifyAccount(a);
 		CPPUNIT_ASSERT(m->dirty() == true);
 		CPPUNIT_ASSERT(a.institutionId() == i.id());
 		b = m->account("A000001");
 		CPPUNIT_ASSERT(b.institutionId() == i.id());
-		CPPUNIT_ASSERT(i.accountList().count() == 1);
+		CPPUNIT_ASSERT(i.accountList().count() == 0);
 	} catch (MyMoneyException *e) {
 		delete e;
 		CPPUNIT_FAIL("Unexpected exception");
@@ -378,12 +404,12 @@ void MyMoneySeqAccessMgrTest::testReparentAccount() {
 	ch.setName("My checkings account");
 
 	try {
-		m->newAccount(ex1);
-		m->newAccount(ex2);
-		m->newAccount(ex3);
-		m->newAccount(ex4);
-		m->newAccount(in);
-		m->newAccount(ch);
+		m->addAccount(ex1);
+		m->addAccount(ex2);
+		m->addAccount(ex3);
+		m->addAccount(ex4);
+		m->addAccount(in);
+		m->addAccount(ch);
 
 		CPPUNIT_ASSERT(ex1.id() == "A000001");
 		CPPUNIT_ASSERT(ex2.id() == "A000002");
@@ -566,10 +592,13 @@ void MyMoneySeqAccessMgrTest::testTransactionCount() {
 void MyMoneySeqAccessMgrTest::testBalance() {
 	testAddTransactions();
 
-	CPPUNIT_ASSERT(m->balance("A000001") == 0);
-	CPPUNIT_ASSERT(m->balance("A000002") == 1200);
-	CPPUNIT_ASSERT(m->balance("A000003") == 400);
-	CPPUNIT_ASSERT(m->totalBalance("A000001") == 1600);
+	CPPUNIT_ASSERT(m->balance("A000001").isZero());
+	CPPUNIT_ASSERT(m->balance("A000002") == MyMoneyMoney(1200));
+	CPPUNIT_ASSERT(m->balance("A000003") == MyMoneyMoney(400));
+	CPPUNIT_ASSERT(m->totalBalance("A000001") == MyMoneyMoney(1600));
+	CPPUNIT_ASSERT(m->balance("A000006", QDate(2002,5,9)) == MyMoneyMoney(-11600));
+	CPPUNIT_ASSERT(m->balance("A000005", QDate(2002,5,10)) == MyMoneyMoney(-100000));
+	CPPUNIT_ASSERT(m->balance("A000006", QDate(2002,5,10)) == MyMoneyMoney(88400));
 }
 
 void MyMoneySeqAccessMgrTest::testModifyTransaction() {
@@ -594,13 +623,13 @@ void MyMoneySeqAccessMgrTest::testModifyTransaction() {
 	t.modifySplit(s);
 
 	try {
-		CPPUNIT_ASSERT(m->balance("A000004") == 10000);
-		CPPUNIT_ASSERT(m->balance("A000006") == 100000-11600);
-		CPPUNIT_ASSERT(m->totalBalance("A000001") == 1600);
+		CPPUNIT_ASSERT(m->balance("A000004") == MyMoneyMoney(10000));
+		CPPUNIT_ASSERT(m->balance("A000006") == MyMoneyMoney(100000-11600));
+		CPPUNIT_ASSERT(m->totalBalance("A000001") == MyMoneyMoney(1600));
 		m->modifyTransaction(t);
-		CPPUNIT_ASSERT(m->balance("A000004") == 11000);
-		CPPUNIT_ASSERT(m->balance("A000006") == 100000-12600);
-		CPPUNIT_ASSERT(m->totalBalance("A000001") == 1600);
+		CPPUNIT_ASSERT(m->balance("A000004") == MyMoneyMoney(11000));
+		CPPUNIT_ASSERT(m->balance("A000006") == MyMoneyMoney(100000-12600));
+		CPPUNIT_ASSERT(m->totalBalance("A000001") == MyMoneyMoney(1600));
 	} catch (MyMoneyException *e) {
 		delete e;
 		CPPUNIT_FAIL("Unexpected exception");
@@ -610,9 +639,9 @@ void MyMoneySeqAccessMgrTest::testModifyTransaction() {
 	t.setPostDate(QDate(2002,5,11));
 	try {
 		m->modifyTransaction(t);
-		CPPUNIT_ASSERT(m->balance("A000004") == 11000);
-		CPPUNIT_ASSERT(m->balance("A000006") == 100000-12600);
-		CPPUNIT_ASSERT(m->totalBalance("A000001") == 1600);
+		CPPUNIT_ASSERT(m->balance("A000004") == MyMoneyMoney(11000));
+		CPPUNIT_ASSERT(m->balance("A000006") == MyMoneyMoney(100000-12600));
+		CPPUNIT_ASSERT(m->totalBalance("A000001") == MyMoneyMoney(1600));
 
 		QMap<QCString, QCString>::ConstIterator it_k;
 		QMap<QCString, MyMoneyTransaction>::ConstIterator it_t;
@@ -703,13 +732,24 @@ void MyMoneySeqAccessMgrTest::testRemoveUnusedAccount() {
 		delete e;
 	}
 
-	// now really remove an account
+	// try to remove the account still attached to the institution
 	try {
-		CPPUNIT_ASSERT(i.accountCount() == 1);
-		CPPUNIT_ASSERT(m->accountCount() == 6);
-
 		m->removeAccount(a);
-		CPPUNIT_ASSERT(m->accountCount() == 5);
+		CPPUNIT_FAIL("Exception expected");
+	} catch (MyMoneyException *e) {
+		delete e;
+	}
+
+	// now really remove an account
+
+	try {
+		CPPUNIT_ASSERT(i.accountCount() == 0);
+		CPPUNIT_ASSERT(m->accountCount() == 7);
+
+		a.setInstitutionId(QCString());
+		m->modifyAccount(a);
+		m->removeAccount(a);
+		CPPUNIT_ASSERT(m->accountCount() == 6);
 		CPPUNIT_ASSERT(m->dirty() == true);
 		i = m->institution("I000001");
 		CPPUNIT_ASSERT(i.accountCount() == 0);
@@ -743,8 +783,9 @@ void MyMoneySeqAccessMgrTest::testRemoveInstitution() {
 	try {
 		i = m->institution("I000001");
 		a = m->account("A000006");
-		m->addAccount(i, a);
-		CPPUNIT_ASSERT(i.accountCount() == 1);
+		a.setInstitutionId(i.id());
+		m->modifyAccount(a);
+		CPPUNIT_ASSERT(i.accountCount() == 0);
 	} catch (MyMoneyException *e) {
 		delete e;
 		CPPUNIT_FAIL("Unexpected exception");
@@ -754,6 +795,8 @@ void MyMoneySeqAccessMgrTest::testRemoveInstitution() {
 	// now remove the institution and see if the account survived ;-)
 	try {
 		m->removeInstitution(i);
+		a.setInstitutionId(QCString());
+		m->modifyAccount(a);
 		a = m->account("A000006");
 		CPPUNIT_ASSERT(m->dirty() == true);
 		CPPUNIT_ASSERT(a.institutionId().isEmpty());
@@ -950,9 +993,9 @@ void MyMoneySeqAccessMgrTest::testRemoveAccountFromTree() {
 	// remains in the storag manager
 
 	try {
-		m->newAccount(a);
-		m->newAccount(b);
-		m->newAccount(c);
+		m->addAccount(a);
+		m->addAccount(b);
+		m->addAccount(c);
 		m->reparentAccount(b, a);
 		m->reparentAccount(c, b);
 
@@ -1014,7 +1057,9 @@ void MyMoneySeqAccessMgrTest::testPayeeName() {
 void MyMoneySeqAccessMgrTest::testAssignment() {
 	testAddTransactions();
 
-	m->setUserName("Thomas");
+	MyMoneyPayee user;
+	user.setName("Thomas");
+	m->setUser(user);
 
 	MyMoneySeqAccessMgr test = *m;
 	testEquality(&test);
@@ -1022,21 +1067,21 @@ void MyMoneySeqAccessMgrTest::testAssignment() {
 
 void MyMoneySeqAccessMgrTest::testEquality(const MyMoneySeqAccessMgr *t)
 {
-	CPPUNIT_ASSERT( m->m_userName == t->m_userName);
-	CPPUNIT_ASSERT( m->m_userStreet == t->m_userStreet);
-	CPPUNIT_ASSERT( m->m_userTown == t->m_userTown);
-	CPPUNIT_ASSERT( m->m_userCounty == t->m_userCounty);
-	CPPUNIT_ASSERT( m->m_userPostcode == t->m_userPostcode);
-	CPPUNIT_ASSERT( m->m_userTelephone == t->m_userTelephone);
-	CPPUNIT_ASSERT( m->m_userEmail == t->m_userEmail);
-	CPPUNIT_ASSERT( m->m_nextInstitutionID == t->m_nextInstitutionID);
-	CPPUNIT_ASSERT( m->m_nextAccountID == t->m_nextAccountID);
-	CPPUNIT_ASSERT( m->m_nextTransactionID == t->m_nextTransactionID);
-	CPPUNIT_ASSERT( m->m_nextPayeeID == t->m_nextPayeeID);
-	CPPUNIT_ASSERT( m->m_nextScheduleID == t->m_nextScheduleID);
-	CPPUNIT_ASSERT( m->m_dirty == t->m_dirty);
-	CPPUNIT_ASSERT( m->m_creationDate == t->m_creationDate);
-	CPPUNIT_ASSERT( m->m_lastModificationDate == t->m_lastModificationDate);
+	CPPUNIT_ASSERT(m->user().name() == t->user().name());
+	CPPUNIT_ASSERT(m->user().address() == t->user().address());
+	CPPUNIT_ASSERT(m->user().city() == t->user().city());
+	CPPUNIT_ASSERT(m->user().state() == t->user().state());
+	CPPUNIT_ASSERT(m->user().postcode() == t->user().postcode());
+	CPPUNIT_ASSERT(m->user().telephone() == t->user().telephone());
+	CPPUNIT_ASSERT(m->user().email() == t->user().email());
+	CPPUNIT_ASSERT(m->m_nextInstitutionID == t->m_nextInstitutionID);
+	CPPUNIT_ASSERT(m->m_nextAccountID == t->m_nextAccountID);
+	CPPUNIT_ASSERT(m->m_nextTransactionID == t->m_nextTransactionID);
+	CPPUNIT_ASSERT(m->m_nextPayeeID == t->m_nextPayeeID);
+	CPPUNIT_ASSERT(m->m_nextScheduleID == t->m_nextScheduleID);
+	CPPUNIT_ASSERT(m->m_dirty == t->m_dirty);
+	CPPUNIT_ASSERT(m->m_creationDate == t->m_creationDate);
+	CPPUNIT_ASSERT(m->m_lastModificationDate == t->m_lastModificationDate);
 
 	/*
 	 * make sure, that the keys and values are the same
@@ -1090,10 +1135,11 @@ void MyMoneySeqAccessMgrTest::testAddSchedule() {
 				 MyMoneySchedule::TYPE_DEPOSIT,
 				 MyMoneySchedule::OCCUR_DAILY,
 				 MyMoneySchedule::STYPE_MANUALDEPOSIT,
-				 QDate(2003,7,10),
+				 QDate(),
 				 QDate(),
 				 true,
 				 false);
+	t1.setPostDate(QDate(2003,7,10));
 	schedule.setTransaction(t1);
 
 		m->addSchedule(schedule);
@@ -1111,7 +1157,7 @@ void MyMoneySeqAccessMgrTest::testAddSchedule() {
 					 MyMoneySchedule::TYPE_DEPOSIT,
 					 MyMoneySchedule::OCCUR_DAILY,
 					 MyMoneySchedule::STYPE_MANUALDEPOSIT,
-					 QDate(2003,7,10),
+					 QDate(),
 					 QDate(),
 					 true,
 					 false);
@@ -1205,10 +1251,11 @@ void MyMoneySeqAccessMgrTest::testScheduleList() {
 				 MyMoneySchedule::TYPE_BILL,
 				 MyMoneySchedule::OCCUR_ONCE,
 				 MyMoneySchedule::STYPE_DIRECTDEBIT,
-				 notOverdue,
+				 QDate(),
 				 QDate(),
 				 false,
 				 false);
+	t1.setPostDate(notOverdue);
 	schedule1.setTransaction(t1);
 	schedule1.setLastPayment(notOverdue);
 
@@ -1222,10 +1269,11 @@ void MyMoneySeqAccessMgrTest::testScheduleList() {
 				 MyMoneySchedule::TYPE_DEPOSIT,
 				 MyMoneySchedule::OCCUR_DAILY,
 				 MyMoneySchedule::STYPE_DIRECTDEPOSIT,
-				 notOverdue.addDays(1),
+				 QDate(),
 				 QDate(),
 				 false,
 				 false);
+	t2.setPostDate(notOverdue.addDays(1));
 	schedule2.setTransaction(t2);
 	schedule2.setLastPayment(notOverdue.addDays(1));
 
@@ -1239,10 +1287,11 @@ void MyMoneySeqAccessMgrTest::testScheduleList() {
 				 MyMoneySchedule::TYPE_TRANSFER,
 				 MyMoneySchedule::OCCUR_WEEKLY,
 				 MyMoneySchedule::STYPE_OTHER,
-				 notOverdue.addDays(2),
+				 QDate(),
 				 QDate(),
 				 false,
 				 false);
+	t3.setPostDate(notOverdue.addDays(2));
 	schedule3.setTransaction(t3);
 	schedule3.setLastPayment(notOverdue.addDays(2));
 
@@ -1256,10 +1305,11 @@ void MyMoneySeqAccessMgrTest::testScheduleList() {
 				 MyMoneySchedule::TYPE_BILL,
 				 MyMoneySchedule::OCCUR_WEEKLY,
 				 MyMoneySchedule::STYPE_WRITECHEQUE,
-				 overdue.addDays(-7),
+				 QDate(),
 				 notOverdue.addDays(31),
 				 false,
 				 false);
+	t4.setPostDate(overdue.addDays(-7));
 	schedule4.setTransaction(t4);
 
 	try {
@@ -1347,3 +1397,146 @@ void MyMoneySeqAccessMgrTest::testScheduleList() {
 	CPPUNIT_ASSERT(list.count() == 1);
 	CPPUNIT_ASSERT(list[0].name() == "Schedule 4");
 }
+
+void MyMoneySeqAccessMgrTest::testAddCurrency()
+{
+	MyMoneySecurity curr("EUR", "Euro", "?", 100, 100);
+	CPPUNIT_ASSERT(m->m_currencyList.count() == 0);
+	m->m_dirty = false;
+	try {
+		m->addCurrency(curr);
+		CPPUNIT_ASSERT(m->m_currencyList.count() == 1);
+		CPPUNIT_ASSERT(m->m_currencyList["EUR"].name() == "Euro");
+		CPPUNIT_ASSERT(m->dirty() == true);
+	} catch(MyMoneyException *e) {
+                delete e;
+                CPPUNIT_FAIL("Unexpected exception");
+	}
+
+	m->m_dirty = false;
+	try {
+		m->addCurrency(curr);
+                CPPUNIT_FAIL("Expected exception missing");
+	} catch(MyMoneyException *e) {
+		CPPUNIT_ASSERT(m->dirty() == false);
+                delete e;
+	}
+}
+
+void MyMoneySeqAccessMgrTest::testModifyCurrency()
+{
+	MyMoneySecurity curr("EUR", "Euro", "?", 100, 100);
+	testAddCurrency();
+	m->m_dirty = false;
+	curr.setName("EURO");
+	try {
+		m->modifyCurrency(curr);
+		CPPUNIT_ASSERT(m->m_currencyList.count() == 1);
+		CPPUNIT_ASSERT(m->m_currencyList["EUR"].name() == "EURO");
+		CPPUNIT_ASSERT(m->dirty() == true);
+	} catch(MyMoneyException *e) {
+                delete e;
+                CPPUNIT_FAIL("Unexpected exception");
+	}
+
+	m->m_dirty = false;
+
+	MyMoneySecurity unknownCurr("DEM", "Deutsche Mark", "DM", 100, 100);
+	try {
+		m->modifyCurrency(unknownCurr);
+                CPPUNIT_FAIL("Expected exception missing");
+	} catch(MyMoneyException *e) {
+		CPPUNIT_ASSERT(m->dirty() == false);
+                delete e;
+	}
+}
+
+void MyMoneySeqAccessMgrTest::testRemoveCurrency()
+{
+	MyMoneySecurity curr("EUR", "Euro", "?", 100, 100);
+	testAddCurrency();
+	m->m_dirty = false;
+	try {
+		m->removeCurrency(curr);
+		CPPUNIT_ASSERT(m->m_currencyList.count() == 0);
+		CPPUNIT_ASSERT(m->dirty() == true);
+	} catch(MyMoneyException *e) {
+                delete e;
+                CPPUNIT_FAIL("Unexpected exception");
+	}
+
+	m->m_dirty = false;
+
+	MyMoneySecurity unknownCurr("DEM", "Deutsche Mark", "DM", 100, 100);
+	try {
+		m->removeCurrency(unknownCurr);
+                CPPUNIT_FAIL("Expected exception missing");
+	} catch(MyMoneyException *e) {
+		CPPUNIT_ASSERT(m->dirty() == false);
+                delete e;
+	}
+}
+
+void MyMoneySeqAccessMgrTest::testCurrency()
+{
+	MyMoneySecurity curr("EUR", "Euro", "?", 100, 100);
+	MyMoneySecurity newCurr;
+	testAddCurrency();
+	m->m_dirty = false;
+	try {
+		newCurr = m->currency("EUR");
+		CPPUNIT_ASSERT(m->dirty() == false);
+		CPPUNIT_ASSERT(newCurr.id() == curr.id());
+		CPPUNIT_ASSERT(newCurr.name() == curr.name());
+	} catch(MyMoneyException *e) {
+                delete e;
+                CPPUNIT_FAIL("Unexpected exception");
+	}
+
+	try {
+		m->currency("DEM");
+                CPPUNIT_FAIL("Expected exception missing");
+	} catch(MyMoneyException *e) {
+		CPPUNIT_ASSERT(m->dirty() == false);
+                delete e;
+	}
+}
+
+void MyMoneySeqAccessMgrTest::testCurrencyList()
+{
+	CPPUNIT_ASSERT(m->currencyList().count() == 0);
+
+	testAddCurrency();
+	CPPUNIT_ASSERT(m->currencyList().count() == 1);
+
+	MyMoneySecurity unknownCurr("DEM", "Deutsche Mark", "DM", 100, 100);
+	try {
+		m->addCurrency(unknownCurr);
+		m->m_dirty = false;
+		CPPUNIT_ASSERT(m->m_currencyList.count() == 2);
+		CPPUNIT_ASSERT(m->currencyList().count() == 2);
+		CPPUNIT_ASSERT(m->dirty() == false);
+	} catch(MyMoneyException *e) {
+                delete e;
+                CPPUNIT_FAIL("Unexpected exception");
+	}
+}
+
+void MyMoneySeqAccessMgrTest::testAccountList()
+{
+	QValueList<MyMoneyAccount> accounts;
+	m->accountList(accounts);
+	CPPUNIT_ASSERT(accounts.count() == 0);
+	testAddNewAccount();
+	accounts.clear();
+	m->accountList(accounts);
+	CPPUNIT_ASSERT(accounts.count() == 2);
+
+	MyMoneyAccount a = m->account("A000001");
+	MyMoneyAccount b = m->account("A000002");
+	m->reparentAccount(b, a);
+	accounts.clear();
+	m->accountList(accounts);
+	CPPUNIT_ASSERT(accounts.count() == 2);
+}
+
