@@ -54,16 +54,17 @@ void MyMoneyStorageDump::writeStream(QDataStream& _s, IMyMoneySerialize* _storag
 {
   QTextStream s(_s.device());
   IMyMoneyStorage* storage = dynamic_cast<IMyMoneyStorage *> (_storage);
+  MyMoneyPayee user = storage->user();
 
   s << "File-Info\n";
   s << "---------\n";
-  s << "user name = " << storage->userName() << "\n";
-  s << "user street = " << storage->userStreet() << "\n";
-  s << "user city = " << storage->userTown() << "\n";
-  s << "user city = " << storage->userCounty() << "\n";
-  s << "user zip = " << storage->userPostcode() << "\n";
-  s << "user telephone = " << storage->userTelephone() << "\n";
-  s << "user e-mail = " << storage->userEmail() << "\n";
+  s << "user name = " << user.name() << "\n";
+  s << "user street = " << user.address() << "\n";
+  s << "user city = " << user.city() << "\n";
+  s << "user city = " << user.state() << "\n";
+  s << "user zip = " << user.postcode() << "\n";
+  s << "user telephone = " << user.telephone() << "\n";
+  s << "user e-mail = " << user.email() << "\n";
   s << "creation date = " << storage->creationDate().toString(Qt::ISODate) << "\n";
   s << "last modification date = " << storage->lastModificationDate().toString(Qt::ISODate) << "\n";
   s << "base currency = " << storage->value("kmm-baseCurrency") << "\n";
@@ -71,9 +72,13 @@ void MyMoneyStorageDump::writeStream(QDataStream& _s, IMyMoneySerialize* _storag
 
   s << "Internal-Info\n";
   s << "-------------\n";
-  s << "accounts = " << _storage->accountList().count() <<", next id = " << _storage->accountId() << "\n";
+  QValueList<MyMoneyAccount> list_a;
+  storage->accountList(list_a);
+  s << "accounts = " << list_a.count() <<", next id = " << _storage->accountId() << "\n";
   MyMoneyTransactionFilter filter;
-  QValueList<MyMoneyTransaction> list_t = storage->transactionList(filter);
+  filter.setReportAllSplits(false);
+  QValueList<MyMoneyTransaction> list_t;
+  storage->transactionList(list_t, filter);
   QValueList<MyMoneyTransaction>::ConstIterator it_t;
   s << "transactions = " << list_t.count() << ", next id = " << _storage->transactionId() << "\n";
   QMap<int,int> xferCount;
@@ -134,7 +139,6 @@ void MyMoneyStorageDump::writeStream(QDataStream& _s, IMyMoneySerialize* _storag
   s << "Accounts" << "\n";
   s << "--------" << "\n";
 
-  QValueList<MyMoneyAccount> list_a = storage->accountList();
   list_a.push_front(storage->equity());
   list_a.push_front(storage->expense());
   list_a.push_front(storage->income());
@@ -150,7 +154,7 @@ void MyMoneyStorageDump::writeStream(QDataStream& _s, IMyMoneySerialize* _storag
     if((*it_a).currencyId().isEmpty()) {
       s << "  Currency = unknown\n";
     } else {
-      if((*it_a).accountType() == MyMoneyAccount::Stock) {
+      if((*it_a).isInvest()) {
         s << "  Equity = " << storage->security((*it_a).currencyId()).name() << "\n";
       } else {
         s << "  Currency = " << storage->currency((*it_a).currencyId()).name() << "\n";
@@ -175,9 +179,9 @@ void MyMoneyStorageDump::writeStream(QDataStream& _s, IMyMoneySerialize* _storag
     s << "\n";
 
     s << "  Opening data = " << (*it_a).openingDate().toString(Qt::ISODate) << "\n";
-    s << "  Opening balance = " << (*it_a).openingBalance().formatMoney() << "\n";
     s << "  Last modified = " << (*it_a).lastModified().toString(Qt::ISODate) << "\n";
     s << "  Last reconciled = " << (*it_a).lastReconciliationDate().toString(Qt::ISODate) << "\n";
+    s << "  Balance = " << (*it_a).balance().formatMoney("", 2) << "\n";
 
     s << "  KVP: " << "\n";
     QMap<QCString, QString>kvp = (*it_a).pairs();
@@ -264,7 +268,7 @@ void MyMoneyStorageDump::writeStream(QDataStream& _s, IMyMoneySerialize* _storag
     MyMoneyPriceEntries::ConstIterator it_pre;
     for(it_pre = (*it_pr).begin(); it_pre != (*it_pr).end(); ++it_pre) {
       s << "      Date = " << (*it_pre).date().toString() << "\n";
-      s << "        Price = " << (*it_pre).rate().formatMoney("", 8) << "\n";
+      s << "        Price = " << (*it_pre).rate(QCString()).formatMoney("", 8) << "\n";
       s << "        Source = " << (*it_pre).source() << "\n";
       s << "        From = " << (*it_pre).from() << "\n";
       s << "        To   = " << (*it_pre).to() << "\n";
@@ -308,7 +312,7 @@ void MyMoneyStorageDump::writeStream(QDataStream& _s, IMyMoneySerialize* _storag
     if((*it_s).isFinished())
       s << "  Next payment = payment finished" << "\n";
     else {
-      s << "  Next payment = " << (*it_s).nextPayment().toString(Qt::ISODate) << "\n";
+      s << "  Next payment = " << (*it_s).nextDueDate().toString(Qt::ISODate) << "\n";
       if((*it_s).isOverdue())
       s << "               = overdue!" << "\n";
     }
@@ -348,6 +352,7 @@ void MyMoneyStorageDump::dumpTransaction(QTextStream& s, IMyMoneyStorage* storag
   QValueList<MyMoneySplit>::ConstIterator it_s;
   for(it_s = it_t.splits().begin(); it_s != it_t.splits().end(); ++it_s) {
     s << "   ID = " << (*it_s).id() << "\n";
+    s << "    Transaction = " << (*it_s).transactionId() << "\n";
     s << "    Payee = " << (*it_s).payeeId();
     if(!(*it_s).payeeId().isEmpty()) {
       MyMoneyPayee p = storage->payee((*it_s).payeeId());
@@ -367,8 +372,10 @@ void MyMoneyStorageDump::dumpTransaction(QTextStream& s, IMyMoneyStorage* storag
     if((*it_s).value() == MyMoneyMoney::autoCalc)
       s << "    Value = will be calculated" << "\n";
     else
-      s << "    Value = " << (*it_s).value().formatMoney() << "\n";
-    s << "    Shares = " <<  (*it_s).shares().formatMoney() << "\n";
+      s << "    Value = " << (*it_s).value().formatMoney("", 2)
+                          << " (" << (*it_s).value().toString() << ")\n";
+    s << "    Shares = " <<  (*it_s).shares().formatMoney("", 2)
+                         << " (" << (*it_s).shares().toString() << ")\n";
     s << "    Action = '" << (*it_s).action() << "'\n";
     s << "    Nr = '" << (*it_s).number() << "'\n";
     s << "    ReconcileFlag = '" << reconcileToString((*it_s).reconcileFlag()) << "'\n";
@@ -406,12 +413,21 @@ const QString MyMoneyStorageDump::occurenceToString(const MyMoneySchedule::occur
     case MyMoneySchedule::OCCUR_EVERYOTHERWEEK:
       text = i18n("Every other week");
       break;
+    case MyMoneySchedule::OCCUR_EVERYTHREEWEEKS:
+      text = i18n("Every three weeks");
+      break;
+    case MyMoneySchedule::OCCUR_EVERYTHIRTYDAYS:
+      text = i18n("Every thirty days");
+      break;
     case MyMoneySchedule::OCCUR_EVERYFOURWEEKS:
       text = i18n("Every four weeks");
       break;
     case MyMoneySchedule::OCCUR_MONTHLY:
       text = i18n("Monthly");
       break;
+    case MyMoneySchedule::OCCUR_EVERYEIGHTWEEKS:
+      text = i18n("Every eight weeks");
+      break;
     case MyMoneySchedule::OCCUR_EVERYOTHERMONTH:
       text = i18n("Every two months");
       break;
@@ -486,6 +502,12 @@ const QString MyMoneyStorageDump::paymentMethodToString(MyMoneySchedule::payment
     case MyMoneySchedule::STYPE_WRITECHEQUE:
       text = i18n("Write check");
       break;
+    case MyMoneySchedule::STYPE_STANDINGORDER:
+      text = i18n("Standing order");
+      break;
+    case MyMoneySchedule::STYPE_BANKTRANSFER:
+      text = i18n("Bank transfer");
+      break;
     case MyMoneySchedule::STYPE_ANY:
       text = i18n("Any (Error)");
       break;
