@@ -23,6 +23,7 @@
 
 #include <kinstance.h>
 #include <kaboutdata.h>
+#include <kaction.h>
 
 // ----------------------------------------------------------------------------
 // Project Includes
@@ -39,6 +40,39 @@ KMyMoneyPlugin::Plugin::~Plugin()
 {
 }
 
+KAction* KMyMoneyPlugin::Plugin::action(const QString& actionName) const
+{
+  static KShortcut shortcut("");
+  static KAction dummyAction(QString("Dummy"), QString(), shortcut, static_cast<const QObject*>(this), 0, static_cast<KActionCollection*>(0), "");
+
+  KAction* p = actionCollection()->action(actionName.latin1());
+  if(p)
+    return p;
+
+  qWarning("Action with name '%s' not found!", actionName.latin1());
+  return &dummyAction;
+}
+
+KToggleAction* KMyMoneyPlugin::Plugin::toggleAction(const QString& actionName) const
+{
+  static KShortcut shortcut("");
+  static KToggleAction dummyAction(QString("Dummy"), QString(), shortcut, static_cast<const QObject*>(this), 0, static_cast<KActionCollection*>(0), "");
+
+  KAction* q = actionCollection()->action(actionName.latin1());
+
+  if(q) {
+    KToggleAction* p = dynamic_cast<KToggleAction*>(q);
+    if(!p) {
+      qWarning("Action '%s' is not of type KToggleAction", actionName.latin1());
+      p = &dummyAction;
+    }
+    return p;
+  }
+
+  qWarning("Action with name '%s' not found!", actionName.latin1());
+  return &dummyAction;
+}
+
 KMyMoneyPlugin::ViewInterface* KMyMoneyPlugin::Plugin::viewInterface()
 {
   return static_cast<ViewInterface*>( parent()->child( 0, "KMyMoneyPlugin::ViewInterface" ) );
@@ -49,6 +83,8 @@ KMyMoneyPlugin::StatementInterface* KMyMoneyPlugin::Plugin::statementInterface()
   return static_cast<StatementInterface*>( parent()->child( 0, "KMyMoneyPlugin::StatementInterface" ) );
 }
 
+
+
 KMyMoneyPlugin::OnlinePlugin::OnlinePlugin(QObject* o, const char* name) :
   KMyMoneyPlugin::Plugin(o, name)
 {
@@ -58,6 +94,10 @@ KMyMoneyPlugin::OnlinePlugin::~OnlinePlugin()
 {
 }
 
+
+
+
+
 KMyMoneyPlugin::ImporterPlugin::ImporterPlugin(QObject* o, const char* name) :
   QObject(o, name)
 {
@@ -69,6 +109,8 @@ KMyMoneyPlugin::ImporterPlugin::~ImporterPlugin()
 
 QString KMyMoneyPlugin::ImporterPlugin::formatName(void) const /*= 0*/
 { return QString(); }
+QString KMyMoneyPlugin::ImporterPlugin::formatFilenameFilter(void) const /*= 0*/
+{ return QString(); }
 bool KMyMoneyPlugin::ImporterPlugin::isMyFormat( const QString& /*filename*/ ) const /*= 0*/
 { return false; }
 bool KMyMoneyPlugin::ImporterPlugin::import( const QString& /*filename*/, QValueList<MyMoneyStatement>& /*result*/ ) /*= 0*/
