@@ -47,54 +47,17 @@
 using namespace reports;
 using namespace test;
 
-namespace pivottabletest
-{
-const MyMoneyMoney moCheckingOpen(1418.0);
-const MyMoneyMoney moCreditOpen(-418.0);
-const MyMoneyMoney moZero(0.0);
-const MyMoneyMoney moSolo(234.12);
-const MyMoneyMoney moParent1(88.01);
-const MyMoneyMoney moParent2(133.22);
-const MyMoneyMoney moParent(moParent1+moParent2);
-const MyMoneyMoney moChild(14.00);
-const MyMoneyMoney moThomas(5.11);
-const MyMoneyMoney moNoPayee(8944.70);
-
-QCString acAsset;
-QCString acLiability;
-QCString acExpense;
-QCString acIncome;
-QCString acChecking;
-QCString acCredit;
-QCString acSolo;
-QCString acParent;
-QCString acChild;
-QCString acForeign;
-QCString acCanChecking;
-QCString acJpyChecking;
-QCString acCanCash;
-QCString acJpyCash;
-QCString inBank;
-QCString eqStock1;
-QCString eqStock2;
-QCString acInvestment;
-QCString acStock1;
-QCString acStock2;
-QCString acDividends;
-
-}
-using namespace pivottabletest;
-
 PivotTableTest::PivotTableTest()
 {
 }
 
-void PivotTableTest::setUp () 
+void PivotTableTest::setUp ()
 {
   storage = new MyMoneySeqAccessMgr;
   file = MyMoneyFile::instance();
   file->attachStorage(storage);
 
+  MyMoneyFileTransaction ft;
   file->addCurrency(MyMoneySecurity("CAD", "Canadian Dollar",        "C$"));
   file->addCurrency(MyMoneySecurity("USD", "US Dollar",              "$"));
   file->addCurrency(MyMoneySecurity("JPY", "Japanese Yen",           QChar(0x00A5), 100, 1));
@@ -116,19 +79,20 @@ void PivotTableTest::setUp ()
   acParent = makeAccount(QString("Parent"),MyMoneyAccount::Expense,0,QDate(2004,1,11),acExpense);
   acChild = makeAccount(QString("Child"),MyMoneyAccount::Expense,0,QDate(2004,2,11),acParent);
   acForeign = makeAccount(QString("Foreign"),MyMoneyAccount::Expense,0,QDate(2004,1,11),acExpense);
-  
+
+  acSecondChild = makeAccount(QString("Second Child"),MyMoneyAccount::Expense,0,QDate(2004,2,11),acParent);
+  acGrandChild1 = makeAccount(QString("Grand Child 1"),MyMoneyAccount::Expense,0,QDate(2004,2,11),acChild);
+  acGrandChild2 = makeAccount(QString("Grand Child 2"),MyMoneyAccount::Expense,0,QDate(2004,2,11),acChild);
+
   MyMoneyInstitution i("Bank of the World","","","","","","");
-  MyMoneyKeyValueContainer ofxsettings;
-  ofxsettings.setValue("iban","123456789");
-  i.setOfxConnectionSettings(ofxsettings);
   file->addInstitution(i);
   inBank = i.id();
+  ft.commit();
 }
 
 void PivotTableTest::tearDown ()
 {
   file->detachStorage(storage);
-  delete file;
   delete storage;
 }
 
@@ -169,7 +133,7 @@ void PivotTableTest::testNetWorthOfsetting()
   filter.setDateFilter(QDate(2004,1,1),QDate(2005,1,1).addDays(-1));
   XMLandback(filter);
   PivotTable networth_f( filter );
-  CPPUNIT_ASSERT(networth_f.m_grid["Liability"]["Credit Card"][acCredit][7]==moCreditOpen);
+  CPPUNIT_ASSERT(networth_f.m_grid["Liability"]["Credit Card"][acCredit][7]==-moCreditOpen);
   CPPUNIT_ASSERT(networth_f.m_grid.m_total[0]==moZero);
   CPPUNIT_ASSERT(networth_f.m_grid.m_total[12]==moCheckingOpen+moCreditOpen);
 
@@ -188,7 +152,7 @@ void PivotTableTest::testNetWorthOpeningPrior()
   PivotTable networth_f( filter );
   writeTabletoCSV(networth_f);
 
-  CPPUNIT_ASSERT(networth_f.m_grid["Liability"]["Credit Card"].m_total[0]==moCreditOpen);
+  CPPUNIT_ASSERT(networth_f.m_grid["Liability"]["Credit Card"].m_total[0]==-moCreditOpen);
   CPPUNIT_ASSERT(networth_f.m_grid["Asset"]["Checking Account"].m_total[0]==moCheckingOpen);
   CPPUNIT_ASSERT(networth_f.m_grid.m_total[0]==moCheckingOpen+moCreditOpen);
   CPPUNIT_ASSERT(networth_f.m_grid.m_total[1]==moCheckingOpen+moCreditOpen);
@@ -203,9 +167,9 @@ void PivotTableTest::testNetWorthOpeningPrior()
   filter.setName("Net Worth Opening Prior 2");
   PivotTable networth_f2( filter );
   writeTabletoCSV(networth_f2);
-  CPPUNIT_ASSERT(networth_f2.m_grid["Liability"]["Credit Card"].m_total[0]==moCreditOpen-moParent);
-  CPPUNIT_ASSERT(networth_f2.m_grid["Asset"]["Checking Account"].m_total[0]==moCheckingOpen-moChild);
-  CPPUNIT_ASSERT(networth_f2.m_grid.m_total[0]==moCheckingOpen+moCreditOpen-moChild-moParent);
+  CPPUNIT_ASSERT(networth_f2.m_grid["Liability"]["Credit Card"].m_total[1]==-moCreditOpen+moParent);
+  CPPUNIT_ASSERT(networth_f2.m_grid["Asset"]["Checking Account"].m_total[1]==moCheckingOpen-moChild);
+  CPPUNIT_ASSERT(networth_f2.m_grid.m_total[1]==moCheckingOpen+moCreditOpen-moChild-moParent);
 }
 
 void PivotTableTest::testNetWorthDateFilter()
@@ -250,8 +214,8 @@ void PivotTableTest::testSingleTransaction()
   PivotTable spending_f( filter );
   writeTabletoHTML(spending_f,"Spending with Single Transaction.html");
 
-  CPPUNIT_ASSERT(spending_f.m_grid["Expense"]["Solo"][acSolo][2]==(-moSolo));
-  CPPUNIT_ASSERT(spending_f.m_grid["Expense"]["Solo"].m_total[2]==(-moSolo));
+  CPPUNIT_ASSERT(spending_f.m_grid["Expense"]["Solo"][acSolo][2]==moSolo);
+  CPPUNIT_ASSERT(spending_f.m_grid["Expense"]["Solo"].m_total[2]==moSolo);
   CPPUNIT_ASSERT(spending_f.m_grid["Expense"]["Solo"].m_total[1]==moZero);
   CPPUNIT_ASSERT(spending_f.m_grid.m_total[2]==(-moSolo));
   CPPUNIT_ASSERT(spending_f.m_grid.m_total.m_total==(-moSolo));
@@ -281,11 +245,11 @@ void PivotTableTest::testSubAccount()
   PivotTable spending_f( filter );
   writeTabletoHTML(spending_f,"Spending with Sub-Account.html");
 
-  CPPUNIT_ASSERT(spending_f.m_grid["Expense"]["Parent"][acParent][3]==(-moParent));
-  CPPUNIT_ASSERT(spending_f.m_grid["Expense"]["Parent"][acChild][3]==(-moChild));
-  CPPUNIT_ASSERT(spending_f.m_grid["Expense"]["Parent"].m_total[3]==(-moParent-moChild));
+  CPPUNIT_ASSERT(spending_f.m_grid["Expense"]["Parent"][acParent][3]==moParent);
+  CPPUNIT_ASSERT(spending_f.m_grid["Expense"]["Parent"][acChild][3]==moChild);
+  CPPUNIT_ASSERT(spending_f.m_grid["Expense"]["Parent"].m_total[3]==moParent+moChild);
   CPPUNIT_ASSERT(spending_f.m_grid["Expense"]["Parent"].m_total[2]==moZero);
-  CPPUNIT_ASSERT(spending_f.m_grid["Expense"]["Parent"].m_total.m_total==(-moParent-moChild));
+  CPPUNIT_ASSERT(spending_f.m_grid["Expense"]["Parent"].m_total.m_total==moParent+moChild);
   CPPUNIT_ASSERT(spending_f.m_grid.m_total[3]==(-moParent-moChild));
   CPPUNIT_ASSERT(spending_f.m_grid.m_total.m_total==(-moParent-moChild));
 
@@ -296,7 +260,7 @@ void PivotTableTest::testSubAccount()
   XMLandback(filter);
   PivotTable networth_f( filter );
   writeTabletoHTML(networth_f,"Net Worth with Sub-Account.html");
-  CPPUNIT_ASSERT(networth_f.m_grid["Liability"]["Credit Card"].m_total[3]==-moParent-moChild+moCreditOpen );
+  CPPUNIT_ASSERT(networth_f.m_grid["Liability"]["Credit Card"].m_total[3]==moParent+moChild-moCreditOpen );
   CPPUNIT_ASSERT(networth_f.m_grid.m_total[4] == -moParent-moChild+moCreditOpen+moCheckingOpen );
 
 }
@@ -316,8 +280,8 @@ void PivotTableTest::testFilterIEvsIE()
   XMLandback(filter);
   PivotTable spending_f( filter );
 
-  CPPUNIT_ASSERT(spending_f.m_grid["Expense"]["Parent"].m_total[3]==-moChild);
-  CPPUNIT_ASSERT(spending_f.m_grid["Expense"].m_total[2]==-moSolo);
+  CPPUNIT_ASSERT(spending_f.m_grid["Expense"]["Parent"].m_total[3]==moChild);
+  CPPUNIT_ASSERT(spending_f.m_grid["Expense"].m_total[2]==moSolo);
   CPPUNIT_ASSERT(spending_f.m_grid.m_total.m_total==-moSolo-moChild);
 
 }
@@ -357,7 +321,7 @@ void PivotTableTest::testFilterALvsIE()
   PivotTable spending_f( filter );
 
   CPPUNIT_ASSERT(spending_f.m_grid["Expense"].m_total[3]==moZero);
-  CPPUNIT_ASSERT(spending_f.m_grid["Expense"].m_total[2]==-moSolo);
+  CPPUNIT_ASSERT(spending_f.m_grid["Expense"].m_total[2]==moSolo);
   CPPUNIT_ASSERT(spending_f.m_grid.m_total.m_total==-moSolo);
 }
 
@@ -377,7 +341,7 @@ void PivotTableTest::testFilterAllvsIE()
   PivotTable spending_f( filter );
 
   CPPUNIT_ASSERT(spending_f.m_grid["Expense"].m_total[2]==moZero);
-  CPPUNIT_ASSERT(spending_f.m_grid["Expense"].m_total[3]==-moChild);
+  CPPUNIT_ASSERT(spending_f.m_grid["Expense"].m_total[3]==moChild);
   CPPUNIT_ASSERT(spending_f.m_grid.m_total.m_total==-moChild);
 }
 
@@ -418,8 +382,8 @@ void PivotTableTest::testFilterBasics()
 
 void PivotTableTest::testMultipleCurrencies()
 {
-  MyMoneyMoney moCanOpening( 1000.0 );
-  MyMoneyMoney moJpyOpening( 1000.0 );
+  MyMoneyMoney moCanOpening( 0.0 );
+  MyMoneyMoney moJpyOpening( 0.0 );
   MyMoneyMoney moCanPrice( 0.75 );
   MyMoneyMoney moJpyPrice( 0.010 );
   MyMoneyMoney moJpyPrice2( 0.011 );
@@ -453,30 +417,31 @@ void PivotTableTest::testMultipleCurrencies()
   IMyMoneyStorageFormat& interface = xml;
   interface.writeFile(&g, dynamic_cast<IMyMoneySerialize*> (MyMoneyFile::instance()->storage()));
   g.close();
-#endif  
+#endif
 
   MyMoneyReport filter;
   filter.setRowType( MyMoneyReport::eExpenseIncome );
   filter.setDateFilter(QDate(2004,1,1),QDate(2005,1,1).addDays(-1));
   filter.setShowSubAccounts(true);
+  filter.setConvertCurrency(true);
   filter.setName("Multiple Currency Spending Rerport (with currency conversion)");
   XMLandback(filter);
-  
+
   PivotTable spending_f( filter );
-  
+
   writeTabletoCSV(spending_f);
 
   // test single foreign currency
-  CPPUNIT_ASSERT(spending_f.m_grid["Expense"]["Foreign"][acCanCash][2]==(-moCanTransaction*moCanPrice));
-  CPPUNIT_ASSERT(spending_f.m_grid["Expense"]["Foreign"][acCanCash][3]==(-moCanTransaction*moCanPrice));
-  CPPUNIT_ASSERT(spending_f.m_grid["Expense"]["Foreign"][acCanCash][4]==(-moCanTransaction*moCanPrice));
+  CPPUNIT_ASSERT(spending_f.m_grid["Expense"]["Foreign"][acCanCash][2]==(moCanTransaction*moCanPrice));
+  CPPUNIT_ASSERT(spending_f.m_grid["Expense"]["Foreign"][acCanCash][3]==(moCanTransaction*moCanPrice));
+  CPPUNIT_ASSERT(spending_f.m_grid["Expense"]["Foreign"][acCanCash][4]==(moCanTransaction*moCanPrice));
 
   // test multiple foreign currencies under a common parent
-  CPPUNIT_ASSERT(spending_f.m_grid["Expense"]["Foreign"][acJpyCash][2]==(-moJpyTransaction*moJpyPrice));
-  CPPUNIT_ASSERT(spending_f.m_grid["Expense"]["Foreign"][acJpyCash][3]==(-moJpyTransaction*moJpyPrice));
-  CPPUNIT_ASSERT(spending_f.m_grid["Expense"]["Foreign"][acJpyCash][4]==(-moJpyTransaction*moJpyPrice));
-  CPPUNIT_ASSERT(spending_f.m_grid["Expense"]["Foreign"].m_total[2]==(-moJpyTransaction*moJpyPrice-moCanTransaction*moCanPrice));
-  CPPUNIT_ASSERT(spending_f.m_grid["Expense"]["Foreign"].m_total.m_total==(-moJpyTransaction*moJpyPrice-moCanTransaction*moCanPrice-moJpyTransaction*moJpyPrice-moCanTransaction*moCanPrice-moJpyTransaction*moJpyPrice-moCanTransaction*moCanPrice));
+  CPPUNIT_ASSERT(spending_f.m_grid["Expense"]["Foreign"][acJpyCash][2]==(moJpyTransaction*moJpyPrice));
+  CPPUNIT_ASSERT(spending_f.m_grid["Expense"]["Foreign"][acJpyCash][3]==(moJpyTransaction*moJpyPrice));
+  CPPUNIT_ASSERT(spending_f.m_grid["Expense"]["Foreign"][acJpyCash][4]==(moJpyTransaction*moJpyPrice));
+  CPPUNIT_ASSERT(spending_f.m_grid["Expense"]["Foreign"].m_total[2]==(moJpyTransaction*moJpyPrice + moCanTransaction*moCanPrice));
+  CPPUNIT_ASSERT(spending_f.m_grid["Expense"]["Foreign"].m_total.m_total==(moJpyTransaction*moJpyPrice + moCanTransaction*moCanPrice + moJpyTransaction*moJpyPrice + moCanTransaction*moCanPrice + moJpyTransaction*moJpyPrice + moCanTransaction*moCanPrice));
 
   // Test the report type where we DO NOT convert the currency
   filter.setConvertCurrency(false);
@@ -486,12 +451,12 @@ void PivotTableTest::testMultipleCurrencies()
   PivotTable spending_fnc( filter );
   writeTabletoCSV(spending_fnc);
 
-  CPPUNIT_ASSERT(spending_fnc.m_grid["Expense"]["Foreign"][acCanCash][2]==(-moCanTransaction));
-  CPPUNIT_ASSERT(spending_fnc.m_grid["Expense"]["Foreign"][acCanCash][3]==(-moCanTransaction));
-  CPPUNIT_ASSERT(spending_fnc.m_grid["Expense"]["Foreign"][acCanCash][4]==(-moCanTransaction));
-  CPPUNIT_ASSERT(spending_fnc.m_grid["Expense"]["Foreign"][acJpyCash][2]==(-moJpyTransaction));
-  CPPUNIT_ASSERT(spending_fnc.m_grid["Expense"]["Foreign"][acJpyCash][3]==(-moJpyTransaction));
-  CPPUNIT_ASSERT(spending_fnc.m_grid["Expense"]["Foreign"][acJpyCash][4]==(-moJpyTransaction));
+  CPPUNIT_ASSERT(spending_fnc.m_grid["Expense"]["Foreign"][acCanCash][2]==(moCanTransaction));
+  CPPUNIT_ASSERT(spending_fnc.m_grid["Expense"]["Foreign"][acCanCash][3]==(moCanTransaction));
+  CPPUNIT_ASSERT(spending_fnc.m_grid["Expense"]["Foreign"][acCanCash][4]==(moCanTransaction));
+  CPPUNIT_ASSERT(spending_fnc.m_grid["Expense"]["Foreign"][acJpyCash][2]==(moJpyTransaction));
+  CPPUNIT_ASSERT(spending_fnc.m_grid["Expense"]["Foreign"][acJpyCash][3]==(moJpyTransaction));
+  CPPUNIT_ASSERT(spending_fnc.m_grid["Expense"]["Foreign"][acJpyCash][4]==(moJpyTransaction));
 
   filter.setConvertCurrency(true);
   filter.clear();
@@ -534,7 +499,7 @@ void PivotTableTest::testAdvancedFilter()
   {
     TransactionHelper t1( QDate(2004,11,7), MyMoneySplit::ActionWithdrawal, moParent1, acCredit, acParent );
     TransactionHelper t2( QDate(2004,11,7), MyMoneySplit::ActionWithdrawal, moChild, acCredit, acChild );
-  
+
     MyMoneyReport filter;
     filter.setRowType( MyMoneyReport::eExpenseIncome );
     filter.setDateFilter(QDate(2004,1,1),QDate(2005,1,1).addDays(-1));
@@ -550,7 +515,7 @@ void PivotTableTest::testAdvancedFilter()
     TransactionHelper t2( QDate(2004,11,7), MyMoneySplit::ActionWithdrawal, moParent1, acCredit, acParent );
     TransactionHelper t3( QDate(2004,11,7), MyMoneySplit::ActionWithdrawal, moChild, acCredit, acChild );
     TransactionHelper t4( QDate(2004,11,7), MyMoneySplit::ActionWithdrawal, moThomas, acCredit, acParent, QCString(), "Thomas Baumgart" );
-    
+
     MyMoneyReport filter;
     filter.setRowType( MyMoneyReport::eExpenseIncome );
     filter.setDateFilter(QDate(2004,1,1),QDate(2005,1,1).addDays(-1));
@@ -559,8 +524,8 @@ void PivotTableTest::testAdvancedFilter()
     XMLandback(filter);
     PivotTable spending_f( filter );
     writeTabletoHTML(spending_f,"Spending with Payee Filter.html");
-    
-    CPPUNIT_ASSERT(spending_f.m_grid["Expense"]["Parent"][acParent][11]==-moThomas);
+
+    CPPUNIT_ASSERT(spending_f.m_grid["Expense"]["Parent"][acParent][11]==moThomas);
     CPPUNIT_ASSERT(spending_f.m_grid.m_total.m_total==-moThomas);
   }
   // payee (no payee)
@@ -576,7 +541,7 @@ void PivotTableTest::testAdvancedFilter()
     filter.addPayee(QCString());
     XMLandback(filter);
     PivotTable spending_f( filter );
-    CPPUNIT_ASSERT(spending_f.m_grid["Expense"]["Parent"][acParent][11]==-moNoPayee);
+    CPPUNIT_ASSERT(spending_f.m_grid["Expense"]["Parent"][acParent][11]==moNoPayee);
     CPPUNIT_ASSERT(spending_f.m_grid.m_total.m_total==-moNoPayee);
   }
 
@@ -629,7 +594,7 @@ void PivotTableTest::testAdvancedFilter()
     PivotTable networth_f4( filter );
 
     CPPUNIT_ASSERT(networth_f4.m_grid["Asset"].m_total[11] == moCheckingOpen + moChild);
-    CPPUNIT_ASSERT(networth_f4.m_grid["Liability"].m_total[11] == moCreditOpen - moChild);
+    CPPUNIT_ASSERT(networth_f4.m_grid["Liability"].m_total[11] == - moCreditOpen + moChild);
     CPPUNIT_ASSERT(networth_f4.m_grid.m_total[10] == moCheckingOpen + moCreditOpen);
     CPPUNIT_ASSERT(networth_f4.m_grid.m_total[11] == moCheckingOpen + moCreditOpen);
   }
@@ -842,16 +807,66 @@ void PivotTableTest::testColumnType()
   CPPUNIT_ASSERT(networth_y.m_grid.m_total[2] == -moSolo-moParent-moSolo-moParent+moCheckingOpen+moCreditOpen);
   CPPUNIT_ASSERT(networth_y.m_grid.m_total[3] == -moSolo-moParent-moSolo-moParent-moSolo-moParent+moCheckingOpen+moCreditOpen);
 
+  // Test days-based reports
+
+  TransactionHelper t1d1( QDate(2004,7,1), MyMoneySplit::ActionWithdrawal, moSolo, acChecking, acSolo );
+  TransactionHelper t2d1( QDate(2004,7,1), MyMoneySplit::ActionWithdrawal, moParent1, acCredit, acParent );
+  TransactionHelper t3d1( QDate(2004,7,5), MyMoneySplit::ActionWithdrawal, moParent2, acCredit, acParent );
+
+  TransactionHelper t1d2( QDate(2004,7,14), MyMoneySplit::ActionWithdrawal, moSolo, acChecking, acSolo );
+  TransactionHelper t2d2( QDate(2004,7,15), MyMoneySplit::ActionWithdrawal, moParent1, acCredit, acParent );
+  TransactionHelper t3d2( QDate(2004,7,20), MyMoneySplit::ActionWithdrawal, moParent2, acCredit, acParent );
+
+  TransactionHelper t1d3( QDate(2004,8,2), MyMoneySplit::ActionWithdrawal, moSolo, acChecking, acSolo );
+  TransactionHelper t2d3( QDate(2004,8,3), MyMoneySplit::ActionWithdrawal, moParent1, acCredit, acParent );
+  TransactionHelper t3d3( QDate(2004,8,4), MyMoneySplit::ActionWithdrawal, moParent2, acCredit, acParent );
+
+  filter.setDateFilter(QDate(2004,7,2),QDate(2004,7,14));
+  filter.setRowType( MyMoneyReport::eExpenseIncome );
+  filter.setColumnType(MyMoneyReport::eMonths);
+  filter.setColumnsAreDays(true);
+
+  XMLandback(filter);
+  PivotTable spending_days( filter );
+  writeTabletoHTML(spending_days,"Spending by Days.html");
+
+  CPPUNIT_ASSERT(spending_days.m_grid.m_total[4] == -moParent2);
+  CPPUNIT_ASSERT(spending_days.m_grid.m_total[13] == -moSolo);
+  CPPUNIT_ASSERT(spending_days.m_grid.m_total.m_total == -moSolo-moParent2);
+
+  unsigned save_dayweekstart = KGlobal::locale()->weekStartDay();
+  KGlobal::locale()->setWeekStartDay(2);
+
+  filter.setDateFilter(QDate(2004,7,2),QDate(2004,8,1));
+  filter.setRowType( MyMoneyReport::eExpenseIncome );
+  filter.setColumnType(static_cast<MyMoneyReport::EColumnType>(7));
+  filter.setColumnsAreDays(true);
+
+  XMLandback(filter);
+  PivotTable spending_weeks( filter );
+  writeTabletoHTML(spending_weeks,"Spending by Weeks.html");
+
+  KGlobal::locale()->setWeekStartDay(save_dayweekstart);
+
+  CPPUNIT_ASSERT(spending_weeks.m_grid.m_total[0] == moZero);
+  CPPUNIT_ASSERT(spending_weeks.m_grid.m_total[1] == -moParent2);
+  CPPUNIT_ASSERT(spending_weeks.m_grid.m_total[2] == moZero);
+  CPPUNIT_ASSERT(spending_weeks.m_grid.m_total[3] == -moSolo-moParent1);
+  CPPUNIT_ASSERT(spending_weeks.m_grid.m_total[4] == -moParent2);
+  CPPUNIT_ASSERT(spending_weeks.m_grid.m_total[5] == moZero);
+  CPPUNIT_ASSERT(spending_weeks.m_grid.m_total.m_total == -moSolo-moParent-moParent2);
+
+
 }
 
 void PivotTableTest::testInvestment(void)
 {
-  try 
+  try
   {
   // Equities
   eqStock1 = makeEquity("Stock1","STK1");
   eqStock2 = makeEquity("Stock2","STK2");
-  
+
   // Accounts
   acInvestment = makeAccount("Investment",MyMoneyAccount::Investment,moZero,QDate(2004,1,1),acAsset);
   acStock1 = makeAccount("Stock 1",MyMoneyAccount::Stock,moZero,QDate(2004,1,1),acInvestment,eqStock1);
@@ -881,8 +896,8 @@ void PivotTableTest::testInvestment(void)
   XMLandback(networth_r);
   PivotTable networth(networth_r);
 
-  networth.dump("networth_i.html");  
-  
+  networth.dump("networth_i.html");
+
   CPPUNIT_ASSERT(networth.m_grid["Asset"]["Investment"].m_total[1]==moZero);
   // 1000 shares @ $100.00
   CPPUNIT_ASSERT(networth.m_grid["Asset"]["Investment"].m_total[2]==MyMoneyMoney(100000.0));
@@ -912,16 +927,80 @@ void PivotTableTest::testInvestment(void)
   interface.writeFile(&g, dynamic_cast<IMyMoneySerialize*> (MyMoneyFile::instance()->storage()));
   g.close();
 
-  invtran.dump("invtran.html","<html><head></head><body>%1</body></html>");  
+  invtran.dump("invtran.html","<html><head></head><body>%1</body></html>");
   invhold.dump("invhold.html","<html><head></head><body>%1</body></html>");
-#endif    
-  
+#endif
+
   }
-  catch(MyMoneyException *e) 
+  catch(MyMoneyException *e)
   {
     CPPUNIT_FAIL(e->what());
     delete e;
   }
 }
 
+void PivotTableTest::testBudget(void)
+{
+
+  // 1. Budget on A, transations on A
+  {
+    BudgetHelper budget;
+    budget += BudgetEntryHelper( QDate(2006,1,1), acSolo, false, 100.0 );
+
+    MyMoneyReport report(MyMoneyReport::eBudgetActual,MyMoneyReport::eMonths,MyMoneyTransactionFilter::yearToDate,
+      false,"Yearly Budgeted vs. Actual","Default Report");
+    PivotTable table(report);
+  }
+
+  // 2. Budget on B, not applying to sub accounts, transactions on B and B:1
+  {
+    BudgetHelper budget;
+    budget += BudgetEntryHelper( QDate(2006,1,1), acParent, false, 100.0 );
+    MyMoneyReport report(MyMoneyReport::eBudgetActual,MyMoneyReport::eMonths,MyMoneyTransactionFilter::yearToDate,
+      false,"Yearly Budgeted vs. Actual","Default Report");
+    PivotTable table(report);
+  }
+
+  //  - Both B and B:1 totals should show up
+  //  - B actuals compare against B budget
+  //  - B:1 actuals compare against 0
+
+  // 3. Budget on C, applying to sub accounts, transactions on C and C:1 and C:1:a
+  {
+    BudgetHelper budget;
+    budget += BudgetEntryHelper( QDate(2006,1,1), acParent, true, 100.0 );
+    MyMoneyReport report(MyMoneyReport::eBudgetActual,MyMoneyReport::eMonths,MyMoneyTransactionFilter::yearToDate,
+      false,"Yearly Budgeted vs. Actual","Default Report");
+    PivotTable table(report);
+  }
+
+  //  - Only C totals show up, not C:1 or C:1:a totals
+  //  - C + C:1 totals compare against C budget
+
+  // 4. Budget on D, not applying to sub accounts, budget on D:1 not applying, budget on D:2 applying.  Transactions on D, D:1, D:2, D:2:a, D:2:b
+  {
+    BudgetHelper budget;
+    budget += BudgetEntryHelper( QDate(2006,1,1), acParent, false, 100.0 );
+    budget += BudgetEntryHelper( QDate(2006,1,1), acChild, false, 100.0 );
+    budget += BudgetEntryHelper( QDate(2006,1,1), acSecondChild, true, 100.0 );
+    MyMoneyReport report(MyMoneyReport::eBudgetActual,MyMoneyReport::eMonths,MyMoneyTransactionFilter::yearToDate,
+      false,"Yearly Budgeted vs. Actual","Default Report");
+    PivotTable table(report);
+  }
+
+  //  - Totals for D, D:1, D:2 show up.  D:2:a and D:2:b do not
+  //  - D actuals (only) compare against D budget
+  //  - Ditto for D:1
+  //  - D:2 acutals and children compare against D:2 budget
+
+  // 5. Budget on E, no transactions on E
+  {
+    BudgetHelper budget;
+    budget += BudgetEntryHelper( QDate(2006,1,1), acSolo, false, 100.0 );
+    MyMoneyReport report(MyMoneyReport::eBudgetActual,MyMoneyReport::eMonths,MyMoneyTransactionFilter::yearToDate,
+      false,"Yearly Budgeted vs. Actual","Default Report");
+    PivotTable table(report);
+  }
+}
+
 // vim:cin:si:ai:et:ts=2:sw=2:
