@@ -50,53 +50,7 @@ class QFrame;
 class KHomeView : public KMyMoneyViewBase
 {
   Q_OBJECT
-
-private:
-  KHTMLPart *m_part;
-  QVBoxLayout *m_qvboxlayoutPage;
-  QString m_filename;
-  bool    m_showAllSchedules;
-
-signals:
-  /**
-    * This signal is emitted whenever this view is activated.
-    */
-  void signalViewActivated(void);
-
-  void ledgerSelected(const QCString& id, const QCString& transaction);
-  void scheduleSelected(const QCString& id);
-  void reportSelected(const QCString& id);
-
 public:
-  /**
-    * Standard constructor.
-    *
-    * @param parent The QWidget this is used in.
-    * @param name The QT name.
-    *
-    * @return An object of type KHomeView
-    *
-    * @see ~KHomeView
-  **/
-  KHomeView(QWidget *parent=0, const char *name=0);
-
-  /**
-    * Standard destructor.
-    *
-    * @return Nothing.
-    *
-    * @see KHomeView
-  **/
-  ~KHomeView();
-
-  /**
-    * Overridden so we can emit the activated signal.
-    *
-    * @return Nothing.
-    */
-  void show(void);
-
-protected:
   /**
     * Definition of bitmap used as argument for showAccounts().
     */
@@ -105,25 +59,56 @@ protected:
     Payment = 2             ///< show payment accounts
   };
 
+  KHomeView(QWidget *parent=0, const char *name=0);
+  ~KHomeView();
+
+protected:
   void showPayments(void);
-  void showPaymentEntry(const MyMoneySchedule&);
+  void showPaymentEntry(const MyMoneySchedule&, int cnt = 1);
   void showAccounts(paymentTypeE type, const QString& hdr);
   void showAccountEntry(const MyMoneyAccount&);
-  void showFavoriteReports();
+  void showFavoriteReports(void);
+  void showForecast(void);
+  void showNetWorthGraph(void);
+  void showSummary(void);
+  void showBudget(void);
 
   const QString link(const QString& view, const QString& query) const;
   const QString linkend(void) const;
+  void loadView(void);
 
 public slots:
-  void slotOpenURL(const KURL &url, const KParts::URLArgs& args);
+  /**
+    * Overridden so we can emit the activated signal.
+    *
+    * @return Nothing.
+    */
+  void show(void);
 
-  void slotRefreshView(void);
-  void slotReloadView(void) { slotRefreshView(); };
+  void slotOpenURL(const KURL &url, const KParts::URLArgs& args);
+  void slotLoadView(void);
 
   /**
     * Print the current view
     */
   void slotPrintView(void);
+
+signals:
+  void ledgerSelected(const QCString& id, const QCString& transaction);
+  void scheduleSelected(const QCString& id);
+  void reportSelected(const QCString& id);
+
+private:
+
+  void showAccountEntry(const MyMoneyAccount& acc, const MyMoneyMoney& value, const MyMoneyMoney& valueToMinBal, const bool showMinBal);
+  MyMoneyMoney investmentBalance(const MyMoneyAccount& acc);
+  QString showColoredAmount(const QString& amount, bool isNegative);
+
+  KHTMLPart*      m_part;
+  QVBoxLayout*    m_qvboxlayoutPage;
+  QString         m_filename;
+  bool            m_showAllSchedules;
+  bool            m_needReload;
 };
 
 #endif
