@@ -23,14 +23,15 @@
 
 // ----------------------------------------------------------------------------
 // QT Includes
+
 #include <qwidget.h>
-#include <qmessagebox.h>
-#include <qvbox.h>
+class QVBox;
 class QFile;
 class QVBoxLayout;
 
 // ----------------------------------------------------------------------------
 // KDE Includes
+
 #include <kpopupmenu.h>
 #include <kjanuswidget.h>
 
@@ -44,20 +45,24 @@ class QVBoxLayout;
 #include <kmymoney/mymoneytransaction.h>
 #include <kmymoney/mymoneyscheduled.h>
 #include <kmymoney/mymoneysecurity.h>
+#include <kmymoney/selectedtransaction.h>
 
 class KHomeView;
 class KAccountsView;
 class KCategoriesView;
+class KInstitutionsView;
 class KPayeesView;
+class KBudgetView;
 class KScheduledView;
-class KNewAccountWizard;
 class KGlobalLedgerView;
 class IMyMoneyStorageFormat;
 class MyMoneyTransaction;
 class KInvestmentView;
 class KReportsView;
 class KMyMoneyViewBase;
-class KFindTransactionDlg;
+class MyMoneyReport;
+class TransactionEditor;
+class KForecastView;
 
 /**
   * This class represents the view of the MyMoneyFile which contains
@@ -69,9 +74,9 @@ class KFindTransactionDlg;
   *
   * @short Handles the view of the MyMoneyFile.
   */
-class KMyMoneyView : public KJanusWidget {
-   Q_OBJECT
-
+class KMyMoneyView : public KJanusWidget
+{
+  Q_OBJECT
 
 public:
   enum viewID {
@@ -83,7 +88,13 @@ public:
     PayeesView,
     LedgersView,
     InvestmentsView,
-    ReportsView
+    ReportsView,
+    BudgetView,
+    ForecastView
+  };
+  // file actions for plugin
+  enum fileActions {
+    preOpen, postOpen, preSave, postSave, preClose, postClose
   };
 
 private:
@@ -99,53 +110,61 @@ private:
     CategoryNew
   };
 
+  typedef enum storageTypeE { // not used but keep for future implementation
+    Memory = 0,
+    Database
+  } _storageType;
+
   KHomeView *m_homeView;
   KAccountsView *m_accountsView;
-  KAccountsView *m_institutionsView;
+  KInstitutionsView *m_institutionsView;
   KCategoriesView *m_categoriesView;
   KPayeesView *m_payeesView;
+  KBudgetView *m_budgetView;
   KScheduledView *m_scheduledView;
   KGlobalLedgerView *m_ledgerView;
   KInvestmentView *m_investmentView;
   KReportsView* m_reportsView;
-  KFindTransactionDlg* m_searchDlg;
+  KForecastView* m_forecastView;
 
   QVBox* m_homeViewFrame;
   QVBox* m_accountsViewFrame;
   QVBox* m_institutionsViewFrame;
   QVBox* m_categoriesViewFrame;
   QVBox* m_payeesViewFrame;
+  QVBox* m_budgetViewFrame;
   QVBox* m_scheduleViewFrame;
   QVBox* m_ledgerViewFrame;
   QVBox* m_investmentViewFrame;
   QVBox* m_reportsViewFrame;
+  QVBox* m_forecastViewFrame;
+
+  bool m_inConstructor;
 
   bool m_fileOpen;
 
-  KPopupMenu* m_accountMenu;
-  KPopupMenu* m_bankMenu;
-  KPopupMenu* m_rightMenu;
+  int  m_fmode;
+
+  // bool m_bankRightClick;
+  // MyMoneyInstitution m_accountsInstitution;
 
-  bool m_bankRightClick;
-  MyMoneyInstitution m_accountsInstitution;
   // Keep a note of the file type
   typedef enum _fileTypeE {
     KmmBinary = 0, // native, binary
     KmmXML,        // native, XML
+    KmmDbFile,     // database used as file, pretty much redundant now
+    KmmDbSingleUser,    // database with some data maintained in memory
+    KmmDbMultiUser,   // 'proper' database support not implemented
     /* insert new native file types above this line */
     MaxNativeFileType,
     /* and non-native types below */
-    GncXML
+    GncXML         // Gnucash XML
   }fileTypeE;
   fileTypeE m_fileType;
+
 private:
   void ungetString(QIODevice *qfile, char * buf, int len);
 
-  /**
-    * This method preloads all known currencies into the engine.
-    */
-  void loadDefaultCurrencies(void);
-
   /**
     * This method creates the currency @p curr if it does not exist and
     * @p create is @p true. If the currency already exists, it checks
@@ -163,8 +182,6 @@ private:
     */
   void loadAncientCurrency(const QCString& id, const QString& name, const QString& sym, const QDate& date, const MyMoneyMoney& rate, const QCString& newId, const int partsPerUnit = 100, const int smallestCashFraction = 100, const int smallestAccountFraction = 0);
 
-  void loadAncientCurrencies(void);
-
   /**
     * if no base currency is defined, start the dialog and force it to be set
     */
@@ -175,7 +192,7 @@ private:
     * object. It calls removeStorage() to remove a possibly attached
     * storage object.
     */
-  void newStorage(void);
+  void newStorage(storageTypeE = Memory);
 
   /**
     * This method removes an attached storage from the MyMoneyFile
@@ -189,31 +206,23 @@ private:
 
   /**
     */
-  void fixFile(void);
+  void fixFile_0(void);
+  void fixFile_1(void);
 
   /**
     */
-  void fixLoanAccount(MyMoneyAccount acc);
-
-  /**
-    * This method converts a possible opening balance held with @p
-    * account into a transaction between the account in question and
-    * the opening balances account for the currency of @p account.
-    * @p account is updated. If the opening balance is 0, then nothing
-    * is changed.
-    *
-    * @param account reference to the account object to be checked
-    */
-  void fixOpeningBalance(MyMoneyAccount& account);
+  void fixLoanAccount_0(MyMoneyAccount acc);
 
   /**
     */
-  void fixTransactions(void);
-  void fixSchedule(MyMoneySchedule sched);
-  void fixDuplicateAccounts(MyMoneyTransaction& t);
+  void fixTransactions_0(void);
+  void fixSchedule_0(MyMoneySchedule sched);
+  void fixDuplicateAccounts_0(MyMoneyTransaction& t);
 
   void createSchedule(MyMoneySchedule s, MyMoneyAccount& a);
 
+  void checkAccountName(const MyMoneyAccount& acc, const QString& name) const;
+
 public:
   /**
     * The constructor for KMyMoneyView. Just creates all the tabs for the
@@ -256,11 +265,33 @@ public:
     *
     * @param url The URL to save into.
     *            If no protocol is specified, file:// is assumed.
+    * @param key QString containing the key to be used for encryption
+    *            If @p key is empty, the file will be saved unencrypted
+    *            (the default)
     *
     * @retval false save operation failed
     * @retval true save operation was successful
     */
-  const bool saveFile(const KURL& url);
+  const bool saveFile(const KURL& url, const QString& key = QString());
+  /**
+   * Saves the data into permanent storage on a new or empty SQL database.
+   *
+   * @param url The pseudo of tyhe database
+   *
+   * @retval false save operation failed
+   * @retval true save operation was successful
+   */
+  //const bool saveDatabase(const KURL& url); This no longer relevant
+  /**
+   * Saves the data into permanent storage on a new or empty SQL database.
+   *
+   * @param url The pseudo URL of the database
+   *
+   * @retval false save operation failed
+   * @retval true save operation was successful
+   */
+  const bool saveAsDatabase(const KURL& url);
+
   /**
     * Call this to find out if the currently open file is native KMM
     *
@@ -269,6 +300,25 @@ public:
     */
   const bool isNativeFile() { return (m_fileType < MaxNativeFileType);};
 
+  /**
+   * Call this to find out if the currently open file is a sql database
+   *
+   * @retval true file is database
+   * @retval false file is serial
+   */
+  const bool isDatabase()
+    { return ((m_fileType == KmmDbFile) || (m_fileType == KmmDbSingleUser) || (m_fileType == KmmDbMultiUser));};
+
+  /**
+   * Call this to find out if the currently open file is a SQL database
+    * opened in sync mode (i.e. updates writen to database as they happen)
+   *
+   * @retval true file is synchronous
+   * @retval false file is asynchronous
+   */
+  const bool isSyncDatabase()
+  { return ((m_fileType == KmmDbSingleUser) || (m_fileType == KmmDbMultiUser));};
+
   /**
     * Call this to see if the MyMoneyFile contains any unsaved data.
     *
@@ -278,55 +328,17 @@ public:
   bool dirty(void);
 
   /**
-    * Creates a new file first making sure that one isn't open already.  Opens
-    * up a KNewFileDlg to input the new details.
-    *
-    * @param createEmtpyFile if false (default) the user will be asked for personal
-    *                        information and will be offered to load default income
-    *                        and expense accounts. If true, a completely empty file
-    *                        will be created without user interaction.
-    * @retval false User pressed Cancel button
-    * @retval true User pressed Ok
+    * Close the currently opened file and create an empty new file.
     *
     * @see MyMoneyFile
     */
-  bool newFile(const bool createEmtpyFile = false);
-
-  /**
-    * Brings up a dialog that displays information about the user who created
-    * the MyMoneyFile if set.
-    *
-    * @see KNewFileDlg
-    */
-  void viewPersonal(void);
+  void newFile(void);
 
   /**
     * Moves the view up from transaction to Bank/Account view.
     */
   void viewUp(void);
 
-  /**
-    * Utility method to retrieve the currently selected account name.
-    *
-    * @return The currently selected account name.
-    */
-  QString currentAccountName(void);
-
-  /**
-    * utility method to suspend/activate updates of the MyMoney engine on
-    * all views. This is used to speed up operations with lot's of updates
-    * of engine data in a short time (e.g. importing data, creating a
-    * new file).
-    *
-    * @param suspend Suspend updates or not. Possible values are
-    *
-    * @li true updates are suspended
-    * @li false updates will be performed immediately
-    */
-  void suspendUpdate(const bool suspend);
-
-  void memoryDump();
-
   /**
     * This method allows to set the enable state of all views (except home view)
     * The argument @p state controls the availability.
@@ -337,12 +349,90 @@ public:
     */
   void enableViews(int state = -1);
 
-  KPopupMenu* accountContextMenu(void) const { return m_accountMenu; };
-
-  KMyMoneyViewBase* addPage(const QString& title, const QPixmap& pixmap = QPixmap());
+  KMyMoneyViewBase* addPage(const QString& title, const QString& icon = QString());
 
   void addWidget(QWidget* w);
 
+  virtual bool showPage(int index);
+
+  /**
+    * check if the current view allows to create a transaction
+    *
+    * @param list list of selected transactions
+    * @param tooltip reference to string receiving the tooltip text
+    *        which explains why the modify function is not available (in case
+    *        of returning @c false)
+    *
+    * @retval true Yes, view allows to create a transaction (tooltip is not changed)
+    * @retval false No, view cannot to create a transaction (tooltip is updated with message)
+    */
+  bool canCreateTransactions(const QValueList<KMyMoneyRegister::SelectedTransaction>& list, QString& tooltip) const;
+
+  /**
+    * check if the current view allows to modify (edit/delete) the selected transactions
+    *
+    * @param list list of selected transactions
+    * @param tooltip reference to string receiving the tooltip text
+    *        which explains why the modify function is not available (in case
+    *        of returning @c false)
+    *
+    * @retval true Yes, view allows to edit/delete transactions (tooltip is not changed)
+    * @retval false No, view cannot edit/delete transactions (tooltip is updated with message)
+    */
+  bool canModifyTransactions(const QValueList<KMyMoneyRegister::SelectedTransaction>& list, QString& tooltip) const;
+
+  /**
+    * check if the current view allows to edit the selected transactions
+    *
+    * @param list list of selected transactions
+    * @param tooltip reference to string receiving the tooltip text
+    *        which explains why the edit function is not available (in case
+    *        of returning @c false)
+    *
+    * @retval true Yes, view allows to enter/edit transactions
+    * @retval false No, view cannot enter/edit transactions
+    */
+  bool canEditTransactions(const QValueList<KMyMoneyRegister::SelectedTransaction>& list, QString& tooltip) const;
+
+  /**
+    * check if the current view allows to print something
+    *
+    * @retval true Yes, view allows to print
+    * @retval false No, view cannot print
+    */
+  bool canPrint(void);
+
+  TransactionEditor* startEdit(const QValueList<KMyMoneyRegister::SelectedTransaction>&);
+
+  bool createNewTransaction(void);
+
+  /**
+    * Used to start reconciliation of account @a account. It switches the
+    * ledger view into reconciliation mode and updates the view.
+    *
+    * @param account account which should be reconciled
+    * @param endingBalance the ending balance entered for this account
+    *
+    * @retval true Reconciliation started
+    * @retval false Account cannot be reconciled
+    */
+  bool startReconciliation(const MyMoneyAccount& account, const MyMoneyMoney& endingBalance);
+
+  /**
+    * Used to finish reconciliation of account @a account. It switches the
+    * ledger view to normal mode and updates the view.
+    *
+    * @param account account which should be reconciled
+    */
+  void finishReconciliation(const MyMoneyAccount& account);
+
+  /**
+    * This method preloads all known currencies into the engine.
+    */
+  void loadDefaultCurrencies(void);
+
+  void loadAncientCurrencies(void);
+
 public slots:
   /**
     * This slot writes information about the page passed as argument @p widget
@@ -363,18 +453,6 @@ public slots:
     **/
   void slotRefreshViews();
 
-  /**
-    * Brings up a dialog to let the user search for specific transaction(s).  It then
-    * opens a results window to display those transactions.
-    */
-  void slotFindTransaction();
-
-  /**
-    * Called whenever the user 'executes' an account. This operation opens the account
-    * and shows the register view.
-    **/
-  void slotAccountDoubleClick(void);
-
   /**
     * Called, whenever the ledger view should pop up and a specific
     * transaction in an account should be shown. If @p transaction
@@ -409,77 +487,15 @@ public slots:
     *
     * @param reportid The ID of the report to be shown
     */
-  void slotReportSelected(const QCString& reportid);
+  void slotShowReport(const QCString& reportid);
 
   /**
-    * Called whenever the user wishes to create a new bank.  Brings up the input
-    * dialog and saves the information.  It then enables the banks view.
+    * Same as the above, but the caller passes in an actual report
+    * definition to be shown.
     *
-    * @see KBanksView
-    * @see KNewBankDlg
-    * @see MyMoneyFile
-    * @see MyMoneyInstitution
+    * @param report The report to be shown
     */
-  void slotBankNew(void);
-
-  /**
-    * Called whenever the user wishes to create a new account.  Brings up the input
-    * dialog and saves the information.
-    *
-    * @see KBanksView
-    * @see KNewAccountDlg
-    * @see MyMoneyFile
-    * @see MyMoneyAccount
-    */
-  void slotAccountNew(void);
-
-  /**
-    * Called whenever the user wishes to create a new account by right clicking on
-    * an institution.  Brings up the input dialog and saves the information.
-    *
-    * This exists so that the institution can be pre-set in the account wizard.
-    *
-    * @see KBanksView
-    * @see KNewAccountDlg
-    * @see MyMoneyFile
-    * @see MyMoneyAccount
-    */
-  void slotBankAccountNew(void);
-
-  /**
-    * Called whenever the user wishes to create a new category.  Brings up the input
-    * dialog and saves the information.
-    *
-    * @see KBanksView
-    * @see KNewAccountDlg
-    * @see MyMoneyFile
-    * @see MyMoneyAccount
-    */
-  void slotCategoryNew(void);
-
-  /**
-    * Called whenever the user wishes to reconcile the open account.  It first get some
-    * required input and then opens the reconciliation dialog.  The user can edit transactions
-    * as normal in the main view because this dialog is modeless and is updated whenever the
-    * transaction list is changed.
-    *
-    * @see KEndingBalanceDlg
-    * @see KReconcileDlg
-    * @see KMyMoneyFile
-    * @see MyMoneyAccount
-    */
-  void slotAccountReconcile(void);
-
-  // Not implemented, not documented!
-
-  void slotAccountImportAscii(void);
-  void slotAccountExportAscii(void);
-
-  /**
-    * This slot cancels any edit activity in any view. It will
-    * be called e.g. before entering the settings dialog.
-    */
-  void slotCancelEdit(void) const;
+  void slotShowReport(const MyMoneyReport& report);
 
   /**
     * This slot prints the current view.
@@ -492,52 +508,6 @@ public slots:
   void slotShowHomePage(void) { showPage(0); }
 
 protected slots:
-  /**
-    * Called whenever the user right clicks on an account.  It brings up
-    * a context menu.  TODO: move the context menu into kmymoney2ui.rc, move
-    * this method into KBanksView, remove the param inList.
-    */
-  void slotAccountRightMouse();
-  void slotBankRightMouse();
-  void slotRightMouse();
-
-  /**
-    * Called by the context menu created in slotAccountRightMouse.  Brings up
-    * a dialog which allows the user to edit the account details.  TODO: move this
-    * method into KBanksView.
-    */
-  void slotAccountEdit();
-
-  /**
-    * Called by the context menu created in slotAccountRightMouse.  Deletes the currently
-    * selected account. TODO: move this method into KBanksView.
-    */
-  void slotAccountDelete();
-
-  /**
-    * Called by the context menu created in slotAccountRightMouse.  Connects to the users
-    * bank via OFX.  Only valid if ofxConnectionSettings have been set in the institution.
-    */
-  void slotAccountOfxConnect();
-
-  /**
-    * Called by the context menu created in slotBankRightMouse.  Brings up
-    * a dialog which allows the user to edit the bank details.  TODO: move this
-    * method into KBanksView.
-    */
-  void slotBankEdit();
-
-  /**
-    * Called by the context menu created in slotBankRightMouse.  Deletes the currently
-    * selected bank. TODO: move this method into KBanksView.
-    */
-  void slotBankDelete();
-
-  /**
-    * Destroys the search dialog
-    */
-  void slotCloseSearchDialog(void);
-
   /**
     * Called when the user changes the detail
     * setting of the transaction register
@@ -546,7 +516,33 @@ protected slots:
     */
   void slotShowTransactionDetail(bool detailed);
 
+  /**
+   * eventually replace this with KMyMoney2App::slotCurrencySetBase(void).
+   * it contains the same code
+   *
+   * @deprecated
+   */
+  void slotSetBaseCurrency(const MyMoneySecurity& baseCurrency);
+
 private:
+  /**
+   * This method is called from readFile to open a database file which
+   * is to be processed in 'proper' database mode, i.e. in-place updates
+   *
+   * @param dbaseURL pseudo-KURL representation of database
+   *
+   * @retval true Database opened successfully
+   * @retval false Could not open or read database
+   */
+  bool openDatabase (const KURL& dbaseURL);
+  /**
+   * This method is used after a file or database has been
+   * read into storage, and performs various initialization tasks
+   *
+   * @retval true all went okay
+   * @retval false an exception occurred during this process
+   */
+  bool initializeStorage();
   /**
     * This method is used by saveFile() to store the data
     * either directly in the destination file if it is on
@@ -557,10 +553,12 @@ private:
     * @param qf pointer to QFile representing the opened file
     * @param writer pointer to the formatter
     * @param plaintext whether to override any compression & encryption settings
+    * @param key QString containing the key to be used for encryption
+    *            If @p key is empty, the file will be saved unencrypted
     *
     * @note This method will close the file when it is written.
     */
-  void saveToLocalFile(QFile* qf, IMyMoneyStorageFormat* writer, bool plaintext=false);
+  void saveToLocalFile(QFile* qf, IMyMoneyStorageFormat* writer, bool plaintext=false, const QString& key = QString());
 
   /**
     * Internal method used by slotAccountNew() and slotAccountCategory().
@@ -577,9 +575,26 @@ signals:
   void accountSelectedForContextMenu(const MyMoneyAccount& acc);
 
   void viewStateChanged(bool enabled);
+   /**
+     * This signal is emitted to inform the kmmFile plugin when various file actions
+     * occur. The Action parameter distinguishes between them.
+     */
+  void kmmFilePlugin (unsigned int action);
+
+  /**
+    * Signal is emitted when reconciliation starts or ends. In case of end,
+    * @a account is MyMoneyAccount()
+    *
+    * @param account account for which reconciliation starts or MyMoneyAccount()
+    *                if reconciliation ends.
+    * @param endingBalance collected ending balance when reconciliation starts
+    *                0 otherwise
+    */
+  void reconciliationStarts(const MyMoneyAccount& account, const MyMoneyMoney& endingBalance);
+
 };
 
-class kMyMoneyTitleLabel;
+class KMyMoneyTitleLabel;
 
 /**
   * This class is an abstract base class that all specific views
@@ -596,8 +611,18 @@ public:
   QVBoxLayout* layout(void) const { return m_viewLayout; }
   void addWidget(QWidget* w);
 
+  /**
+    * This method is used to edit the currently selected transactions
+    * The default implementation returns @p false which signals to the caller, that
+    * the view was not capable to edit the transactions.
+    *
+    * @retval false view was not capable to edit transactions
+    * @retval true view was capable to edit the transactions and did so
+    */
+  bool editTransactions(const QValueList<MyMoneyTransaction>& transactions) const { Q_UNUSED(transactions)  return false; }
+
 protected:
-  kMyMoneyTitleLabel*    m_titleLabel;
+  KMyMoneyTitleLabel*    m_titleLabel;
   QVBoxLayout*           m_viewLayout;
 };
 #endif
