@@ -31,6 +31,7 @@
 
 #include <kconfig.h>
 #include <klocale.h>
+#include <kglobal.h>
 #include <kglobalsettings.h>
 
 // ----------------------------------------------------------------------------
@@ -38,31 +39,27 @@
 
 #include "kscheduledlistitem.h"
 #include "../mymoney/mymoneyfile.h"
+#include "../kmymoneyglobalsettings.h"
 #include "../kmymoneyutils.h"
 
-KScheduledListItem::KScheduledListItem(KListView *parent, const QString& name)
- : KListViewItem(parent, name)/*, m_even(false)*/, m_base(true)
+KScheduledListItem::KScheduledListItem(KListView *parent, const QString& name, const QPixmap& pixmap, const QString& sortKey) :
+  KListViewItem(parent, name),
+  m_sortKey(sortKey)
 {
-  if (name == i18n("Bills"))
-    setPixmap(0, KMyMoneyUtils::billScheduleIcon(KIcon::Small));
-  else if (name == i18n("Deposits"))
-    setPixmap(0, KMyMoneyUtils::depositScheduleIcon(KIcon::Small));
-  else if (name == i18n("Transfers"))
-    setPixmap(0, KMyMoneyUtils::transferScheduleIcon(KIcon::Small));
-  else if (name == i18n("Loans"))
-    setPixmap(0, KMyMoneyUtils::transferScheduleIcon(KIcon::Small));
+  setPixmap(0, pixmap);
+  if(m_sortKey.isEmpty())
+    m_sortKey = name;
 }
 
 KScheduledListItem::KScheduledListItem(KScheduledListItem *parent, const MyMoneySchedule& schedule/*, bool even*/)
- : KListViewItem(parent), m_base(false)
+ : KListViewItem(parent)
 {
   m_schedule = schedule;
+  m_sortKey = schedule.name();
   setPixmap(0, KMyMoneyUtils::scheduleIcon(KIcon::Small));
 
-//  m_even = even;
   try
   {
-    m_id = schedule.id();
     MyMoneyTransaction transaction = schedule.transaction();
     MyMoneySplit s1 = transaction.splits()[0];
     MyMoneySplit s2 = transaction.splits()[1];
@@ -72,10 +69,10 @@ KScheduledListItem::KScheduledListItem(KScheduledListItem *parent, const MyMoney
 
     switch(schedule.type()) {
       case MyMoneySchedule::TYPE_DEPOSIT:
-        if (!s1.value().isNegative())
-          split = s1;
-        else
+        if (s1.value().isNegative())
           split = s2;
+        else
+          split = s1;
         break;
 
       case MyMoneySchedule::TYPE_LOANPAYMENT:
@@ -96,7 +93,7 @@ KScheduledListItem::KScheduledListItem(KScheduledListItem *parent, const MyMoney
         break;
 
       default:
-        if (s1.value().isNegative())
+        if (!s1.value().isPositive())
           split = s1;
         else
           split = s2;
@@ -132,16 +129,15 @@ KScheduledListItem::KScheduledListItem(KScheduledListItem *parent, const MyMoney
       setText(2, MyMoneyFile::instance()->payee(split.payeeId()).name());
     else
       setText(2, "---");
-    MyMoneyMoney amount = split.value();
-    amount = amount.abs();
-    setText(3, amount.formatMoney(currency.tradingSymbol()));
+    m_amount = split.value().abs();
+    setText(3, QString("%1  ").arg(m_amount.formatMoney(acc, currency)));
     // Do the real next payment like ms-money etc
     if (schedule.isFinished())
     {
       setText(4, i18n("Finished"));
     }
     else
-      setText(4, schedule.nextPayment(schedule.lastPayment()).toString());
+      setText(4, KGlobal::locale()->formatDate(schedule.nextDueDate(), true));
 
     setText(5, KMyMoneyUtils::occurenceToString(schedule.occurence()));
     setText(6, KMyMoneyUtils::paymentMethodToString(schedule.paymentType()));
@@ -162,170 +158,74 @@ void KScheduledListItem::paintCell(QPainter* p, const QColorGroup& cg, int colum
 {
   QColorGroup cg2(cg);
 
-  QColor colour = KMyMoneyUtils::listColour();
-  QColor bgColour = KMyMoneyUtils::backgroundColour();
   QColor textColour = KGlobalSettings::textColor();
-  QFont cellFont = KMyMoneyUtils::cellFont();
+  QFont cellFont = KMyMoneyGlobalSettings::listCellFont();
 
-  if (m_schedule.isFinished())
-  {
-    textColour = Qt::darkGreen;
+  // avoid colorizing lines that do not contain a schedule
+  if(!m_schedule.id().isEmpty()) {
+    if (m_schedule.isFinished())
+      textColour = Qt::darkGreen;
+    else if (m_schedule.isOverdue())
+      textColour = Qt::red;
   }
-  else if (m_schedule.isOverdue())
-  {
-    textColour = Qt::red;
-  }
-  // else
-  //   keep the same colour
 
   cg2.setColor(QColorGroup::Text, textColour);
 
-  if (m_base)
-  {
-    QFont font(p->font());
-    font.setBold(true);
-    p->setFont(font);
-  }
+  // display group items in bold
+  if (!parent())
+    cellFont.setBold(true);
+
+  p->setFont(cellFont);
 
   if (isAlternate())
-  {
-    cg2.setColor(QColorGroup::Base, colour);
-  }
+    cg2.setColor(QColorGroup::Base, KMyMoneyGlobalSettings::listColor());
   else
-  {
-    cg2.setColor(QColorGroup::Base, bgColour);
-  }
+    cg2.setColor(QColorGroup::Base, KMyMoneyGlobalSettings::listBGColor());
 
   QListViewItem::paintCell(p, cg2, column, width, align);
-
-  if (column == 0)
-  {
-    int ts = listView()->treeStepSize();
-    int indent = ts * (depth()+1);
-    p->save();
-    p->translate(-indent, 0);
-
-    paintBranches(p, cg, 0, 0, 0);
-
-    p->restore();
-  }
 }
 
-void KScheduledListItem::paintBranches(QPainter* p, const QColorGroup& cg, int/* w*/, int/* y*/, int/* h*/)
-// void KScheduledListItem::paintBranches(QPainter* p, const QColorGroup& cg, int w, int y, int h)
+int KScheduledListItem::compare(QListViewItem* i, int col, bool ascending) const
 {
-  // qDebug("paintBranches(%d,%d,%d)", w, y, h);
-  QColorGroup cg2(cg);
+  KScheduledListItem* item = dynamic_cast<KScheduledListItem*>(i);
+  int rc;
+  // do special sorting only if
+  // a) date
+  // b) amount
+  // c) name/group
+  // d) occurence
+  // in all other cases use the standard sorting
+  MyMoneyMoney diff;
+  switch(col) {
+    case 0:   // type and name
+      rc = m_sortKey.compare(item->m_sortKey);
+      break;
 
-  QColor colour = KMyMoneyUtils::listColour();
-  QColor bgColour = KMyMoneyUtils::backgroundColour();
+    case 3:   // amount
+      diff = m_amount - item->m_amount;
+      if(diff.isZero())
+        rc = 0;
+      else if(diff.isPositive())
+        rc = 1;
+      else
+        rc = -1;
+      break;
 
-  if (isAlternate())
-  {
-    cg2.setColor(QColorGroup::Base, colour);
-  }
-  else
-  {
-    cg2.setColor(QColorGroup::Base, bgColour);
-  }
+    case 4:   // date
+      rc = item->m_schedule.nextDueDate().daysTo(m_schedule.nextDueDate());
+      break;
 
-  int ts = listView()->treeStepSize();
-  int ofs;
-  int indent = ts * (depth()+1);
+    case 5:   // occurence
+      rc = (m_schedule.occurence() - item->m_schedule.occurence());
+      break;
 
-  if ( isSelected()) {
-    p->fillRect( 0, 0, indent, height(), cg2.brush( QColorGroup::Highlight ) );
-    if ( isEnabled() || !listView() )
-      p->setPen( cg2.highlightedText() );
-    else if ( !isEnabled() && listView())
-      p->setPen( listView()->palette().disabled().highlightedText() );
-
-  } else
-    p->fillRect( 0, 0, indent, height(), cg2.base() );
-
-  // draw dotted lines in upper levels to the left of us
-  QListViewItem *parent = this;
-  for(int j = depth()-1; j >= 0; --j) {
-    if(!parent)
+    default:
+      rc = KListViewItem::compare(i, col, ascending);
       break;
-    parent = parent->parent();
-    if(parent->nextSibling()) {
-      ofs = (j * ts) + ts/2 - 1;
-      for(int j = 0; j < height(); j += 2)
-        p->drawPoint(ofs, j);
-    }
   }
-
-  if(childCount() == 0) {
-    // if we have no children, the we need to draw a vertical line
-    // which length depends if we have a sibling or not.
-    // also a horizontal line to the right is required.
-    ofs = depth()*ts + ts/2 - 1;
-    int end = nextSibling() ? height() : height()/2;
-    for(int i = 0; i < end; i += 2)
-      p->drawPoint(ofs, i);
-
-    for(int i = ofs; i < (depth()+1)*ts; i += 2)
-      p->drawPoint(i, height()/2);
-
-  } else {
-    // draw upper part of vertical line
-    ofs = depth()*ts + ts/2 - 1;
-    for(int i = 0; i < height()/2-(ts-2)/4; i += 2)
-      p->drawPoint(ofs, i);
-
-    // draw horizontal part
-    for(int i = ofs + ts/4 ; i < (depth()+1)*ts; i += 2)
-      p->drawPoint(i, height()/2);
-
-    // need to draw box with +/- in it
-    ofs = depth() * ts;
-    p->drawRect( ofs + ts/4, height() / 2 - (ts-2)/4, (ts-2)/2, (ts-2)/2 );
-    p->drawLine( ofs + ts/2-3, height() / 2, ofs + ts/2+1, height() / 2 );
-    if ( !isOpen() )
-        p->drawLine( ofs + ts/2-1, height() / 2 - 2, ofs + ts/2-1, height() / 2 + 2 );
-
-    // if there are more siblings, we need to draw
-    // the remainder of the vertical line
-    if(nextSibling()) {
-      ofs = depth()*ts + ts/2 - 1;
-      for(int i = height() / 2 + (ts-2)/4; i < height(); i += 2)
-        p->drawPoint(ofs, i);
-    }
+  // adjust to [-1..1]
+  if(rc != 0) {
+    rc = (rc > 0) ? 1 : -1;
   }
-}
-
-void KScheduledListItem::paintFocus(QPainter* p, const QColorGroup& cg, const QRect& r)
-{
-  QColorGroup cg2(cg);
-
-  KConfig *config = KGlobal::config();
-  config->setGroup("List Options");
-
-  QColor textColour = cg2.highlightedText();
-  textColour = config->readColorEntry("listGridColor", &textColour);
-
-  if (m_schedule.isFinished())
-    textColour = Qt::darkGreen;
-  else if (m_schedule.isOverdue())
-    textColour = Qt::red;
-
-  cg2.setColor(QColorGroup::HighlightedText, textColour);
-
-  int indent = listView()->treeStepSize() * (depth()+1);
-
-  QRect r2(r);
-  r2.setLeft(r2.left() + -indent);
-  if (isSelected())
-    p->fillRect(  r2.left(),
-                  r2.top(),
-                  -indent,
-                  r2.height(),
-                  cg2.highlight());
-
-  listView()->style().drawPrimitive(
-                QStyle::PE_FocusRect, p, r2, cg2,
-                (isSelected() ? QStyle::Style_FocusAtBorder : QStyle::Style_Default),
-                QStyleOption(isSelected() ? cg2.highlight() : cg2.base()));
-
+  return rc;
 }
