@@ -19,8 +19,10 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
+
 // ----------------------------------------------------------------------------
 // QT Includes
+
 #include <qheader.h>
 #include <qtoolbutton.h>
 #include <qcombobox.h>
@@ -30,6 +32,7 @@
 
 // ----------------------------------------------------------------------------
 // KDE Includes
+
 #include <kglobal.h>
 #include <klocale.h>
 #include <kconfig.h>
@@ -38,25 +41,32 @@
 #include <kmessagebox.h>
 #include <klistview.h>
 #include <kpushbutton.h>
+#include <klistviewsearchline.h>
 
 // ----------------------------------------------------------------------------
 // Project Includes
+
 #include "kscheduledview.h"
 #include "kscheduledlistitem.h"
 #include "../widgets/kmymoneyscheduleddatetbl.h"
-#include "../dialogs/ieditscheduledialog.h"
-#include "../dialogs/keditloanwizard.h"
-#include "../dialogs/kenterscheduledialog.h"
-#include "../kmymoneyutils.h"
+// #include "../dialogs/kenterscheduledialog.h"
+#include <kmymoney/kmymoneyutils.h>
+#include <kmymoney/kmymoneyglobalsettings.h>
 
-KScheduledView::KScheduledView(QWidget *parent, const char *name )
- : kScheduledViewDecl(parent,name, false),
- m_openBills(true),
- m_openDeposits(true),
- m_openTransfers(true),
- m_openLoans(true)
+#include "../kmymoney2.h"
+
+KScheduledView::KScheduledView(QWidget *parent, const char *name ) :
+  KScheduledViewDecl(parent,name, false),
+  m_openBills(true),
+  m_openDeposits(true),
+  m_openTransfers(true),
+  m_openLoans(true)
 {
-  m_qlistviewScheduled->setRootIsDecorated(true);
+  // create the searchline widget
+  // and insert it into the existing layout
+  m_searchWidget = new KListViewSearchLineWidget(m_qlistviewScheduled, m_listTab);
+  m_listTabLayout->insertWidget(0, m_searchWidget);
+
   m_qlistviewScheduled->addColumn(i18n("Type/Name"));
   m_qlistviewScheduled->addColumn(i18n("Account"));
   m_qlistviewScheduled->addColumn(i18n("Payee"));
@@ -64,20 +74,18 @@ KScheduledView::KScheduledView(QWidget *parent, const char *name )
   m_qlistviewScheduled->addColumn(i18n("Next Due Date"));
   m_qlistviewScheduled->addColumn(i18n("Frequency"));
   m_qlistviewScheduled->addColumn(i18n("Payment Method"));
-  m_qlistviewScheduled->setMultiSelection(false);
-  m_qlistviewScheduled->header()->setResizeEnabled(true);
-  m_qlistviewScheduled->setAllColumnsShowFocus(true);
-  // never show a horizontal scroll bar
-  m_qlistviewScheduled->setHScrollBarMode(QScrollView::AlwaysOff);
-  m_qlistviewScheduled->setSorting(-1);
   m_qlistviewScheduled->setColumnAlignment(3, Qt::AlignRight);
 
-  KPopupMenu* kpopupmenuNew = new KPopupMenu(this);
-  kpopupmenuNew->insertItem(KMyMoneyUtils::billScheduleIcon(KIcon::SizeSmall), i18n("Bill"), this, SLOT(slotNewBill()));
-  kpopupmenuNew->insertItem(KMyMoneyUtils::depositScheduleIcon(KIcon::SizeSmall), i18n("Deposit"), this, SLOT(slotNewDeposit()));
-  kpopupmenuNew->insertItem(KMyMoneyUtils::transferScheduleIcon(KIcon::SizeSmall), i18n("Transfer"), this, SLOT(slotNewTransfer()));
-  m_qbuttonNew->setPopup(kpopupmenuNew);
+  readConfig();
 
+  m_qlistviewScheduled->setMultiSelection(false);
+  m_qlistviewScheduled->header()->setResizeEnabled(true);
+  if(m_qlistviewScheduled->sortColumn() == -1)
+    m_qlistviewScheduled->setSorting(0);
+
+  connect(m_qbuttonNew, SIGNAL(clicked()), kmymoney2->action("schedule_new"), SLOT(activate()));
+
+  // attach popup to 'Filter...' button
   m_kaccPopup = new KPopupMenu(this);
   m_kaccPopup->setCheckable(true);
   m_accountsCombo->setPopup(m_kaccPopup);
@@ -87,37 +95,34 @@ KScheduledView::KScheduledView(QWidget *parent, const char *name )
   m_accountsCombo->setGuiItem(KMyMoneyUtils::accountsFilterGuiItem());
 
   KIconLoader *il = KGlobal::iconLoader();
-  m_tabWidget->setTabIconSet(listTab, QIconSet(il->loadIcon("contents", KIcon::Small, KIcon::SizeSmall)));
-  m_tabWidget->setTabIconSet(calendarTab, QIconSet(il->loadIcon("calendartab", KIcon::User, KIcon::SizeSmall)));
+  m_tabWidget->setTabIconSet(m_listTab, QIconSet(il->loadIcon("contents", KIcon::Small, KIcon::SizeSmall)));
+  m_tabWidget->setTabIconSet(m_calendarTab, QIconSet(il->loadIcon("calendartab", KIcon::User, KIcon::SizeSmall)));
 
-  readConfig();
+  connect(m_qlistviewScheduled, SIGNAL(contextMenu(KListView*, QListViewItem*, const QPoint&)),
+    this, SLOT(slotListViewContextMenu(KListView*, QListViewItem*, const QPoint&)));
+  connect(m_qlistviewScheduled, SIGNAL(selectionChanged(QListViewItem*)),
+    this, SLOT(slotSetSelectedItem(QListViewItem*)));
 
-  connect(m_qlistviewScheduled, SIGNAL(contextMenuRequested(QListViewItem*, const QPoint&, int)),
-    this, SLOT(slotListViewContextMenu(QListViewItem*, const QPoint&, int)));
   connect(m_qlistviewScheduled, SIGNAL(doubleClicked(QListViewItem*, const QPoint&, int)),
     this, SLOT(slotListItemExecuted(QListViewItem*, const QPoint&, int)));
   connect(m_qlistviewScheduled, SIGNAL(expanded(QListViewItem*)),
     this, SLOT(slotListViewExpanded(QListViewItem*)));
   connect(m_qlistviewScheduled, SIGNAL(collapsed(QListViewItem*)),
     this, SLOT(slotListViewCollapsed(QListViewItem*)));
-  connect(m_qlistviewScheduled, SIGNAL(selectionChanged(QListViewItem*)),
-    this, SLOT(slotSetSelectedItem(QListViewItem*)));
 
   connect(m_calendar, SIGNAL(enterClicked(const MyMoneySchedule&, const QDate&)), this, SLOT(slotBriefEnterClicked(const MyMoneySchedule&, const QDate&)));
 
-  MyMoneyFile::instance()->attach(MyMoneyFile::NotifyClassSchedule, this);
+  connect(MyMoneyFile::instance(), SIGNAL(dataChanged()), this, SLOT(slotReloadView()));
 }
 
 KScheduledView::~KScheduledView()
 {
-  MyMoneyFile::instance()->detach(MyMoneyFile::NotifyClassSchedule, this);
   writeConfig();
 }
 
 void KScheduledView::refresh(bool full, const QCString schedId)
 {
-  m_qlistviewScheduled->header()->setFont(KMyMoneyUtils::headerFont());
-  m_qlistviewScheduled->setFont(KMyMoneyUtils::cellFont());
+  m_qlistviewScheduled->header()->setFont(KMyMoneyGlobalSettings::listHeaderFont());
 
   m_qlistviewScheduled->clear();
 
@@ -159,10 +164,10 @@ void KScheduledView::refresh(bool full, const QCString schedId)
 
     MyMoneyFile *file = MyMoneyFile::instance();
 
-    KScheduledListItem *itemTransfers = new KScheduledListItem(m_qlistviewScheduled, i18n("Transfers"));
-    KScheduledListItem *itemDeposits = new KScheduledListItem(m_qlistviewScheduled, i18n("Deposits"));
-    KScheduledListItem *itemBills = new KScheduledListItem(m_qlistviewScheduled, i18n("Bills"));
-    KScheduledListItem *itemLoans = new KScheduledListItem(m_qlistviewScheduled, i18n("Loans"));
+    KScheduledListItem *itemBills = new KScheduledListItem(m_qlistviewScheduled, i18n("Bills"), KMyMoneyUtils::billScheduleIcon(KIcon::Small), "0");
+    KScheduledListItem *itemDeposits = new KScheduledListItem(m_qlistviewScheduled, i18n("Deposits"), KMyMoneyUtils::depositScheduleIcon(KIcon::Small), "1");
+    KScheduledListItem *itemLoans = new KScheduledListItem(m_qlistviewScheduled, i18n("Loans"), KMyMoneyUtils::transferScheduleIcon(KIcon::Small), "2");
+    KScheduledListItem *itemTransfers = new KScheduledListItem(m_qlistviewScheduled, i18n("Transfers"), KMyMoneyUtils::transferScheduleIcon(KIcon::Small), "3");
 
     QValueList<MyMoneySchedule> scheduledItems = file->scheduleList();
 
@@ -189,35 +194,35 @@ void KScheduledView::refresh(bool full, const QCString schedId)
       if (!bContinue)
         continue;
 
+      KScheduledListItem* parent = 0;
       switch (schedData.type())
       {
         case MyMoneySchedule::TYPE_BILL:
-          item = new KScheduledListItem(itemBills, schedData);
-          if (schedData.id() == schedId)
-            openItem = item;
+          parent = itemBills;
           break;
 
         case MyMoneySchedule::TYPE_DEPOSIT:
-          item = new KScheduledListItem(itemDeposits, schedData);
-          if (schedData.id() == schedId)
-            openItem = item;
+          parent = itemDeposits;
           break;
 
         case MyMoneySchedule::TYPE_TRANSFER:
-          item = new KScheduledListItem(itemTransfers, schedData);
-          if (schedData.id() == schedId)
-            openItem = item;
+          parent = itemTransfers;
           break;
 
         case MyMoneySchedule::TYPE_LOANPAYMENT:
-          item = new KScheduledListItem(itemLoans, schedData);
-          if (schedData.id() == schedId)
-            openItem = item;
+          parent = itemLoans;
           break;
 
         case MyMoneySchedule::TYPE_ANY:
           break; // Should we display an error ?
       }
+      if(parent) {
+        if(!KMyMoneyGlobalSettings::hideFinishedSchedules() || !schedData.isFinished()) {
+          item = new KScheduledListItem(parent, schedData);
+          if (schedData.id() == schedId)
+            openItem = item;
+        }
+      }
     }
 
     if (openItem)
@@ -268,20 +273,24 @@ void KScheduledView::slotTimerDone(void)
 
 void KScheduledView::slotReloadView(void)
 {
-  m_qbuttonNew->setEnabled(true);
-  m_tabWidget->setEnabled(true);
+  m_needReload = true;
+  if(isVisible()) {
+    m_qbuttonNew->setEnabled(true);
+    m_tabWidget->setEnabled(true);
 
-  refresh(true, m_selectedSchedule);
+    refresh(true, m_selectedSchedule);
 
-  QTimer::singleShot(50, this, SLOT(slotRearrange()));
-  QWidget::show();
+    m_needReload = false;
+    QTimer::singleShot(50, this, SLOT(slotRearrange()));
+  }
 }
 
 void KScheduledView::show()
 {
-  slotReloadView();
+  KScheduledViewDecl::show();
 
-  emit signalViewActivated();
+  if(m_needReload)
+    slotReloadView();
 }
 
 void KScheduledView::slotRearrange(void)
@@ -289,115 +298,6 @@ void KScheduledView::slotRearrange(void)
   resizeEvent(0);
 }
 
-void KScheduledView::resizeEvent(QResizeEvent* e)
-{
-  m_qlistviewScheduled->setColumnWidth(1, 100);
-  m_qlistviewScheduled->setColumnWidth(2, 100);
-  m_qlistviewScheduled->setColumnWidth(3, 80);
-  m_qlistviewScheduled->setColumnWidth(4, 120);
-  m_qlistviewScheduled->setColumnWidth(5, 100);
-  m_qlistviewScheduled->setColumnWidth(6, 120);
-  m_qlistviewScheduled->setColumnWidth(0, m_qlistviewScheduled->visibleWidth()-620);
-
-  // call base class resizeEvent()
-  kScheduledViewDecl::resizeEvent(e);
-}
-
-void KScheduledView::slotDeleteClicked()
-{
-  if (!m_selectedSchedule.isEmpty())
-  {
-    try
-    {
-      MyMoneySchedule sched = MyMoneyFile::instance()->schedule(m_selectedSchedule);
-      QString msg = i18n("Are you sure you want to delete the selected schedule?");
-      if(sched.type() == MyMoneySchedule::TYPE_LOANPAYMENT)
-      {
-        msg += QString(" ");
-        msg += i18n("In case of loan payments it is currently not possible to recreate the schedules");
-      }
-      if (KMessageBox::questionYesNo(this, msg) == KMessageBox::No)
-        return;
-
-      MyMoneyFile::instance()->removeSchedule(sched);
-
-      refresh();
-
-    } catch (MyMoneyException *e)
-    {
-      KMessageBox::detailedSorry(this, i18n("Unable to remove schedule"), e->what());
-      delete e;
-    }
-  }
-}
-
-void KScheduledView::slotEditClicked()
-{
-  if (!m_selectedSchedule.isEmpty())
-  {
-    try
-    {
-      MyMoneySchedule schedule = MyMoneyFile::instance()->schedule(m_selectedSchedule);
-
-      const char *action = 0;
-      switch (schedule.type())
-      {
-        case MyMoneySchedule::TYPE_BILL:
-          action = MyMoneySplit::ActionWithdrawal;
-          break;
-
-        case MyMoneySchedule::TYPE_DEPOSIT:
-          action = MyMoneySplit::ActionDeposit;
-          break;
-
-        case MyMoneySchedule::TYPE_TRANSFER:
-          action = MyMoneySplit::ActionTransfer;
-          break;
-
-        case MyMoneySchedule::TYPE_LOANPAYMENT:
-        case MyMoneySchedule::TYPE_ANY:
-          break;
-      }
-
-      KEditScheduleDialog* sched_dlg;
-      KEditLoanWizard* loan_wiz;
-
-      switch (schedule.type())
-      {
-        case MyMoneySchedule::TYPE_BILL:
-        case MyMoneySchedule::TYPE_DEPOSIT:
-        case MyMoneySchedule::TYPE_TRANSFER:
-          sched_dlg = new KEditScheduleDialog(action, schedule, this);
-          if (sched_dlg->exec() == QDialog::Accepted)
-          {
-            MyMoneySchedule sched = sched_dlg->schedule();
-            MyMoneyFile::instance()->modifySchedule(sched);
-            refresh(false, m_selectedSchedule);
-          }
-          delete sched_dlg;
-          break;
-
-        case MyMoneySchedule::TYPE_LOANPAYMENT:
-          loan_wiz = new KEditLoanWizard(schedule.account(2));
-          if (loan_wiz->exec() == QDialog::Accepted)
-          {
-            MyMoneyFile::instance()->modifySchedule(loan_wiz->schedule());
-            MyMoneyFile::instance()->modifyAccount(loan_wiz->account());
-            refresh(false, m_selectedSchedule);
-          }
-          delete loan_wiz;
-          break;
-
-        case MyMoneySchedule::TYPE_ANY:
-          break;
-      }
-    } catch (MyMoneyException *e)
-    {
-      delete e;
-    }
-  }
-}
-
 void KScheduledView::readConfig(void)
 {
   KConfig *config = KGlobal::config();
@@ -406,11 +306,11 @@ void KScheduledView::readConfig(void)
   m_openDeposits = config->readBoolEntry("KScheduleView_openDeposits", true);
   m_openTransfers = config->readBoolEntry("KScheduleView_openTransfers", true);
   m_openLoans = config->readBoolEntry("KScheduleView_openLoans", true);
+  m_tabWidget->setCurrentPage(config->readNumEntry("KScheduleView_tab", 0));
 
-  config->setGroup("List Options");
+  m_qlistviewScheduled->header()->setFont(KMyMoneyGlobalSettings::listHeaderFont());
+  m_qlistviewScheduled->restoreLayout(KGlobal::config(), "Schedule View Settings");
 
-  m_qlistviewScheduled->header()->setFont(KMyMoneyUtils::headerFont());
-  m_qlistviewScheduled->setFont(KMyMoneyUtils::cellFont());
 }
 
 void KScheduledView::writeConfig(void)
@@ -421,154 +321,37 @@ void KScheduledView::writeConfig(void)
   config->writeEntry("KScheduleView_openDeposits", m_openDeposits);
   config->writeEntry("KScheduleView_openTransfers", m_openTransfers);
   config->writeEntry("KScheduleView_openLoans", m_openLoans);
+  config->writeEntry("KScheduleView_tab", m_tabWidget->currentPageIndex());
   config->sync();
+
+  m_qlistviewScheduled->saveLayout(KGlobal::config(), "Schedule View Settings");
 }
 
-void KScheduledView::slotNewBill()
+void KScheduledView::slotListViewContextMenu(KListView* /* view */, QListViewItem *item, const QPoint& /* pos */)
 {
-  MyMoneySchedule schedule;
-
-  KEditScheduleDialog *m_keditschedbilldlg = new KEditScheduleDialog(MyMoneySplit::ActionWithdrawal, schedule, this);
-  if (m_keditschedbilldlg->exec() == QDialog::Accepted)
-  {
-    MyMoneySchedule sched = m_keditschedbilldlg->schedule();
-    try
-    {
-      MyMoneyFile::instance()->addSchedule(sched);
-      refresh(false, sched.id());
-    } catch (MyMoneyException *e)
-    {
-      KMessageBox::information(this, i18n("Unable to add schedule: "), e->what());
-      delete e;
-    }
-  }
-
-  delete m_keditschedbilldlg;
-}
-
-void KScheduledView::slotNewDeposit()
-{
-  MyMoneySchedule schedule;
-
-  KEditScheduleDialog *m_keditscheddepdlg = new KEditScheduleDialog(MyMoneySplit::ActionDeposit, schedule, this);
-  if (m_keditscheddepdlg->exec() == QDialog::Accepted)
-  {
-    MyMoneySchedule sched = m_keditscheddepdlg->schedule();
-
-    try
-    {
-      MyMoneyFile::instance()->addSchedule(sched);
-      refresh(false, sched.id());
-    } catch (MyMoneyException *e)
-    {
-      KMessageBox::information(this, i18n("Unable to add schedule: "), e->what());
-      delete e;
-    }
-  }
-
-  delete m_keditscheddepdlg;
-}
-
-void KScheduledView::slotNewTransfer()
-{
-  MyMoneySchedule schedule;
-
-  KEditScheduleDialog *m_keditschedtransdlg = new KEditScheduleDialog(MyMoneySplit::ActionTransfer, schedule, this);
-  if (m_keditschedtransdlg->exec() == QDialog::Accepted)
-  {
-    MyMoneySchedule sched = m_keditschedtransdlg->schedule();
-
-    try
-    {
-      MyMoneyFile::instance()->addSchedule(sched);
-      refresh(false, sched.id());
-    } catch (MyMoneyException *e)
-    {
-      KMessageBox::information(this, i18n("Unable to add schedule: "), e->what());
-      delete e;
-    }
-  }
-
-  delete m_keditschedtransdlg;
-}
-
-void KScheduledView::slotListViewContextMenu(QListViewItem *item, const QPoint& pos, int/* col*/)
-{
-  KScheduledListItem *scheduleItem = (KScheduledListItem*)item;
+  KScheduledListItem *scheduleItem = dynamic_cast<KScheduledListItem *>(item);
   if (scheduleItem)
   {
     try
     {
       QCString scheduleId = scheduleItem->scheduleId();
 
-      KIconLoader *kiconloader = KGlobal::iconLoader();
-      KPopupMenu *listViewMenu = new KPopupMenu(m_qlistviewScheduled);
-
-      if (scheduleId.isEmpty()) // Top level item
-      {
-        QString text = scheduleItem->text(0);
-        if (text == i18n("Bills"))
-        {
-          listViewMenu->insertTitle(i18n("Bill Options"));
-          listViewMenu->insertItem(kiconloader->loadIcon("new_bill", KIcon::Small), i18n("New Bill..."), this, SLOT(slotNewBill()));
-        }
-        else if (text == i18n("Deposits"))
-        {
-          listViewMenu->insertTitle(i18n("Deposit Options"));
-          listViewMenu->insertItem(kiconloader->loadIcon("new_deposit", KIcon::Small), i18n("New Deposit..."), this, SLOT(slotNewDeposit()));
-        }
-        else if (text == i18n("Transfers"))
-        {
-          listViewMenu->insertTitle(i18n("Transfer Options"));
-          listViewMenu->insertItem(kiconloader->loadIcon("new_transfer", KIcon::Small), i18n("New Transfer..."), this, SLOT(slotNewTransfer()));
-        }
-      }
-      else // schedule item
+      if (!scheduleId.isEmpty()) // Top level item
       {
         MyMoneySchedule schedule = MyMoneyFile::instance()->schedule(scheduleId);
+        emit scheduleSelected(schedule);
         m_selectedSchedule = schedule.id();
-
-        switch (schedule.type())
-        {
-          case MyMoneySchedule::TYPE_BILL:
-            listViewMenu->insertTitle(i18n("Bill Options"));
-            listViewMenu->insertItem(kiconloader->loadIcon("new_bill", KIcon::Small), i18n("New Bill..."), this, SLOT(slotNewBill()));
-            break;
-          case MyMoneySchedule::TYPE_DEPOSIT:
-            listViewMenu->insertTitle(i18n("Deposit Options"));
-            listViewMenu->insertItem(kiconloader->loadIcon("new_deposit", KIcon::Small), i18n("New Deposit..."), this, SLOT(slotNewDeposit()));
-            break;
-          case MyMoneySchedule::TYPE_TRANSFER:
-            listViewMenu->insertTitle(i18n("Transfer Options"));
-            listViewMenu->insertItem(kiconloader->loadIcon("new_transfer", KIcon::Small), i18n("New Transfer..."), this, SLOT(slotNewTransfer()));
-            break;
-          case MyMoneySchedule::TYPE_LOANPAYMENT:
-          case MyMoneySchedule::TYPE_ANY:
-            break;
-        }
-        listViewMenu->insertSeparator();
-        listViewMenu->insertItem(kiconloader->loadIcon("enter", KIcon::Small), i18n("Enter..."), this, SLOT(slotEnterClicked()));
-        listViewMenu->insertSeparator();
-        listViewMenu->insertItem(kiconloader->loadIcon("edit", KIcon::Small), i18n("Edit..."), this, SLOT(slotEditClicked()));
-        listViewMenu->insertItem(kiconloader->loadIcon("delete", KIcon::Small), i18n("Delete..."), this, SLOT(slotDeleteClicked()));
       }
-
-      listViewMenu->popup(pos);
+      emit openContextMenu();
     } catch (MyMoneyException *e)
     {
-      KMessageBox::detailedSorry(this, i18n("Error building context menu"), e->what());
+      KMessageBox::detailedSorry(this, i18n("Error activating context menu"), e->what());
       delete e;
     }
   }
   else
   {
-    KIconLoader *kiconloader = KGlobal::iconLoader();
-    KPopupMenu *listViewMenu = new KPopupMenu(m_qlistviewScheduled);
-
-    listViewMenu->insertItem(kiconloader->loadIcon("new_bill", KIcon::Small), i18n("New Bill..."), this, SLOT(slotNewBill()));
-    listViewMenu->insertItem(kiconloader->loadIcon("new_deposit", KIcon::Small), i18n("New Deposit..."), this, SLOT(slotNewDeposit()));
-    listViewMenu->insertItem(kiconloader->loadIcon("new_transfer", KIcon::Small), i18n("New Transfer..."), this, SLOT(slotNewTransfer()));
-    listViewMenu->popup(pos);
+    emit openContextMenu();
   }
 }
 
@@ -585,10 +368,8 @@ void KScheduledView::slotListItemExecuted(QListViewItem* item, const QPoint&, in
     if (!scheduleId.isEmpty()) // Top level item
     {
       MyMoneySchedule schedule = MyMoneyFile::instance()->schedule(scheduleId);
-
-      m_calendar->setDate(schedule.nextPayment(schedule.lastPayment()));
-      m_tabWidget->showPage(calendarTab);
       m_selectedSchedule = schedule.id();
+      emit editSchedule();
     }
   } catch (MyMoneyException *e)
   {
@@ -662,55 +443,35 @@ void KScheduledView::slotListViewCollapsed(QListViewItem* item)
       m_openLoans = false;
   }
 }
+
 void KScheduledView::slotSelectSchedule(const QCString& schedule)
 {
   refresh(true, schedule);
 }
 
-void KScheduledView::slotEnterClicked()
+void KScheduledView::slotBriefEnterClicked(const MyMoneySchedule& schedule, const QDate& date)
 {
-  if (!m_selectedSchedule.isEmpty())
-  {
-    try
-    {
-      MyMoneySchedule schedule = MyMoneyFile::instance()->schedule(m_selectedSchedule);
+  Q_UNUSED(date);
 
-      KEnterScheduleDialog *dlg = new KEnterScheduleDialog(this, schedule);
-      if (dlg->exec())
-      {
-        refresh(false/*, schedule.id()*/);
-      }
-    } catch (MyMoneyException *e)
-    {
+  emit scheduleSelected(schedule);
+  emit enterSchedule();
+}
+
+void KScheduledView::slotSetSelectedItem(QListViewItem* item)
+{
+  emit scheduleSelected(MyMoneySchedule());
+  KScheduledListItem* schedItem = static_cast<KScheduledListItem*>(item);
+  if(item) {
+    try {
+      MyMoneySchedule schedule = MyMoneyFile::instance()->schedule(schedItem->scheduleId());
+      emit scheduleSelected(schedule);
+      m_selectedSchedule = schedItem->scheduleId();
+    } catch(MyMoneyException* e) {
+      qDebug("KScheduledView::slotSetSelectedItem: %s", e->what().data());
       delete e;
     }
   }
 }
 
-void KScheduledView::slotBriefEnterClicked(const MyMoneySchedule& schedule, const QDate& date)
-{
-  KEnterScheduleDialog *dlg = new KEnterScheduleDialog(this, schedule, date);
-  if (dlg->exec())
-  {
-    refresh(false);
-  }
-}
-
-void KScheduledView::slotSetSelectedItem(QListViewItem* item)
-{
-  KScheduledListItem* schedItem = static_cast<KScheduledListItem*>(item);
-  if(item)
-    m_selectedSchedule = schedItem->scheduleId();
-}
-
-void KScheduledView::update(const QCString& /* id */)
-{
-  KScheduledListItem *p = dynamic_cast<KScheduledListItem*>(m_qlistviewScheduled->selectedItem());
-  QCString schedId;
-  if(p)
-    schedId = p->scheduleId();
-  refresh(true, schedId);
-}
-
 
 #include "kscheduledview.moc"
