@@ -37,9 +37,10 @@ class KPushButton;
 // ----------------------------------------------------------------------------
 // Project Includes
 
+#include <kmymoney/kmymoneylineedit.h>
+#include <kmymoney/mymoneysecurity.h>
 class MyMoneyMoney;
 class kMyMoneyCalculator;
-#include "../mymoney/mymoneysecurity.h"
 
 #if KDE_VERSION <= KDE_MAKE_VERSION(3,1,0)
   #define KDoubleValidator QDoubleValidator
@@ -90,15 +91,22 @@ public:
 class kMyMoneyEdit : public QHBox
 {
   Q_OBJECT
+  Q_PROPERTY(bool calculatorButtonVisibility READ isCalculatorButtonVisible WRITE setCalculatorButtonVisible);
+  Q_PROPERTY(bool resetButtonVisibility READ isResetButtonVisible WRITE setResetButtonVisible);
+  Q_PROPERTY(bool allowEmpty READ isEmptyAllowed WRITE setAllowEmpty);
+  Q_PROPERTY(bool readOnly READ isReadOnly WRITE setReadOnly )
 
 private:
   QString previousText; // keep track of what has been typed
   QString m_text;       // keep track of what was the original value
   kMyMoneyCalculator* m_calculator;
   QVBox*              m_calculatorFrame;
-  KLineEdit*          m_edit;
+  kMyMoneyLineEdit*   m_edit;
   KPushButton*        m_calcButton;
+  KPushButton*        m_resetButton;
   int                 m_prec;
+  bool                calculatorButtonVisibility;
+  bool                allowEmpty;
 
 private:
   /**
@@ -133,26 +141,28 @@ protected slots:
   void slotCalculatorOpen(void);
 
 public:
-  kMyMoneyEdit(QWidget *parent=0, const char *name=0, const int prec = -1);
+  kMyMoneyEdit(QWidget *parent=0, const char *name=0, const int prec = -2);
   kMyMoneyEdit(const MyMoneySecurity& eq, QWidget *parent=0, const char *name=0);
   ~kMyMoneyEdit();
 
   /**
     * @deprecated Use value() instead
     */
-  MyMoneyMoney getMoneyValue(void) KDE_DEPRECATED;
+  // MyMoneyMoney getMoneyValue(void) KDE_DEPRECATED;
 
   MyMoneyMoney value(void) const;
 
   void setValue(const MyMoneyMoney& value);
 
-  void resetText(void);
-
   const bool isValid(void) const;
 
   virtual bool eventFilter(QObject * , QEvent * );
 
-  QString text(void) const { return m_edit->text(); };
+  /**
+    * This method returns the value of the edit field in "numerator/denominator" format.
+    * If you want to get the text of the edit field, use lineedit()->text() instead.
+    */
+  QString text(void) const { return value().toString(); };
 
   void setMinimumWidth(int w) { m_edit->setMinimumWidth(w); };
 
@@ -174,6 +184,39 @@ public:
 
   QWidget* focusWidget(void) const;
 
+  /**
+    * This method allows to modify the behavior of the widget
+    * such that it accepts an empty value (all blank) or not.
+    * The default is to not accept an emtpy input and to
+    * convert an empty field into 0.00 upon loss of focus.
+    *
+    * @param allowed if @a true, empty input is allowed, if @a false
+    *                emtpy input will be converted to 0.00
+    */
+  void setAllowEmpty(bool allowed = true);
+
+  /** Overloaded for internal reasons. The API is not affected. */
+  void setValidator(const QValidator* v);
+
+  bool isCalculatorButtonVisible(void) const;
+
+  bool isResetButtonVisible(void) const;
+
+  bool isEmptyAllowed(void) const;
+
+  KLineEdit* lineedit(void) const;
+
+  void setHint(const QString& hint) const;
+
+  bool isReadOnly(void) const;
+
+public slots:
+  void loadText(const QString& text);
+  void resetText(void);
+  void clearText(void);
+
+  void setText(const QString& txt) { setValue(MyMoneyMoney(txt)); };
+
   /**
     * This method allows to show/hide the calculator button of the widget.
     * The parameter @p show controls the behavior. Default is to show the
@@ -181,32 +224,17 @@ public:
     *
     * @param show if true, button is shown, if false it is hidden
     */
-  void showCalculatorButton(const bool show = true);
+  void setCalculatorButtonVisible(const bool show);
 
-  /**
-    * This method allows to hide the calculator button. It is
-    * provided as convenience function and is equivilant to @p
-    * showCalculatorButton(false).
-    */
-  void hideCalculatorButton(void) { showCalculatorButton(false); }
+  void setResetButtonVisible(const bool show);
 
-  /** Overloaded for internal reasons. The API is not affected. */
-  void setValidator(const QValidator* v);
-
-  KLineEdit* lineedit(void) const;
-
-public slots:
-  void loadText(const QString& text);
-  void setReadOnly(bool ro) { m_edit->setReadOnly(ro); };
-  void setText(const QString& txt) { m_edit->setText(txt); };
+  void setReadOnly(bool readOnly);
 
 signals: // Signals
-
-  /** signal is sent, when the tab key is pressed */
-  void signalTab();
-  /** signal is sent, when the Back-tab (Shift-Tab) key is pressed */
-  void signalBackTab();
-  /** signal is sent, when amount has been changed by user */
+  /**
+    * This signal is sent, when the focus leaves this widget and
+    * the amount has been changed by user during this session.
+    */
   void valueChanged(const QString& text);
 
   void textChanged(const QString& text);
